{% load i18n %}
{% spaceless %}
<div id="openassessment__message__{{ xblock_id }}" class="openassessment__message message">
    <h4 class="message__title">{% trans "Status" %}</h4>
    <div class="message__content">
        <p>
            {% if is_team_assignment %}
                {% trans "Your team’s submission has been cancelled. Your team will receive a grade of zero unless course staff resets your assessment to allow the team to resubmit a response." %}
            {% else %}
                {% trans "Your submission has been cancelled. You will receive a grade of zero unless course staff resets your assessment to allow you to resubmit a response." %}
            {% endif %}
        </p>
    </div>
</div>
{% endspaceless %}
