{% load i18n %}
{% spaceless %}
<li class="openassessment_training_example">

    <div class="openassessment_training_example_header view-outline">
        <h6 class="openassessment_training_example_header_title">
            {% trans "Scored Response" %}
        </h6>
        <div class="openassessment_training_example_remove">
            <h2>{% trans "Remove" %}</h2>
        </div>
    </div>

    <div class="openassessment_training_example_body">
        <div class="openassessment_training_example_scored_rubric wrapper-comp-settings">
            <h2>{% trans "Response Score" %}</h2>
            <ol class="openassessment_training_example_criteria_selections list-input settings-list">
                {% for criterion in example.criteria %}
                    {%  if criterion.options %}
                        {% include "openassessmentblock/edit/oa_training_example_criterion.html" with criterion=criterion %}
                    {% endif %}
                {% endfor %}
            </ol>
        </div>

        <div class="openassessment_training_example_essay_wrapper">
            <h2>{% trans "Response" %}</h2>
            <ol class="openassessment_training_example_essay">
                {% for part in example.answer.parts %}
                    <li class="openassessment_training_example_essay_part">
                        <textarea maxlength="100000">{{ part.text }}</textarea>
                    </li>
                {% endfor %}
            </ol>
        </div>
    </div>
</li>
{% endspaceless %}
