{%  extends "openassessmentblock/response/oa_response.html" %}
{% load i18n %}

{% block list_item %}
<li id="openassessment__response__{{ xblock_id }}"
    class="openassessment__steps__step step--response is--empty is--unavailable ui-slidable__container"
    tabindex="-1">
{% endblock %}

{% block button %}
    <button class="ui-slidable" aria-expanded="false" disabled/>
{% endblock %}

{% block title %}
<span class="step__status">
  <span id="oa_step_status_response" class="step__status__value">
    <span class="copy">{% trans "Not Available" %}</span>
  </span>
</span>
{% endblock %}

{% block body %}
{% endblock %}
