{% load i18n %}
{% spaceless %}
<div id="openassessment__message__{{ xblock_id }}" class="openassessment__message message">
    <h4 class="message__title">{% trans "Instructions Unavailable" %} </h4>
    <div class="message__content">
        <p> {% trans "The instructions for this step could not be loaded." %}</p>
    </div>
</div>
{% endspaceless %}
