# Generated by Django 2.2.10 on 2020-03-27 19:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('assessment', '0005_add_filename_to_sharedupload'),
    ]

    operations = [
        migrations.CreateModel(
            name='TeamStaffWorkflow',
            fields=[
                ('staffworkflow_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='assessment.StaffWorkflow')),
                ('team_submission_uuid', models.CharField(max_length=128, unique=True)),
            ],
            bases=('assessment.staffworkflow',),
        ),
    ]
