{% extends "openassessmentblock/student_training/student_training.html" %}
{% load i18n %}
{% load tz %}

{% block list_item %}
    <li class="openassessment__steps__step step--student-training ui-slidable__container"
        tabindex="-1">
{% endblock %}

{% block button %}
    <button class="ui-slidable" aria-expanded="false" disabled>
{% endblock %}

{% block title %}
  <span class="step__status">
    <span class="step__status__value">
        <span class="icon fa fa-exclamation-triangle" aria-hidden="true"></span>
        <span class="copy">{% trans "Cancelled" %}</span>
    </span>
  </span>
{% endblock %}

{% block body %}
{% endblock %}
