{% load i18n %}
{% spaceless %}
<div id="openassessment__message__{{ xblock_id }}" class="openassessment__message message">
    <h4 class="message__title">{% trans "Status" %}</h4>
    <div class="message__content">
        <p>
            {% if not_yet_open %}
                {% trans "This task is not yet available. Check back to complete the assignment once this section has opened." %}
            {% else %}
                {% trans "This assignment has closed. One or more deadlines for this assignment have passed. You will receive an incomplete grade for this assignment." %}
            {% endif %}
        </p>
    </div>
</div>
{% endspaceless %}
