@charset "UTF-8";
/* Neat 1.7.0
 * http://neat.bourbon.io
 * Copyright 2012-2014 thoughtbot, inc.
 * MIT License */
html {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}

*, *:before, *:after {
  -webkit-box-sizing: inherit;
  -moz-box-sizing: inherit;
  box-sizing: inherit;
}

.wrapper--xblock {
  font-family: sans-serif;
  /* 1 */
  -ms-text-size-adjust: 100%;
  /* 2 */
  -webkit-text-size-adjust: 100%;
  /* 2 */
  /* HTML5 display definitions
     ========================================================================== */
  /**
   * Correct `block` display not defined in IE 8/9.
   */
  /**
   * 1. Correct `inline-block` display not defined in IE 8/9.
   * 2. Normalize vertical alignment of `progress` in Chrome, Firefox, and Opera.
   */
  /**
   * Prevent modern browsers from displaying `audio` without controls.
   * Remove excess height in iOS 5 devices.
   */
  /**
   * Address `[hidden]` styling not present in IE 8/9.
   * Hide the `template` element in IE, Safari, and Firefox < 22.
   */
  /* Links
     ========================================================================== */
  /**
   * Remove the gray background color from active links in IE 10.
   */
  /* Text-level semantics
     ========================================================================== */
  /**
   * Address styling not present in IE 8/9, Safari 5, and Chrome.
   */
  /**
   * Address style set to `bolder` in Firefox 4+, Safari 5, and Chrome.
   */
  /**
   * Address styling not present in Safari 5 and Chrome.
   */
  /**
   * Address variable `h1` font-size and margin within `section` and `article`
   * contexts in Firefox 4+, Safari 5, and Chrome.
   */
  /**
   * Address styling not present in IE 8/9.
   */
  /**
   * Address inconsistent and variable font size in all browsers.
   */
  /**
   * Prevent `sub` and `sup` affecting `line-height` in all browsers.
   */
  /* Embedded content
     ========================================================================== */
  /**
   * Remove border when inside `a` element in IE 8/9.
   */
  /**
   * Correct overflow displayed oddly in IE 9.
   */
  /* Grouping content
     ========================================================================== */
  /**
   * Address margin not present in IE 8/9 and Safari 5.
   */
  /**
   * Address differences between Firefox and other browsers.
   */
  /**
   * Contain overflow in all browsers.
   */
  /**
   * Address odd `em`-unit font size rendering in all browsers.
   */
  /* Forms
     ========================================================================== */
  /**
   * Known limitation: by default, Chrome and Safari on OS X allow very limited
   * styling of `select`, unless a `border` property is set.
   */
  /**
   * 1. Correct color not being inherited.
   *    Known issue: affects color of disabled elements.
   * 2. Correct font properties not being inherited.
   * 3. Address margins set differently in Firefox 4+, Safari 5, and Chrome.
   */
  /**
   * Address `overflow` set to `hidden` in IE 8/9/10.
   */
  /**
   * Address inconsistent `text-transform` inheritance for `button` and `select`.
   * All other form control elements do not inherit `text-transform` values.
   * Correct `button` style inheritance in Firefox, IE 8+, and Opera
   * Correct `select` style inheritance in Firefox.
   */
  /**
   * 1. Avoid the WebKit bug in Android 4.0.* where (2) destroys native `audio`
   *    and `video` controls.
   * 2. Correct inability to style clickable `input` types in iOS.
   * 3. Improve usability and consistency of cursor style between image-type
   *    `input` and others.
   */
  /**
   * Re-set default cursor for disabled elements.
   */
  /**
   * Set opacity on buttons to 50% (which matches LMS styling).
   */
  /**
   * Remove inner padding and border in Firefox 4+.
   */
  /**
   * Address Firefox 4+ setting `line-height` on `input` using `!important` in
   * the UA stylesheet.
   */
  /**
   * It's recommended that you don't attempt to style these elements.
   * Firefox's implementation doesn't respect box-sizing, padding, or width.
   *
   * 1. Address box sizing set to `content-box` in IE 8/9/10.
   * 2. Remove excess padding in IE 8/9/10.
   */
  /**
   * Fix the cursor style for Chrome's increment/decrement buttons. For certain
   * `font-size` values of the `input`, it causes the cursor style of the
   * decrement button to change from `default` to `text`.
   */
  /**
   * 1. Address `appearance` set to `searchfield` in Safari 5 and Chrome.
   * 2. Address `box-sizing` set to `border-box` in Safari 5 and Chrome
   *    (include `-moz` to future-proof).
   */
  /**
   * Remove inner padding and search cancel button in Safari and Chrome on OS X.
   * Safari (but not Chrome) clips the cancel button when the search input has
   * padding (and `textfield` appearance).
   */
  /**
   * Define consistent border, margin, and padding.
   */
  /**
   * 1. Correct `color` not being inherited in IE 8/9.
   * 2. Remove padding so people aren't caught out if they zero out fieldsets.
   */
  /**
   * Remove default vertical scrollbar in IE 8/9.
   */
  /**
   * Don't inherit the `font-weight` (applied by a rule above).
   * NOTE: the default cannot safely be changed in Chrome and Safari on OS X.
   */
  /* Tables
     ========================================================================== */
  /**
   * Remove most spacing between table cells.
   */
}
.wrapper--xblock article,
.wrapper--xblock aside,
.wrapper--xblock details,
.wrapper--xblock figcaption,
.wrapper--xblock figure,
.wrapper--xblock footer,
.wrapper--xblock header,
.wrapper--xblock hgroup,
.wrapper--xblock main,
.wrapper--xblock nav,
.wrapper--xblock section,
.wrapper--xblock summary {
  display: block;
}
.wrapper--xblock audio,
.wrapper--xblock canvas,
.wrapper--xblock progress,
.wrapper--xblock video {
  display: inline-block;
  /* 1 */
  vertical-align: baseline;
  /* 2 */
}
.wrapper--xblock audio:not([controls]) {
  display: none;
  height: 0;
}
.wrapper--xblock [hidden],
.wrapper--xblock template {
  display: none;
}
.wrapper--xblock a {
  background: transparent;
}
.wrapper--xblock abbr[title] {
  border-bottom: 1px dotted;
}
.wrapper--xblock b,
.wrapper--xblock strong {
  font-weight: bold;
}
.wrapper--xblock dfn {
  font-style: italic;
}
.wrapper--xblock h1 {
  font-size: 2em;
  margin: 0.67em 0;
}
.wrapper--xblock mark {
  background: #ff0;
  color: #000;
}
.wrapper--xblock small {
  font-size: 80%;
}
.wrapper--xblock sub,
.wrapper--xblock sup {
  font-size: 75%;
  line-height: 0;
  position: relative;
  vertical-align: baseline;
}
.wrapper--xblock sup {
  top: -0.5em;
}
.wrapper--xblock sub {
  bottom: -0.25em;
}
.wrapper--xblock img {
  border: 0;
}
.wrapper--xblock svg:not(:root) {
  overflow: hidden;
}
.wrapper--xblock figure {
  margin: 1em 40px;
}
.wrapper--xblock hr {
  -moz-box-sizing: content-box;
  box-sizing: content-box;
  height: 0;
}
.wrapper--xblock pre {
  overflow: auto;
}
.wrapper--xblock code,
.wrapper--xblock kbd,
.wrapper--xblock pre,
.wrapper--xblock samp {
  font-family: monospace, monospace;
  font-size: 1em;
}
.wrapper--xblock button,
.wrapper--xblock input,
.wrapper--xblock optgroup,
.wrapper--xblock select,
.wrapper--xblock textarea {
  color: inherit;
  /* 1 */
  font: inherit;
  /* 2 */
  margin: 0;
  /* 3 */
}
.wrapper--xblock button {
  overflow: visible;
}
.wrapper--xblock button,
.wrapper--xblock select {
  text-transform: none;
}
.wrapper--xblock button,
.wrapper--xblock html input[type=button],
.wrapper--xblock input[type=reset],
.wrapper--xblock input[type=submit] {
  -webkit-appearance: button;
  /* 2 */
  cursor: pointer;
  /* 3 */
}
.wrapper--xblock button[disabled],
.wrapper--xblock html input[disabled] {
  cursor: default;
}
.wrapper--xblock button[disabled] {
  opacity: 0.5;
}
.wrapper--xblock button::-moz-focus-inner,
.wrapper--xblock input::-moz-focus-inner {
  border: 0;
  padding: 0;
}
.wrapper--xblock input {
  line-height: normal;
}
.wrapper--xblock input[type=checkbox],
.wrapper--xblock input[type=radio] {
  box-sizing: border-box;
  /* 1 */
  padding: 0;
  /* 2 */
}
.wrapper--xblock input[type=number]::-webkit-inner-spin-button,
.wrapper--xblock input[type=number]::-webkit-outer-spin-button {
  height: auto;
}
.wrapper--xblock input[type=search] {
  -webkit-appearance: textfield;
  /* 1 */
  -moz-box-sizing: content-box;
  -webkit-box-sizing: content-box;
  /* 2 */
  box-sizing: content-box;
}
.wrapper--xblock input[type=search]::-webkit-search-cancel-button,
.wrapper--xblock input[type=search]::-webkit-search-decoration {
  -webkit-appearance: none;
}
.wrapper--xblock fieldset {
  border: 1px solid #c0c0c0;
  margin: 0 2px;
  padding: 0.35em 0.625em 0.75em 0.625em;
}
.wrapper--xblock legend {
  border: 0;
  /* 1 */
  padding: 0;
  /* 2 */
}
.wrapper--xblock textarea {
  overflow: auto;
}
.wrapper--xblock optgroup {
  font-weight: bold;
}
.wrapper--xblock table {
  border-collapse: collapse;
  border-spacing: 0;
}
.wrapper--xblock td,
.wrapper--xblock th {
  padding: 0;
}
.wrapper--xblock * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
.wrapper--xblock div, .wrapper--xblock span, .wrapper--xblock applet, .wrapper--xblock object, .wrapper--xblock iframe,
.wrapper--xblock h1, .wrapper--xblock h2, .wrapper--xblock h3, .wrapper--xblock h4, .wrapper--xblock h5, .wrapper--xblock h6, .wrapper--xblock p, .wrapper--xblock blockquote, .wrapper--xblock pre,
.wrapper--xblock a, .wrapper--xblock abbr, .wrapper--xblock acronym, .wrapper--xblock address, .wrapper--xblock big, .wrapper--xblock cite, .wrapper--xblock code,
.wrapper--xblock del, .wrapper--xblock dfn, .wrapper--xblock em, .wrapper--xblock img, .wrapper--xblock ins, .wrapper--xblock kbd, .wrapper--xblock q, .wrapper--xblock s, .wrapper--xblock samp,
.wrapper--xblock small, .wrapper--xblock strike, .wrapper--xblock strong, .wrapper--xblock sub, .wrapper--xblock sup, .wrapper--xblock tt, .wrapper--xblock var,
.wrapper--xblock b, .wrapper--xblock u, .wrapper--xblock i, .wrapper--xblock center,
.wrapper--xblock dl, .wrapper--xblock dt, .wrapper--xblock dd, .wrapper--xblock ol, .wrapper--xblock ul, .wrapper--xblock li,
.wrapper--xblock fieldset, .wrapper--xblock form, .wrapper--xblock label, .wrapper--xblock legend,
.wrapper--xblock table, .wrapper--xblock caption, .wrapper--xblock tbody, .wrapper--xblock tfoot, .wrapper--xblock thead, .wrapper--xblock tr, .wrapper--xblock th, .wrapper--xblock td,
.wrapper--xblock article, .wrapper--xblock aside, .wrapper--xblock canvas, .wrapper--xblock details, .wrapper--xblock figcaption, .wrapper--xblock figure,
.wrapper--xblock footer, .wrapper--xblock header, .wrapper--xblock hgroup, .wrapper--xblock menu, .wrapper--xblock nav, .wrapper--xblock section, .wrapper--xblock summary,
.wrapper--xblock time, .wrapper--xblock mark, .wrapper--xblock audio, .wrapper--xblock video {
  margin: 0;
  vertical-align: baseline;
  border: 0;
  padding: 0;
  font-weight: 400;
}
.wrapper--xblock abbr[title] {
  border-bottom: none;
  text-decoration: none;
  cursor: help;
}
.openassessment .answer--feedback .feedback:last-child, .openassessment .submission__peer-evaluations__questions .question__answers:last-child, .openassessment .submission__peer-evaluations__questions .question:last-child, .openassessment .submission__answer__part__prompt__copy > :last-child, .openassessment .openassessment__steps__step .step__content__section:last-child, .openassessment .ui-staff__content__section:last-child, .step--response .step__rubric .question__answers .answer:last-child, .step--student-training .student-training__assessment .question__answers .answer:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer:last-child, .openassessment .staff-assessment__assessment .question__answers .answer:last-child, .step--response .step__rubric .question__answers .answer--feedback:last-child, .step--student-training .student-training__assessment .question__answers .answer--feedback:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback:last-child, .openassessment .staff-assessment__assessment .question__answers .answer--feedback:last-child, .step--response .step__rubric .assessment__rubric__question:last-child, .step--student-training .student-training__assessment .assessment__rubric__question:last-child, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question:last-child, .openassessment .staff-assessment__assessment .assessment__rubric__question:last-child, .wrapper--xblock .message .message__content p:last-child, .wrapper--xblock .field--radio .option:last-child, .wrapper--xblock .field--checkbox .option:last-child, .list--actions:last-child,
.list--docs:last-child,
.list--steps:last-child,
.list--fields:last-child,
.list--controls:last-child,
.list--help:last-child,
.list--options:last-child,
.nav--wizard:last-child,
.list--tips:last-child,
.field--group:last-child {
  margin-bottom: 0;
  border-bottom: none;
  padding-bottom: 0;
  margin-right: 0;
  border-right: none;
  padding-right: 0;
}

.openassessment .answer--feedback .feedback, .openassessment .submission__peer-evaluations__questions .answer, .openassessment .submission__peer-evaluations__questions .question__answers, .openassessment .submission__peer-evaluations__questions, .openassessment .step--peer-assessment .list--peer-assessments, .openassessment .openassessment__steps, .list--actions,
.list--docs,
.list--steps,
.list--fields,
.list--controls,
.list--help,
.list--options,
.nav--wizard,
.list--tips,
.field--group {
  list-style: none;
  margin: 0;
  padding: 0;
  text-indent: 0;
}

.openassessment .step--response .response__submission__content label, .openassessment .submission__answer__part__prompt__title, .system__element, .sr,
hr.divider,
.nav--contents {
  border: 0;
  clip: rect(0 0 0 0);
  height: 1px;
  margin: -1px;
  overflow: hidden;
  padding: 0;
  position: absolute;
  width: 1px;
}

.ui-hint {
  cursor: pointer;
}

.wrapper--xblock .is--transitioning, .is--hidden {
  pointer-events: none;
  outline: none;
}

.is--hidden {
  display: none;
  visibility: hidden;
}

.wrapper--xblock .message--error, .wrapper--xblock [placeholder]:focus::-webkit-input-placeholder {
  -webkit-transition: opacity 0.5s ease-in-out;
  -moz-transition: opacity 0.5s ease-in-out;
  transition: opacity 0.5s ease-in-out;
}

.openassessment .submission__peer-evaluations__questions .answer__value__value .ui-hint {
  -webkit-transition: color 0.5s ease-in-out;
  -moz-transition: color 0.5s ease-in-out;
  transition: color 0.5s ease-in-out;
}

.openassessment .ui-hint:after {
  box-shadow: 0 1px 2px 0 rgba(3, 3, 3, 0.4);
}

@-webkit-keyframes pulse {
  0% {
    opacity: 0.5;
  }
  50% {
    opacity: 1;
  }
  100% {
    opacity: 0.5;
  }
}
@-moz-keyframes pulse {
  0% {
    opacity: 0.5;
  }
  50% {
    opacity: 1;
  }
  100% {
    opacity: 0.5;
  }
}
@-o-keyframes pulse {
  0% {
    opacity: 0.5;
  }
  50% {
    opacity: 1;
  }
  100% {
    opacity: 0.5;
  }
}
@keyframes pulse {
  0% {
    opacity: 0.5;
  }
  50% {
    opacity: 1;
  }
  100% {
    opacity: 0.5;
  }
}
.wrapper--xblock .problem__header {
  margin-bottom: 20px;
  color: #414141;
}

.wrapper--xblock {
  background: transparent;
}

.openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title .step__counter:before, .staff-info__student .staff-info__student__report .title, .openassessment .answer--feedback .feedback__source, .openassessment .submission__peer-evaluations__questions .answer__value__value, .openassessment .submission__peer-evaluations__questions .answer__source, .openassessment .submission__peer-evaluations__questions .question__score__potential .unit, .openassessment .submission__peer-evaluations__questions .question__score__potential, .openassessment .submission__peer-evaluations__questions .question__score__value, .openassessment .step--grade .step__title .grade__value__title .grade__value__potential, .openassessment .step--grade .step__title .grade__value__title .grade__value__earned, .openassessment .step--peer-assessment .peer-assessment__display__header span, .openassessment .openassessment__steps__step .step__title .step__counter:before, .openassessment .staff-assessment__display__header span, .openassessment .staff-info__status__table .value, .openassessment .staff-info__classifierset__table .value, .openassessment .staff-info__summary .value, .openassessment .staff-info__title__copy, .step--response .step__rubric .question__answers .wrapper--input .answer__label, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .submission__peer-evaluations__questions .answer__score, .openassessment .submission__peer-evaluations__questions .answer__score__value, .openassessment .submission__peer-evaluations__questions .answer__source .answer__source__value-with-points, .openassessment .submission__peer-evaluations__questions .question__score, .openassessment .step--response .response__submission__content .tip, .openassessment .openassessment__steps__step .step__status .copy, .openassessment .ui-staff .staff__grade__control .staff__grade__status .copy, .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .response__submission__label, .openassessment .step--peer-assessment .peer-assessment__assessment .response__submission__label, .openassessment .step--response .response__submission__actions .response__submission__label, .openassessment .step--response .response__submission__content .response__submission__label, .step--response .step__rubric #read_only___assessment__rubric__header, .step--student-training .student-training__display__title, .openassessment .submission__feedback__title__copy, .openassessment .submission__feedback__title, .openassessment .submission__peer-evaluations__questions .question__title__copy, .openassessment .submission__peer-evaluations__title, .openassessment .submission__answer__display__title, .openassessment .step--self-assessment .self-assessment__display__title, .openassessment .step--peer-assessment .peer-assessment__display__header .peer-assessment__display__title, .openassessment .submission__upload__files__title,
.openassessment .submission__answer__part__tip__title, .openassessment .submission__answer__part__text__title, .openassessment .staff-assessment__display__header .staff-assessment__display__title, .openassessment .ui-staff__subtitle, .openassessment .ui-staff__title, .step--response .step__rubric .assessment__rubric__question .question__title__copy, .step--student-training .student-training__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .staff-assessment__assessment .assessment__rubric__question .question__title__copy, .wrapper--xblock .message .message__title, .openassessment .step--grade .step__title .grade__value__title, .openassessment .openassessment__steps__step .step__title .step__label, .openassessment .ui-staff .staff__grade__control .staff__grade__title .staff__grade__show-form, .wrapper--xblock .is--transitioning .transition__status .copy, .wrapper--xblock .field--radio .option__label, .wrapper--xblock .field--checkbox .option__label, .wrapper--xblock .field-group__label, .wrapper--xblock .field--textarea .label, .wrapper--xblock .field--textarea label, .wrapper--xblock input[type=text],
.wrapper--xblock input[type=email],
.wrapper--xblock input[type=password],
.wrapper--xblock textarea, .step--response .delete__uploaded__file, .step--response .action--upload, .openassessment .action--submit, .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save, .openassessment .list--actions .action--submit, .openassessment .openassessment_student_info_form .action--submit-username, .wrapper--xblock strong, .wrapper--xblock .emphasis {
  font-weight: 600;
}

.openassessment .openassessment__title, .wrapper--xblock input[type=text]:-ms-input-placeholder,
.wrapper--xblock input[type=email]:-ms-input-placeholder,
.wrapper--xblock input[type=password]:-ms-input-placeholder,
.wrapper--xblock textarea:-ms-input-placeholder, .wrapper--xblock input[type=text]::-moz-placeholder,
.wrapper--xblock input[type=email]::-moz-placeholder,
.wrapper--xblock input[type=password]::-moz-placeholder,
.wrapper--xblock textarea::-moz-placeholder, .wrapper--xblock input[type=text]:-moz-placeholder,
.wrapper--xblock input[type=email]:-moz-placeholder,
.wrapper--xblock input[type=password]:-moz-placeholder,
.wrapper--xblock textarea:-moz-placeholder, .wrapper--xblock input[type=text]::-webkit-input-placeholder,
.wrapper--xblock input[type=email]::-webkit-input-placeholder,
.wrapper--xblock input[type=password]::-webkit-input-placeholder,
.wrapper--xblock textarea::-webkit-input-placeholder {
  font-weight: 400;
}

#openassessment-editor .openassessment_container_guide, .openassessment .ui-hint:after, .openassessment .staff-info__status__table th, .openassessment .staff-info__classifierset__table th, .openassessment .submission__peer-evaluations__questions .answer__score, .openassessment .submission__peer-evaluations__questions .answer__score__value, .openassessment .submission__peer-evaluations__questions .answer__source .answer__source__value-with-points, .openassessment .submission__peer-evaluations__questions .question__score, .openassessment .step--response .response__submission__content .tip, .openassessment .openassessment__steps__step .step__status .copy, .openassessment .ui-staff .staff__grade__control .staff__grade__status .copy, .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .response__submission__label, .openassessment .step--peer-assessment .peer-assessment__assessment .response__submission__label, .openassessment .step--response .response__submission__actions .response__submission__label, .openassessment .step--response .response__submission__content .response__submission__label, .wrapper--xblock .field .tip, .openassessment .answer--feedback .feedback__source, .openassessment .submission__peer-evaluations__questions .answer__source {
  font-size: 12px;
  line-height: 18px;
}

.openassessment .openassessment__steps__step .step__status .icon, .step--response .delete__uploaded__file, .step--response .action--upload, .openassessment .action--submit, .openassessment .answer--feedback .answer--feedback__title, .openassessment .list--actions .action--submit, .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save, .step--student-training .student-training__display__response, .openassessment .answer--feedback .feedback__value, .openassessment .submission__answer__display__content, .openassessment .self-assessment__display__image,
.openassessment .peer-assessment__display__image,
.openassessment .staff-assessment__display__image,
.openassessment .submission__answer__display__image, .openassessment .step--self-assessment .self-assessment__display__response, .openassessment .step--peer-assessment .peer-assessment__display__response, .openassessment .staff-assessment__display__response, .openassessment .staff-info__status__table, .openassessment .staff-info__classifierset__table, .step--response .step__rubric .question__answers .answer__tip, .step--student-training .student-training__assessment .question__answers .answer__tip, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip, .openassessment .staff-assessment__assessment .question__answers .answer__tip, .step--response .step__rubric .question__answers .wrapper--input .answer__label, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__label, .wrapper--xblock .message .message__content, .wrapper--xblock .field--radio .option__label, .wrapper--xblock .field--checkbox .option__label, .wrapper--xblock .field label, .wrapper--xblock .field .label, .wrapper--xblock .field .field-group__label, .staff-info__student .title--sub, .openassessment .openassessment__steps__step .step__title .step__deadline, .step--response .step__rubric #read_only___assessment__rubric__header, .step--student-training .student-training__display__title, .openassessment .submission__feedback__title__copy, .openassessment .submission__feedback__title, .openassessment .submission__peer-evaluations__questions .question__title__copy, .openassessment .submission__peer-evaluations__title, .openassessment .submission__answer__display__title, .openassessment .step--self-assessment .self-assessment__display__title, .openassessment .step--peer-assessment .peer-assessment__display__header .peer-assessment__display__title, .openassessment .submission__upload__files__title,
.openassessment .submission__answer__part__tip__title, .openassessment .submission__answer__part__text__title, .openassessment .staff-assessment__display__header .staff-assessment__display__title, .openassessment .ui-staff__subtitle, .openassessment .ui-staff__title, .step--response .step__rubric .assessment__rubric__question .question__title__copy, .step--student-training .student-training__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .staff-assessment__assessment .assessment__rubric__question .question__title__copy, .wrapper--xblock .message .message__title {
  font-size: 14px;
  line-height: 21px;
}

.openassessment .list--actions .action--submit .icon, .openassessment .submission__feedback__instructions, .openassessment .step--grade .grade__summary, .openassessment .step--grade .grade__value__description, .openassessment .submission__answer__part__prompt__copy, .openassessment .openassessment__steps__step .step__instruction, .wrapper--xblock .problem__header {
  font-size: 16px;
  line-height: 24px;
}

.step--response .step__rubric #read-only-rubric-button, .openassessment .submission__feedback__title .icon, .openassessment .submission__peer-evaluations__questions .question__title .icon, .step--response .step__rubric .assessment__rubric__question .question__title .icon, .step--student-training .student-training__assessment .assessment__rubric__question .question__title .icon, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question .question__title .icon, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question .question__title .icon, .openassessment .staff-assessment__assessment .assessment__rubric__question .question__title .icon, .openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title .step__counter:before, .staff-info__student .title, .openassessment .submission__peer-evaluations__questions .answer__value__value, .openassessment .openassessment__steps__step .step__title .step__counter:before, .openassessment .staff-info__status__table .title, .openassessment .staff-info__classifierset__table .title, .openassessment .staff-info__summary .label, .openassessment .staff-info__summary .value, .openassessment .step--grade .step__title .grade__value__title, .openassessment .openassessment__steps__step .step__title .step__label, .openassessment .ui-staff .staff__grade__control .staff__grade__title .staff__grade__show-form, .wrapper--xblock .is--transitioning .transition__status .copy {
  font-size: 18px;
  line-height: 27px;
}

.wrapper--xblock .is--transitioning .transition__status .icon {
  font-size: 22px;
  line-height: 33px;
}

.openassessment .answer--feedback .feedback__source, .openassessment .submission__peer-evaluations__questions .answer__source, .openassessment .staff-info__status__table th, .openassessment .staff-info__classifierset__table th, .openassessment .submission__peer-evaluations__questions .answer__score, .openassessment .submission__peer-evaluations__questions .answer__score__value, .openassessment .submission__peer-evaluations__questions .answer__source .answer__source__value-with-points, .openassessment .submission__peer-evaluations__questions .question__score, .openassessment .step--response .response__submission__content .tip, .openassessment .openassessment__steps__step .step__status .copy, .openassessment .ui-staff .staff__grade__control .staff__grade__status .copy, .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .response__submission__label, .openassessment .step--peer-assessment .peer-assessment__assessment .response__submission__label, .openassessment .step--response .response__submission__actions .response__submission__label, .openassessment .step--response .response__submission__content .response__submission__label, .wrapper--xblock .is--transitioning .transition__status .copy, .wrapper--xblock .problem__header {
  text-transform: uppercase;
  letter-spacing: 0.05rem;
}

.openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title .step__counter:before, .staff-info__student .title, .openassessment .submission__peer-evaluations__questions .answer__value__value, .openassessment .openassessment__steps__step .step__title .step__counter:before, .openassessment .staff-info__status__table .title, .openassessment .staff-info__classifierset__table .title, .openassessment .staff-info__summary .label, .openassessment .staff-info__summary .value, .openassessment .step--grade .step__title .grade__value__title, .openassessment .openassessment__steps__step .step__title .step__label, .openassessment .ui-staff .staff__grade__control .staff__grade__title .staff__grade__show-form, .wrapper--xblock .is--transitioning .transition__status .copy {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.wrapper--xblock .problem__header {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.staff-info__student .title--sub, .openassessment .openassessment__steps__step .step__title .step__deadline, .step--response .step__rubric #read_only___assessment__rubric__header, .step--student-training .student-training__display__title, .openassessment .submission__feedback__title__copy, .openassessment .submission__feedback__title, .openassessment .submission__peer-evaluations__questions .question__title__copy, .openassessment .submission__peer-evaluations__title, .openassessment .submission__answer__display__title, .openassessment .step--self-assessment .self-assessment__display__title, .openassessment .step--peer-assessment .peer-assessment__display__header .peer-assessment__display__title, .openassessment .submission__upload__files__title,
.openassessment .submission__answer__part__tip__title, .openassessment .submission__answer__part__text__title, .openassessment .staff-assessment__display__header .staff-assessment__display__title, .openassessment .ui-staff__subtitle, .openassessment .ui-staff__title, .step--response .step__rubric .assessment__rubric__question .question__title__copy, .step--student-training .student-training__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .staff-assessment__assessment .assessment__rubric__question .question__title__copy, .wrapper--xblock .message .message__title {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.openassessment .answer--feedback .feedback__source, .openassessment .submission__peer-evaluations__questions .answer__source {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.openassessment .submission__feedback__instructions, .openassessment .step--grade .grade__summary, .openassessment .step--grade .grade__value__description, .openassessment .submission__answer__part__prompt__copy, .openassessment .openassessment__steps__step .step__instruction {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.step--student-training .student-training__display__response, .openassessment .answer--feedback .feedback__value, .openassessment .submission__answer__display__content, .openassessment .self-assessment__display__image,
.openassessment .peer-assessment__display__image,
.openassessment .staff-assessment__display__image,
.openassessment .submission__answer__display__image, .openassessment .step--self-assessment .self-assessment__display__response, .openassessment .step--peer-assessment .peer-assessment__display__response, .openassessment .staff-assessment__display__response, .openassessment .staff-info__status__table, .openassessment .staff-info__classifierset__table, .step--response .step__rubric .question__answers .answer__tip, .step--student-training .student-training__assessment .question__answers .answer__tip, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip, .openassessment .staff-assessment__assessment .question__answers .answer__tip, .step--response .step__rubric .question__answers .wrapper--input .answer__label, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__label, .wrapper--xblock .message .message__content, .wrapper--xblock .field--radio .option__label, .wrapper--xblock .field--checkbox .option__label, .wrapper--xblock .field label, .wrapper--xblock .field .label, .wrapper--xblock .field .field-group__label {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.openassessment .ui-hint:after, .openassessment .staff-info__status__table th, .openassessment .staff-info__classifierset__table th, .openassessment .submission__peer-evaluations__questions .answer__score, .openassessment .submission__peer-evaluations__questions .answer__score__value, .openassessment .submission__peer-evaluations__questions .answer__source .answer__source__value-with-points, .openassessment .submission__peer-evaluations__questions .question__score, .openassessment .step--response .response__submission__content .tip, .openassessment .openassessment__steps__step .step__status .copy, .openassessment .ui-staff .staff__grade__control .staff__grade__status .copy, .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .response__submission__label, .openassessment .step--peer-assessment .peer-assessment__assessment .response__submission__label, .openassessment .step--response .response__submission__actions .response__submission__label, .openassessment .step--response .response__submission__content .response__submission__label, .wrapper--xblock .field .tip {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.step--response .delete__uploaded__file, .step--response .action--upload, .openassessment .action--submit, .openassessment .answer--feedback .answer--feedback__title, .openassessment .list--actions .action--submit, .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.openassessment .submission__answer__part__prompt__copy a, .wrapper--xblock .message .message__content a {
  -webkit-transition: color 0.5s ease-in-out, border 0.5s ease-in-out;
  -moz-transition: color 0.5s ease-in-out, border 0.5s ease-in-out;
  transition: color 0.5s ease-in-out, border 0.5s ease-in-out;
  border-bottom: 1px solid transparent;
  color: #0075b4;
  text-decoration: none;
}
.openassessment .submission__answer__part__prompt__copy a:hover, .wrapper--xblock .message .message__content a:hover, .openassessment .submission__answer__part__prompt__copy a:focus, .wrapper--xblock .message .message__content a:focus {
  color: #065683;
  border-color: #065683;
}
.openassessment .submission__answer__part__prompt__copy a:active, .wrapper--xblock .message .message__content a:active, .openassessment .submission__answer__part__prompt__copy a.is--current, .wrapper--xblock .message .message__content a.is--current {
  color: #0075b4;
  border-color: #0075b4;
}

.step--response .delete__uploaded__file, .step--response .action--upload, .openassessment .action--submit, .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save, .openassessment .list--actions .action--submit, .openassessment .openassessment_student_info_form .action--submit-username {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  -webkit-transition: color 0.25s ease-in-out, background 0.25s ease-in-out, box-shadow 0.25s ease-in-out;
  -moz-transition: color 0.25s ease-in-out, background 0.25s ease-in-out, box-shadow 0.25s ease-in-out;
  transition: color 0.25s ease-in-out, background 0.25s ease-in-out, box-shadow 0.25s ease-in-out;
  display: inline-block;
  cursor: pointer;
  text-decoration: none;
}
.step--response .delete__uploaded__file:hover, .step--response .action--upload:hover, .openassessment .action--submit:hover, .openassessment .step--self-assessment .self-assessment__assessment .action--save:hover, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save:hover, .openassessment .step--response .response__submission__actions .action--save:hover, .openassessment .step--response .response__submission__content .action--save:hover, .openassessment .openassessment_student_info_form .action--submit-username:hover, .step--response .delete__uploaded__file:active, .step--response .action--upload:active, .openassessment .action--submit:active, .openassessment .step--self-assessment .self-assessment__assessment .action--save:active, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save:active, .openassessment .step--response .response__submission__actions .action--save:active, .openassessment .step--response .response__submission__content .action--save:active, .openassessment .openassessment_student_info_form .action--submit-username:active, .step--response .delete__uploaded__file:focus, .step--response .action--upload:focus, .openassessment .action--submit:focus, .openassessment .step--self-assessment .self-assessment__assessment .action--save:focus, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save:focus, .openassessment .step--response .response__submission__actions .action--save:focus, .openassessment .step--response .response__submission__content .action--save:focus, .openassessment .openassessment_student_info_form .action--submit-username:focus {
  text-decoration: none;
}
.step--response .delete__uploaded__file .icon, .step--response .action--upload .icon, .openassessment .action--submit .icon, .openassessment .step--self-assessment .self-assessment__assessment .action--save .icon, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save .icon, .openassessment .step--response .response__submission__actions .action--save .icon, .openassessment .step--response .response__submission__content .action--save .icon, .openassessment .openassessment_student_info_form .action--submit-username .icon {
  display: inline-block;
  vertical-align: middle;
}

.step--response .delete__uploaded__file, .step--response .action--upload, .openassessment .action--submit, .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save, .openassessment .list--actions .action--submit, .openassessment .openassessment_student_info_form .action--submit-username {
  border-radius: 4px;
}

.wrapper--openassessment .wrapper--staff-area .wrapper--staff-toolbar .ui-staff__button, .openassessment .ui-slidable__container .ui-slidable, .openassessment .ui-staff .staff__grade__control .staff__grade__title .staff__grade__show-form {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 14px;
  -webkit-appearance: none;
  background-image: none;
  text-shadow: none;
  box-shadow: none;
  border: none;
  border-image: none;
}

.openassessment .list--actions .action--submit, .openassessment .openassessment_student_info_form .action--submit-username {
  padding: 10px 40px;
  background: #0075b4;
  border: 1px solid #0075b4;
  color: white;
}
.openassessment .list--actions .action--submit:hover, .openassessment .openassessment_student_info_form .action--submit-username:hover, .openassessment .list--actions .action--submit:active, .openassessment .openassessment_student_info_form .action--submit-username:active, .openassessment .list--actions .action--submit:focus, .openassessment .openassessment_student_info_form .action--submit-username:focus {
  background: #065683;
  border-color: #065683;
}
.openassessment .list--actions .current.action--submit, .openassessment .openassessment_student_info_form .current.action--submit-username, .openassessment .list--actions .active.action--submit, .openassessment .openassessment_student_info_form .active.action--submit-username {
  background: #0075b4;
  border-color: #0075b4;
}
.openassessment .list--actions [disabled].action--submit, .openassessment .openassessment_student_info_form [disabled].action--submit-username {
  color: #b4b3b3;
  border-color: #b4b3b3;
  background: whitesmoke;
  opacity: 1;
}

.step--response .delete__uploaded__file, .step--response .action--upload, .openassessment .action--submit, .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save {
  padding: 10px 40px;
  background: white;
  border: 1px solid #065683;
  color: #065683;
}
.step--response .delete__uploaded__file:hover, .step--response .action--upload:hover, .openassessment .action--submit:hover, .openassessment .step--self-assessment .self-assessment__assessment .action--save:hover, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save:hover, .openassessment .step--response .response__submission__actions .action--save:hover, .openassessment .step--response .response__submission__content .action--save:hover, .step--response .delete__uploaded__file:active, .step--response .action--upload:active, .openassessment .action--submit:active, .openassessment .step--self-assessment .self-assessment__assessment .action--save:active, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save:active, .openassessment .step--response .response__submission__actions .action--save:active, .openassessment .step--response .response__submission__content .action--save:active, .step--response .delete__uploaded__file:focus, .step--response .action--upload:focus, .openassessment .action--submit:focus, .openassessment .step--self-assessment .self-assessment__assessment .action--save:focus, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save:focus, .openassessment .step--response .response__submission__actions .action--save:focus, .openassessment .step--response .response__submission__content .action--save:focus {
  background: #0075b4;
  border-color: #0075b4;
  color: white;
}
.step--response .current.delete__uploaded__file, .step--response .current.action--upload, .openassessment .current.action--submit, .openassessment .step--self-assessment .self-assessment__assessment .current.action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .current.action--save, .openassessment .step--response .response__submission__actions .current.action--save, .openassessment .step--response .response__submission__content .current.action--save, .step--response .active.delete__uploaded__file, .step--response .active.action--upload, .openassessment .active.action--submit, .openassessment .step--self-assessment .self-assessment__assessment .active.action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .active.action--save, .openassessment .step--response .response__submission__actions .active.action--save, .openassessment .step--response .response__submission__content .active.action--save {
  background: white;
  border-color: #065683;
  color: #065683;
}
.step--response [disabled].delete__uploaded__file, .step--response [disabled].action--upload, .openassessment [disabled].action--submit, .openassessment .step--self-assessment .self-assessment__assessment [disabled].action--save, .openassessment .step--peer-assessment .peer-assessment__assessment [disabled].action--save, .openassessment .step--response .response__submission__actions [disabled].action--save, .openassessment .step--response .response__submission__content [disabled].action--save {
  background: white;
  border-color: #b4b3b3;
  color: #b4b3b3;
  opacity: 1;
}

.wrapper--xblock [placeholder]:focus::-webkit-input-placeholder {
  opacity: 0;
}
.wrapper--xblock input[type=text],
.wrapper--xblock input[type=email],
.wrapper--xblock input[type=password],
.wrapper--xblock textarea {
  border: 1px solid #b9c1c8;
  padding: 10px;
  color: #414141;
}
.wrapper--xblock input[type=text]::-webkit-input-placeholder,
.wrapper--xblock input[type=email]::-webkit-input-placeholder,
.wrapper--xblock input[type=password]::-webkit-input-placeholder,
.wrapper--xblock textarea::-webkit-input-placeholder {
  color: #414141;
}
.wrapper--xblock input[type=text]:-moz-placeholder,
.wrapper--xblock input[type=email]:-moz-placeholder,
.wrapper--xblock input[type=password]:-moz-placeholder,
.wrapper--xblock textarea:-moz-placeholder {
  color: #414141;
}
.wrapper--xblock input[type=text]::-moz-placeholder,
.wrapper--xblock input[type=email]::-moz-placeholder,
.wrapper--xblock input[type=password]::-moz-placeholder,
.wrapper--xblock textarea::-moz-placeholder {
  color: #414141;
}
.wrapper--xblock input[type=text]:-ms-input-placeholder,
.wrapper--xblock input[type=email]:-ms-input-placeholder,
.wrapper--xblock input[type=password]:-ms-input-placeholder,
.wrapper--xblock textarea:-ms-input-placeholder {
  color: #414141;
}
.wrapper--xblock input[type=text]:focus,
.wrapper--xblock input[type=email]:focus,
.wrapper--xblock input[type=password]:focus,
.wrapper--xblock textarea:focus {
  box-shadow: 0 0 6px 0 rgba(29, 157, 217, 0.4), inset 0 0 4px 0 rgba(0, 0, 0, 0.15);
  border-color: black;
}
.wrapper--xblock .field {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}
.wrapper--xblock .field .field-group__label {
  display: block;
  margin-bottom: 10px;
}
.wrapper--xblock .field textarea {
  width: 100%;
}
.wrapper--xblock .field .tip {
  display: block;
  color: #414141;
}
.wrapper--xblock .field--textarea .label, .wrapper--xblock .field--textarea label {
  margin-bottom: 10px;
}
.wrapper--xblock .field-group__label {
  margin-bottom: 10px;
}
.wrapper--xblock .field--radio .option, .wrapper--xblock .field--checkbox .option {
  margin-bottom: 5px;
}
.wrapper--xblock .field--radio .option__input, .wrapper--xblock .field--radio .option__label, .wrapper--xblock .field--checkbox .option__input, .wrapper--xblock .field--checkbox .option__label {
  display: inline-block;
  vertical-align: middle;
}
.wrapper--xblock .field--radio .option__label, .wrapper--xblock .field--checkbox .option__label {
  margin-bottom: 0;
  color: #414141;
}
.wrapper--xblock .field--radio .option__input, .wrapper--xblock .field--checkbox .option__input {
  margin-left: 10px;
}
.wrapper--xblock .field--radio .option__input:checked + .option__label, .wrapper--xblock .field--checkbox .option__input:checked + .option__label {
  color: black;
}

.wrapper--xblock .message {
  margin-bottom: 20px;
  border-radius: 2px;
  padding: 20px 20px;
  background: #f5f6f7;
}
.wrapper--xblock .message .message__title {
  margin-bottom: 5px;
  border-bottom: 2px solid #d9d9d9;
  padding-bottom: 5px;
}
.wrapper--xblock .message .message__content {
  color: #414141;
}
.wrapper--xblock .message .message__content p {
  margin-bottom: 10px;
}
.wrapper--xblock .message--error {
  background: #feeced;
}
.wrapper--xblock .message--error .message__title {
  color: #414141;
  border-bottom-color: #b20610;
}
.wrapper--xblock .message--warning {
  background: #fff9eb;
}
.wrapper--xblock .message--warning .message__title {
  color: #414141;
  border-bottom-color: #ffc01f;
}
.wrapper--xblock .message--complete, .wrapper--xblock .step--student-training .message--correct, .step--student-training .wrapper--xblock .message--correct {
  background: #ecfaec;
}
.wrapper--xblock .message--complete .message__title, .wrapper--xblock .step--student-training .message--correct .message__title, .step--student-training .wrapper--xblock .message--correct .message__title {
  color: #414141;
  border-bottom-color: #008100;
}
.wrapper--xblock .message--incomplete, .wrapper--xblock .step--student-training .message--incorrect, .step--student-training .wrapper--xblock .message--incorrect {
  background: #fff9eb;
}
.wrapper--xblock .message--incomplete .message__title, .wrapper--xblock .step--student-training .message--incorrect .message__title, .step--student-training .wrapper--xblock .message--incorrect .message__title {
  color: #414141;
  border-bottom-color: #ffc01f;
}
.wrapper--xblock .message--error {
  display: none;
  opacity: 0;
}
.wrapper--xblock .has--error .message--error {
  display: block;
  opacity: 1;
}
.wrapper--xblock .message--inline {
  padding: 10px 20px;
  background: #5e5e5e;
}
.wrapper--xblock .message--inline .message__title {
  margin-bottom: 0;
  border: none;
  padding-bottom: 0;
  color: white;
}
.wrapper--xblock .message--inline.message--error {
  background: #b20610;
}
.wrapper--xblock .message--inline.message--error .message__content {
  color: white;
}
.wrapper--xblock .message--inline.message--warning {
  background: #ffc01f;
}
.wrapper--xblock .message--inline.message--warning .message__content {
  color: #111111;
}
.wrapper--xblock .message--inline.message--warning .message__title {
  color: #111111;
}
.wrapper--xblock .is--transitioning {
  padding: 40px 40px;
  background: #f5f6f7;
}
.wrapper--xblock .is--transitioning .transition__status {
  position: relative;
  top: 50%;
  -webkit-transform: translateY(-50%);
  -moz-transform: translateY(-50%);
  -ms-transform: translateY(-50%);
  -o-transform: translateY(-50%);
  transform: translateY(-50%);
  text-align: center;
}
.wrapper--xblock .is--transitioning .transition__status .wrapper--anim {
  display: block;
  -webkit-animation: pulse 3s ease-in-out infinite;
  -moz-animation: pulse 3s ease-in-out infinite;
  animation: pulse 3s ease-in-out infinite;
}
.wrapper--xblock .is--transitioning .transition__status .icon, .wrapper--xblock .is--transitioning .transition__status .copy {
  display: block;
  color: #414141;
}
.wrapper--xblock .is--transitioning .transition__status .icon {
  margin-bottom: 10px;
}
.wrapper--grid {
  *zoom: 1;
  max-width: 100%;
  margin-left: auto;
  margin-right: auto;
}
.wrapper--grid:before, .wrapper--grid:after {
  content: " ";
  display: table;
}
.wrapper--grid:after {
  clear: both;
}

/*! Hint.css - v1.3.1 - 2013-11-23
* http://kushagragour.in/lab/hint/
* Copyright (c) 2013 Kushagra Gour; Licensed MIT */
/*-------------------------------------*\
	HINT.css - A CSS tooltip library
\*-------------------------------------*/
/**
 * HINT.css is a tooltip library made in pure CSS.
 *
 * Source: https://github.com/chinchang/hint.css
 * Demo: http://kushagragour.in/lab/hint/
 *
 * Release under The MIT License
 *
 */
/**
 * source: hint-core.scss
 *
 * Defines the basic styling for the tooltip.
 * Each tooltip is made of 2 parts:
 * 	1) body (:after)
 * 	2) arrow (:before)
 * 
 * Classes added:
 * 	1) hint
 */
.hint, [data-hint] {
  position: relative;
  display: inline-block;
  /**
   * tooltip arrow
   */
  /**
   * tooltip body
   */
}

.hint:before, .hint:after, [data-hint]:before, [data-hint]:after {
  position: absolute;
  -webkit-transform: translate3d(0, 0, 0);
  -moz-transform: translate3d(0, 0, 0);
  transform: translate3d(0, 0, 0);
  visibility: hidden;
  opacity: 0;
  z-index: 1000000;
  pointer-events: none;
  -webkit-transition: 0.3s ease;
  -moz-transition: 0.3s ease;
  transition: 0.3s ease;
}

.hint:hover:before, .hint:hover:after, .hint:focus:before, .hint:focus:after, [data-hint]:hover:before, [data-hint]:hover:after, [data-hint]:focus:before, [data-hint]:focus:after {
  visibility: visible;
  opacity: 1;
}

.hint:before, [data-hint]:before {
  content: "";
  position: absolute;
  background: transparent;
  border: 6px solid transparent;
  z-index: 1000001;
}

.hint:after, [data-hint]:after {
  content: attr(data-hint);
  background: #383838;
  color: white;
  text-shadow: 0 -1px 0px black;
  padding: 8px 10px;
  font-size: 12px;
  line-height: 12px;
  white-space: nowrap;
  box-shadow: 4px 4px 8px rgba(0, 0, 0, 0.3);
}

/**
 * source: hint-position.scss
 *
 * Defines the positoning logic for the tooltips.
 *
 * Classes added:
 * 	1) hint--top
 * 	2) hint--bottom
 * 	3) hint--left
 * 	4) hint--right
 */
/**
 * set default color for tooltip arrows
 */
.hint--top:before {
  border-top-color: #383838;
}

.hint--bottom:before {
  border-bottom-color: #383838;
}

.hint--left:before {
  border-left-color: #383838;
}

.hint--right:before {
  border-right-color: #383838;
}

/**
 * top tooltip
 */
.hint--top:before {
  margin-bottom: -12px;
}

.hint--top:after {
  margin-left: -18px;
}

.hint--top:before, .hint--top:after {
  bottom: 100%;
  left: 50%;
}

.hint--top:hover:after, .hint--top:hover:before, .hint--top:focus:after, .hint--top:focus:before {
  -webkit-transform: translateY(-8px);
  -moz-transform: translateY(-8px);
  transform: translateY(-8px);
}

/**
 * bottom tooltip
 */
.hint--bottom:before {
  margin-top: -12px;
}

.hint--bottom:after {
  margin-left: -18px;
}

.hint--bottom:before, .hint--bottom:after {
  top: 100%;
  left: 50%;
}

.hint--bottom:hover:after, .hint--bottom:hover:before, .hint--bottom:focus:after, .hint--bottom:focus:before {
  -webkit-transform: translateY(8px);
  -moz-transform: translateY(8px);
  transform: translateY(8px);
}

/**
 * right tooltip
 */
.hint--right:before {
  margin-left: -12px;
  margin-bottom: -6px;
}

.hint--right:after {
  margin-bottom: -14px;
}

.hint--right:before, .hint--right:after {
  left: 100%;
  bottom: 50%;
}

.hint--right:hover:after, .hint--right:hover:before, .hint--right:focus:after, .hint--right:focus:before {
  -webkit-transform: translateX(8px);
  -moz-transform: translateX(8px);
  transform: translateX(8px);
}

/**
 * left tooltip
 */
.hint--left:before {
  margin-right: -12px;
  margin-bottom: -6px;
}

.hint--left:after {
  margin-bottom: -14px;
}

.hint--left:before, .hint--left:after {
  right: 100%;
  bottom: 50%;
}

.hint--left:hover:after, .hint--left:hover:before, .hint--left:focus:after, .hint--left:focus:before {
  -webkit-transform: translateX(-8px);
  -moz-transform: translateX(-8px);
  transform: translateX(-8px);
}

/**
 * source: hint-color-types.scss
 *
 * Contains tooltips of various types based on color differences.
 *
 * Classes added:
 * 	1) hint--error
 * 	2) hint--warning
 * 	3) hint--info
 * 	4) hint--success
 *
 */
/**
 * Error
 */
.hint--error:after {
  background-color: #b34e4d;
  text-shadow: 0 -1px 0px #592726;
}

.hint--error.hint--top:before {
  border-top-color: #b34e4d;
}

.hint--error.hint--bottom:before {
  border-bottom-color: #b34e4d;
}

.hint--error.hint--left:before {
  border-left-color: #b34e4d;
}

.hint--error.hint--right:before {
  border-right-color: #b34e4d;
}

/**
 * Warning
 */
.hint--warning:after {
  background-color: #c09854;
  text-shadow: 0 -1px 0px #6c5328;
}

.hint--warning.hint--top:before {
  border-top-color: #c09854;
}

.hint--warning.hint--bottom:before {
  border-bottom-color: #c09854;
}

.hint--warning.hint--left:before {
  border-left-color: #c09854;
}

.hint--warning.hint--right:before {
  border-right-color: #c09854;
}

/**
 * Info
 */
.hint--info:after {
  background-color: #3986ac;
  text-shadow: 0 -1px 0px #193b4d;
}

.hint--info.hint--top:before {
  border-top-color: #3986ac;
}

.hint--info.hint--bottom:before {
  border-bottom-color: #3986ac;
}

.hint--info.hint--left:before {
  border-left-color: #3986ac;
}

.hint--info.hint--right:before {
  border-right-color: #3986ac;
}

/**
 * Success
 */
.hint--success:after {
  background-color: #458746;
  text-shadow: 0 -1px 0px #1a321a;
}

.hint--success.hint--top:before {
  border-top-color: #458746;
}

.hint--success.hint--bottom:before {
  border-bottom-color: #458746;
}

.hint--success.hint--left:before {
  border-left-color: #458746;
}

.hint--success.hint--right:before {
  border-right-color: #458746;
}

/**
 * source: hint-always.scss
 *
 * Defines a persisted tooltip which shows always.
 *
 * Classes added:
 * 	1) hint--always
 *
 */
.hint--always:after, .hint--always:before {
  opacity: 1;
  visibility: visible;
}

.hint--always.hint--top:after, .hint--always.hint--top:before {
  -webkit-transform: translateY(-8px);
  -moz-transform: translateY(-8px);
  transform: translateY(-8px);
}

.hint--always.hint--bottom:after, .hint--always.hint--bottom:before {
  -webkit-transform: translateY(8px);
  -moz-transform: translateY(8px);
  transform: translateY(8px);
}

.hint--always.hint--left:after, .hint--always.hint--left:before {
  -webkit-transform: translateX(-8px);
  -moz-transform: translateX(-8px);
  transform: translateX(-8px);
}

.hint--always.hint--right:after, .hint--always.hint--right:before {
  -webkit-transform: translateX(8px);
  -moz-transform: translateX(8px);
  transform: translateX(8px);
}

/**
 * source: hint-rounded.scss
 *
 * Defines rounded corner tooltips.
 *
 * Classes added:
 * 	1) hint--rounded
 *
 */
.hint--rounded:after {
  border-radius: 4px;
}

/**
 * source: hint-effects.scss
 *
 * Defines various transition effects for the tooltips.
 *
 * Classes added:
 * 	1) hint--bounce
 *
 */
.hint--bounce:before, .hint--bounce:after {
  -webkit-transition: opacity 0.3s ease, visibility 0.3s ease, -webkit-transform 0.3s cubic-bezier(0.71, 1.7, 0.77, 1.24);
  -moz-transition: opacity 0.3s ease, visibility 0.3s ease, -moz-transform 0.3s cubic-bezier(0.71, 1.7, 0.77, 1.24);
  transition: opacity 0.3s ease, visibility 0.3s ease, transform 0.3s cubic-bezier(0.71, 1.7, 0.77, 1.24);
}

.openassessment .openassessment__steps__step {
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
  -webkit-transform: scale(0.99);
  -moz-transform: scale(0.99);
  -ms-transform: scale(0.99);
  -o-transform: scale(0.99);
  transform: scale(0.99);
  -webkit-border-radius: 2px;
  -moz-border-radius: 2px;
  -ms-border-radius: 2px;
  -o-border-radius: 2px;
  border-radius: 2px;
  box-shadow: 0 1px 1px 0 rgba(3, 3, 3, 0.05);
  margin-bottom: 20px;
  border: 1px solid #d9d9d9;
  border-top: 5px solid #d9d9d9;
  padding: 10px 20px;
  background: white;
}
.openassessment .openassessment__steps__step:last-child {
  margin-bottom: 0;
}
.openassessment .openassessment__steps__step:hover, .openassessment .openassessment__steps__step:focus {
  -webkit-transform: scale(1);
  -moz-transform: scale(1);
  -ms-transform: scale(1);
  -o-transform: scale(1);
  transform: scale(1);
}
.openassessment .has--error.openassessment__steps__step:hover, .openassessment .has--error.openassessment__steps__step:focus {
  -webkit-transform: scale(0.99);
  -moz-transform: scale(0.99);
  -ms-transform: scale(0.99);
  -o-transform: scale(0.99);
  transform: scale(0.99);
}
.openassessment .is--showing.openassessment__steps__step {
  -webkit-transform: scale(1);
  -moz-transform: scale(1);
  -ms-transform: scale(1);
  -o-transform: scale(1);
  transform: scale(1);
  box-shadow: 0 1px 1px 0 rgba(3, 3, 3, 0.1);
}
.openassessment .is--loading.openassessment__steps__step {
  -webkit-transform: scale(0.99);
  -moz-transform: scale(0.99);
  -ms-transform: scale(0.99);
  -o-transform: scale(0.99);
  transform: scale(0.99);
  box-shadow: 0 1px 1px 0 rgba(3, 3, 3, 0.05);
}
.openassessment .is--empty.openassessment__steps__step:hover, .openassessment .is--empty.openassessment__steps__step:focus {
  -webkit-transform: scale(0.99);
  -moz-transform: scale(0.99);
  -ms-transform: scale(0.99);
  -o-transform: scale(0.99);
  transform: scale(0.99);
}
.openassessment .is--unavailable.openassessment__steps__step {
  border-top-color: #d9d9d9;
}
.openassessment .is--unavailable.openassessment__steps__step:hover, .openassessment .is--unavailable.openassessment__steps__step:focus {
  -webkit-transform: scale(0.99);
  -moz-transform: scale(0.99);
  -ms-transform: scale(0.99);
  -o-transform: scale(0.99);
  transform: scale(0.99);
}
.openassessment .is--in-progress.openassessment__steps__step {
  border-top-color: #0075b4;
}
.openassessment .is--complete.openassessment__steps__step {
  border-top-color: #008100;
}
.openassessment .is--incomplete.openassessment__steps__step {
  border-top-color: #ffc01f;
}
.openassessment .has--error.openassessment__steps__step {
  border-top-color: #b20610;
}
.openassessment .is--closed.openassessment__steps__step, .openassessment .is--incomplete.openassessment__steps__step {
  border-top-color: #ffc01f;
}

.step--student-training .student-training__display, .openassessment .submission__feedback, .openassessment .submission__peer-evaluations, .openassessment .step--self-assessment .self-assessment__display, .openassessment .step--peer-assessment .peer-assessment__display, .openassessment .staff-assessment__display {
  margin-bottom: 20px;
}

.openassessment .submission__feedback__title {
  margin-bottom: 10px;
  border-bottom: 2px solid #d9d9d9;
  padding-bottom: 10px;
}

.step--student-training .student-training__display__response p, .openassessment .submission__feedback__actions p, .openassessment .submission__feeedback__content p, .openassessment .submission__peer-evaluations__questions p, .openassessment .submission__answer__display__content p, .openassessment .self-assessment__display__image p,
.openassessment .peer-assessment__display__image p,
.openassessment .staff-assessment__display__image p,
.openassessment .submission__answer__display__image p, .openassessment .step--self-assessment .self-assessment__display__response p, .openassessment .step--peer-assessment .peer-assessment__display__response p, .openassessment .staff-assessment__display__response p {
  margin-bottom: 20px;
}
.step--student-training .student-training__display__response p:last-child, .openassessment .submission__feedback__actions p:last-child, .openassessment .submission__feeedback__content p:last-child, .openassessment .submission__peer-evaluations__questions p:last-child, .openassessment .submission__answer__display__content p:last-child, .openassessment .self-assessment__display__image p:last-child,
.openassessment .peer-assessment__display__image p:last-child,
.openassessment .staff-assessment__display__image p:last-child,
.openassessment .submission__answer__display__image p:last-child, .openassessment .step--self-assessment .self-assessment__display__response p:last-child, .openassessment .step--peer-assessment .peer-assessment__display__response p:last-child, .openassessment .staff-assessment__display__response p:last-child {
  margin-bottom: 0;
}

.step--student-training .student-training__display__response, .openassessment .step--self-assessment .self-assessment__display__response, .openassessment .step--peer-assessment .peer-assessment__display__response, .openassessment .submission__answer__part__prompt, .openassessment .staff-assessment__display__response {
  box-shadow: inset 0 1px 2px 1px rgba(3, 3, 3, 0.1);
  padding: 20px 20px;
  background: #fdfdfd;
}

.step--student-training .student-training__assessment .assessment__rubric__question--feedback textarea, .step--student-training .student-training__display__response, .openassessment .submission__feedback__fields textarea, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question--feedback textarea, .openassessment .step--self-assessment .self-assessment__display__response, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question--feedback textarea, .openassessment .step--peer-assessment .peer-assessment__display__response, .openassessment .step--response .response__submission__content .submission__file__description textarea, .openassessment .step--response .response__submission__content .submission__answer__part__text textarea, .openassessment .staff-assessment__assessment .assessment__rubric__question--feedback textarea, .openassessment .staff-assessment__display__response, .step--response .step__rubric .question__answers .answer--feedback .answer__value, .step--student-training .student-training__assessment .question__answers .answer--feedback .answer__value, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback .answer__value, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback .answer__value, .openassessment .staff-assessment__assessment .question__answers .answer--feedback .answer__value {
  max-height: 300px;
  overflow-y: scroll;
}

.openassessment .step--self-assessment .self-assessment__assessment .response__submission__label, .openassessment .step--peer-assessment .peer-assessment__assessment .response__submission__label, .openassessment .step--response .response__submission__actions .response__submission__label, .openassessment .step--response .response__submission__content .response__submission__label {
  color: #414141;
  display: inline-block;
  vertical-align: middle;
  margin-right: 10px;
}
.openassessment .step--self-assessment .self-assessment__assessment .response__submission__label .list--actions, .openassessment .step--peer-assessment .peer-assessment__assessment .response__submission__label .list--actions, .openassessment .step--response .response__submission__actions .response__submission__label .list--actions, .openassessment .step--response .response__submission__content .response__submission__label .list--actions {
  text-align: right;
}
.openassessment .step--self-assessment .self-assessment__assessment .list--actions__item, .openassessment .step--peer-assessment .peer-assessment__assessment .list--actions__item, .openassessment .step--response .response__submission__actions .list--actions__item, .openassessment .step--response .response__submission__content .list--actions__item {
  text-align: right;
  margin-top: 5px;
}
.openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save {
  display: block;
  margin-bottom: 10px;
  min-width: 215px;
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save {
    display: inline-block;
    margin-left: 10px;
    margin-bottom: 0;
  }
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save {
    display: inline-block;
    margin-left: 10px;
    margin-bottom: 0;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save {
    display: inline-block;
    margin-left: 10px;
    margin-bottom: 0;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .step--self-assessment .self-assessment__assessment .action--save, .openassessment .step--peer-assessment .peer-assessment__assessment .action--save, .openassessment .step--response .response__submission__actions .action--save, .openassessment .step--response .response__submission__content .action--save {
    display: inline-block;
    margin-left: 10px;
    margin-bottom: 0;
  }
}

.step--response .step__rubric .assessment__rubric__question, .step--student-training .student-training__assessment .assessment__rubric__question, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question, .openassessment .staff-assessment__assessment .assessment__rubric__question {
  *zoom: 1;
  margin-bottom: 0px;
}
.step--response .step__rubric .assessment__rubric__question:before, .step--student-training .student-training__assessment .assessment__rubric__question:before, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question:before, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question:before, .openassessment .staff-assessment__assessment .assessment__rubric__question:before, .step--response .step__rubric .assessment__rubric__question:after, .step--student-training .student-training__assessment .assessment__rubric__question:after, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question:after, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question:after, .openassessment .staff-assessment__assessment .assessment__rubric__question:after {
  content: " ";
  display: table;
}
.step--response .step__rubric .assessment__rubric__question:after, .step--student-training .student-training__assessment .assessment__rubric__question:after, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question:after, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question:after, .openassessment .staff-assessment__assessment .assessment__rubric__question:after {
  clear: both;
}
.step--response .step__rubric .assessment__rubric__question .question__title, .step--student-training .student-training__assessment .assessment__rubric__question .question__title, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question .question__title, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question .question__title, .openassessment .staff-assessment__assessment .assessment__rubric__question .question__title {
  border-bottom: 2px solid #414141;
  padding-bottom: 5px;
}
.step--response .step__rubric .assessment__rubric__question .question__title .icon, .step--student-training .student-training__assessment .assessment__rubric__question .question__title .icon, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question .question__title .icon, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question .question__title .icon, .openassessment .staff-assessment__assessment .assessment__rubric__question .question__title .icon {
  display: inline-block;
  vertical-align: middle;
}
.step--response .step__rubric .assessment__rubric__question .question__title__copy, .step--student-training .student-training__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question .question__title__copy, .openassessment .staff-assessment__assessment .assessment__rubric__question .question__title__copy {
  margin-right: 10px;
  white-space: pre-wrap;
  display: inline;
}

.step--response .step__rubric .question__answers, .step--student-training .student-training__assessment .question__answers, .openassessment .step--self-assessment .self-assessment__assessment .question__answers, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers, .openassessment .staff-assessment__assessment .question__answers {
  margin-top: 20px;
  margin-bottom: 20px;
  margin-right: 20px;
}
.step--response .step__rubric .question__answers .answer, .step--student-training .student-training__assessment .question__answers .answer, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer, .openassessment .staff-assessment__assessment .question__answers .answer, .step--response .step__rubric .question__answers .answer--feedback, .step--student-training .student-training__assessment .question__answers .answer--feedback, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback, .openassessment .staff-assessment__assessment .question__answers .answer--feedback {
  *zoom: 1;
  display: block;
  margin-bottom: 10px;
  border-bottom: 1px solid #f5f6f7;
  padding-bottom: 10px;
}
.step--response .step__rubric .question__answers .answer:before, .step--student-training .student-training__assessment .question__answers .answer:before, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer:before, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer:before, .openassessment .staff-assessment__assessment .question__answers .answer:before, .step--response .step__rubric .question__answers .answer:after, .step--student-training .student-training__assessment .question__answers .answer:after, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer:after, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer:after, .openassessment .staff-assessment__assessment .question__answers .answer:after, .step--response .step__rubric .question__answers .answer--feedback:before, .step--student-training .student-training__assessment .question__answers .answer--feedback:before, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback:before, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback:before, .openassessment .staff-assessment__assessment .question__answers .answer--feedback:before, .step--response .step__rubric .question__answers .answer--feedback:after, .step--student-training .student-training__assessment .question__answers .answer--feedback:after, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback:after, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback:after, .openassessment .staff-assessment__assessment .question__answers .answer--feedback:after {
  content: " ";
  display: table;
}
.step--response .step__rubric .question__answers .answer:after, .step--student-training .student-training__assessment .question__answers .answer:after, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer:after, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer:after, .openassessment .staff-assessment__assessment .question__answers .answer:after, .step--response .step__rubric .question__answers .answer--feedback:after, .step--student-training .student-training__assessment .question__answers .answer--feedback:after, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback:after, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback:after, .openassessment .staff-assessment__assessment .question__answers .answer--feedback:after {
  clear: both;
}
.step--response .step__rubric .question__answers .wrapper--input, .step--student-training .student-training__assessment .question__answers .wrapper--input, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input, .openassessment .staff-assessment__assessment .question__answers .wrapper--input {
  margin-bottom: 5px;
}
.step--response .step__rubric .question__answers .wrapper--input .answer__value, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__value, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__value, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__value, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__value, .step--response .step__rubric .question__answers .wrapper--input .answer__label, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__label {
  display: inline-block;
  vertical-align: middle;
}
.step--response .step__rubric .question__answers .wrapper--input .answer__label, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__label, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__label {
  margin-bottom: 0;
  color: #414141;
}
.step--response .step__rubric .question__answers .wrapper--input .answer__value, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__value, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__value, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__value, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__value {
  margin-left: 10px;
}
.step--response .step__rubric .question__answers .wrapper--input .answer__value:checked + .answer__label, .step--student-training .student-training__assessment .question__answers .wrapper--input .answer__value:checked + .answer__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--input .answer__value:checked + .answer__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--input .answer__value:checked + .answer__label, .openassessment .staff-assessment__assessment .question__answers .wrapper--input .answer__value:checked + .answer__label {
  color: black;
}
.step--response .step__rubric .question__answers .wrapper--metadata, .step--student-training .student-training__assessment .question__answers .wrapper--metadata, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--metadata, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--metadata, .openassessment .staff-assessment__assessment .question__answers .wrapper--metadata {
  display: block;
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .step--response .step__rubric .question__answers .wrapper--metadata, .step--student-training .student-training__assessment .question__answers .wrapper--metadata, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--metadata, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--metadata, .openassessment .staff-assessment__assessment .question__answers .wrapper--metadata {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 65.8807828007%;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .wrapper--metadata:last-child, .step--student-training .student-training__assessment .question__answers .wrapper--metadata:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--metadata:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--metadata:last-child, .openassessment .staff-assessment__assessment .question__answers .wrapper--metadata:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .step--response .step__rubric .question__answers .wrapper--metadata, .step--student-training .student-training__assessment .question__answers .wrapper--metadata, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--metadata, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--metadata, .openassessment .staff-assessment__assessment .question__answers .wrapper--metadata {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 65.8807828007%;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .wrapper--metadata:last-child, .step--student-training .student-training__assessment .question__answers .wrapper--metadata:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--metadata:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--metadata:last-child, .openassessment .staff-assessment__assessment .question__answers .wrapper--metadata:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 1300px) {
  .step--response .step__rubric .question__answers .wrapper--metadata, .step--student-training .student-training__assessment .question__answers .wrapper--metadata, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--metadata, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--metadata, .openassessment .staff-assessment__assessment .question__answers .wrapper--metadata {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 65.8807828007%;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .wrapper--metadata:last-child, .step--student-training .student-training__assessment .question__answers .wrapper--metadata:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .wrapper--metadata:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .wrapper--metadata:last-child, .openassessment .staff-assessment__assessment .question__answers .wrapper--metadata:last-child {
    margin-left: 0;
  }
}
.step--response .step__rubric .question__answers .answer__tip, .step--student-training .student-training__assessment .question__answers .answer__tip, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip, .openassessment .staff-assessment__assessment .question__answers .answer__tip, .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points {
  display: block;
}
.step--response .step__rubric .question__answers .answer__tip, .step--student-training .student-training__assessment .question__answers .answer__tip, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip, .openassessment .staff-assessment__assessment .question__answers .answer__tip {
  margin-bottom: 10px;
  white-space: pre-wrap;
  margin-left: 20px;
  color: #414141;
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .step--response .step__rubric .question__answers .answer__tip, .step--student-training .student-training__assessment .question__answers .answer__tip, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip, .openassessment .staff-assessment__assessment .question__answers .answer__tip {
    float: right;
    display: block;
    margin-left: 3.5786636068%;
    width: 74.1053340983%;
    float: right;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .answer__tip:last-child, .step--student-training .student-training__assessment .question__answers .answer__tip:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip:last-child, .openassessment .staff-assessment__assessment .question__answers .answer__tip:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .step--response .step__rubric .question__answers .answer__tip, .step--student-training .student-training__assessment .question__answers .answer__tip, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip, .openassessment .staff-assessment__assessment .question__answers .answer__tip {
    float: right;
    display: block;
    margin-left: 3.5786636068%;
    width: 74.1053340983%;
    float: right;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .answer__tip:last-child, .step--student-training .student-training__assessment .question__answers .answer__tip:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip:last-child, .openassessment .staff-assessment__assessment .question__answers .answer__tip:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 1300px) {
  .step--response .step__rubric .question__answers .answer__tip, .step--student-training .student-training__assessment .question__answers .answer__tip, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip, .openassessment .staff-assessment__assessment .question__answers .answer__tip {
    float: right;
    display: block;
    margin-left: 3.5786636068%;
    width: 74.1053340983%;
    float: right;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .answer__tip:last-child, .step--student-training .student-training__assessment .question__answers .answer__tip:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip:last-child, .openassessment .staff-assessment__assessment .question__answers .answer__tip:last-child {
    margin-left: 0;
  }
}
.step--response .step__rubric .question__answers .answer__tip:after, .step--student-training .student-training__assessment .question__answers .answer__tip:after, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__tip:after, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__tip:after, .openassessment .staff-assessment__assessment .question__answers .answer__tip:after {
  content: "";
}
.step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points {
  color: #414141;
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points {
    float: left;
    text-align: left;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points {
    float: left;
    text-align: left;
  }
}
@media screen and (min-width: 1300px) {
  .step--response .step__rubric .question__answers .answer__points, .step--student-training .student-training__assessment .question__answers .answer__points, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points, .openassessment .staff-assessment__assessment .question__answers .answer__points {
    float: left;
    text-align: left;
  }
}
.step--response .step__rubric .question__answers .answer__points__label, .step--student-training .student-training__assessment .question__answers .answer__points__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer__points__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer__points__label, .openassessment .staff-assessment__assessment .question__answers .answer__points__label {
  display: inline-block;
  margin-right: 5px;
  color: #414141;
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .step--response .step__rubric .question__answers .answer .wrapper--input, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input {
    float: right;
    display: block;
    margin-left: 4.8291579146%;
    width: 100%;
  }
  .step--response .step__rubric .question__answers .answer .wrapper--input:last-child, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .step--response .step__rubric .question__answers .answer .wrapper--input, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 31.7615656014%;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .answer .wrapper--input:last-child, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .step--response .step__rubric .question__answers .answer .wrapper--input, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 31.7615656014%;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .answer .wrapper--input:last-child, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 1300px) {
  .step--response .step__rubric .question__answers .answer .wrapper--input, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 31.7615656014%;
    margin-bottom: 0;
  }
  .step--response .step__rubric .question__answers .answer .wrapper--input:last-child, .step--student-training .student-training__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer .wrapper--input:last-child, .openassessment .staff-assessment__assessment .question__answers .answer .wrapper--input:last-child {
    margin-left: 0;
  }
}
.step--response .step__rubric .question__answers .answer--feedback .answer__label, .step--student-training .student-training__assessment .question__answers .answer--feedback .answer__label, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback .answer__label, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback .answer__label, .openassessment .staff-assessment__assessment .question__answers .answer--feedback .answer__label {
  margin-bottom: 5px;
}
.step--response .step__rubric .question__answers .answer--feedback .answer__value, .step--student-training .student-training__assessment .question__answers .answer--feedback .answer__value, .openassessment .step--self-assessment .self-assessment__assessment .question__answers .answer--feedback .answer__value, .openassessment .step--peer-assessment .peer-assessment__assessment .question__answers .answer--feedback .answer__value, .openassessment .staff-assessment__assessment .question__answers .answer--feedback .answer__value {
  min-height: 100px;
  margin-left: 0;
}

.has--hints {
  overflow: visible;
}

.ui-hint .icon {
  margin-right: 5px;
}
.ui-hint:hover, .ui-hint:focus {
  color: #065683;
}
.ui-hint:after {
  right: 0;
  display: block;
  white-space: normal;
  width: 200px;
  background: rgba(3, 3, 3, 0.75);
}
.ui-hint.hint--top:after {
  right: 0;
  margin-right: -20px;
}

.openassessment .wrapper--ui-staff {
  box-shadow: inset 0 1px -2px 1px rgba(3, 3, 3, 0.4);
  margin-top: 40px;
  -webkit-border-radius: 2px;
  -moz-border-radius: 2px;
  -ms-border-radius: 2px;
  -o-border-radius: 2px;
  border-radius: 2px;
  border: 1px solid #891c4e;
  border-top: 5px solid #b62568;
  padding: 20px 20px;
  background: #2c2f36;
  display: none;
}
.openassessment .ui-staff .ui-slidable__content {
  padding: 10px 10px;
}
.openassessment .ui-staff__title {
  color: white;
}
.openassessment .ui-staff__subtitle {
  font-size: 16px;
  color: white !important;
  margin-bottom: 10px;
}
.openassessment .ui-staff__subtitle span {
  font-weight: inherit;
}
.openassessment .ui-staff__content {
  margin-top: 20px;
  color: white;
}
.openassessment .ui-staff__content__section {
  padding-bottom: 10px;
}
.openassessment .ui-staff__content__section .wrapper--input,
.openassessment .ui-staff__content__section .wrapper--content {
  padding: 20px 20px;
  background-color: white;
  color: #414141;
}
.openassessment .staff-info__summary .label, .openassessment .staff-info__summary .value {
  display: inline-block;
  vertical-align: center;
  color: white;
}
.openassessment .staff-info__summary .label {
  margin-left: 10px;
}
.openassessment .openassessment_student_info_form {
  margin-bottom: 10px;
}
.openassessment .openassessment_student_info_form .action--submit-username {
  margin-right: 0;
}
.openassessment .staff-info__status__table, .openassessment .staff-info__classifierset__table {
  -webkit-border-radius: 2px;
  -moz-border-radius: 2px;
  -ms-border-radius: 2px;
  -o-border-radius: 2px;
  border-radius: 2px;
  word-break: initial;
}
.openassessment .staff-info__status__table .title, .openassessment .staff-info__classifierset__table .title {
  text-align: right;
  color: white;
  margin-bottom: 10px;
}
.openassessment .staff-info__status__table .label, .openassessment .staff-info__classifierset__table .label {
  color: white;
}
.openassessment .staff-info__status__table .value, .openassessment .staff-info__classifierset__table .value {
  color: white;
}
.openassessment .staff-info__status__table th, .openassessment .staff-info__status__table td, .openassessment .staff-info__classifierset__table th, .openassessment .staff-info__classifierset__table td {
  border: 1px solid rgba(255, 255, 255, 0.25);
  padding: 10px 10px;
}
.openassessment .staff-info__status__table th, .openassessment .staff-info__status__table td[scope], .openassessment .staff-info__classifierset__table th, .openassessment .staff-info__classifierset__table td[scope] {
  text-align: right;
}
.openassessment .wrapper--staff-assessment {
  margin-top: 10px;
  padding-top: 10px;
  border-top: 1px solid #d9d9d9;
}
.openassessment .staff-assessment__display__header {
  *zoom: 1;
}
.openassessment .staff-assessment__display__header:before, .openassessment .staff-assessment__display__header:after {
  content: " ";
  display: table;
}
.openassessment .staff-assessment__display__header:after {
  clear: both;
}
.openassessment .staff-assessment__display__header .staff-assessment__display__title {
  margin-bottom: 10px;
  color: #414141;
}
.openassessment .staff-assessment__display__response {
  color: #414141;
}
.openassessment .staff-assessment__assessment .assessment__fields {
  margin-bottom: 20px;
}
.openassessment .staff-assessment__assessment .assessment__rubric__question {
  margin-bottom: 0px;
}
.openassessment .staff-assessment__assessment .assessment__rubric__question--feedback .wrapper--input {
  margin-top: 20px;
}
.openassessment .staff-assessment__assessment .assessment__rubric__question--feedback .question__title__copy {
  margin-right: 0;
  white-space: pre-wrap;
}
.openassessment .staff-assessment__assessment .assessment__rubric__question--feedback textarea {
  min-height: 100px;
}
.openassessment .ui-staff .staff__grade__control {
  border-top: 5px solid #d9d9d9;
  background: white;
  background-color: white;
}
.openassessment .ui-staff .staff__grade__control .staff__grade__header {
  margin-bottom: 0;
}
.openassessment .ui-staff .staff__grade__control .staff__grade__title {
  text-align: right;
  float: none;
  padding: 0 20px;
  display: block;
  width: 100%;
}
.openassessment .ui-staff .staff__grade__control .staff__grade__title .staff__grade__show-form {
  font-size: 16px;
  background-color: white;
  text-transform: none;
  letter-spacing: normal;
  padding: 0;
}
.openassessment .ui-staff .staff__grade__control .staff__grade__status {
  display: inline-block;
  padding: 0 20px;
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .ui-staff .staff__grade__control .staff__grade__status {
    margin-top: 0;
    float: left;
    position: relative;
    top: -40px;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .ui-staff .staff__grade__control .staff__grade__status {
    margin-top: 0;
    float: left;
    position: relative;
    top: -40px;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .ui-staff .staff__grade__control .staff__grade__status {
    margin-top: 0;
    float: left;
    position: relative;
    top: -40px;
  }
}
.openassessment .ui-staff .staff__grade__control .staff__grade__status .staff__grade__value {
  border-radius: 2px;
  padding: 5px 10px;
  background: #d9d9d9;
  line-height: 0;
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .openassessment .ui-staff .staff__grade__control .staff__grade__status .staff__grade__value {
    display: block;
  }
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .ui-staff .staff__grade__control .staff__grade__status .staff__grade__value {
    display: block;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .ui-staff .staff__grade__control .staff__grade__status .staff__grade__value {
    display: block;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .ui-staff .staff__grade__control .staff__grade__status .staff__grade__value {
    display: block;
  }
}
.openassessment .ui-staff .staff__grade__control .staff__grade__status .copy {
  color: #414141;
}
.openassessment .ui-staff .staff__grade__control .submit_assessment--action {
  display: inline;
}
.openassessment .ui-staff .staff__grade__control .staff__grade__content {
  padding: 0;
}
.openassessment .staff-info__workflow-cancellation .staff-info__cancel-submission__content .comments__label {
  color: #414141;
}
.openassessment .staff-info__workflow-cancellation .staff-info__cancel-submission__content .cancel_submission_comments {
  width: 100%;
  min-height: 100px;
  text-align: left;
}
.openassessment .staff-info__student__report .staff-info__final__grade__table {
  margin-top: 10px;
}
.openassessment .staff-info__student__report .staff_area_override_unaavilable_msg {
  margin-top: 10px;
}

#oa_basic_settings_editor .warning-banner {
  background: #fffaed;
  border: 1px solid #ffd875;
  padding: 0.75rem 1.25rem;
}
#oa_basic_settings_editor .warning-banner.is--hidden {
  display: none;
}
#oa_basic_settings_editor .warning-banner span.icon.fa-exclamation-circle {
  font-size: 1.5rem;
  color: #ffd875;
}

.wrapper--openassessment {
  position: relative;
}

.openassessment {
  direction: rtl;
}
.openassessment .openassessment__title {
  *zoom: 1;
  display: block;
}
.openassessment .openassessment__title:before, .openassessment .openassessment__title:after {
  content: " ";
  display: table;
}
.openassessment .openassessment__title:after {
  clear: both;
}
.openassessment .openassessment__message {
  *zoom: 1;
  display: block;
  background: none;
  padding: 0;
}
.openassessment .openassessment__message:before, .openassessment .openassessment__message:after {
  content: " ";
  display: table;
}
.openassessment .openassessment__message:after {
  clear: both;
}
.openassessment .list--actions {
  margin-bottom: 10px;
  text-align: center;
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .openassessment .list--actions {
    text-align: left;
  }
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .list--actions {
    text-align: left;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .list--actions {
    text-align: left;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .list--actions {
    text-align: left;
  }
}
.openassessment .list--actions.has--error .message {
  margin-bottom: 20px;
  text-align: right;
}
.openassessment .list--actions .action--submit {
  margin-right: 10px;
  margin-top: 10px;
}
.openassessment .list--actions .action--submit .copy, .openassessment .list--actions .action--submit .icon {
  display: inline;
}
.openassessment .list--actions .action--submit .fa-caret-right:before {
  -webkit-transform: rotate(180deg);
  -moz-transform: rotate(180deg);
  -ms-transform: rotate(180deg);
  -o-transform: rotate(180deg);
  transform: rotate(180deg);
}
.openassessment .openassessment__steps {
  counter-reset: item;
}
.openassessment .openassessment__steps__step .step__title {
  display: inline;
  margin-bottom: 10px;
  width: 100%;
  margin-bottom: 0;
}
.openassessment .openassessment__steps__step .step__title .step__counter, .openassessment .openassessment__steps__step .step__title .wrapper--copy {
  display: inline-block;
  vertical-align: middle;
}
.openassessment .openassessment__steps__step .step__title .wrapper--copy {
  margin-right: 20px;
  border-right: 1px solid #d9d9d9;
  padding-right: 20px;
}
@media screen and (max-width: 529px) {
  .openassessment .openassessment__steps__step .step__title .wrapper--copy {
    margin-right: 0;
    border-right: none;
    padding-right: 0;
  }
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .openassessment .openassessment__steps__step .step__title .wrapper--copy {
    margin-right: 0;
    border-right: none;
    padding-right: 0;
  }
}
.openassessment .openassessment__steps__step .step__title .step__counter {
  display: none;
}
.openassessment .openassessment__steps__step .step__title .step__counter:before {
  display: none;
  content: counter(item, decimal);
  counter-increment: item;
  color: #414141;
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .openassessment__steps__step .step__title .step__counter {
    display: inline-block;
  }
  .openassessment .openassessment__steps__step .step__title .step__counter:before {
    display: inline;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .openassessment__steps__step .step__title .step__counter {
    display: inline-block;
  }
  .openassessment .openassessment__steps__step .step__title .step__counter:before {
    display: inline;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .openassessment__steps__step .step__title .step__counter {
    display: inline-block;
  }
  .openassessment .openassessment__steps__step .step__title .step__counter:before {
    display: inline;
  }
}
.openassessment .openassessment__steps__step .step__title .step__label, .openassessment .openassessment__steps__step .step__title .step__deadline {
  display: block;
}
.openassessment .openassessment__steps__step .step__title .step__deadline {
  color: #414141;
  margin-left: 5px;
}
.openassessment .openassessment__steps__step .step__status {
  display: inline;
  margin-top: 5px;
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .openassessment__steps__step .step__status {
    margin-top: 0;
    float: left;
    position: relative;
    top: -32px;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .openassessment__steps__step .step__status {
    margin-top: 0;
    float: left;
    position: relative;
    top: -32px;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .openassessment__steps__step .step__status {
    margin-top: 0;
    float: left;
    position: relative;
    top: -32px;
  }
}
.openassessment .openassessment__steps__step .step__status .step__status__value {
  border-radius: 2px;
  padding: 4px 10px;
  background: #d9d9d9;
  position: relative;
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .openassessment .openassessment__steps__step .step__status .step__status__value {
    display: block;
  }
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .openassessment__steps__step .step__status .step__status__value {
    display: block;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .openassessment__steps__step .step__status .step__status__value {
    display: block;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .openassessment__steps__step .step__status .step__status__value {
    display: block;
  }
}
.openassessment .openassessment__steps__step .step__status .icon {
  display: inline-block;
  vertical-align: baseline;
  margin-left: 10px;
}
.openassessment .openassessment__steps__step .step__status .copy {
  color: #414141;
}
.openassessment .openassessment__steps__step .wrapper--step__content {
  margin-top: 10px;
  padding-top: 10px;
  border-top: 1px solid #d9d9d9;
}
.openassessment .openassessment__steps__step .step__message {
  *zoom: 1;
  display: block;
  margin-bottom: 20px;
}
.openassessment .openassessment__steps__step .step__message:before, .openassessment .openassessment__steps__step .step__message:after {
  content: " ";
  display: table;
}
.openassessment .openassessment__steps__step .step__message:after {
  clear: both;
}
.openassessment .openassessment__steps__step .step__instruction {
  margin-bottom: 20px;
  color: #414141;
}
.openassessment .openassessment__steps__step .step__content {
  margin-bottom: 20px;
}
.openassessment .openassessment__steps__step .step__content__section {
  margin-bottom: 40px;
}
.openassessment .openassessment__steps__step.is--loading .step__header {
  padding-bottom: 0;
  border-bottom: none;
  margin-bottom: 0;
}
.openassessment .openassessment__steps__step.is--loading .step__counter:before, .openassessment .openassessment__steps__step.is--loading .step__label, .openassessment .openassessment__steps__step.is--loading .step__deadline {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--loading .step__status__value {
  background: #d9d9d9;
}
.openassessment .openassessment__steps__step.is--loading .step__status__value .wrapper--anim {
  -webkit-animation: pulse 3s ease-in-out infinite;
  -moz-animation: pulse 3s ease-in-out infinite;
  animation: pulse 3s ease-in-out infinite;
}
.openassessment .openassessment__steps__step.is--loading .step__status__value .icon {
  display: inline-block;
  color: #414141;
}
.openassessment .openassessment__steps__step.is--loading .step__status__value .copy {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--incomplete .step__counter:before {
  border-color: #ffc01f;
  color: #414141;
}
.openassessment .openassessment__steps__step.is--incomplete .step__status__value {
  background: #ffc01f;
}
.openassessment .openassessment__steps__step.is--incomplete .step__status__value .copy, .openassessment .openassessment__steps__step.is--incomplete .step__status__value .icon {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--unavailable .step__counter:before, .openassessment .openassessment__steps__step.is--unavailable .step__label, .openassessment .openassessment__steps__step.is--unavailable .step__deadline {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--unavailable .step__status__value {
  background: #d9d9d9;
}
.openassessment .openassessment__steps__step.is--unavailable .step__status__value .icon {
  display: inline-block;
  color: #414141;
}
.openassessment .openassessment__steps__step.is--unavailable .step__status__value .copy {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--waiting .step__counter:before, .openassessment .openassessment__steps__step.is--waiting .step__label, .openassessment .openassessment__steps__step.is--waiting .step__deadline {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--waiting .step__status__value {
  background: #d9d9d9;
}
.openassessment .openassessment__steps__step.is--waiting .step__status__value .icon {
  display: inline-block;
  color: #414141;
}
.openassessment .openassessment__steps__step.is--waiting .step__status__value .copy {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--waiting .step__title .grade__value__title {
  color: #d9d9d9;
}
.openassessment .openassessment__steps__step.is--waiting .step__instruction, .openassessment .openassessment__steps__step.is--waiting .grade__value__description {
  color: #414141;
}
.openassessment .openassessment__steps__step.is--complete .step__counter:before {
  color: #008100;
}
.openassessment .openassessment__steps__step.is--complete .step__status__value {
  background: #008100;
}
.openassessment .openassessment__steps__step.is--complete .step__status__value .icon {
  display: inline-block;
}
.openassessment .openassessment__steps__step.is--complete .step__status__value .copy, .openassessment .openassessment__steps__step.is--complete .step__status__value .icon {
  color: white;
}
.openassessment .openassessment__steps__step.is--complete button {
  opacity: 1;
}
.openassessment .openassessment__steps__step.is--in-progress .step__counter:before {
  color: #0075b4;
}
.openassessment .openassessment__steps__step.is--in-progress .step__status__value {
  background: #0075b4;
}
.openassessment .openassessment__steps__step.is--in-progress .step__status__value .icon {
  display: inline-block;
}
.openassessment .openassessment__steps__step.is--in-progress .step__status__value .copy, .openassessment .openassessment__steps__step.is--in-progress .step__status__value .icon {
  color: white;
}
.openassessment .openassessment__steps__step.is--in-progress button {
  opacity: 1;
}
.openassessment .openassessment__steps__step.has--error .step__counter:before {
  border-color: rgba(178, 6, 16, 0.5);
  color: #b20610;
}
.openassessment .openassessment__steps__step.has--error .step__status__value {
  background: #b20610;
}
.openassessment .openassessment__steps__step.has--error .step__status__value .copy, .openassessment .openassessment__steps__step.has--error .step__status__value .icon {
  color: white;
}
.openassessment .openassessment__steps__step.has--error .step__label {
  color: #b20610;
}
.openassessment .ui-slidable__container {
  -webkit-transition: color 0.25s ease-in;
  -moz-transition: color 0.25s ease-in;
  transition: color 0.25s ease-in;
}
.openassessment .ui-slidable__container .ui-slidable__content {
  width: 100%;
}
.openassessment .ui-slidable__container .icon {
  -webkit-transition: all 0.25s ease-in-out 0s;
  -moz-transition: all 0.25s ease-in-out 0s;
  transition: all 0.25s ease-in-out 0s;
  -webkit-transform: rotate(180deg);
  -moz-transform: rotate(180deg);
  -ms-transform: rotate(180deg);
  -o-transform: rotate(180deg);
  transform: rotate(180deg);
}
.openassessment .ui-slidable__container.is--initially--collapsed .ui-slidable__content {
  display: none;
}
.openassessment .ui-slidable__container.is--showing .ui-slidable .icon {
  -webkit-transform: rotate(90deg);
  -moz-transform: rotate(90deg);
  -ms-transform: rotate(90deg);
  -o-transform: rotate(90deg);
  transform: rotate(90deg);
  -webkit-transform-origin: 50% 50%;
  -moz-transform-origin: 50% 50%;
  -ms-transform-origin: 50% 50%;
  -o-transform-origin: 50% 50%;
  transform-origin: 50% 50%;
}
.openassessment .ui-slidable__container.is--showing .ui-slidable__container:not(.is--showing) .icon {
  -webkit-transition: all 0.25s ease-in-out 0s;
  -moz-transition: all 0.25s ease-in-out 0s;
  transition: all 0.25s ease-in-out 0s;
  -webkit-transform: rotate(180deg);
  -moz-transform: rotate(180deg);
  -ms-transform: rotate(180deg);
  -o-transform: rotate(180deg);
  transform: rotate(180deg);
}
.openassessment .ui-slidable__container .ui-slidable {
  background: none;
  border-radius: 5px;
  padding: 5px;
}
.openassessment .ui-slidable__container .ui-slidable:active, .openassessment .ui-slidable__container .ui-slidable:focus:not(:disabled), .openassessment .ui-slidable__container .ui-slidable:hover:not(:disabled) {
  background: none;
  border: none;
  box-shadow: none;
}
.openassessment .ui-slidable__container .ui-slidable:not([disabled]) {
  color: #0075b4;
}
.openassessment .ui-slidable__container .ui-slidable:not([disabled]):hover, .openassessment .ui-slidable__container .ui-slidable:not([disabled]):focus {
  color: #065683;
}
.openassessment .ui-slidable__container .ui-slidable .wrapper--copy {
  text-align: left;
}
.openassessment .submission__answer__part__prompt {
  position: relative;
  margin: 10px 0;
  border: 1px solid #d9d9d9;
  border-right: 10px solid #b9c1c8;
  padding-right: 30px;
}
.openassessment .submission__answer__part__prompt__copy {
  color: #414141;
}
.openassessment .submission__answer__part__prompt__copy > * {
  margin-bottom: 20px;
}
.openassessment .submission__answer__part__text__value {
  margin: 10px 0;
  border-color: #7c8f9c;
}
.openassessment .submission__answer__part__text__value:not(textarea) {
  border: 1px solid #b9c1c8;
  background-color: white;
  padding: 20px;
}
.openassessment .submission__file__description__label {
  margin-bottom: 5px;
}
.openassessment .submission__answer__file__block {
  margin-bottom: 8px;
}
.openassessment .submission__img__preview {
  float: left;
  margin-right: 10px;
}
.openassessment .step--response .response__submission {
  padding: 10px 10px 20px 10px;
  background: #f5f6f7;
  border-radius: 2px;
}
.openassessment .step--response .response__submission__content {
  margin-bottom: 20px;
}
.openassessment .step--response .response__submission__content .files__descriptions {
  display: none;
}
.openassessment .step--response .response__submission__content .files__descriptions .submission__file__description {
  padding-bottom: 10px;
}
.openassessment .step--response .response__submission__content .submission__answer__part__text textarea {
  min-height: 200px;
}
.openassessment .step--response .response__submission__content .submission__file__description textarea {
  min-height: 80px;
  width: 70%;
}
.openassessment .step--response .response__submission__content .tip {
  padding: 5px 10px;
  background: #b9c1c8;
  color: white;
}
.openassessment .step--peer-assessment .peer-assessment__display__header {
  *zoom: 1;
}
.openassessment .step--peer-assessment .peer-assessment__display__header:before, .openassessment .step--peer-assessment .peer-assessment__display__header:after {
  content: " ";
  display: table;
}
.openassessment .step--peer-assessment .peer-assessment__display__header:after {
  clear: both;
}
.openassessment .step--peer-assessment .peer-assessment__display__header .peer-assessment__display__title {
  margin-bottom: 10px;
  color: #414141;
}
.openassessment .step--peer-assessment .peer-assessment__display__response {
  color: #414141;
}
.openassessment .step--peer-assessment .peer-assessment__assessment .assessment__fields {
  margin-bottom: 20px;
}
.openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question--feedback .wrapper--input {
  margin-top: 20px;
}
.openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question--feedback .question__title__copy {
  margin-right: 0;
  white-space: pre-wrap;
}
.openassessment .step--peer-assessment .peer-assessment__assessment .assessment__rubric__question--feedback textarea {
  min-height: 100px;
}
.openassessment .step--self-assessment .self-assessment__display__header {
  *zoom: 1;
}
.openassessment .step--self-assessment .self-assessment__display__header:before, .openassessment .step--self-assessment .self-assessment__display__header:after {
  content: " ";
  display: table;
}
.openassessment .step--self-assessment .self-assessment__display__header:after {
  clear: both;
}
.openassessment .step--self-assessment .self-assessment__display__title {
  margin-bottom: 10px;
  color: #414141;
}
.openassessment .step--self-assessment .self-assessment__display__response {
  color: #414141;
}
.openassessment .step--self-assessment .self-assessment__assessment .assessment__fields {
  margin-bottom: 20px;
}
.openassessment .step--self-assessment .self-assessment__assessment .question__answers {
  overflow: visible;
}
.openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question--feedback .wrapper--input {
  margin-top: 20px;
}
.openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question--feedback .question__title__copy {
  margin-right: 0;
  white-space: pre-wrap;
}
.openassessment .step--self-assessment .self-assessment__assessment .assessment__rubric__question--feedback textarea {
  min-height: 100px;
}
@media screen and (max-width: 529px) {
  .openassessment .step--grade .step__title {
    float: right;
    display: block;
    margin-left: 7.4229703521%;
    width: 100%;
  }
  .openassessment .step--grade .step__title:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .openassessment .step--grade .step__title {
    float: right;
    display: block;
    margin-left: 4.8291579146%;
    width: 100%;
  }
  .openassessment .step--grade .step__title:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .step--grade .step__title {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 100%;
  }
  .openassessment .step--grade .step__title:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .step--grade .step__title {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 100%;
  }
  .openassessment .step--grade .step__title:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .step--grade .step__title {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 100%;
  }
  .openassessment .step--grade .step__title:last-child {
    margin-left: 0;
  }
}
.openassessment .step--grade .step__title .step__counter, .openassessment .step--grade .step__title .step__counter:before {
  display: none;
}
.openassessment .step--grade .step__title .wrapper--copy {
  margin-right: 0;
  border-right: none;
  padding-right: 0;
}
.openassessment .step--grade .step__title .step__label, .openassessment .step--grade .step__title .grade__value {
  display: inline;
  vertical-align: middle;
}
.openassessment .step--grade .step__title .step__label {
  margin-left: 10px;
}
.openassessment .step--grade .grade__value__description {
  margin-bottom: 20px;
  color: #414141;
}
.openassessment .submission__answer__display__title {
  margin-bottom: 10px;
  color: #414141;
}
.openassessment .submission__answer__display__content, .openassessment .self-assessment__display__image,
.openassessment .peer-assessment__display__image,
.openassessment .staff-assessment__display__image,
.openassessment .submission__answer__display__image {
  padding: 10px 10px 20px 10px;
  border-radius: 2px;
  list-style: none;
}
.openassessment .submission__answer__display__content .submission__answer__part, .openassessment .self-assessment__display__image .submission__answer__part,
.openassessment .peer-assessment__display__image .submission__answer__part,
.openassessment .staff-assessment__display__image .submission__answer__part,
.openassessment .submission__answer__display__image .submission__answer__part {
  margin: 30px 0px 0px;
}
.openassessment .submission__answer__display__content .submission__answer__part:first-child, .openassessment .self-assessment__display__image .submission__answer__part:first-child,
.openassessment .peer-assessment__display__image .submission__answer__part:first-child,
.openassessment .staff-assessment__display__image .submission__answer__part:first-child,
.openassessment .submission__answer__display__image .submission__answer__part:first-child {
  margin-top: 0px;
}
.openassessment .submission__peer-evaluations__title {
  margin-bottom: 10px;
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions {
  margin-bottom: 10px;
}
.openassessment .submission__peer-evaluations__questions:last-child {
  margin-bottom: 0;
  border-bottom: none;
  padding-bottom: 0;
}
.openassessment .submission__peer-evaluations__questions .question {
  margin-bottom: 30px;
}
.openassessment .submission__peer-evaluations__questions .question__title {
  *zoom: 1;
  margin-bottom: 20px;
  border-bottom: 2px solid #414141;
  padding-bottom: 5px;
}
.openassessment .submission__peer-evaluations__questions .question__title:before, .openassessment .submission__peer-evaluations__questions .question__title:after {
  content: " ";
  display: table;
}
.openassessment .submission__peer-evaluations__questions .question__title:after {
  clear: both;
}
.openassessment .submission__peer-evaluations__questions .question__title .icon {
  display: inline-block;
  vertical-align: middle;
}
.openassessment .submission__peer-evaluations__questions .question__title__copy {
  margin-right: 10px;
  white-space: pre-wrap;
}
.openassessment .submission__peer-evaluations__questions .question__score {
  float: left;
  margin-right: 20px;
}
.openassessment .submission__peer-evaluations__questions .question__score__value {
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions .question__score__value:after {
  content: "/";
  margin-right: 5px;
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions .question__score__potential {
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions .question__score__potential .unit {
  margin-right: 5px;
}
.openassessment .submission__peer-evaluations__questions .question__answers {
  *zoom: 1;
  display: block;
  overflow: visible;
}
.openassessment .submission__peer-evaluations__questions .question__answers:before, .openassessment .submission__peer-evaluations__questions .question__answers:after {
  content: " ";
  display: table;
}
.openassessment .submission__peer-evaluations__questions .question__answers:after {
  clear: both;
}
.openassessment .submission__peer-evaluations__questions .answer {
  margin-bottom: 20px;
}
.openassessment .submission__peer-evaluations__questions .answer.self-assessment {
  border-right: 5px solid #f5f6f7;
  padding-right: 20px;
}
.openassessment .submission__peer-evaluations__questions .answer.self-assessment .answer__source {
  color: #d9d9d9;
}
.openassessment .submission__peer-evaluations__questions .answer__source {
  display: block;
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions .answer__source .answer__source__value-with-points {
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions .answer__value {
  display: block;
  margin-bottom: 10px;
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions .answer__value .answer__value__raw {
  white-space: pre-wrap;
}
.openassessment .submission__peer-evaluations__questions .answer__value__value {
  color: #414141;
}
.openassessment .submission__peer-evaluations__questions .answer__value__value .ui-hint {
  color: #d9d9d9;
}
.openassessment .submission__peer-evaluations__questions .answer__value__value .ui-hint:hover, .openassessment .submission__peer-evaluations__questions .answer__value__value .ui-hint:focus {
  color: #065683;
}
.openassessment .submission__peer-evaluations__questions .answer__score, .openassessment .submission__peer-evaluations__questions .answer__score__value {
  display: block;
  color: #414141;
}
@media screen and (min-width: 530px) and (max-width: 759px) {
  .openassessment .answer--feedback {
    float: right;
    display: block;
    margin-left: 4.8291579146%;
    width: 100%;
  }
  .openassessment .answer--feedback:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 760px) and (max-width: 924px) {
  .openassessment .answer--feedback {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 100%;
  }
  .openassessment .answer--feedback:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 925px) and (max-width: 1299px) {
  .openassessment .answer--feedback {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 100%;
  }
  .openassessment .answer--feedback:last-child {
    margin-left: 0;
  }
}
@media screen and (min-width: 1300px) {
  .openassessment .answer--feedback {
    float: right;
    display: block;
    margin-left: 2.3576515979%;
    width: 100%;
  }
  .openassessment .answer--feedback:last-child {
    margin-left: 0;
  }
}
.openassessment .answer--feedback .answer--feedback__title__copy {
  margin-right: 10px;
}
.openassessment .answer--feedback .answer--feedback__content {
  margin-top: 5px;
}
.openassessment .answer--feedback .feedback {
  margin-bottom: 20px;
}
.openassessment .answer--feedback .feedback__source {
  display: block;
  color: #414141;
}
.openassessment .answer--feedback .feedback__value {
  display: block;
}
.openassessment .answer--feedback .feedback__value .feedback__value__raw {
  white-space: pre-wrap;
}
.openassessment .submission__feedback__title {
  border-bottom-color: solid #414141;
  color: #414141;
}
.openassessment .submission__feedback__title .icon {
  display: inline-block;
  vertical-align: middle;
}
.openassessment .submission__feedback__title__copy {
  margin-right: 10px;
}
.openassessment .submission__feeedback__content {
  margin-bottom: 20px;
}
.openassessment .submission__feedback__instructions {
  margin-bottom: 20px;
  color: #414141;
}
.openassessment .submission__feedback__elements {
  padding: 20px 20px 20px 20px;
  background: #f5f6f7;
  border-radius: 2px;
}
.openassessment .submission__feedback__fields {
  margin-bottom: 20px;
}
.openassessment .submission__feedback__fields .field {
  margin-bottom: 30px;
}
.openassessment .submission__feedback__fields .field:last-child {
  margin-bottom: 0;
}
.openassessment .submission__feedback__fields select {
  display: block;
}
.openassessment .submission__feedback__fields textarea {
  min-height: 200px;
}
.openassessment .submission__feedback__actions {
  padding-top: 0;
}
.openassessment .submission__feedback__actions .list--actions {
  padding: 0;
}
.openassessment .action--submit {
  margin-left: 10px;
}

.open-response-assessment-block table {
  table-layout: fixed;
}
.open-response-assessment-block .open-response-assessment-msg {
  margin-bottom: 20px;
}
.open-response-assessment-block .open-response-assessment-content,
.open-response-assessment-block .open-response-assessment-item {
  display: none;
}
.open-response-assessment-block .open-response-assessment-summary td {
  border: none;
}
.open-response-assessment-block .open-response-assessment-item-breadcrumbs {
  padding-bottom: 20px;
}
.open-response-assessment-block .open-response-assessment-item-breadcrumbs a {
  cursor: pointer;
}
.open-response-assessment-block .open-response-assessment-main-table th {
  background-color: #ffffff;
  text-align: left;
}
.open-response-assessment-block .open-response-assessment-main-table th button {
  background: none;
  border: none;
  box-shadow: none;
  outline: none;
  font-size: 14px;
  padding-left: 0px;
  text-align: left;
}
.open-response-assessment-block .open-response-assessment-main-table th a {
  color: #000000 !important;
}
.open-response-assessment-block .open-response-assessment-main-table td a {
  cursor: pointer;
}
.open-response-assessment-block .open-response-assessment-main-table tbody tr:nth-child(odd) {
  background-color: #f9f9f9;
}
.open-response-assessment-block .backgrid, .open-response-assessment-block .ora-summary-value {
  text-overflow: ellipsis;
  overflow: hidden;
}
.open-response-assessment-block .ora-summary-title {
  color: #0079bc;
  font-weight: bold;
  text-transform: uppercase;
  font-size: 0.8em;
}
.open-response-assessment-block .ora-summary-value {
  font-size: 1.5em;
  padding-top: 3px;
}
.open-response-assessment-block .backgrid .peer, .open-response-assessment-block .backgrid .self, .open-response-assessment-block .backgrid .staff, .open-response-assessment-block .backgrid .waiting, .open-response-assessment-block .backgrid .training {
  width: 80px;
}
.open-response-assessment-block .backgrid .total {
  width: 140px;
}
.open-response-assessment-block .backgrid .done {
  width: 170px;
}

/*! jQuery UI - v1.12.1 - 2021-07-13
* http://jqueryui.com
* Includes: theme.css
* To view and modify this theme, visit http://jqueryui.com/themeroller/?bgImgOpacityError=&bgImgOpacityHighlight=&bgImgOpacityActive=&bgImgOpacityHover=&bgImgOpacityDefault=&bgImgOpacityContent=&bgImgOpacityHeader=30&cornerRadiusShadow=8px&offsetLeftShadow=0px&offsetTopShadow=0px&thicknessShadow=5px&opacityShadow=30&bgImgOpacityShadow=0&bgTextureShadow=flat&bgColorShadow=%23666666&opacityOverlay=30&bgImgOpacityOverlay=0&bgTextureOverlay=flat&bgColorOverlay=%23aaaaaa&iconColorError=%23cc0000&fcError=%235f3f3f&borderColorError=%23f1a899&bgTextureError=flat&bgColorError=%23fddfdf&iconColorHighlight=%23ebebeb&fcHighlight=%23000000&borderColorHighlight=%23000000&bgTextureHighlight=flat&bgColorHighlight=%23d7d7d7&iconColorActive=%23777777&fcActive=%23454545&borderColorActive=%23c5c5c5&bgTextureActive=flat&bgColorActive=%23ffffff&iconColorHover=%23555555&fcHover=%232b2b2b&borderColorHover=%23cccccc&bgTextureHover=flat&bgColorHover=%23f3f3f3&iconColorDefault=%23777777&fcDefault=%23454545&borderColorDefault=%23c5c5c5&bgTextureDefault=flat&bgColorDefault=%23ffffff&iconColorContent=%23444444&fcContent=%23454545&borderColorContent=%23c5c5c5&bgTextureContent=flat&bgColorContent=%23ffffff&iconColorHeader=%23444444&fcHeader=%23454545&borderColorHeader=%23D7D3D1&bgTextureHeader=flat&bgColorHeader=%23ffffff&cornerRadius=1px&fwDefault=normal&fsDefault=1em&ffDefault=Arial%2CHelvetica%2Csans-serif
* Copyright jQuery Foundation and other contributors; Licensed MIT */
/*! jQuery UI - v1.12.1 - 2021-07-13
* http://jqueryui.com
* Includes: theme.css
* To view and modify this theme, visit http://jqueryui.com/themeroller/?bgImgOpacityError=&bgImgOpacityHighlight=&bgImgOpacityActive=&bgImgOpacityHover=&bgImgOpacityDefault=&bgImgOpacityContent=&bgImgOpacityHeader=30&cornerRadiusShadow=8px&offsetLeftShadow=0px&offsetTopShadow=0px&thicknessShadow=5px&opacityShadow=30&bgImgOpacityShadow=0&bgTextureShadow=flat&bgColorShadow=%23666666&opacityOverlay=30&bgImgOpacityOverlay=0&bgTextureOverlay=flat&bgColorOverlay=%23aaaaaa&iconColorError=%23cc0000&fcError=%235f3f3f&borderColorError=%23f1a899&bgTextureError=flat&bgColorError=%23fddfdf&iconColorHighlight=%23ebebeb&fcHighlight=%23000000&borderColorHighlight=%23000000&bgTextureHighlight=flat&bgColorHighlight=%23d7d7d7&iconColorActive=%23777777&fcActive=%23454545&borderColorActive=%23c5c5c5&bgTextureActive=flat&bgColorActive=%23ffffff&iconColorHover=%23555555&fcHover=%232b2b2b&borderColorHover=%23cccccc&bgTextureHover=flat&bgColorHover=%23f3f3f3&iconColorDefault=%23777777&fcDefault=%23454545&borderColorDefault=%23c5c5c5&bgTextureDefault=flat&bgColorDefault=%23ffffff&iconColorContent=%23444444&fcContent=%23454545&borderColorContent=%23c5c5c5&bgTextureContent=flat&bgColorContent=%23ffffff&iconColorHeader=%23444444&fcHeader=%23454545&borderColorHeader=%23D7D3D1&bgTextureHeader=flat&bgColorHeader=%23ffffff&cornerRadius=1px&fwDefault=normal&fsDefault=1em&ffDefault=Arial%2CHelvetica%2Csans-serif
* Copyright jQuery Foundation and other contributors; Licensed MIT */
.view--placeholder {
  background: transparent;
  margin: 40px 40px;
}

.ora-confirmation-alert {
  /* Interaction states
  ----------------------------------*/
  /* Interaction Cues
  ----------------------------------*/
  /* Misc visuals
  ----------------------------------*/
  /* Corner radius */
  /* Overlays */
}
.ora-confirmation-alert .ui-widget {
  font-family: Arial, Helvetica, sans-serif;
  font-size: 1em;
}
.ora-confirmation-alert .ui-widget .ui-widget {
  font-size: 1em;
}
.ora-confirmation-alert .ui-widget input,
.ora-confirmation-alert .ui-widget select,
.ora-confirmation-alert .ui-widget textarea,
.ora-confirmation-alert .ui-widget button {
  font-family: Arial, Helvetica, sans-serif;
  font-size: 1em;
}
.ora-confirmation-alert .ui-widget.ui-widget-content {
  border: 1px solid #c5c5c5;
}
.ora-confirmation-alert .ui-widget-content {
  border: 1px solid #c5c5c5;
  background: #ffffff;
  color: #454545;
}
.ora-confirmation-alert .ui-widget-content a {
  color: #454545;
}
.ora-confirmation-alert .ui-widget-header {
  border: 1px solid #D7D3D1;
  background: #ffffff;
  color: #454545;
  font-weight: bold;
}
.ora-confirmation-alert .ui-widget-header a {
  color: #454545;
}
.ora-confirmation-alert .ui-state-default,
.ora-confirmation-alert .ui-widget-content .ui-state-default,
.ora-confirmation-alert .ui-widget-header .ui-state-default,
.ora-confirmation-alert .ui-button,
.ora-confirmation-alert html .ui-button.ui-state-disabled:hover,
.ora-confirmation-alert html .ui-button.ui-state-disabled:active {
  border: 1px solid #c5c5c5;
  background: #ffffff;
  font-weight: normal;
  color: #454545;
}
.ora-confirmation-alert .ui-state-default a,
.ora-confirmation-alert .ui-state-default a:link,
.ora-confirmation-alert .ui-state-default a:visited,
.ora-confirmation-alert a .ui-button,
.ora-confirmation-alert a:link .ui-button,
.ora-confirmation-alert a:visited .ui-button,
.ora-confirmation-alert .ui-button {
  color: #454545;
  text-decoration: none;
}
.ora-confirmation-alert .ui-state-hover,
.ora-confirmation-alert .ui-widget-content .ui-state-hover,
.ora-confirmation-alert .ui-widget-header .ui-state-hover,
.ora-confirmation-alert .ui-state-focus,
.ora-confirmation-alert .ui-widget-content .ui-state-focus,
.ora-confirmation-alert .ui-widget-header .ui-state-focus,
.ora-confirmation-alert .ui-button:hover,
.ora-confirmation-alert .ui-button:focus {
  border: 1px solid #cccccc;
  background: #f3f3f3;
  font-weight: normal;
  color: #2b2b2b;
}
.ora-confirmation-alert .ui-state-hover a,
.ora-confirmation-alert .ui-state-hover a:hover,
.ora-confirmation-alert .ui-state-hover a:link,
.ora-confirmation-alert .ui-state-hover a:visited,
.ora-confirmation-alert .ui-state-focus a,
.ora-confirmation-alert .ui-state-focus a:hover,
.ora-confirmation-alert .ui-state-focus a:link,
.ora-confirmation-alert .ui-state-focus a:visited,
.ora-confirmation-alert a .ui-button:hover,
.ora-confirmation-alert a .ui-button:focus {
  color: #2b2b2b;
  text-decoration: none;
}
.ora-confirmation-alert .ui-visual-focus {
  box-shadow: 0 0 3px 1px #5e9ed6;
}
.ora-confirmation-alert .ui-state-active,
.ora-confirmation-alert .ui-widget-content .ui-state-active,
.ora-confirmation-alert .ui-widget-header .ui-state-active,
.ora-confirmation-alert a .ui-button:active,
.ora-confirmation-alert .ui-button:active,
.ora-confirmation-alert .ui-button.ui-state-active:hover {
  border: 1px solid #c5c5c5;
  background: #ffffff;
  font-weight: normal;
  color: #454545;
}
.ora-confirmation-alert .ui-icon-background,
.ora-confirmation-alert .ui-state-active .ui-icon-background {
  border: #c5c5c5;
  background-color: #454545;
}
.ora-confirmation-alert .ui-state-active a,
.ora-confirmation-alert .ui-state-active a:link,
.ora-confirmation-alert .ui-state-active a:visited {
  color: #454545;
  text-decoration: none;
}
.ora-confirmation-alert .ui-state-highlight,
.ora-confirmation-alert .ui-widget-content .ui-state-highlight,
.ora-confirmation-alert .ui-widget-header .ui-state-highlight {
  border: 1px solid #000000;
  background: #d7d7d7;
  color: #000000;
}
.ora-confirmation-alert .ui-state-checked {
  border: 1px solid #000000;
  background: #d7d7d7;
}
.ora-confirmation-alert .ui-state-highlight a,
.ora-confirmation-alert .ui-widget-content .ui-state-highlight a,
.ora-confirmation-alert .ui-widget-header .ui-state-highlight a {
  color: #000000;
}
.ora-confirmation-alert .ui-state-error,
.ora-confirmation-alert .ui-widget-content .ui-state-error,
.ora-confirmation-alert .ui-widget-header .ui-state-error {
  border: 1px solid #f1a899;
  background: #fddfdf;
  color: #5f3f3f;
}
.ora-confirmation-alert .ui-state-error a,
.ora-confirmation-alert .ui-widget-content .ui-state-error a,
.ora-confirmation-alert .ui-widget-header .ui-state-error a {
  color: #5f3f3f;
}
.ora-confirmation-alert .ui-state-error-text,
.ora-confirmation-alert .ui-widget-content .ui-state-error-text,
.ora-confirmation-alert .ui-widget-header .ui-state-error-text {
  color: #5f3f3f;
}
.ora-confirmation-alert .ui-priority-primary,
.ora-confirmation-alert .ui-widget-content .ui-priority-primary,
.ora-confirmation-alert .ui-widget-header .ui-priority-primary {
  font-weight: bold;
}
.ora-confirmation-alert .ui-priority-secondary,
.ora-confirmation-alert .ui-widget-content .ui-priority-secondary,
.ora-confirmation-alert .ui-widget-header .ui-priority-secondary {
  opacity: 0.7;
  filter: Alpha(Opacity=70);
  /* support: IE8 */
  font-weight: normal;
}
.ora-confirmation-alert .ui-state-disabled,
.ora-confirmation-alert .ui-widget-content .ui-state-disabled,
.ora-confirmation-alert .ui-widget-header .ui-state-disabled {
  opacity: 0.35;
  filter: Alpha(Opacity=35);
  /* support: IE8 */
  background-image: none;
}
.ora-confirmation-alert .ui-state-disabled .ui-icon {
  filter: Alpha(Opacity=35);
  /* support: IE8 - See #6059 */
}
.ora-confirmation-alert .ui-corner-all,
.ora-confirmation-alert .ui-corner-top,
.ora-confirmation-alert .ui-corner-left,
.ora-confirmation-alert .ui-corner-tl {
  border-top-left-radius: 1px;
}
.ora-confirmation-alert .ui-corner-all,
.ora-confirmation-alert .ui-corner-top,
.ora-confirmation-alert .ui-corner-right,
.ora-confirmation-alert .ui-corner-tr {
  border-top-right-radius: 1px;
}
.ora-confirmation-alert .ui-corner-all,
.ora-confirmation-alert .ui-corner-bottom,
.ora-confirmation-alert .ui-corner-left,
.ora-confirmation-alert .ui-corner-bl {
  border-bottom-left-radius: 1px;
}
.ora-confirmation-alert .ui-corner-all,
.ora-confirmation-alert .ui-corner-bottom,
.ora-confirmation-alert .ui-corner-right,
.ora-confirmation-alert .ui-corner-br {
  border-bottom-right-radius: 1px;
}
.ora-confirmation-alert .ui-widget-overlay {
  background: #aaaaaa;
  opacity: 0.3;
  filter: Alpha(Opacity=30);
  /* support: IE8 */
}
.ora-confirmation-alert .ui-widget-shadow {
  -webkit-box-shadow: 0px 0px 5px #666666;
  box-shadow: 0px 0px 5px #666666;
}

.no-close .ui-dialog-titlebar-close {
  display: none;
}

.wrapper--openassessment .wrapper--staff-area {
  width: 100%;
  overflow: auto;
  margin-top: 20px;
  padding-top: 20px;
}
.wrapper--openassessment .wrapper--staff-area .wrapper--staff-toolbar {
  position: relative;
  text-align: right;
  margin: 0 0 8px;
  padding: 10px;
}
.wrapper--openassessment .wrapper--staff-area .wrapper--staff-toolbar .ui-staff__button {
  padding: 5px 10px;
  font-size: 12px;
  border-radius: 5px;
  text-transform: uppercase;
  color: #414141;
  background-color: rgba(3, 3, 3, 0.05);
}
.wrapper--openassessment .wrapper--staff-area .wrapper--staff-toolbar .ui-staff__button.is--active {
  color: white;
  background-color: #b62568;
}
.wrapper--openassessment .wrapper--staff-area .wrapper--staff-toolbar .ui-staff__button.is--active:hover {
  background-color: #921e53;
}
.wrapper--openassessment .wrapper--staff-area .wrapper--staff-toolbar .ui-staff__button:hover {
  color: white;
  background-color: #0075b4;
}
.wrapper--openassessment .wrapper--staff-area .wrapper--ui-staff {
  margin-top: 0;
}
.wrapper--openassessment .wrapper--staff-area .wrapper--ui-staff .ui-staff_close_button {
  margin: 0;
  padding: 0;
  border: 0;
  float: right;
  background-color: #2c2f36;
  font-size: 12px;
  -webkit-appearance: none;
  background-image: none;
  text-shadow: none;
  box-shadow: none;
  border: none;
  border-image: none;
}
.wrapper--openassessment .wrapper--staff-area .wrapper--ui-staff .ui-staff_close_button:hover {
  color: white;
}

.staff-info__student .label {
  color: white;
  margin-bottom: 10px;
}
.staff-info__student .label input {
  display: block;
  padding-top: 0;
  padding-bottom: 0;
}
.staff-info__student .title {
  color: white;
  margin-bottom: 10px;
}
.staff-info__student .title--sub {
  color: white;
  margin-top: 10px;
  margin-bottom: 10px;
}
.staff-info__student .student__answer__display__content {
  border: 1px solid rgba(255, 255, 255, 0.25);
  padding: 10px 10px;
  margin-bottom: 10px;
}
.staff-info__student .staff-info__student__report {
  list-style-type: none;
}
.staff-info__student .staff-info__student__report .title {
  margin-top: 10px;
  border-top: 1px solid white;
  padding: 10px 20px 10px 0;
}
.staff-info__student .staff-info__student__report .title span {
  font-weight: inherit;
}
.staff-info__student .value {
  width: 50%;
}

.step--student-training .student-training__display__header {
  *zoom: 1;
}
.step--student-training .student-training__display__header:before, .step--student-training .student-training__display__header:after {
  content: " ";
  display: table;
}
.step--student-training .student-training__display__header:after {
  clear: both;
}
.step--student-training .student-training__display__title {
  margin-bottom: 10px;
  color: #414141;
}
.step--student-training .student-training__display__response {
  color: #414141;
}
.step--student-training .student-training__assessment .assessment__fields {
  margin-bottom: 20px;
}
.step--student-training .student-training__assessment .question__answers {
  overflow: visible;
}
.step--student-training .student-training__assessment .assessment__rubric__question--feedback textarea {
  min-height: 100px;
}
.step--student-training .message--correct {
  margin-top: 20px;
}
.step--student-training .message--correct.is--hidden {
  height: 0;
  width: 0;
  padding: 0;
  margin: 0;
  display: none;
}
.step--student-training .message--correct.is--hidden .step__header {
  border-bottom: none;
}
.step--student-training .message--incorrect {
  margin-top: 20px;
}
.step--student-training .message--incorrect.is--hidden {
  height: 0;
  width: 0;
  padding: 0;
  margin: 0;
  display: none;
}
.step--student-training .message--incorrect.is--hidden .step__header {
  border-bottom: none;
}

#openassessment-editor {
  margin-bottom: 0;
  height: 100%;
}
#openassessment-editor .openassessment_editor_content_and_tabs {
  width: 100%;
  height: calc(100% - 55px);
}
#openassessment-editor #oa_editor_window_title {
  float: right;
}
#openassessment-editor input[type=number] {
  background-image: -moz-linear-gradient(#F2F2F2, #FFF);
  background-image: -webkit-linear-gradient(#F2F2F2, #FFF);
  background-image: linear, false, #F2F2F2, #FFF;
  width: 45%;
}
#openassessment-editor #openassessment_editor_header {
  background-color: #e5e5e5;
  width: 100%;
  top: 0;
  height: 42px;
}
#openassessment-editor #openassessment_editor_header .editor_tabs {
  display: flex;
  justify-content: flex-end;
  /* Inactive tab styling */
  /* Active tab styling */
}
#openassessment-editor #openassessment_editor_header .editor_tabs .oa_editor_tab {
  padding: 0 10px;
  margin: 0px 1%;
  height: 35px;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  -ms-border-radius: 5px;
  -o-border-radius: 5px;
  border-radius: 5px;
  box-shadow: none;
  border: 0;
  text-align: center;
  /* For an invalid tab, drop a validation error icon in the button */
}
#openassessment-editor #openassessment_editor_header .editor_tabs .oa_editor_tab span.validation-warning {
  display: none;
  padding-right: 4px;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .oa_editor_tab span.validation-warning svg {
  width: 1em;
  vertical-align: middle;
  stroke: #424243;
  fill: #F0CC00;
  paint-order: stroke;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .oa_editor_tab span.validation-warning svg path:nth-child(2) {
  stroke: #F0CC00;
  fill: #424243;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .oa_editor_tab a {
  padding: 8px 0 0 0;
  width: 100%;
  text-transform: uppercase;
  outline-color: transparent;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .ui-state-default.oa_editor_tab {
  background: #e5e5e5 none;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .ui-state-default.oa_editor_tab a {
  color: #424243;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .ui-state-active.ui-state-default.oa_editor_tab:hover,
#openassessment-editor #openassessment_editor_header .editor_tabs .ui-state-active.oa_editor_tab {
  background: #626365 none;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .ui-state-active.ui-state-default.oa_editor_tab:hover a,
#openassessment-editor #openassessment_editor_header .editor_tabs .ui-state-active.oa_editor_tab a {
  color: white;
}
#openassessment-editor #openassessment_editor_header .editor_tabs .ui-state-default.oa_editor_tab:hover {
  background-color: rgba(164, 165, 168, 0.75);
}
#openassessment-editor #openassessment_validation_alert {
  width: 100%;
  -webkit-border-top-right-radius: 2px;
  -moz-border-topright-radius: 2px;
  border-top-right-radius: 2px;
  -webkit-border-top-left-radius: 2px;
  -moz-border-topleft-radius: 2px;
  border-top-left-radius: 2px;
  background-color: #333333;
  border-bottom: 3px solid #F0CC00;
  padding: 10px;
  position: absolute;
  z-index: 10;
  max-height: 200px;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_icon {
  position: absolute;
  right: 3%;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_icon svg {
  width: 3em;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_header {
  width: 85%;
  margin: 0 10% 0 5%;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_header .openassessment_alert_title {
  width: auto;
  color: white;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_header .openassessment_alert_message {
  font-size: 80%;
  color: #D7D3D1;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_close {
  display: inline-block;
  position: absolute;
  top: 14px;
  left: 0px;
  color: #e9e9e9;
  font-size: 1.5em;
  text-align: center;
  margin: 5px 10px;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_close [class^=icon] {
  width: auto;
  margin: 0;
  padding: 2px;
}
#openassessment-editor #openassessment_validation_alert .openassessment_alert_close:hover {
  color: #80a9cc;
}
#openassessment-editor .oa_editor_content_wrapper {
  height: calc(100% - 42px);
  width: 100%;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  -ms-border-radius: 3px;
  -o-border-radius: 3px;
  border-radius: 3px;
  border: 1px solid #838486;
  background-color: white;
  overflow-y: scroll;
  position: absolute;
  bottom: 0;
  z-index: 11;
  transition: height 1s ease-in-out 0;
  -webkit-transition: height 1s ease-in-out 0;
  -moz-transition: height 1s ease-in-out 0;
}
#openassessment-editor #openassessment_rubric_editor {
  width: 100%;
  height: 100%;
}
#openassessment-editor #oa_basic_settings_editor #openassessment_title_editor_wrapper label {
  width: 25%;
  text-align: right;
}
#openassessment-editor #oa_basic_settings_editor #openassessment_title_editor_wrapper input {
  width: 45%;
  min-width: 100px;
}
#openassessment-editor #openassessment_step_select_description {
  padding: 10px 10px 0 10px;
  font-size: 80%;
  border-top: 1px solid #838486;
  margin: 7.5px;
}
#openassessment-editor .openassessment_assessment_module_settings_editor.openassessment_shrink_for_drag {
  height: 40px;
}
#openassessment-editor #ai_training_examples {
  margin: 5px 10px 10px 20px;
  height: 300px;
  width: calc(100% - 20px);
  resize: none;
  font: inherit;
  line-height: 110%;
  font-size: 80%;
  padding: 5px 10px;
}
#openassessment-editor .openassessment_assessment_module_settings_editor {
  padding: 0 0 10px 15px;
  margin: 10px;
  border: 1px solid #dbdbdc;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  -ms-border-radius: 3px;
  -o-border-radius: 3px;
  border-radius: 3px;
  position: relative;
  background-color: white;
}
#openassessment-editor .openassessment_assessment_module_settings_editor .drag-handle {
  position: absolute;
  background: center rgba(164, 165, 168, 0.25);
  display: block;
  top: 0px;
  left: 0px;
  z-index: 10;
  width: 15px;
  height: 100%;
  border-right: 1px solid #dbdbdc;
  cursor: move;
  -webkit-transition: none;
  -moz-transition: none;
  transition: none;
}
#openassessment-editor .openassessment_assessment_module_settings_editor .openassessment_description_closed {
  padding: 5px 15px 5px 15px;
  font-size: 75%;
  margin: 0;
}
#openassessment-editor .openassessment_assessment_module_settings_editor .openassessment_description {
  padding: 5px 15px 0 15px;
  font-size: 75%;
  margin: 0;
}
#openassessment-editor .openassessment_assessment_module_settings_editor .openassessment_view_details {
  padding: 5px 15px 0 15px;
  font-size: 75%;
  margin: 0;
}
#openassessment-editor .openassessment_assessment_module_settings_editor .openassessment_view_details a {
  color: #009fe6;
  border-bottom: 1px dotted;
}
#openassessment-editor .openassessment_assessment_module_settings_editor.ui-sortable-helper {
  opacity: 50%;
  border-color: #009fe6;
}
#openassessment-editor .openassessment_assessment_module_settings_editor:hover {
  border-color: #80a9cc;
}
#openassessment-editor .openassessment_assessment_module_settings_editor:hover .drag-handle {
  background-color: #009fe6;
  border-color: #009fe6;
  -webkit-transition: none;
  -moz-transition: none;
  transition: none;
}
#openassessment-editor .openassessment_description {
  font-size: 75%;
  margin: 0;
}
#openassessment-editor .openassessment_inclusion_wrapper {
  margin: 0;
  padding: 15px 8px 5px 5px;
  -webkit-border-top-right-radius: 2px;
  -moz-border-topright-radius: 2px;
  border-top-right-radius: 2px;
  -webkit-border-top-left-radius: 2px;
  -moz-border-topleft-radius: 2px;
  border-top-left-radius: 2px;
}
#openassessment-editor .openassessment_inclusion_wrapper label {
  font-size: 19px;
  cursor: pointer;
  margin-right: 20px;
}
#openassessment-editor label {
  padding-left: 10px;
}
#openassessment-editor .xblock_actions {
  background-color: #c8c9cb;
  position: absolute;
  width: 100%;
  bottom: 0;
}
#openassessment-editor .openassessment_tab_instructions {
  background-color: rgba(164, 165, 168, 0.25);
  border-bottom: 1px solid #838486;
  padding: 10px;
}
#openassessment-editor .openassessment_container_header {
  margin: 10px 10px;
  padding-bottom: 5px;
  border-bottom: 1px solid;
  overflow: auto;
  color: #626365;
}
#openassessment-editor .action.expand-collapse {
  float: right;
}
#openassessment-editor .action.expand-collapse .ui-toggle-expansion {
  color: #626365;
  cursor: pointer;
}
#openassessment-editor .action.expand-collapse .ui-toggle-expansion:hover {
  color: #838486;
  cursor: pointer;
}
#openassessment-editor .openassessment_container_header_title_box {
  float: right;
  width: 80%;
  display: inline-block;
}
#openassessment-editor .openassessment_container_header_title {
  text-transform: uppercase;
  cursor: default;
  padding-top: 2px;
}
#openassessment-editor .openassessment_container_remove_button, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header .openassessment_option_header_remove, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_remove_button {
  cursor: pointer;
  float: left;
}
#openassessment-editor .openassessment_container_remove_button h2:after, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header .openassessment_option_header_remove h2:after, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_remove_button h2:after {
  font-family: FontAwesome;
  content: "";
  display: inline-block;
  color: inherit;
  margin: 0 5px;
}
#openassessment-editor .openassessment_container_remove_button h2, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header .openassessment_option_header_remove h2, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_remove_button h2 {
  text-transform: uppercase;
  font-size: 80%;
  float: left;
  display: inline-block;
  padding: 3px 8px 3px 13px;
}
#openassessment-editor .openassessment_container_remove_button:hover, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header .openassessment_option_header_remove:hover, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_remove_button:hover {
  background-color: #626365;
  border-radius: 4px;
  color: white;
}
#openassessment-editor .openassessment_container_add_button {
  background-color: white;
  border: 1px solid;
  border-radius: 4px;
  text-align: center;
  color: #009fe6;
  padding: 10px;
  margin: 15px 10px;
}
#openassessment-editor .openassessment_container_add_button h6:before {
  font-family: "FontAwesome";
  display: inline-block;
  margin-right: 5px;
  margin-left: 10px;
  width: auto;
  height: auto;
  content: "";
}
#openassessment-editor .openassessment_container_add_button.openassessment_highlighted_field {
  color: red;
  border-width: 2px;
}
#openassessment-editor .openassessment_container_add_button:hover {
  color: white;
  background-color: #009fe6;
  cursor: pointer;
}
#openassessment-editor .openassessment_container_remove_button.is--disabled, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header .is--disabled.openassessment_option_header_remove, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .is--disabled.openassessment_criterion_option_remove_button,
#openassessment-editor .openassessment_container_remove_button.is--disabled:hover,
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header .is--disabled.openassessment_option_header_remove:hover,
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .is--disabled.openassessment_criterion_option_remove_button:hover,
#openassessment-editor .openassessment_container_add_button.is--disabled,
#openassessment-editor .openassessment_container_add_button.is--disabled:hover {
  color: #c8c9cb;
  background-color: transparent;
  cursor: auto;
}
#openassessment-editor #oa_prompts_editor_wrapper .wrapper-comp-settings {
  display: block;
}
#openassessment-editor #oa_prompts_editor_wrapper .openassessment_prompt_description {
  width: 100%;
  min-height: 100px;
  resize: none;
  border: 1px solid #b2b2b2;
  border-radius: 4px;
  padding: 10px;
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 14px;
  overflow: auto;
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
}
#openassessment-editor #oa_prompts_editor_wrapper .openassessment_prompt_description.openassessment_highlighted_field {
  border: 2px solid red;
}
#openassessment-editor #oa_rubric_editor_wrapper .wrapper-comp-settings {
  display: block;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone {
  /* Interaction states
  ----------------------------------*/
  /* Interaction Cues
  ----------------------------------*/
  /* Misc visuals
  ----------------------------------*/
  /* Corner radius */
  /* Overlays */
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget {
  font-family: Arial, Helvetica, sans-serif;
  font-size: 1em;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget .ui-widget {
  font-size: 1em;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget input,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget select,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget textarea,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget button {
  font-family: Arial, Helvetica, sans-serif;
  font-size: 1em;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget.ui-widget-content {
  border: 1px solid #c5c5c5;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content {
  border: 1px solid #c5c5c5;
  background: #ffffff;
  color: #454545;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content a {
  color: #454545;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header {
  border: 1px solid #D7D3D1;
  background: #ffffff;
  color: #454545;
  font-weight: bold;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header a {
  color: #454545;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-default,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-default,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-default,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-button,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone html .ui-button.ui-state-disabled:hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone html .ui-button.ui-state-disabled:active {
  border: 1px solid #c5c5c5;
  background: #ffffff;
  font-weight: normal;
  color: #454545;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-default a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-default a:link,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-default a:visited,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone a .ui-button,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone a:link .ui-button,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone a:visited .ui-button,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-button {
  color: #454545;
  text-decoration: none;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-focus,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-focus,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-focus,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-button:hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-button:focus {
  border: 1px solid #cccccc;
  background: #f3f3f3;
  font-weight: normal;
  color: #2b2b2b;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-hover a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-hover a:hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-hover a:link,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-hover a:visited,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-focus a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-focus a:hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-focus a:link,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-focus a:visited,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone a .ui-button:hover,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone a .ui-button:focus {
  color: #2b2b2b;
  text-decoration: none;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-visual-focus {
  box-shadow: 0 0 3px 1px #5e9ed6;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-active,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-active,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-active,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone a .ui-button:active,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-button:active,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-button.ui-state-active:hover {
  border: 1px solid #c5c5c5;
  background: #ffffff;
  font-weight: normal;
  color: #454545;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-icon-background,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-active .ui-icon-background {
  border: #c5c5c5;
  background-color: #454545;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-active a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-active a:link,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-active a:visited {
  color: #454545;
  text-decoration: none;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-highlight,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-highlight,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-highlight {
  border: 1px solid #000000;
  background: #d7d7d7;
  color: #000000;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-checked {
  border: 1px solid #000000;
  background: #d7d7d7;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-highlight a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-highlight a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-highlight a {
  color: #000000;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-error,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-error,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-error {
  border: 1px solid #f1a899;
  background: #fddfdf;
  color: #5f3f3f;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-error a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-error a,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-error a {
  color: #5f3f3f;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-error-text,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-error-text,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-error-text {
  color: #5f3f3f;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-priority-primary,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-priority-primary,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-priority-primary {
  font-weight: bold;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-priority-secondary,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-priority-secondary,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-priority-secondary {
  opacity: 0.7;
  filter: Alpha(Opacity=70);
  /* support: IE8 */
  font-weight: normal;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-disabled,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-content .ui-state-disabled,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-header .ui-state-disabled {
  opacity: 0.35;
  filter: Alpha(Opacity=35);
  /* support: IE8 */
  background-image: none;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-state-disabled .ui-icon {
  filter: Alpha(Opacity=35);
  /* support: IE8 - See #6059 */
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-all,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-top,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-left,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-tl {
  border-top-left-radius: 1px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-all,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-top,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-right,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-tr {
  border-top-right-radius: 1px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-all,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-bottom,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-left,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-bl {
  border-bottom-left-radius: 1px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-all,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-bottom,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-right,
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-corner-br {
  border-bottom-right-radius: 1px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-overlay {
  background: #aaaaaa;
  opacity: 0.3;
  filter: Alpha(Opacity=30);
  /* support: IE8 */
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-widget-shadow {
  -webkit-box-shadow: 0px 0px 5px #666666;
  box-shadow: 0px 0px 5px #666666;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-header {
  padding: 5px;
  font-weight: 500;
  font-size: 18px;
  line-height: 28px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-header #rubric-reuse-collapse-icon::after {
  float: left;
  font-family: FontAwesome;
  display: inline-block;
  color: inherit;
  padding: 0 5px;
  content: "";
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-header-active #rubric-reuse-collapse-icon::after {
  content: "";
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-header-icon {
  display: none;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .openassessment_rubric_clone_block {
  padding: 8px 0px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .openassessment_rubric_clone_block .no-select {
  user-select: none;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content {
  overflow: visible;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container {
  display: flex;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container #openassessment_rubric_clone_input_container {
  flex: 9;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container #openassessment_rubric_clone_input_container #openassessment_rubric_clone_input {
  padding: 6px;
  width: 90%;
  height: 40px;
  vertical-align: middle;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container #openassessment_rubric_clone_input_container .openassessment_clone_combobox {
  position: relative;
  display: inline-block;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container #openassessment_rubric_clone_input_container .openassessment_clone_combobox_toggle {
  vertical-align: middle;
  font-family: FontAwesome;
  margin-left: -1px;
  padding: 6px;
  height: 40px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container #openassessment_rubric_clone_input_container .openassessment_clone_combobox_toggle span {
  padding: 0;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container #openassessment_rubric_clone_button_container {
  flex: 3;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_form_container #openassessment_rubric_clone_button_container #openassessment_rubric_clone_button {
  margin-top: 0;
  float: left;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_clone .ui-accordion-content #openassessment_rubric_clone_alert {
  background: #BBE6D7;
  border: 1px solid #175B3C;
  box-sizing: border-box;
  width: 100%;
  padding: 10px;
  margin: 5px 0;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_add_option {
  text-align: center;
  border: 1px solid;
  color: #009fe6;
  background-color: white;
  padding: 7.5px;
  margin: 10px 20px 10px 10px;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  -ms-border-radius: 3px;
  -o-border-radius: 3px;
  border-radius: 3px;
  cursor: pointer;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_add_option h2:before {
  font-family: FontAwesome;
  content: "";
  display: inline-block;
  margin: 0 5px;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_add_option:hover {
  background-color: #009fe6;
  color: white;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_basic_editor .comp-setting-entry {
  padding-left: 0;
  margin-left: 10px;
  overflow: auto;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_basic_editor .comp-setting-entry .wrapper-comp-settings label {
  padding: 0;
  margin: 0;
  width: 100%;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_basic_editor .comp-setting-entry .wrapper-comp-settings label input {
  font-size: 11px;
  float: left;
  width: 70%;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_basic_editor .comp-setting-entry .wrapper-comp-settings .openassessment_criterion_prompt {
  padding: 10px;
  width: 70%;
  float: left;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_basic_editor .comp-setting-entry .wrapper-comp-settings .openassessment_criterion_prompt.openassessment_highlighted_field {
  border: 2px solid red;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_feedback_wrapper .openassessment_criterion_feedback_header {
  background-color: #c8c9cb;
  padding: 5px;
  margin: 10px;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  -ms-border-radius: 3px;
  -o-border-radius: 3px;
  border-radius: 3px;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_feedback_wrapper label {
  width: 100%;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_feedback_wrapper label select {
  float: left;
  margin-left: 24%;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_feedback_wrapper .openassessment_criterion_feedback_direction label {
  margin-right: 15px;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option {
  padding: 5px 15px 5px 5px;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header {
  background-color: rgba(164, 165, 168, 0.25);
  padding: 5px 10px 5px 5px;
  margin: 5px 5px 8px 5px;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  -ms-border-radius: 3px;
  -o-border-radius: 3px;
  border-radius: 3px;
  overflow: auto;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_option_header .openassessment_option_header_title {
  cursor: default;
  float: right;
  padding: 2.5px 5px 0 0;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_point_wrapper {
  width: 30%;
  border-top: none;
  padding: 10px 0px 10px 5px;
  float: right;
  margin: 0;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_point_wrapper .wrapper-comp-setting {
  min-width: 150px;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_point_wrapper label {
  width: 100%;
  padding-right: calc((100% - 150px)/2);
  margin: 0;
  float: left;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_point_wrapper label input {
  min-width: 50px;
  padding: 10px;
  float: left;
  width: 30%;
  font-size: 11px;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_name_wrapper {
  float: right;
  width: 70%;
  padding: 10px 20px 10px 10px;
  border-top: 0;
  margin: 0;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_name_wrapper label {
  width: 100%;
  vertical-align: middle;
  padding: 0;
  margin: 0;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_name_wrapper label input {
  padding: 10px;
  font-size: 11px;
  width: 56%;
  float: left;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_explanation_wrapper {
  padding: 15px 20px 0px 5px;
  width: 100%;
  display: inline-block;
  margin: 0;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_explanation_wrapper label {
  width: 100%;
  text-align: right;
  margin: 0;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_explanation_wrapper label textarea {
  padding: 10px;
  height: 70px;
  width: 69.5%;
  float: left;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_large_text_input, #openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_feedback_wrapper textarea, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_basic_editor .comp-setting-entry .wrapper-comp-settings .openassessment_criterion_prompt {
  height: 70px;
  width: 70%;
}
#openassessment-editor #oa_rubric_editor_wrapper .openassessment_input_styling, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion_option .openassessment_criterion_option_explanation_wrapper label textarea, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_large_text_input, #openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_feedback_wrapper textarea, #openassessment-editor #oa_rubric_editor_wrapper .openassessment_criterion .openassessment_criterion_basic_editor .comp-setting-entry .wrapper-comp-settings .openassessment_criterion_prompt {
  resize: none;
  box-sizing: border-box;
  border: 1px solid #b2b2b2;
  border-radius: 2px;
  background-image: -webkit-linear-gradient(#F2F2F2, #FFF);
  background-image: -moz-linear-gradient(#F2F2F2, #FFF);
  background-image: linear, false, #F2F2F2, #FFF;
  box-shadow: inset 0 1px 2px 0 rgba(0, 0, 0, 0.1);
  font-family: "Open Sans", sans-serif;
  font-size: 11px;
  color: #4c4c4c;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_feedback_wrapper {
  padding: 0;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_feedback_wrapper #openassessment_rubric_feedback_header {
  background-color: rgba(164, 165, 168, 0.25);
  padding: 7.5px 15px 7.5px 7.5px;
  margin: 10px 10px 0 10px;
  border-radius: 3px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_feedback_wrapper label {
  padding-top: 10px;
}
#openassessment-editor #oa_rubric_editor_wrapper #openassessment_rubric_feedback_wrapper textarea {
  padding: 10px;
  width: 70%;
  float: left;
}
#openassessment-editor #oa_schedule_editor_wrapper .openassessment_schedule_setting_header {
  font-size: 19px;
  margin: 15px 30px 10px 0;
}
#openassessment-editor #oa_schedule_editor_wrapper li:nth-child(2) {
  border-bottom: 1px solid #e5e5e5;
}
#openassessment-editor #oa_schedule_editor_wrapper #peer_assessment_schedule_editor #self_assessment_schedule_editor li:nth-child(2) {
  border-bottom: 1px solid #e5e5e5;
}
#openassessment-editor hr {
  background-color: transparent;
  color: #424243;
  height: 1px;
  border: 0px;
  clear: both;
}

.modal-lg.modal-window.confirm.openassessment_modal_window {
  height: 80%;
  top: 10%;
  min-width: 600px;
}

.openassessment_full_height.edit-xblock-modal,
.openassessment_full_height.xblock-editor,
.openassessment_full_height.xblock.xblock-studio_view.xblock-initialized,
[class*=view-] .modal-lg.modal-editor .openassessment_full_height.modal-content {
  height: 100%;
}

#openassessment-editor .openassessment_highlighted_field {
  border-color: red;
  border-width: 2px;
}

#student_training_settings_editor .openassessment_training_example {
  padding: 5px;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_header {
  margin: 10px;
  padding: 5px;
  border-bottom: 1px solid;
  overflow: auto;
  color: #626365;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_header .openassessment_training_example_header_title {
  text-transform: uppercase;
  width: 50%;
  display: inline-block;
  float: right;
  padding-top: 2px;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_header .openassessment_training_example_remove {
  padding: 3px 7px 3px 5px;
  cursor: pointer;
  border-radius: 4px;
  float: left;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_header .openassessment_training_example_remove h2:after {
  font-family: FontAwesome;
  content: "";
  display: inline-block;
  color: inherit;
  margin: 0 5px;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_header .openassessment_training_example_remove h2 {
  text-transform: uppercase;
  font-size: 80%;
  float: left;
  display: inline-block;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_header .openassessment_training_example_remove:hover {
  background-color: lightgray;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body {
  padding: 0 15px 15px 15px;
  position: relative;
  overflow: scroll;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body .openassessment_training_example_essay_wrapper {
  width: 58%;
  display: inline-block;
  position: absolute;
  right: 15px;
  height: 100%;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body .openassessment_training_example_essay_wrapper textarea {
  min-height: 90px;
  height: calc(100% - 35px);
  resize: none;
  box-sizing: border-box;
  border: 1px solid #b2b2b2;
  border-radius: 2px;
  padding: 6px 8px 8px 8px;
  background-color: white;
  box-shadow: inset 0 1px 2px 0 rgba(0, 0, 0, 0.1);
  font-family: "Open Sans", sans-serif;
  font-size: 11px;
  color: #4c4c4c;
  margin: 5px 10px 0 15px;
  width: calc(100% - 25px);
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body .openassessment_training_example_scored_rubric {
  width: 40%;
  display: inline-block;
  float: left;
  min-height: 150px;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body .openassessment_training_example_scored_rubric .openassessment_training_example_criteria_selections .comp-setting-entry {
  margin: 0;
  padding: 10px;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body .openassessment_training_example_scored_rubric .openassessment_training_example_criteria_selections .comp-setting-entry .wrapper-comp-setting {
  min-width: 100px;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body .openassessment_training_example_scored_rubric .openassessment_training_example_criteria_selections .comp-setting-entry label {
  width: 100%;
  margin-right: 0;
}
#student_training_settings_editor .openassessment_training_example .openassessment_training_example_body .openassessment_training_example_scored_rubric .openassessment_training_example_criteria_selections .comp-setting-entry label select {
  width: 60%;
  float: left;
}
#student_training_settings_editor .openassessment_add_training_example {
  color: #009fe6;
  text-align: center;
  padding: 5px;
  border: 1px solid;
  border-radius: 4px;
  margin: 10px;
}
#student_training_settings_editor .openassessment_add_training_example:hover {
  color: white;
  background-color: #009fe6;
  cursor: pointer;
}
#student_training_settings_editor .openassessment_add_training_example:before {
  font-family: "FontAwesome";
  display: inline-block;
  width: auto;
  height: auto;
  content: "";
  margin: 0 10px;
}

#openassessment-editor .wrapper-comp-settings .list-input.settings-list .field.comp-setting-entry select {
  padding: 5px;
}

.step--response .action--upload {
  text-align: center;
  float: left;
  display: inline-block;
  margin: 10px 0;
  box-shadow: none;
}
.step--response .file--upload {
  margin: 20px 10px;
}
.step--response .delete__uploaded__file {
  text-align: center;
  display: inline-block;
  margin: 10px;
  padding: 10px;
  box-shadow: none;
  font-size: 0.75em;
}
.step--response .delete__uploaded__file:active {
  text-shadow: none;
  box-shadow: none;
}
.step--response .delete__uploaded__file:hover {
  text-shadow: none;
  box-shadow: none;
}
.step--response .delete__uploaded__file:focus {
  text-shadow: none;
  box-shadow: none;
}
.step--response #team_name_and_users {
  margin: 10px 0;
  border: 1px solid #b9c1c8;
  background-color: white;
  padding: 20px;
}
.step--response #team_concurrency_notice {
  margin: 10px 0;
  border: 1px solid #ffc01f;
  background-color: #fff9eb;
  padding: 20px;
}
.step--response #team_user_has_previous_submission {
  margin: 10px 0;
  border: 1px solid #b20610;
  background-color: #feeced;
  color: #b20610;
  padding: 20px;
}
.step--response .step__rubric {
  padding: 10px 10px 0 10px;
  background: #f5f6f7;
  border-radius: 2px;
}
.step--response .step__rubric #read-only-rubric-button {
  display: inline-block;
  vertical-align: middle;
}
.step--response .step__rubric #read_only___assessment__rubric__header {
  margin-right: 10px;
  white-space: pre-wrap;
  display: inline;
}
.step--response .step__rubric .assessment__fields {
  margin-bottom: 20px;
}
.step--response .step__rubric .question__answers {
  overflow: visible;
}

.self-assessment__display__header .self-assessment__display__title,
.peer-assessment__display__header .peer-assessment__display__title,
.staff-assessment__display__header .staff-assessment__display__title,
.submission__answer__display .submission__answer__display__title {
  margin: 10px 0;
}

.self-assessment__display__image,
.peer-assessment__display__image,
.staff-assessment__display__image,
.submission__answer__display__image {
  max-height: 400px;
  text-align: right;
  overflow: auto;
}
.self-assessment__display__image img,
.peer-assessment__display__image img,
.staff-assessment__display__image img,
.submission__answer__display__image img {
  max-height: 100%;
  max-width: 100%;
}

.submission__answer__display__image .submission--image {
  max-height: 250px;
  max-width: 100%;
}

.openassessment .step--leaderboard {
  font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
}
.openassessment .step--leaderboard .step__title .step__counter, .openassessment .step--leaderboard .step__title .step__counter:before {
  display: none;
}
.openassessment .step--leaderboard .step__title .wrapper--copy {
  margin-right: 0;
  padding-right: 0;
  border-right: 0;
}
.openassessment .step--leaderboard .step__label, .openassessment .step--leaderboard .grade__value {
  display: inline-block;
  vertical-align: middle;
}
.openassessment .step--leaderboard .step__label {
  margin-left: 10px;
}
.openassessment .step--leaderboard .list.leaderboard__score__list {
  list-style-type: none;
}
.openassessment .step--leaderboard .list.leaderboard__score__list li.leaderboard__score__item {
  margin: 15px 0;
}
.openassessment .step--leaderboard .list.leaderboard__score__list li.leaderboard__score__item .leaderboard__list__number {
  display: inline-block;
  background: #626365;
  color: white;
  padding: 5px 5px 3px 5px;
  font-size: 16px;
  min-width: 35px;
  text-align: center;
  -webkit-border-top-left-radius: 2px;
  -moz-border-topleft-radius: 2px;
  border-top-left-radius: 2px;
  -webkit-border-top-right-radius: 2px;
  -moz-border-topright-radius: 2px;
  border-top-right-radius: 2px;
}
.openassessment .step--leaderboard .list.leaderboard__score__list li.leaderboard__score__item .leaderboard__score__image {
  max-height: 150px;
  max-width: 100%;
  margin-left: 20px;
  margin-bottom: 20px;
}
.openassessment .step--leaderboard .list.leaderboard__score__list li.leaderboard__score__item .leaderboard__score__title {
  font-size: 15px;
  color: #414141;
  text-transform: uppercase;
  display: inline-block;
  padding-right: 15px;
}
.openassessment .step--leaderboard .list.leaderboard__score__list li.leaderboard__score__item .leaderboard__answer {
  border-top: 2px solid #626365;
  box-shadow: inset 0 0 3px 1px rgba(10, 10, 10, 0.1);
  padding: 10px;
  max-height: 200px;
  overflow-y: scroll;
  font-size: 14px;
}

.wrapper--xblock .openassessment h3.staff__grade__title {
  font-size: inherit;
  margin-bottom: 5px;
}

.openassessment .self-assessment__display__header, .openassessment .peer-assessment__display__header, .openassessment .step__header,
.openassessment .staff-assessment__display__header, .openassessment .submission__answer__display__header, .openassessment .student-training__display__header {
  margin-bottom: 0 !important;
  border-radius: 0 !important;
  padding-bottom: 2px !important;
  font-size: initial !important;
  border-bottom: none !important;
}
.openassessment span {
  line-height: normal;
}
.openassessment h2 {
  text-transform: none;
  letter-spacing: normal;
}
.openassessment .openassessment__steps .openassessment__steps__step .step__header .step__deadline {
  color: #414141;
  margin-left: 5px;
}
.openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title {
  text-align: right;
  float: none;
  margin-top: 0 !important;
}
.openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title .step__label {
  text-transform: none;
  letter-spacing: normal;
}
.openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title .step__counter, .openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title .wrapper--copy {
  display: inline-block;
  vertical-align: middle;
}
.openassessment .openassessment__steps .openassessment__steps__step .step__header .step__title .step__counter:before {
  content: counter(item, decimal);
  counter-increment: item;
  color: #414141;
}
.openassessment .step__status__value {
  line-height: 0;
}
.openassessment .leaderboard__answer .submission__answer__display__content .submission__answer__part__text__title, .openassessment .leaderboard__answer .self-assessment__display__image .submission__answer__part__text__title,
.openassessment .leaderboard__answer .peer-assessment__display__image .submission__answer__part__text__title,
.openassessment .leaderboard__answer .staff-assessment__display__image .submission__answer__part__text__title,
.openassessment .leaderboard__answer .submission__answer__display__image .submission__answer__part__text__title {
  text-align: right;
  float: none;
  font-size: inherit;
  margin: 0;
}
.openassessment .list--actions {
  list-style: none !important;
  padding-right: 0 !important;
  text-indent: 0 !important;
}
.openassessment .list--actions li {
  margin-bottom: 0 !important;
  margin-top: 5px !important;
}
.openassessment .list--actions button span, .openassessment .list--actions .button span, .openassessment .list--actions .action span {
  color: inherit;
}
.openassessment span {
  color: inherit;
}
.openassessment .step--peer-assessment .list--peer-assessments {
  list-style: none outside none !important;
  padding-right: 0;
}
.openassessment .self-assessment__display__title, .openassessment .peer-assessment__display__title {
  margin-bottom: 10px !important;
  color: #414141 !important;
}
.openassessment .question__answers {
  padding-right: 0;
}
.openassessment .question__title.ui-slidable .question__title__copy {
  color: inherit !important;
}
.openassessment .step--grade .step__title {
  display: inline;
  width: 100% !important;
}
.openassessment .action--submit, .openassessment .action--save, .openassessment .action--upload {
  text-shadow: none !important;
  box-shadow: none !important;
}
.openassessment .message p {
  color: inherit;
}
.openassessment .icon {
  font-style: normal;
}
.openassessment .step__title .step__deadline {
  letter-spacing: normal;
  text-transform: none;
}
.openassessment .openassessment__steps__step .step__header .step__title {
  margin-bottom: 0;
}
.openassessment .is--collapsed .question__answers {
  overflow: auto;
}
.openassessment .wrapper--staff-assessment .student__answer__display__content p {
  color: inherit;
}

.paragon-styles :root {
  --blue: #80a9cc;
  --indigo: #6610f2;
  --purple: #6f42c1;
  --pink: #e83e8c;
  --red: #C32D3A;
  --orange: #fd7e14;
  --yellow: #D69600;
  --green: #178253;
  --teal: #006DAA;
  --cyan: #17a2b8;
  --white: white;
  --gray: #adb5bd;
  --gray-dark: #333333;
  --dark-gray: #495057;
  --primary: #80a9cc;
  --secondary: #6c757d;
  --success: #178253;
  --info: #006DAA;
  --warning: #D69600;
  --danger: #C32D3A;
  --light: #eee;
  --dark: #495057;
  --brand: #80a9cc;
  --gray: #adb5bd;
  --breakpoint-xs: 0;
  --breakpoint-sm: 576px;
  --breakpoint-md: 768px;
  --breakpoint-lg: 992px;
  --breakpoint-xl: 1200px;
  --font-family-sans-serif: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  --font-family-monospace: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace;
}
.paragon-styles *,
.paragon-styles *::before,
.paragon-styles *::after {
  box-sizing: border-box;
}
.paragon-styles html {
  font-family: sans-serif;
  line-height: 1.15;
  -webkit-text-size-adjust: 100%;
  -webkit-tap-highlight-color: rgba(3, 3, 3, 0);
}
.paragon-styles article, .paragon-styles aside, .paragon-styles figcaption, .paragon-styles figure, .paragon-styles footer, .paragon-styles header, .paragon-styles hgroup, .paragon-styles main, .paragon-styles nav, .paragon-styles section {
  display: block;
}
.paragon-styles body {
  margin: 0;
  font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
  color: #212529;
  text-align: left;
  background-color: white;
}
.paragon-styles [tabindex="-1"]:focus:not(:focus-visible) {
  outline: 0 !important;
}
.paragon-styles hr {
  box-sizing: content-box;
  height: 0;
  overflow: visible;
}
.paragon-styles h1, .paragon-styles h2, .paragon-styles h3, .paragon-styles h4, .paragon-styles h5, .paragon-styles h6 {
  margin-top: 0;
  margin-bottom: 0.5rem;
}
.paragon-styles p {
  margin-top: 0;
  margin-bottom: 1rem;
}
.paragon-styles abbr[title],
.paragon-styles abbr[data-original-title] {
  text-decoration: underline;
  text-decoration: underline dotted;
  cursor: help;
  border-bottom: 0;
  text-decoration-skip-ink: none;
}
.paragon-styles address {
  margin-bottom: 1rem;
  font-style: normal;
  line-height: inherit;
}
.paragon-styles ol,
.paragon-styles ul,
.paragon-styles dl {
  margin-top: 0;
  margin-bottom: 1rem;
}
.paragon-styles ol ol,
.paragon-styles ul ul,
.paragon-styles ol ul,
.paragon-styles ul ol {
  margin-bottom: 0;
}
.paragon-styles dt {
  font-weight: 700;
}
.paragon-styles dd {
  margin-bottom: 0.5rem;
  margin-left: 0;
}
.paragon-styles blockquote {
  margin: 0 0 1rem;
}
.paragon-styles b,
.paragon-styles strong {
  font-weight: bolder;
}
.paragon-styles small {
  font-size: 80%;
}
.paragon-styles sub,
.paragon-styles sup {
  position: relative;
  font-size: 75%;
  line-height: 0;
  vertical-align: baseline;
}
.paragon-styles sub {
  bottom: -0.25em;
}
.paragon-styles sup {
  top: -0.5em;
}
.paragon-styles a {
  color: #80a9cc;
  text-decoration: none;
  background-color: transparent;
}
.paragon-styles a:hover {
  color: #4984b6;
  text-decoration: underline;
}
.paragon-styles a:not([href]):not([class]) {
  color: inherit;
  text-decoration: none;
}
.paragon-styles a:not([href]):not([class]):hover {
  color: inherit;
  text-decoration: none;
}
.paragon-styles pre,
.paragon-styles code,
.paragon-styles kbd,
.paragon-styles samp {
  font-family: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace;
  font-size: 1em;
}
.paragon-styles pre {
  margin-top: 0;
  margin-bottom: 1rem;
  overflow: auto;
  -ms-overflow-style: scrollbar;
}
.paragon-styles figure {
  margin: 0 0 1rem;
}
.paragon-styles img {
  vertical-align: middle;
  border-style: none;
}
.paragon-styles svg {
  overflow: hidden;
  vertical-align: middle;
}
.paragon-styles table {
  border-collapse: collapse;
}
.paragon-styles caption {
  padding-top: 0.75rem;
  padding-bottom: 0.75rem;
  color: #adb5bd;
  text-align: left;
  caption-side: bottom;
}
.paragon-styles th {
  text-align: inherit;
  text-align: -webkit-match-parent;
}
.paragon-styles label {
  display: inline-block;
  margin-bottom: 0.5rem;
}
.paragon-styles button {
  border-radius: 0;
}
.paragon-styles button:focus:not(:focus-visible) {
  outline: 0;
}
.paragon-styles input,
.paragon-styles button,
.paragon-styles select,
.paragon-styles optgroup,
.paragon-styles textarea {
  margin: 0;
  font-family: inherit;
  font-size: inherit;
  line-height: inherit;
}
.paragon-styles button,
.paragon-styles input {
  overflow: visible;
}
.paragon-styles button,
.paragon-styles select {
  text-transform: none;
}
.paragon-styles [role=button] {
  cursor: pointer;
}
.paragon-styles select {
  word-wrap: normal;
}
.paragon-styles button,
.paragon-styles [type=button],
.paragon-styles [type=reset],
.paragon-styles [type=submit] {
  -webkit-appearance: button;
}
.paragon-styles button:not(:disabled),
.paragon-styles [type=button]:not(:disabled),
.paragon-styles [type=reset]:not(:disabled),
.paragon-styles [type=submit]:not(:disabled) {
  cursor: pointer;
}
.paragon-styles button::-moz-focus-inner,
.paragon-styles [type=button]::-moz-focus-inner,
.paragon-styles [type=reset]::-moz-focus-inner,
.paragon-styles [type=submit]::-moz-focus-inner {
  padding: 0;
  border-style: none;
}
.paragon-styles input[type=radio],
.paragon-styles input[type=checkbox] {
  box-sizing: border-box;
  padding: 0;
}
.paragon-styles textarea {
  overflow: auto;
  resize: vertical;
}
.paragon-styles fieldset {
  min-width: 0;
  padding: 0;
  margin: 0;
  border: 0;
}
.paragon-styles legend {
  display: block;
  width: 100%;
  max-width: 100%;
  padding: 0;
  margin-bottom: 0.5rem;
  font-size: 1.5rem;
  line-height: inherit;
  color: inherit;
  white-space: normal;
}
.paragon-styles progress {
  vertical-align: baseline;
}
.paragon-styles [type=number]::-webkit-inner-spin-button,
.paragon-styles [type=number]::-webkit-outer-spin-button {
  height: auto;
}
.paragon-styles [type=search] {
  outline-offset: -2px;
  -webkit-appearance: none;
}
.paragon-styles [type=search]::-webkit-search-decoration {
  -webkit-appearance: none;
}
.paragon-styles ::-webkit-file-upload-button {
  font: inherit;
  -webkit-appearance: button;
}
.paragon-styles output {
  display: inline-block;
}
.paragon-styles summary {
  display: list-item;
  cursor: pointer;
}
.paragon-styles template {
  display: none;
}
.paragon-styles [hidden] {
  display: none !important;
}
.paragon-styles h1, .paragon-styles h2, .paragon-styles h3, .paragon-styles h4, .paragon-styles h5, .paragon-styles h6,
.paragon-styles .h1, .paragon-styles .h2, .paragon-styles .h3, .paragon-styles .h4, .paragon-styles .h5, .paragon-styles .h6 {
  margin-bottom: 0.5rem;
  font-weight: 700;
  line-height: 1.25;
}
.paragon-styles h1, .paragon-styles .h1 {
  font-size: 2.5rem;
}
.paragon-styles h2, .paragon-styles .h2 {
  font-size: 2rem;
}
.paragon-styles h3, .paragon-styles .h3 {
  font-size: 1.75rem;
}
.paragon-styles h4, .paragon-styles .h4 {
  font-size: 1.5rem;
}
.paragon-styles h5, .paragon-styles .h5 {
  font-size: 1.25rem;
}
.paragon-styles h6, .paragon-styles .h6 {
  font-size: 1rem;
}
.paragon-styles .lead {
  font-size: 1.25rem;
  font-weight: 300;
}
.paragon-styles .display-1 {
  font-size: 6rem;
  font-weight: 700;
  line-height: 1.25;
}
.paragon-styles .display-2 {
  font-size: 5.5rem;
  font-weight: 700;
  line-height: 1.25;
}
.paragon-styles .display-3 {
  font-size: 4.5rem;
  font-weight: 700;
  line-height: 1.25;
}
.paragon-styles .display-4 {
  font-size: 3.5rem;
  font-weight: 700;
  line-height: 1.25;
}
.paragon-styles hr {
  margin-top: 1rem;
  margin-bottom: 1rem;
  border: 0;
  border-top: 1px solid rgba(3, 3, 3, 0.1);
}
.paragon-styles small,
.paragon-styles .small {
  font-size: 87.5%;
  font-weight: 400;
}
.paragon-styles mark,
.paragon-styles .mark {
  padding: 0.2em;
  background-color: #fff243;
}
.paragon-styles .list-unstyled {
  padding-left: 0;
  list-style: none;
}
.paragon-styles .list-inline {
  padding-left: 0;
  list-style: none;
}
.paragon-styles .list-inline-item {
  display: inline-block;
}
.paragon-styles .list-inline-item:not(:last-child) {
  margin-right: 0.5rem;
}
.paragon-styles .initialism {
  font-size: 90%;
  text-transform: uppercase;
}
.paragon-styles .blockquote {
  margin-bottom: 1rem;
  font-size: 1.25rem;
}
.paragon-styles .blockquote-footer {
  display: block;
  font-size: 87.5%;
  color: #adb5bd;
}
.paragon-styles .blockquote-footer::before {
  content: "— ";
}
.paragon-styles .img-fluid {
  max-width: 100%;
  height: auto;
}
.paragon-styles .img-thumbnail {
  padding: 0.25rem;
  background-color: white;
  border: 1px solid #e9ecef;
  border-radius: 0.3rem;
  max-width: 100%;
  height: auto;
}
.paragon-styles .figure {
  display: inline-block;
}
.paragon-styles .figure-img {
  margin-bottom: 0.5rem;
  line-height: 1;
}
.paragon-styles .figure-caption {
  font-size: 90%;
  color: #adb5bd;
}
.paragon-styles code {
  font-size: 87.5%;
  color: #e83e8c;
  word-wrap: break-word;
}
a > .paragon-styles code {
  color: inherit;
}
.paragon-styles kbd {
  padding: 0.2rem 0.4rem;
  font-size: 87.5%;
  color: white;
  background-color: #495057;
  border-radius: 0.2rem;
}
.paragon-styles kbd kbd {
  padding: 0;
  font-size: 100%;
  font-weight: 700;
}
.paragon-styles pre {
  display: block;
  font-size: 87.5%;
  color: #212529;
}
.paragon-styles pre code {
  font-size: inherit;
  color: inherit;
  word-break: normal;
}
.paragon-styles .pre-scrollable {
  max-height: 340px;
  overflow-y: scroll;
}
.paragon-styles .container,
.paragon-styles .container-fluid,
.paragon-styles .container-xl,
.paragon-styles .container-lg,
.paragon-styles .container-md,
.paragon-styles .container-sm {
  width: 100%;
  padding-right: 15px;
  padding-left: 15px;
  margin-right: auto;
  margin-left: auto;
}
@media (min-width: 576px) {
  .paragon-styles .container-sm, .paragon-styles .container {
    max-width: 540px;
  }
}
@media (min-width: 768px) {
  .paragon-styles .container-md, .paragon-styles .container-sm, .paragon-styles .container {
    max-width: 720px;
  }
}
@media (min-width: 992px) {
  .paragon-styles .container-lg, .paragon-styles .container-md, .paragon-styles .container-sm, .paragon-styles .container {
    max-width: 960px;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .container-xl, .paragon-styles .container-lg, .paragon-styles .container-md, .paragon-styles .container-sm, .paragon-styles .container {
    max-width: 1600px;
  }
}
.paragon-styles .row {
  display: flex;
  flex-wrap: wrap;
  margin-right: -15px;
  margin-left: -15px;
}
.paragon-styles .no-gutters {
  margin-right: 0;
  margin-left: 0;
}
.paragon-styles .no-gutters > .col,
.paragon-styles .no-gutters > [class*=col-] {
  padding-right: 0;
  padding-left: 0;
}
.paragon-styles .col-xl,
.paragon-styles .col-xl-auto, .paragon-styles .col-xl-12, .paragon-styles .col-xl-11, .paragon-styles .col-xl-10, .paragon-styles .col-xl-9, .paragon-styles .col-xl-8, .paragon-styles .col-xl-7, .paragon-styles .col-xl-6, .paragon-styles .col-xl-5, .paragon-styles .col-xl-4, .paragon-styles .col-xl-3, .paragon-styles .col-xl-2, .paragon-styles .col-xl-1, .paragon-styles .col-lg,
.paragon-styles .col-lg-auto, .paragon-styles .col-lg-12, .paragon-styles .col-lg-11, .paragon-styles .col-lg-10, .paragon-styles .col-lg-9, .paragon-styles .col-lg-8, .paragon-styles .col-lg-7, .paragon-styles .col-lg-6, .paragon-styles .col-lg-5, .paragon-styles .col-lg-4, .paragon-styles .col-lg-3, .paragon-styles .col-lg-2, .paragon-styles .col-lg-1, .paragon-styles .col-md,
.paragon-styles .col-md-auto, .paragon-styles .col-md-12, .paragon-styles .col-md-11, .paragon-styles .col-md-10, .paragon-styles .col-md-9, .paragon-styles .col-md-8, .paragon-styles .col-md-7, .paragon-styles .col-md-6, .paragon-styles .col-md-5, .paragon-styles .col-md-4, .paragon-styles .col-md-3, .paragon-styles .col-md-2, .paragon-styles .col-md-1, .paragon-styles .col-sm,
.paragon-styles .col-sm-auto, .paragon-styles .col-sm-12, .paragon-styles .col-sm-11, .paragon-styles .col-sm-10, .paragon-styles .col-sm-9, .paragon-styles .col-sm-8, .paragon-styles .col-sm-7, .paragon-styles .col-sm-6, .paragon-styles .col-sm-5, .paragon-styles .col-sm-4, .paragon-styles .col-sm-3, .paragon-styles .col-sm-2, .paragon-styles .col-sm-1, .paragon-styles .col,
.paragon-styles .col-auto, .paragon-styles .col-12, .paragon-styles .col-11, .paragon-styles .col-10, .paragon-styles .col-9, .paragon-styles .col-8, .paragon-styles .col-7, .paragon-styles .col-6, .paragon-styles .col-5, .paragon-styles .col-4, .paragon-styles .col-3, .paragon-styles .col-2, .paragon-styles .col-1 {
  position: relative;
  width: 100%;
  padding-right: 15px;
  padding-left: 15px;
}
.paragon-styles .col {
  flex-basis: 0;
  flex-grow: 1;
  max-width: 100%;
}
.paragon-styles .row-cols-1 > * {
  flex: 0 0 100%;
  max-width: 100%;
}
.paragon-styles .row-cols-2 > * {
  flex: 0 0 50%;
  max-width: 50%;
}
.paragon-styles .row-cols-3 > * {
  flex: 0 0 33.3333333333%;
  max-width: 33.3333333333%;
}
.paragon-styles .row-cols-4 > * {
  flex: 0 0 25%;
  max-width: 25%;
}
.paragon-styles .row-cols-5 > * {
  flex: 0 0 20%;
  max-width: 20%;
}
.paragon-styles .row-cols-6 > * {
  flex: 0 0 16.6666666667%;
  max-width: 16.6666666667%;
}
.paragon-styles .col-auto {
  flex: 0 0 auto;
  width: auto;
  max-width: 100%;
}
.paragon-styles .col-1 {
  flex: 0 0 8.3333333333%;
  max-width: 8.3333333333%;
}
.paragon-styles .col-2 {
  flex: 0 0 16.6666666667%;
  max-width: 16.6666666667%;
}
.paragon-styles .col-3 {
  flex: 0 0 25%;
  max-width: 25%;
}
.paragon-styles .col-4 {
  flex: 0 0 33.3333333333%;
  max-width: 33.3333333333%;
}
.paragon-styles .col-5 {
  flex: 0 0 41.6666666667%;
  max-width: 41.6666666667%;
}
.paragon-styles .col-6 {
  flex: 0 0 50%;
  max-width: 50%;
}
.paragon-styles .col-7 {
  flex: 0 0 58.3333333333%;
  max-width: 58.3333333333%;
}
.paragon-styles .col-8 {
  flex: 0 0 66.6666666667%;
  max-width: 66.6666666667%;
}
.paragon-styles .col-9 {
  flex: 0 0 75%;
  max-width: 75%;
}
.paragon-styles .col-10 {
  flex: 0 0 83.3333333333%;
  max-width: 83.3333333333%;
}
.paragon-styles .col-11 {
  flex: 0 0 91.6666666667%;
  max-width: 91.6666666667%;
}
.paragon-styles .col-12 {
  flex: 0 0 100%;
  max-width: 100%;
}
.paragon-styles .order-first {
  order: -1;
}
.paragon-styles .order-last {
  order: 13;
}
.paragon-styles .order-0 {
  order: 0;
}
.paragon-styles .order-1 {
  order: 1;
}
.paragon-styles .order-2 {
  order: 2;
}
.paragon-styles .order-3 {
  order: 3;
}
.paragon-styles .order-4 {
  order: 4;
}
.paragon-styles .order-5 {
  order: 5;
}
.paragon-styles .order-6 {
  order: 6;
}
.paragon-styles .order-7 {
  order: 7;
}
.paragon-styles .order-8 {
  order: 8;
}
.paragon-styles .order-9 {
  order: 9;
}
.paragon-styles .order-10 {
  order: 10;
}
.paragon-styles .order-11 {
  order: 11;
}
.paragon-styles .order-12 {
  order: 12;
}
.paragon-styles .offset-1 {
  margin-left: 8.3333333333%;
}
.paragon-styles .offset-2 {
  margin-left: 16.6666666667%;
}
.paragon-styles .offset-3 {
  margin-left: 25%;
}
.paragon-styles .offset-4 {
  margin-left: 33.3333333333%;
}
.paragon-styles .offset-5 {
  margin-left: 41.6666666667%;
}
.paragon-styles .offset-6 {
  margin-left: 50%;
}
.paragon-styles .offset-7 {
  margin-left: 58.3333333333%;
}
.paragon-styles .offset-8 {
  margin-left: 66.6666666667%;
}
.paragon-styles .offset-9 {
  margin-left: 75%;
}
.paragon-styles .offset-10 {
  margin-left: 83.3333333333%;
}
.paragon-styles .offset-11 {
  margin-left: 91.6666666667%;
}
@media (min-width: 576px) {
  .paragon-styles .col-sm {
    flex-basis: 0;
    flex-grow: 1;
    max-width: 100%;
  }
  .paragon-styles .row-cols-sm-1 > * {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .row-cols-sm-2 > * {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .row-cols-sm-3 > * {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .row-cols-sm-4 > * {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .row-cols-sm-5 > * {
    flex: 0 0 20%;
    max-width: 20%;
  }
  .paragon-styles .row-cols-sm-6 > * {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-sm-auto {
    flex: 0 0 auto;
    width: auto;
    max-width: 100%;
  }
  .paragon-styles .col-sm-1 {
    flex: 0 0 8.3333333333%;
    max-width: 8.3333333333%;
  }
  .paragon-styles .col-sm-2 {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-sm-3 {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .col-sm-4 {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .col-sm-5 {
    flex: 0 0 41.6666666667%;
    max-width: 41.6666666667%;
  }
  .paragon-styles .col-sm-6 {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .col-sm-7 {
    flex: 0 0 58.3333333333%;
    max-width: 58.3333333333%;
  }
  .paragon-styles .col-sm-8 {
    flex: 0 0 66.6666666667%;
    max-width: 66.6666666667%;
  }
  .paragon-styles .col-sm-9 {
    flex: 0 0 75%;
    max-width: 75%;
  }
  .paragon-styles .col-sm-10 {
    flex: 0 0 83.3333333333%;
    max-width: 83.3333333333%;
  }
  .paragon-styles .col-sm-11 {
    flex: 0 0 91.6666666667%;
    max-width: 91.6666666667%;
  }
  .paragon-styles .col-sm-12 {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .order-sm-first {
    order: -1;
  }
  .paragon-styles .order-sm-last {
    order: 13;
  }
  .paragon-styles .order-sm-0 {
    order: 0;
  }
  .paragon-styles .order-sm-1 {
    order: 1;
  }
  .paragon-styles .order-sm-2 {
    order: 2;
  }
  .paragon-styles .order-sm-3 {
    order: 3;
  }
  .paragon-styles .order-sm-4 {
    order: 4;
  }
  .paragon-styles .order-sm-5 {
    order: 5;
  }
  .paragon-styles .order-sm-6 {
    order: 6;
  }
  .paragon-styles .order-sm-7 {
    order: 7;
  }
  .paragon-styles .order-sm-8 {
    order: 8;
  }
  .paragon-styles .order-sm-9 {
    order: 9;
  }
  .paragon-styles .order-sm-10 {
    order: 10;
  }
  .paragon-styles .order-sm-11 {
    order: 11;
  }
  .paragon-styles .order-sm-12 {
    order: 12;
  }
  .paragon-styles .offset-sm-0 {
    margin-left: 0;
  }
  .paragon-styles .offset-sm-1 {
    margin-left: 8.3333333333%;
  }
  .paragon-styles .offset-sm-2 {
    margin-left: 16.6666666667%;
  }
  .paragon-styles .offset-sm-3 {
    margin-left: 25%;
  }
  .paragon-styles .offset-sm-4 {
    margin-left: 33.3333333333%;
  }
  .paragon-styles .offset-sm-5 {
    margin-left: 41.6666666667%;
  }
  .paragon-styles .offset-sm-6 {
    margin-left: 50%;
  }
  .paragon-styles .offset-sm-7 {
    margin-left: 58.3333333333%;
  }
  .paragon-styles .offset-sm-8 {
    margin-left: 66.6666666667%;
  }
  .paragon-styles .offset-sm-9 {
    margin-left: 75%;
  }
  .paragon-styles .offset-sm-10 {
    margin-left: 83.3333333333%;
  }
  .paragon-styles .offset-sm-11 {
    margin-left: 91.6666666667%;
  }
}
@media (min-width: 768px) {
  .paragon-styles .col-md {
    flex-basis: 0;
    flex-grow: 1;
    max-width: 100%;
  }
  .paragon-styles .row-cols-md-1 > * {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .row-cols-md-2 > * {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .row-cols-md-3 > * {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .row-cols-md-4 > * {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .row-cols-md-5 > * {
    flex: 0 0 20%;
    max-width: 20%;
  }
  .paragon-styles .row-cols-md-6 > * {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-md-auto {
    flex: 0 0 auto;
    width: auto;
    max-width: 100%;
  }
  .paragon-styles .col-md-1 {
    flex: 0 0 8.3333333333%;
    max-width: 8.3333333333%;
  }
  .paragon-styles .col-md-2 {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-md-3 {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .col-md-4 {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .col-md-5 {
    flex: 0 0 41.6666666667%;
    max-width: 41.6666666667%;
  }
  .paragon-styles .col-md-6 {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .col-md-7 {
    flex: 0 0 58.3333333333%;
    max-width: 58.3333333333%;
  }
  .paragon-styles .col-md-8 {
    flex: 0 0 66.6666666667%;
    max-width: 66.6666666667%;
  }
  .paragon-styles .col-md-9 {
    flex: 0 0 75%;
    max-width: 75%;
  }
  .paragon-styles .col-md-10 {
    flex: 0 0 83.3333333333%;
    max-width: 83.3333333333%;
  }
  .paragon-styles .col-md-11 {
    flex: 0 0 91.6666666667%;
    max-width: 91.6666666667%;
  }
  .paragon-styles .col-md-12 {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .order-md-first {
    order: -1;
  }
  .paragon-styles .order-md-last {
    order: 13;
  }
  .paragon-styles .order-md-0 {
    order: 0;
  }
  .paragon-styles .order-md-1 {
    order: 1;
  }
  .paragon-styles .order-md-2 {
    order: 2;
  }
  .paragon-styles .order-md-3 {
    order: 3;
  }
  .paragon-styles .order-md-4 {
    order: 4;
  }
  .paragon-styles .order-md-5 {
    order: 5;
  }
  .paragon-styles .order-md-6 {
    order: 6;
  }
  .paragon-styles .order-md-7 {
    order: 7;
  }
  .paragon-styles .order-md-8 {
    order: 8;
  }
  .paragon-styles .order-md-9 {
    order: 9;
  }
  .paragon-styles .order-md-10 {
    order: 10;
  }
  .paragon-styles .order-md-11 {
    order: 11;
  }
  .paragon-styles .order-md-12 {
    order: 12;
  }
  .paragon-styles .offset-md-0 {
    margin-left: 0;
  }
  .paragon-styles .offset-md-1 {
    margin-left: 8.3333333333%;
  }
  .paragon-styles .offset-md-2 {
    margin-left: 16.6666666667%;
  }
  .paragon-styles .offset-md-3 {
    margin-left: 25%;
  }
  .paragon-styles .offset-md-4 {
    margin-left: 33.3333333333%;
  }
  .paragon-styles .offset-md-5 {
    margin-left: 41.6666666667%;
  }
  .paragon-styles .offset-md-6 {
    margin-left: 50%;
  }
  .paragon-styles .offset-md-7 {
    margin-left: 58.3333333333%;
  }
  .paragon-styles .offset-md-8 {
    margin-left: 66.6666666667%;
  }
  .paragon-styles .offset-md-9 {
    margin-left: 75%;
  }
  .paragon-styles .offset-md-10 {
    margin-left: 83.3333333333%;
  }
  .paragon-styles .offset-md-11 {
    margin-left: 91.6666666667%;
  }
}
@media (min-width: 992px) {
  .paragon-styles .col-lg {
    flex-basis: 0;
    flex-grow: 1;
    max-width: 100%;
  }
  .paragon-styles .row-cols-lg-1 > * {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .row-cols-lg-2 > * {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .row-cols-lg-3 > * {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .row-cols-lg-4 > * {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .row-cols-lg-5 > * {
    flex: 0 0 20%;
    max-width: 20%;
  }
  .paragon-styles .row-cols-lg-6 > * {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-lg-auto {
    flex: 0 0 auto;
    width: auto;
    max-width: 100%;
  }
  .paragon-styles .col-lg-1 {
    flex: 0 0 8.3333333333%;
    max-width: 8.3333333333%;
  }
  .paragon-styles .col-lg-2 {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-lg-3 {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .col-lg-4 {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .col-lg-5 {
    flex: 0 0 41.6666666667%;
    max-width: 41.6666666667%;
  }
  .paragon-styles .col-lg-6 {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .col-lg-7 {
    flex: 0 0 58.3333333333%;
    max-width: 58.3333333333%;
  }
  .paragon-styles .col-lg-8 {
    flex: 0 0 66.6666666667%;
    max-width: 66.6666666667%;
  }
  .paragon-styles .col-lg-9 {
    flex: 0 0 75%;
    max-width: 75%;
  }
  .paragon-styles .col-lg-10 {
    flex: 0 0 83.3333333333%;
    max-width: 83.3333333333%;
  }
  .paragon-styles .col-lg-11 {
    flex: 0 0 91.6666666667%;
    max-width: 91.6666666667%;
  }
  .paragon-styles .col-lg-12 {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .order-lg-first {
    order: -1;
  }
  .paragon-styles .order-lg-last {
    order: 13;
  }
  .paragon-styles .order-lg-0 {
    order: 0;
  }
  .paragon-styles .order-lg-1 {
    order: 1;
  }
  .paragon-styles .order-lg-2 {
    order: 2;
  }
  .paragon-styles .order-lg-3 {
    order: 3;
  }
  .paragon-styles .order-lg-4 {
    order: 4;
  }
  .paragon-styles .order-lg-5 {
    order: 5;
  }
  .paragon-styles .order-lg-6 {
    order: 6;
  }
  .paragon-styles .order-lg-7 {
    order: 7;
  }
  .paragon-styles .order-lg-8 {
    order: 8;
  }
  .paragon-styles .order-lg-9 {
    order: 9;
  }
  .paragon-styles .order-lg-10 {
    order: 10;
  }
  .paragon-styles .order-lg-11 {
    order: 11;
  }
  .paragon-styles .order-lg-12 {
    order: 12;
  }
  .paragon-styles .offset-lg-0 {
    margin-left: 0;
  }
  .paragon-styles .offset-lg-1 {
    margin-left: 8.3333333333%;
  }
  .paragon-styles .offset-lg-2 {
    margin-left: 16.6666666667%;
  }
  .paragon-styles .offset-lg-3 {
    margin-left: 25%;
  }
  .paragon-styles .offset-lg-4 {
    margin-left: 33.3333333333%;
  }
  .paragon-styles .offset-lg-5 {
    margin-left: 41.6666666667%;
  }
  .paragon-styles .offset-lg-6 {
    margin-left: 50%;
  }
  .paragon-styles .offset-lg-7 {
    margin-left: 58.3333333333%;
  }
  .paragon-styles .offset-lg-8 {
    margin-left: 66.6666666667%;
  }
  .paragon-styles .offset-lg-9 {
    margin-left: 75%;
  }
  .paragon-styles .offset-lg-10 {
    margin-left: 83.3333333333%;
  }
  .paragon-styles .offset-lg-11 {
    margin-left: 91.6666666667%;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .col-xl {
    flex-basis: 0;
    flex-grow: 1;
    max-width: 100%;
  }
  .paragon-styles .row-cols-xl-1 > * {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .row-cols-xl-2 > * {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .row-cols-xl-3 > * {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .row-cols-xl-4 > * {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .row-cols-xl-5 > * {
    flex: 0 0 20%;
    max-width: 20%;
  }
  .paragon-styles .row-cols-xl-6 > * {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-xl-auto {
    flex: 0 0 auto;
    width: auto;
    max-width: 100%;
  }
  .paragon-styles .col-xl-1 {
    flex: 0 0 8.3333333333%;
    max-width: 8.3333333333%;
  }
  .paragon-styles .col-xl-2 {
    flex: 0 0 16.6666666667%;
    max-width: 16.6666666667%;
  }
  .paragon-styles .col-xl-3 {
    flex: 0 0 25%;
    max-width: 25%;
  }
  .paragon-styles .col-xl-4 {
    flex: 0 0 33.3333333333%;
    max-width: 33.3333333333%;
  }
  .paragon-styles .col-xl-5 {
    flex: 0 0 41.6666666667%;
    max-width: 41.6666666667%;
  }
  .paragon-styles .col-xl-6 {
    flex: 0 0 50%;
    max-width: 50%;
  }
  .paragon-styles .col-xl-7 {
    flex: 0 0 58.3333333333%;
    max-width: 58.3333333333%;
  }
  .paragon-styles .col-xl-8 {
    flex: 0 0 66.6666666667%;
    max-width: 66.6666666667%;
  }
  .paragon-styles .col-xl-9 {
    flex: 0 0 75%;
    max-width: 75%;
  }
  .paragon-styles .col-xl-10 {
    flex: 0 0 83.3333333333%;
    max-width: 83.3333333333%;
  }
  .paragon-styles .col-xl-11 {
    flex: 0 0 91.6666666667%;
    max-width: 91.6666666667%;
  }
  .paragon-styles .col-xl-12 {
    flex: 0 0 100%;
    max-width: 100%;
  }
  .paragon-styles .order-xl-first {
    order: -1;
  }
  .paragon-styles .order-xl-last {
    order: 13;
  }
  .paragon-styles .order-xl-0 {
    order: 0;
  }
  .paragon-styles .order-xl-1 {
    order: 1;
  }
  .paragon-styles .order-xl-2 {
    order: 2;
  }
  .paragon-styles .order-xl-3 {
    order: 3;
  }
  .paragon-styles .order-xl-4 {
    order: 4;
  }
  .paragon-styles .order-xl-5 {
    order: 5;
  }
  .paragon-styles .order-xl-6 {
    order: 6;
  }
  .paragon-styles .order-xl-7 {
    order: 7;
  }
  .paragon-styles .order-xl-8 {
    order: 8;
  }
  .paragon-styles .order-xl-9 {
    order: 9;
  }
  .paragon-styles .order-xl-10 {
    order: 10;
  }
  .paragon-styles .order-xl-11 {
    order: 11;
  }
  .paragon-styles .order-xl-12 {
    order: 12;
  }
  .paragon-styles .offset-xl-0 {
    margin-left: 0;
  }
  .paragon-styles .offset-xl-1 {
    margin-left: 8.3333333333%;
  }
  .paragon-styles .offset-xl-2 {
    margin-left: 16.6666666667%;
  }
  .paragon-styles .offset-xl-3 {
    margin-left: 25%;
  }
  .paragon-styles .offset-xl-4 {
    margin-left: 33.3333333333%;
  }
  .paragon-styles .offset-xl-5 {
    margin-left: 41.6666666667%;
  }
  .paragon-styles .offset-xl-6 {
    margin-left: 50%;
  }
  .paragon-styles .offset-xl-7 {
    margin-left: 58.3333333333%;
  }
  .paragon-styles .offset-xl-8 {
    margin-left: 66.6666666667%;
  }
  .paragon-styles .offset-xl-9 {
    margin-left: 75%;
  }
  .paragon-styles .offset-xl-10 {
    margin-left: 83.3333333333%;
  }
  .paragon-styles .offset-xl-11 {
    margin-left: 91.6666666667%;
  }
}
.paragon-styles .table {
  width: 100%;
  margin-bottom: 1rem;
  color: #212529;
}
.paragon-styles .table th,
.paragon-styles .table td {
  padding: 0.75rem;
  vertical-align: top;
  border-top: 1px solid #e9ecef;
}
.paragon-styles .table thead th {
  vertical-align: bottom;
  border-bottom: 2px solid #e9ecef;
}
.paragon-styles .table tbody + tbody {
  border-top: 2px solid #e9ecef;
}
.paragon-styles .table-sm th,
.paragon-styles .table-sm td {
  padding: 0.3rem;
}
.paragon-styles .table-bordered {
  border: 1px solid #e9ecef;
}
.paragon-styles .table-bordered th,
.paragon-styles .table-bordered td {
  border: 1px solid #e9ecef;
}
.paragon-styles .table-bordered thead th,
.paragon-styles .table-bordered thead td {
  border-bottom-width: 2px;
}
.paragon-styles .table-borderless th,
.paragon-styles .table-borderless td,
.paragon-styles .table-borderless thead th,
.paragon-styles .table-borderless tbody + tbody {
  border: 0;
}
.paragon-styles .table-striped tbody tr:nth-of-type(odd) {
  background-color: rgba(3, 3, 3, 0.05);
}
.paragon-styles .table-hover tbody tr:hover {
  color: #212529;
  background-color: rgba(3, 3, 3, 0.075);
}
.paragon-styles .table-primary,
.paragon-styles .table-primary > th,
.paragon-styles .table-primary > td {
  background-color: #dbe7f1;
}
.paragon-styles .table-primary th,
.paragon-styles .table-primary td,
.paragon-styles .table-primary thead th,
.paragon-styles .table-primary tbody + tbody {
  border-color: #bdd2e4;
}
.paragon-styles .table-hover .table-primary:hover {
  background-color: #c9dbea;
}
.paragon-styles .table-hover .table-primary:hover > td,
.paragon-styles .table-hover .table-primary:hover > th {
  background-color: #c9dbea;
}
.paragon-styles .table-secondary,
.paragon-styles .table-secondary > th,
.paragon-styles .table-secondary > td {
  background-color: #d6d8db;
}
.paragon-styles .table-secondary th,
.paragon-styles .table-secondary td,
.paragon-styles .table-secondary thead th,
.paragon-styles .table-secondary tbody + tbody {
  border-color: #b3b7bb;
}
.paragon-styles .table-hover .table-secondary:hover {
  background-color: #c8cbcf;
}
.paragon-styles .table-hover .table-secondary:hover > td,
.paragon-styles .table-hover .table-secondary:hover > th {
  background-color: #c8cbcf;
}
.paragon-styles .table-success,
.paragon-styles .table-success > th,
.paragon-styles .table-success > td {
  background-color: #bedccf;
}
.paragon-styles .table-success th,
.paragon-styles .table-success td,
.paragon-styles .table-success thead th,
.paragon-styles .table-success tbody + tbody {
  border-color: #86bea6;
}
.paragon-styles .table-hover .table-success:hover {
  background-color: #add3c3;
}
.paragon-styles .table-hover .table-success:hover > td,
.paragon-styles .table-hover .table-success:hover > th {
  background-color: #add3c3;
}
.paragon-styles .table-info,
.paragon-styles .table-info > th,
.paragon-styles .table-info > td {
  background-color: #b8d6e7;
}
.paragon-styles .table-info th,
.paragon-styles .table-info td,
.paragon-styles .table-info thead th,
.paragon-styles .table-info tbody + tbody {
  border-color: #7ab3d3;
}
.paragon-styles .table-hover .table-info:hover {
  background-color: #a5cbe1;
}
.paragon-styles .table-hover .table-info:hover > td,
.paragon-styles .table-hover .table-info:hover > th {
  background-color: #a5cbe1;
}
.paragon-styles .table-warning,
.paragon-styles .table-warning > th,
.paragon-styles .table-warning > td {
  background-color: #f4e2b8;
}
.paragon-styles .table-warning th,
.paragon-styles .table-warning td,
.paragon-styles .table-warning thead th,
.paragon-styles .table-warning tbody + tbody {
  border-color: #eac87a;
}
.paragon-styles .table-hover .table-warning:hover {
  background-color: #f1d9a2;
}
.paragon-styles .table-hover .table-warning:hover > td,
.paragon-styles .table-hover .table-warning:hover > th {
  background-color: #f1d9a2;
}
.paragon-styles .table-danger,
.paragon-styles .table-danger > th,
.paragon-styles .table-danger > td {
  background-color: #eec4c8;
}
.paragon-styles .table-danger th,
.paragon-styles .table-danger td,
.paragon-styles .table-danger thead th,
.paragon-styles .table-danger tbody + tbody {
  border-color: #e09299;
}
.paragon-styles .table-hover .table-danger:hover {
  background-color: #e8b0b6;
}
.paragon-styles .table-hover .table-danger:hover > td,
.paragon-styles .table-hover .table-danger:hover > th {
  background-color: #e8b0b6;
}
.paragon-styles .table-light,
.paragon-styles .table-light > th,
.paragon-styles .table-light > td {
  background-color: #fafafa;
}
.paragon-styles .table-light th,
.paragon-styles .table-light td,
.paragon-styles .table-light thead th,
.paragon-styles .table-light tbody + tbody {
  border-color: #f6f6f6;
}
.paragon-styles .table-hover .table-light:hover {
  background-color: #ededed;
}
.paragon-styles .table-hover .table-light:hover > td,
.paragon-styles .table-hover .table-light:hover > th {
  background-color: #ededed;
}
.paragon-styles .table-dark,
.paragon-styles .table-dark > th,
.paragon-styles .table-dark > td {
  background-color: #ccced0;
}
.paragon-styles .table-dark th,
.paragon-styles .table-dark td,
.paragon-styles .table-dark thead th,
.paragon-styles .table-dark tbody + tbody {
  border-color: #a0a4a8;
}
.paragon-styles .table-hover .table-dark:hover {
  background-color: #bfc1c4;
}
.paragon-styles .table-hover .table-dark:hover > td,
.paragon-styles .table-hover .table-dark:hover > th {
  background-color: #bfc1c4;
}
.paragon-styles .table-brand,
.paragon-styles .table-brand > th,
.paragon-styles .table-brand > td {
  background-color: #dbe7f1;
}
.paragon-styles .table-brand th,
.paragon-styles .table-brand td,
.paragon-styles .table-brand thead th,
.paragon-styles .table-brand tbody + tbody {
  border-color: #bdd2e4;
}
.paragon-styles .table-hover .table-brand:hover {
  background-color: #c9dbea;
}
.paragon-styles .table-hover .table-brand:hover > td,
.paragon-styles .table-hover .table-brand:hover > th {
  background-color: #c9dbea;
}
.paragon-styles .table-gray,
.paragon-styles .table-gray > th,
.paragon-styles .table-gray > td {
  background-color: #e8eaed;
}
.paragon-styles .table-gray th,
.paragon-styles .table-gray td,
.paragon-styles .table-gray thead th,
.paragon-styles .table-gray tbody + tbody {
  border-color: #d4d9dd;
}
.paragon-styles .table-hover .table-gray:hover {
  background-color: #dadde2;
}
.paragon-styles .table-hover .table-gray:hover > td,
.paragon-styles .table-hover .table-gray:hover > th {
  background-color: #dadde2;
}
.paragon-styles .table-active,
.paragon-styles .table-active > th,
.paragon-styles .table-active > td {
  background-color: rgba(3, 3, 3, 0.075);
}
.paragon-styles .table-hover .table-active:hover {
  background-color: rgba(0, 0, 0, 0.075);
}
.paragon-styles .table-hover .table-active:hover > td,
.paragon-styles .table-hover .table-active:hover > th {
  background-color: rgba(0, 0, 0, 0.075);
}
.paragon-styles .table .thead-dark th {
  color: white;
  background-color: #495057;
  border-color: #5a636c;
}
.paragon-styles .table .thead-light th {
  color: #495057;
  background-color: #f8f9fa;
  border-color: #e9ecef;
}
.paragon-styles .table-dark {
  color: white;
  background-color: #495057;
}
.paragon-styles .table-dark th,
.paragon-styles .table-dark td,
.paragon-styles .table-dark thead th {
  border-color: #5a636c;
}
.paragon-styles .table-dark.table-bordered {
  border: 0;
}
.paragon-styles .table-dark.table-striped tbody tr:nth-of-type(odd) {
  background-color: rgba(255, 255, 255, 0.05);
}
.paragon-styles .table-dark.table-hover tbody tr:hover {
  color: white;
  background-color: rgba(255, 255, 255, 0.075);
}
@media (max-width: 575.98px) {
  .paragon-styles .table-responsive-sm {
    display: block;
    width: 100%;
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
  .paragon-styles .table-responsive-sm > .table-bordered {
    border: 0;
  }
}
@media (max-width: 767.98px) {
  .paragon-styles .table-responsive-md {
    display: block;
    width: 100%;
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
  .paragon-styles .table-responsive-md > .table-bordered {
    border: 0;
  }
}
@media (max-width: 991.98px) {
  .paragon-styles .table-responsive-lg {
    display: block;
    width: 100%;
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
  .paragon-styles .table-responsive-lg > .table-bordered {
    border: 0;
  }
}
@media (max-width: 1199.98px) {
  .paragon-styles .table-responsive-xl {
    display: block;
    width: 100%;
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
  .paragon-styles .table-responsive-xl > .table-bordered {
    border: 0;
  }
}
.paragon-styles .table-responsive {
  display: block;
  width: 100%;
  overflow-x: auto;
  -webkit-overflow-scrolling: touch;
}
.paragon-styles .table-responsive > .table-bordered {
  border: 0;
}
.paragon-styles .form-control {
  display: block;
  width: 100%;
  height: calc(1.5em + 0.75rem + 2px);
  padding: 0.375rem 0.75rem;
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
  color: #495057;
  background-color: white;
  background-clip: padding-box;
  border: 1px solid #e9ecef;
  border-radius: 0.3rem;
  transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .form-control {
    transition: none;
  }
}
.paragon-styles .form-control::-ms-expand {
  background-color: transparent;
  border: 0;
}
.paragon-styles .form-control:-moz-focusring {
  color: transparent;
  text-shadow: 0 0 0 #495057;
}
.paragon-styles .form-control:focus {
  color: #495057;
  background-color: white;
  border-color: #dbe7f1;
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .form-control::placeholder {
  color: #adb5bd;
  opacity: 1;
}
.paragon-styles .form-control:disabled, .paragon-styles .form-control[readonly] {
  background-color: #f8f9fa;
  opacity: 1;
}
.paragon-styles input[type=date].form-control,
.paragon-styles input[type=time].form-control,
.paragon-styles input[type=datetime-local].form-control,
.paragon-styles input[type=month].form-control {
  appearance: none;
}
.paragon-styles select.form-control:focus::-ms-value {
  color: #495057;
  background-color: white;
}
.paragon-styles .form-control-file,
.paragon-styles .form-control-range {
  display: block;
  width: 100%;
}
.paragon-styles .col-form-label {
  padding-top: calc(0.375rem + 1px);
  padding-bottom: calc(0.375rem + 1px);
  margin-bottom: 0;
  font-size: inherit;
  line-height: 1.5;
}
.paragon-styles .col-form-label-lg {
  padding-top: calc(0.5rem + 1px);
  padding-bottom: calc(0.5rem + 1px);
  font-size: 1.25rem;
  line-height: 1.5;
}
.paragon-styles .col-form-label-sm {
  padding-top: calc(0.25rem + 1px);
  padding-bottom: calc(0.25rem + 1px);
  font-size: 0.875rem;
  line-height: 1.5;
}
.paragon-styles .form-control-plaintext {
  display: block;
  width: 100%;
  padding: 0.375rem 0;
  margin-bottom: 0;
  font-size: 1rem;
  line-height: 1.5;
  color: #212529;
  background-color: transparent;
  border: solid transparent;
  border-width: 1px 0;
}
.paragon-styles .form-control-plaintext.form-control-sm, .paragon-styles .form-control-plaintext.form-control-lg {
  padding-right: 0;
  padding-left: 0;
}
.paragon-styles .form-control-sm {
  height: calc(1.5em + 0.5rem + 2px);
  padding: 0.25rem 0.5rem;
  font-size: 0.875rem;
  line-height: 1.5;
  border-radius: 0.2rem;
}
.paragon-styles .form-control-lg {
  height: calc(1.5em + 1rem + 2px);
  padding: 0.5rem 1rem;
  font-size: 1.25rem;
  line-height: 1.5;
  border-radius: 0.35rem;
}
.paragon-styles select.form-control[size], .paragon-styles select.form-control[multiple] {
  height: auto;
}
.paragon-styles textarea.form-control {
  height: auto;
}
.paragon-styles .form-group {
  margin-bottom: 1rem;
}
.paragon-styles .form-text {
  display: block;
  margin-top: 0.25rem;
}
.paragon-styles .form-row {
  display: flex;
  flex-wrap: wrap;
  margin-right: -5px;
  margin-left: -5px;
}
.paragon-styles .form-row > .col,
.paragon-styles .form-row > [class*=col-] {
  padding-right: 5px;
  padding-left: 5px;
}
.paragon-styles .form-check {
  position: relative;
  display: block;
  padding-left: 1.25rem;
}
.paragon-styles .form-check-input {
  position: absolute;
  margin-top: 0.3rem;
  margin-left: -1.25rem;
}
.paragon-styles .form-check-input[disabled] ~ .form-check-label, .paragon-styles .form-check-input:disabled ~ .form-check-label {
  color: #adb5bd;
}
.paragon-styles .form-check-label {
  margin-bottom: 0;
}
.paragon-styles .form-check-inline {
  display: inline-flex;
  align-items: center;
  padding-left: 0;
  margin-right: 0.75rem;
}
.paragon-styles .form-check-inline .form-check-input {
  position: static;
  margin-top: 0;
  margin-right: 0.3125rem;
  margin-left: 0;
}
.paragon-styles .valid-feedback {
  display: none;
  width: 100%;
  margin-top: 0.25rem;
  font-size: 87.5%;
  color: #178253;
}
.paragon-styles .valid-tooltip {
  position: absolute;
  top: 100%;
  left: 0;
  z-index: 5;
  display: none;
  max-width: 100%;
  padding: 0.25rem 0.5rem;
  margin-top: 0.1rem;
  font-size: 0.875rem;
  line-height: 1.5;
  color: white;
  background-color: rgba(23, 130, 83, 0.9);
  border-radius: 0.3rem;
}
.form-row > .col > .paragon-styles .valid-tooltip, .form-row > [class*=col-] > .paragon-styles .valid-tooltip {
  left: 5px;
}
.was-validated .paragon-styles:valid ~ .valid-feedback,
.was-validated .paragon-styles:valid ~ .valid-tooltip, .paragon-styles.is-valid ~ .valid-feedback,
.paragon-styles.is-valid ~ .valid-tooltip {
  display: block;
}
.was-validated .paragon-styles .form-control:valid, .paragon-styles .form-control.is-valid {
  border-color: #178253;
  padding-right: calc(1.5em + 0.75rem);
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23178253' d='M2.3 6.73L.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1z'/%3e%3c/svg%3e");
  background-repeat: no-repeat;
  background-position: right calc(0.375em + 0.1875rem) center;
  background-size: calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .paragon-styles .form-control:valid:focus, .paragon-styles .form-control.is-valid:focus {
  border-color: #178253;
  box-shadow: 0 0 0 0.2rem rgba(23, 130, 83, 0.25);
}
.was-validated .paragon-styles textarea.form-control:valid, .paragon-styles textarea.form-control.is-valid {
  padding-right: calc(1.5em + 0.75rem);
  background-position: top calc(0.375em + 0.1875rem) right calc(0.375em + 0.1875rem);
}
.was-validated .paragon-styles .custom-select:valid, .paragon-styles .custom-select.is-valid {
  border-color: #178253;
  padding-right: calc((1em + 0.75rem) * 3 / 4 + 1.75rem);
  background: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 5'%3e%3cpath fill='%23495057' d='M2 0L0 2h4zm0 5L0 3h4z'/%3e%3c/svg%3e") no-repeat right 0.75rem center/8px 10px, white url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23178253' d='M2.3 6.73L.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1z'/%3e%3c/svg%3e") center right 1.75rem/calc(0.75em + 0.375rem) calc(0.75em + 0.375rem) no-repeat;
}
.was-validated .paragon-styles .custom-select:valid:focus, .paragon-styles .custom-select.is-valid:focus {
  border-color: #178253;
  box-shadow: 0 0 0 0.2rem rgba(23, 130, 83, 0.25);
}
.was-validated .paragon-styles .form-check-input:valid ~ .form-check-label, .paragon-styles .form-check-input.is-valid ~ .form-check-label {
  color: #178253;
}
.was-validated .paragon-styles .form-check-input:valid ~ .valid-feedback,
.was-validated .paragon-styles .form-check-input:valid ~ .valid-tooltip, .paragon-styles .form-check-input.is-valid ~ .valid-feedback,
.paragon-styles .form-check-input.is-valid ~ .valid-tooltip {
  display: block;
}
.was-validated .paragon-styles .custom-control-input:valid ~ .custom-control-label, .paragon-styles .custom-control-input.is-valid ~ .custom-control-label {
  color: #178253;
}
.was-validated .paragon-styles .custom-control-input:valid ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-valid ~ .custom-control-label::before {
  border-color: #178253;
}
.was-validated .paragon-styles .custom-control-input:valid:checked ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-valid:checked ~ .custom-control-label::before {
  border-color: #1fad6f;
  background-color: #1fad6f;
}
.was-validated .paragon-styles .custom-control-input:valid:focus ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-valid:focus ~ .custom-control-label::before {
  box-shadow: 0 0 0 0.2rem rgba(23, 130, 83, 0.25);
}
.was-validated .paragon-styles .custom-control-input:valid:focus:not(:checked) ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-valid:focus:not(:checked) ~ .custom-control-label::before {
  border-color: #178253;
}
.was-validated .paragon-styles .custom-file-input:valid ~ .custom-file-label, .paragon-styles .custom-file-input.is-valid ~ .custom-file-label {
  border-color: #178253;
}
.was-validated .paragon-styles .custom-file-input:valid:focus ~ .custom-file-label, .paragon-styles .custom-file-input.is-valid:focus ~ .custom-file-label {
  border-color: #178253;
  box-shadow: 0 0 0 0.2rem rgba(23, 130, 83, 0.25);
}
.paragon-styles .invalid-feedback {
  display: none;
  width: 100%;
  margin-top: 0.25rem;
  font-size: 87.5%;
  color: #C32D3A;
}
.paragon-styles .invalid-tooltip {
  position: absolute;
  top: 100%;
  left: 0;
  z-index: 5;
  display: none;
  max-width: 100%;
  padding: 0.25rem 0.5rem;
  margin-top: 0.1rem;
  font-size: 0.875rem;
  line-height: 1.5;
  color: white;
  background-color: rgba(195, 45, 58, 0.9);
  border-radius: 0.3rem;
}
.form-row > .col > .paragon-styles .invalid-tooltip, .form-row > [class*=col-] > .paragon-styles .invalid-tooltip {
  left: 5px;
}
.was-validated .paragon-styles:invalid ~ .invalid-feedback,
.was-validated .paragon-styles:invalid ~ .invalid-tooltip, .paragon-styles.is-invalid ~ .invalid-feedback,
.paragon-styles.is-invalid ~ .invalid-tooltip {
  display: block;
}
.was-validated .paragon-styles .form-control:invalid, .paragon-styles .form-control.is-invalid {
  border-color: #C32D3A;
  padding-right: calc(1.5em + 0.75rem);
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='%23C32D3A' viewBox='-2 -2 7 7'%3e%3cpath stroke='%23C32D3A' d='M0 0l3 3m0-3L0 3'/%3e%3ccircle r='.5'/%3e%3ccircle cx='3' r='.5'/%3e%3ccircle cy='3' r='.5'/%3e%3ccircle cx='3' cy='3' r='.5'/%3e%3c/svg%3E");
  background-repeat: no-repeat;
  background-position: right calc(0.375em + 0.1875rem) center;
  background-size: calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .paragon-styles .form-control:invalid:focus, .paragon-styles .form-control.is-invalid:focus {
  border-color: #C32D3A;
  box-shadow: 0 0 0 0.2rem rgba(195, 45, 58, 0.25);
}
.was-validated .paragon-styles textarea.form-control:invalid, .paragon-styles textarea.form-control.is-invalid {
  padding-right: calc(1.5em + 0.75rem);
  background-position: top calc(0.375em + 0.1875rem) right calc(0.375em + 0.1875rem);
}
.was-validated .paragon-styles .custom-select:invalid, .paragon-styles .custom-select.is-invalid {
  border-color: #C32D3A;
  padding-right: calc((1em + 0.75rem) * 3 / 4 + 1.75rem);
  background: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 5'%3e%3cpath fill='%23495057' d='M2 0L0 2h4zm0 5L0 3h4z'/%3e%3c/svg%3e") no-repeat right 0.75rem center/8px 10px, white url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='%23C32D3A' viewBox='-2 -2 7 7'%3e%3cpath stroke='%23C32D3A' d='M0 0l3 3m0-3L0 3'/%3e%3ccircle r='.5'/%3e%3ccircle cx='3' r='.5'/%3e%3ccircle cy='3' r='.5'/%3e%3ccircle cx='3' cy='3' r='.5'/%3e%3c/svg%3E") center right 1.75rem/calc(0.75em + 0.375rem) calc(0.75em + 0.375rem) no-repeat;
}
.was-validated .paragon-styles .custom-select:invalid:focus, .paragon-styles .custom-select.is-invalid:focus {
  border-color: #C32D3A;
  box-shadow: 0 0 0 0.2rem rgba(195, 45, 58, 0.25);
}
.was-validated .paragon-styles .form-check-input:invalid ~ .form-check-label, .paragon-styles .form-check-input.is-invalid ~ .form-check-label {
  color: #C32D3A;
}
.was-validated .paragon-styles .form-check-input:invalid ~ .invalid-feedback,
.was-validated .paragon-styles .form-check-input:invalid ~ .invalid-tooltip, .paragon-styles .form-check-input.is-invalid ~ .invalid-feedback,
.paragon-styles .form-check-input.is-invalid ~ .invalid-tooltip {
  display: block;
}
.was-validated .paragon-styles .custom-control-input:invalid ~ .custom-control-label, .paragon-styles .custom-control-input.is-invalid ~ .custom-control-label {
  color: #C32D3A;
}
.was-validated .paragon-styles .custom-control-input:invalid ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-invalid ~ .custom-control-label::before {
  border-color: #C32D3A;
}
.was-validated .paragon-styles .custom-control-input:invalid:checked ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-invalid:checked ~ .custom-control-label::before {
  border-color: #d64d59;
  background-color: #d64d59;
}
.was-validated .paragon-styles .custom-control-input:invalid:focus ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-invalid:focus ~ .custom-control-label::before {
  box-shadow: 0 0 0 0.2rem rgba(195, 45, 58, 0.25);
}
.was-validated .paragon-styles .custom-control-input:invalid:focus:not(:checked) ~ .custom-control-label::before, .paragon-styles .custom-control-input.is-invalid:focus:not(:checked) ~ .custom-control-label::before {
  border-color: #C32D3A;
}
.was-validated .paragon-styles .custom-file-input:invalid ~ .custom-file-label, .paragon-styles .custom-file-input.is-invalid ~ .custom-file-label {
  border-color: #C32D3A;
}
.was-validated .paragon-styles .custom-file-input:invalid:focus ~ .custom-file-label, .paragon-styles .custom-file-input.is-invalid:focus ~ .custom-file-label {
  border-color: #C32D3A;
  box-shadow: 0 0 0 0.2rem rgba(195, 45, 58, 0.25);
}
.paragon-styles .form-inline {
  display: flex;
  flex-flow: row wrap;
  align-items: center;
}
.paragon-styles .form-inline .form-check {
  width: 100%;
}
@media (min-width: 576px) {
  .paragon-styles .form-inline label {
    display: flex;
    align-items: center;
    justify-content: center;
    margin-bottom: 0;
  }
  .paragon-styles .form-inline .form-group {
    display: flex;
    flex: 0 0 auto;
    flex-flow: row wrap;
    align-items: center;
    margin-bottom: 0;
  }
  .paragon-styles .form-inline .form-control {
    display: inline-block;
    width: auto;
    vertical-align: middle;
  }
  .paragon-styles .form-inline .form-control-plaintext {
    display: inline-block;
  }
  .paragon-styles .form-inline .input-group,
.paragon-styles .form-inline .custom-select {
    width: auto;
  }
  .paragon-styles .form-inline .form-check {
    display: flex;
    align-items: center;
    justify-content: center;
    width: auto;
    padding-left: 0;
  }
  .paragon-styles .form-inline .form-check-input {
    position: relative;
    flex-shrink: 0;
    margin-top: 0;
    margin-right: 0.25rem;
    margin-left: 0;
  }
  .paragon-styles .form-inline .custom-control {
    align-items: center;
    justify-content: center;
  }
  .paragon-styles .form-inline .custom-control-label {
    margin-bottom: 0;
  }
}
.paragon-styles .btn {
  display: inline-block;
  font-weight: 400;
  color: #212529;
  text-align: center;
  vertical-align: middle;
  user-select: none;
  background-color: transparent;
  border: 2px solid transparent;
  padding: 0.375rem 0.75rem;
  font-size: 1rem;
  line-height: 1.5;
  border-radius: 0.3rem;
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .btn {
    transition: none;
  }
}
.paragon-styles .btn:hover {
  color: #212529;
  text-decoration: none;
}
.paragon-styles .btn.disabled, .paragon-styles .btn:disabled {
  opacity: 0.65;
}
.paragon-styles a.btn.disabled,
.paragon-styles fieldset:disabled a.btn {
  pointer-events: none;
}
.paragon-styles .btn-primary {
  color: #495057;
  background-color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .btn-primary:hover {
  color: white;
  background-color: #6687a3;
  border-color: #6687a3;
}
.paragon-styles .btn-primary.disabled, .paragon-styles .btn-primary:disabled {
  color: #495057;
  background-color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .btn-primary:not(:disabled):not(.disabled):active, .paragon-styles .btn-primary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-primary.dropdown-toggle {
  color: white;
  background-color: #5a768f;
  border-color: #5a768f;
}
.paragon-styles .btn-primary.focus, .paragon-styles .btn-primary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-primary.focus:before, .paragon-styles .btn-primary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #c0d4e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-primary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-primary.focus.btn:before, .paragon-styles .btn-primary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-primary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-primary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-primary.focus.btn:before, .paragon-styles .btn-primary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-primary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-primary.focus:active:before, .paragon-styles .btn-primary.focus.active:before, .paragon-styles .btn-primary:focus:active:before, .paragon-styles .btn-primary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-primary.focus:disabled:before, .paragon-styles .btn-primary.focus.disabled:before, .paragon-styles .btn-primary:focus:disabled:before, .paragon-styles .btn-primary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-primary {
  color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .btn-outline-primary:hover {
  color: #6687a3;
  background-color: #f7fafc;
  border-color: #5a768f;
}
.paragon-styles .btn-outline-primary.disabled, .paragon-styles .btn-outline-primary:disabled {
  color: #80a9cc;
  background-color: transparent;
}
.paragon-styles .btn-outline-primary:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-primary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-primary.dropdown-toggle {
  color: #495057;
  background-color: #f7fafc;
  border-color: #5a768f;
}
.paragon-styles .btn-outline-primary.focus, .paragon-styles .btn-outline-primary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-primary.focus:before, .paragon-styles .btn-outline-primary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #c0d4e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-primary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-primary.focus.btn:before, .paragon-styles .btn-outline-primary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-primary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-primary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-primary.focus.btn:before, .paragon-styles .btn-outline-primary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-primary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-primary.focus:active:before, .paragon-styles .btn-outline-primary.focus.active:before, .paragon-styles .btn-outline-primary:focus:active:before, .paragon-styles .btn-outline-primary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-primary.focus:disabled:before, .paragon-styles .btn-outline-primary.focus.disabled:before, .paragon-styles .btn-outline-primary:focus:disabled:before, .paragon-styles .btn-outline-primary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-primary {
  color: #80a9cc;
  border-color: transparent;
  background-color: #495057;
}
.paragon-styles .btn-inverse-primary:hover {
  color: #6597c1;
  background-color: #383d42;
  border-color: transparent;
}
.paragon-styles .btn-inverse-primary.disabled, .paragon-styles .btn-inverse-primary:disabled {
  color: #80a9cc;
  background-color: transparent;
}
.paragon-styles .btn-inverse-primary:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-primary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-primary.dropdown-toggle {
  color: #5c90bd;
  background: #eee;
}
.paragon-styles .btn-inverse-primary.focus, .paragon-styles .btn-inverse-primary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-primary.focus:before, .paragon-styles .btn-inverse-primary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-primary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-primary.focus.btn:before, .paragon-styles .btn-inverse-primary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-primary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-primary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-primary.focus.btn:before, .paragon-styles .btn-inverse-primary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-primary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-primary.focus:active:before, .paragon-styles .btn-inverse-primary.focus.active:before, .paragon-styles .btn-inverse-primary:focus:active:before, .paragon-styles .btn-inverse-primary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-primary.focus:disabled:before, .paragon-styles .btn-inverse-primary.focus.disabled:before, .paragon-styles .btn-inverse-primary:focus:disabled:before, .paragon-styles .btn-inverse-primary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-primary {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-primary:hover {
  color: #6687a3;
  background-color: #f7fafc;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-primary.disabled, .paragon-styles .btn-inverse-outline-primary:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-primary:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-primary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-primary.dropdown-toggle {
  color: #495057;
  background-color: #f7fafc;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-primary.focus, .paragon-styles .btn-inverse-outline-primary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-primary.focus:before, .paragon-styles .btn-inverse-outline-primary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-primary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-primary.focus.btn:before, .paragon-styles .btn-inverse-outline-primary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-primary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-primary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-primary.focus.btn:before, .paragon-styles .btn-inverse-outline-primary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-primary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-primary.focus:active:before, .paragon-styles .btn-inverse-outline-primary.focus.active:before, .paragon-styles .btn-inverse-outline-primary:focus:active:before, .paragon-styles .btn-inverse-outline-primary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-primary.focus:disabled:before, .paragon-styles .btn-inverse-outline-primary.focus.disabled:before, .paragon-styles .btn-inverse-outline-primary:focus:disabled:before, .paragon-styles .btn-inverse-outline-primary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-secondary {
  color: white;
  background-color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .btn-secondary:hover {
  color: white;
  background-color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .btn-secondary.disabled, .paragon-styles .btn-secondary:disabled {
  color: white;
  background-color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .btn-secondary:not(:disabled):not(.disabled):active, .paragon-styles .btn-secondary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-secondary.dropdown-toggle {
  color: white;
  background-color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .btn-secondary.focus, .paragon-styles .btn-secondary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-secondary.focus:before, .paragon-styles .btn-secondary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #6c757d;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-secondary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-secondary.focus.btn:before, .paragon-styles .btn-secondary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-secondary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-secondary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-secondary.focus.btn:before, .paragon-styles .btn-secondary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-secondary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-secondary.focus:active:before, .paragon-styles .btn-secondary.focus.active:before, .paragon-styles .btn-secondary:focus:active:before, .paragon-styles .btn-secondary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-secondary.focus:disabled:before, .paragon-styles .btn-secondary.focus.disabled:before, .paragon-styles .btn-secondary:focus:disabled:before, .paragon-styles .btn-secondary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-secondary {
  color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .btn-outline-secondary:hover {
  color: #6c757d;
  background-color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .btn-outline-secondary.disabled, .paragon-styles .btn-outline-secondary:disabled {
  color: #6c757d;
  background-color: transparent;
}
.paragon-styles .btn-outline-secondary:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-secondary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-secondary.dropdown-toggle {
  color: white;
  background-color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .btn-outline-secondary.focus, .paragon-styles .btn-outline-secondary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-secondary.focus:before, .paragon-styles .btn-outline-secondary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #6c757d;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-secondary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-secondary.focus.btn:before, .paragon-styles .btn-outline-secondary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-secondary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-secondary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-secondary.focus.btn:before, .paragon-styles .btn-outline-secondary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-secondary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-secondary.focus:active:before, .paragon-styles .btn-outline-secondary.focus.active:before, .paragon-styles .btn-outline-secondary:focus:active:before, .paragon-styles .btn-outline-secondary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-secondary.focus:disabled:before, .paragon-styles .btn-outline-secondary.focus.disabled:before, .paragon-styles .btn-outline-secondary:focus:disabled:before, .paragon-styles .btn-outline-secondary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-secondary {
  color: #6c757d;
  border-color: transparent;
  background-color: white;
}
.paragon-styles .btn-inverse-secondary:hover {
  color: #5a6268;
  background-color: #ececec;
  border-color: transparent;
}
.paragon-styles .btn-inverse-secondary.disabled, .paragon-styles .btn-inverse-secondary:disabled {
  color: #6c757d;
  background-color: transparent;
}
.paragon-styles .btn-inverse-secondary:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-secondary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-secondary.dropdown-toggle {
  color: #545b62;
  background: #eee;
}
.paragon-styles .btn-inverse-secondary.focus, .paragon-styles .btn-inverse-secondary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-secondary.focus:before, .paragon-styles .btn-inverse-secondary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-secondary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-secondary.focus.btn:before, .paragon-styles .btn-inverse-secondary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-secondary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-secondary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-secondary.focus.btn:before, .paragon-styles .btn-inverse-secondary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-secondary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-secondary.focus:active:before, .paragon-styles .btn-inverse-secondary.focus.active:before, .paragon-styles .btn-inverse-secondary:focus:active:before, .paragon-styles .btn-inverse-secondary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-secondary.focus:disabled:before, .paragon-styles .btn-inverse-secondary.focus.disabled:before, .paragon-styles .btn-inverse-secondary:focus:disabled:before, .paragon-styles .btn-inverse-secondary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-secondary {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-secondary:hover {
  color: #6c757d;
  background-color: #6c757d;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-secondary.disabled, .paragon-styles .btn-inverse-outline-secondary:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-secondary:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-secondary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-secondary.dropdown-toggle {
  color: white;
  background-color: #6c757d;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-secondary.focus, .paragon-styles .btn-inverse-outline-secondary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-secondary.focus:before, .paragon-styles .btn-inverse-outline-secondary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-secondary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-secondary.focus.btn:before, .paragon-styles .btn-inverse-outline-secondary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-secondary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-secondary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-secondary.focus.btn:before, .paragon-styles .btn-inverse-outline-secondary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-secondary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-secondary.focus:active:before, .paragon-styles .btn-inverse-outline-secondary.focus.active:before, .paragon-styles .btn-inverse-outline-secondary:focus:active:before, .paragon-styles .btn-inverse-outline-secondary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-secondary.focus:disabled:before, .paragon-styles .btn-inverse-outline-secondary.focus.disabled:before, .paragon-styles .btn-inverse-outline-secondary:focus:disabled:before, .paragon-styles .btn-inverse-outline-secondary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-success {
  color: white;
  background-color: #178253;
  border-color: #178253;
}
.paragon-styles .btn-success:hover {
  color: white;
  background-color: #126842;
  border-color: #126842;
}
.paragon-styles .btn-success.disabled, .paragon-styles .btn-success:disabled {
  color: white;
  background-color: #178253;
  border-color: #178253;
}
.paragon-styles .btn-success:not(:disabled):not(.disabled):active, .paragon-styles .btn-success:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-success.dropdown-toggle {
  color: white;
  background-color: #105b3a;
  border-color: #105b3a;
}
.paragon-styles .btn-success.focus, .paragon-styles .btn-success:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-success.focus:before, .paragon-styles .btn-success:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #8bc1a9;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-success.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-success.focus.btn:before, .paragon-styles .btn-success:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-success.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-success.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-success.focus.btn:before, .paragon-styles .btn-success:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-success.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-success.focus:active:before, .paragon-styles .btn-success.focus.active:before, .paragon-styles .btn-success:focus:active:before, .paragon-styles .btn-success:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-success.focus:disabled:before, .paragon-styles .btn-success.focus.disabled:before, .paragon-styles .btn-success:focus:disabled:before, .paragon-styles .btn-success:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-success {
  color: #178253;
  border-color: #178253;
}
.paragon-styles .btn-outline-success:hover {
  color: #126842;
  background-color: #f1f8f5;
  border-color: #105b3a;
}
.paragon-styles .btn-outline-success.disabled, .paragon-styles .btn-outline-success:disabled {
  color: #178253;
  background-color: transparent;
}
.paragon-styles .btn-outline-success:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-success:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-success.dropdown-toggle {
  color: #495057;
  background-color: #f1f8f5;
  border-color: #105b3a;
}
.paragon-styles .btn-outline-success.focus, .paragon-styles .btn-outline-success:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-success.focus:before, .paragon-styles .btn-outline-success:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #8bc1a9;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-success.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-success.focus.btn:before, .paragon-styles .btn-outline-success:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-success.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-success.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-success.focus.btn:before, .paragon-styles .btn-outline-success:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-success.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-success.focus:active:before, .paragon-styles .btn-outline-success.focus.active:before, .paragon-styles .btn-outline-success:focus:active:before, .paragon-styles .btn-outline-success:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-success.focus:disabled:before, .paragon-styles .btn-outline-success.focus.disabled:before, .paragon-styles .btn-outline-success:focus:disabled:before, .paragon-styles .btn-outline-success:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-success {
  color: #178253;
  border-color: transparent;
  background-color: white;
}
.paragon-styles .btn-inverse-success:hover {
  color: #11623e;
  background-color: #ececec;
  border-color: transparent;
}
.paragon-styles .btn-inverse-success.disabled, .paragon-styles .btn-inverse-success:disabled {
  color: #178253;
  background-color: transparent;
}
.paragon-styles .btn-inverse-success:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-success:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-success.dropdown-toggle {
  color: #0f5737;
  background: #eee;
}
.paragon-styles .btn-inverse-success.focus, .paragon-styles .btn-inverse-success:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-success.focus:before, .paragon-styles .btn-inverse-success:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-success.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-success.focus.btn:before, .paragon-styles .btn-inverse-success:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-success.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-success.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-success.focus.btn:before, .paragon-styles .btn-inverse-success:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-success.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-success.focus:active:before, .paragon-styles .btn-inverse-success.focus.active:before, .paragon-styles .btn-inverse-success:focus:active:before, .paragon-styles .btn-inverse-success:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-success.focus:disabled:before, .paragon-styles .btn-inverse-success.focus.disabled:before, .paragon-styles .btn-inverse-success:focus:disabled:before, .paragon-styles .btn-inverse-success:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-success {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-success:hover {
  color: #126842;
  background-color: #f1f8f5;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-success.disabled, .paragon-styles .btn-inverse-outline-success:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-success:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-success:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-success.dropdown-toggle {
  color: #495057;
  background-color: #f1f8f5;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-success.focus, .paragon-styles .btn-inverse-outline-success:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-success.focus:before, .paragon-styles .btn-inverse-outline-success:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-success.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-success.focus.btn:before, .paragon-styles .btn-inverse-outline-success:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-success.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-success.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-success.focus.btn:before, .paragon-styles .btn-inverse-outline-success:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-success.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-success.focus:active:before, .paragon-styles .btn-inverse-outline-success.focus.active:before, .paragon-styles .btn-inverse-outline-success:focus:active:before, .paragon-styles .btn-inverse-outline-success:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-success.focus:disabled:before, .paragon-styles .btn-inverse-outline-success.focus.disabled:before, .paragon-styles .btn-inverse-outline-success:focus:disabled:before, .paragon-styles .btn-inverse-outline-success:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-info {
  color: white;
  background-color: #006DAA;
  border-color: #006DAA;
}
.paragon-styles .btn-info:hover {
  color: white;
  background-color: #005788;
  border-color: #005788;
}
.paragon-styles .btn-info.disabled, .paragon-styles .btn-info:disabled {
  color: white;
  background-color: #006DAA;
  border-color: #006DAA;
}
.paragon-styles .btn-info:not(:disabled):not(.disabled):active, .paragon-styles .btn-info:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-info.dropdown-toggle {
  color: white;
  background-color: #004c77;
  border-color: #004c77;
}
.paragon-styles .btn-info.focus, .paragon-styles .btn-info:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-info.focus:before, .paragon-styles .btn-info:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #80b6d5;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-info.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-info.focus.btn:before, .paragon-styles .btn-info:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-info.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-info.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-info.focus.btn:before, .paragon-styles .btn-info:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-info.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-info.focus:active:before, .paragon-styles .btn-info.focus.active:before, .paragon-styles .btn-info:focus:active:before, .paragon-styles .btn-info:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-info.focus:disabled:before, .paragon-styles .btn-info.focus.disabled:before, .paragon-styles .btn-info:focus:disabled:before, .paragon-styles .btn-info:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-info {
  color: #006DAA;
  border-color: #006DAA;
}
.paragon-styles .btn-outline-info:hover {
  color: #005788;
  background-color: #f0f6fa;
  border-color: #004c77;
}
.paragon-styles .btn-outline-info.disabled, .paragon-styles .btn-outline-info:disabled {
  color: #006DAA;
  background-color: transparent;
}
.paragon-styles .btn-outline-info:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-info:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-info.dropdown-toggle {
  color: #495057;
  background-color: #f0f6fa;
  border-color: #004c77;
}
.paragon-styles .btn-outline-info.focus, .paragon-styles .btn-outline-info:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-info.focus:before, .paragon-styles .btn-outline-info:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #80b6d5;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-info.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-info.focus.btn:before, .paragon-styles .btn-outline-info:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-info.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-info.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-info.focus.btn:before, .paragon-styles .btn-outline-info:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-info.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-info.focus:active:before, .paragon-styles .btn-outline-info.focus.active:before, .paragon-styles .btn-outline-info:focus:active:before, .paragon-styles .btn-outline-info:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-info.focus:disabled:before, .paragon-styles .btn-outline-info.focus.disabled:before, .paragon-styles .btn-outline-info:focus:disabled:before, .paragon-styles .btn-outline-info:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-info {
  color: #006DAA;
  border-color: transparent;
  background-color: white;
}
.paragon-styles .btn-inverse-info:hover {
  color: #005484;
  background-color: #ececec;
  border-color: transparent;
}
.paragon-styles .btn-inverse-info.disabled, .paragon-styles .btn-inverse-info:disabled {
  color: #006DAA;
  background-color: transparent;
}
.paragon-styles .btn-inverse-info:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-info:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-info.dropdown-toggle {
  color: #004c77;
  background: #eee;
}
.paragon-styles .btn-inverse-info.focus, .paragon-styles .btn-inverse-info:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-info.focus:before, .paragon-styles .btn-inverse-info:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-info.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-info.focus.btn:before, .paragon-styles .btn-inverse-info:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-info.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-info.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-info.focus.btn:before, .paragon-styles .btn-inverse-info:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-info.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-info.focus:active:before, .paragon-styles .btn-inverse-info.focus.active:before, .paragon-styles .btn-inverse-info:focus:active:before, .paragon-styles .btn-inverse-info:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-info.focus:disabled:before, .paragon-styles .btn-inverse-info.focus.disabled:before, .paragon-styles .btn-inverse-info:focus:disabled:before, .paragon-styles .btn-inverse-info:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-info {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-info:hover {
  color: #005788;
  background-color: #f0f6fa;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-info.disabled, .paragon-styles .btn-inverse-outline-info:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-info:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-info:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-info.dropdown-toggle {
  color: #495057;
  background-color: #f0f6fa;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-info.focus, .paragon-styles .btn-inverse-outline-info:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-info.focus:before, .paragon-styles .btn-inverse-outline-info:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-info.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-info.focus.btn:before, .paragon-styles .btn-inverse-outline-info:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-info.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-info.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-info.focus.btn:before, .paragon-styles .btn-inverse-outline-info:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-info.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-info.focus:active:before, .paragon-styles .btn-inverse-outline-info.focus.active:before, .paragon-styles .btn-inverse-outline-info:focus:active:before, .paragon-styles .btn-inverse-outline-info:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-info.focus:disabled:before, .paragon-styles .btn-inverse-outline-info.focus.disabled:before, .paragon-styles .btn-inverse-outline-info:focus:disabled:before, .paragon-styles .btn-inverse-outline-info:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-warning {
  color: #495057;
  background-color: #D69600;
  border-color: #D69600;
}
.paragon-styles .btn-warning:hover {
  color: white;
  background-color: #ab7800;
  border-color: #ab7800;
}
.paragon-styles .btn-warning.disabled, .paragon-styles .btn-warning:disabled {
  color: #495057;
  background-color: #D69600;
  border-color: #D69600;
}
.paragon-styles .btn-warning:not(:disabled):not(.disabled):active, .paragon-styles .btn-warning:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-warning.dropdown-toggle {
  color: white;
  background-color: #966900;
  border-color: #966900;
}
.paragon-styles .btn-warning.focus, .paragon-styles .btn-warning:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-warning.focus:before, .paragon-styles .btn-warning:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #ebcb80;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-warning.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-warning.focus.btn:before, .paragon-styles .btn-warning:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-warning.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-warning.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-warning.focus.btn:before, .paragon-styles .btn-warning:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-warning.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-warning.focus:active:before, .paragon-styles .btn-warning.focus.active:before, .paragon-styles .btn-warning:focus:active:before, .paragon-styles .btn-warning:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-warning.focus:disabled:before, .paragon-styles .btn-warning.focus.disabled:before, .paragon-styles .btn-warning:focus:disabled:before, .paragon-styles .btn-warning:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-warning {
  color: #D69600;
  border-color: #D69600;
}
.paragon-styles .btn-outline-warning:hover {
  color: #ab7800;
  background-color: #fdf9f0;
  border-color: #966900;
}
.paragon-styles .btn-outline-warning.disabled, .paragon-styles .btn-outline-warning:disabled {
  color: #D69600;
  background-color: transparent;
}
.paragon-styles .btn-outline-warning:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-warning:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-warning.dropdown-toggle {
  color: #495057;
  background-color: #fdf9f0;
  border-color: #966900;
}
.paragon-styles .btn-outline-warning.focus, .paragon-styles .btn-outline-warning:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-warning.focus:before, .paragon-styles .btn-outline-warning:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #ebcb80;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-warning.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-warning.focus.btn:before, .paragon-styles .btn-outline-warning:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-warning.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-warning.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-warning.focus.btn:before, .paragon-styles .btn-outline-warning:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-warning.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-warning.focus:active:before, .paragon-styles .btn-outline-warning.focus.active:before, .paragon-styles .btn-outline-warning:focus:active:before, .paragon-styles .btn-outline-warning:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-warning.focus:disabled:before, .paragon-styles .btn-outline-warning.focus.disabled:before, .paragon-styles .btn-outline-warning:focus:disabled:before, .paragon-styles .btn-outline-warning:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-warning {
  color: #D69600;
  border-color: transparent;
  background-color: #495057;
}
.paragon-styles .btn-inverse-warning:hover {
  color: #b07b00;
  background-color: #383d42;
  border-color: transparent;
}
.paragon-styles .btn-inverse-warning.disabled, .paragon-styles .btn-inverse-warning:disabled {
  color: #D69600;
  background-color: transparent;
}
.paragon-styles .btn-inverse-warning:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-warning:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-warning.dropdown-toggle {
  color: #a37200;
  background: #eee;
}
.paragon-styles .btn-inverse-warning.focus, .paragon-styles .btn-inverse-warning:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-warning.focus:before, .paragon-styles .btn-inverse-warning:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-warning.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-warning.focus.btn:before, .paragon-styles .btn-inverse-warning:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-warning.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-warning.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-warning.focus.btn:before, .paragon-styles .btn-inverse-warning:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-warning.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-warning.focus:active:before, .paragon-styles .btn-inverse-warning.focus.active:before, .paragon-styles .btn-inverse-warning:focus:active:before, .paragon-styles .btn-inverse-warning:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-warning.focus:disabled:before, .paragon-styles .btn-inverse-warning.focus.disabled:before, .paragon-styles .btn-inverse-warning:focus:disabled:before, .paragon-styles .btn-inverse-warning:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-warning {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-warning:hover {
  color: #ab7800;
  background-color: #fdf9f0;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-warning.disabled, .paragon-styles .btn-inverse-outline-warning:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-warning:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-warning:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-warning.dropdown-toggle {
  color: #495057;
  background-color: #fdf9f0;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-warning.focus, .paragon-styles .btn-inverse-outline-warning:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-warning.focus:before, .paragon-styles .btn-inverse-outline-warning:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-warning.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-warning.focus.btn:before, .paragon-styles .btn-inverse-outline-warning:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-warning.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-warning.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-warning.focus.btn:before, .paragon-styles .btn-inverse-outline-warning:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-warning.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-warning.focus:active:before, .paragon-styles .btn-inverse-outline-warning.focus.active:before, .paragon-styles .btn-inverse-outline-warning:focus:active:before, .paragon-styles .btn-inverse-outline-warning:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-warning.focus:disabled:before, .paragon-styles .btn-inverse-outline-warning.focus.disabled:before, .paragon-styles .btn-inverse-outline-warning:focus:disabled:before, .paragon-styles .btn-inverse-outline-warning:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-danger {
  color: white;
  background-color: #C32D3A;
  border-color: #C32D3A;
}
.paragon-styles .btn-danger:hover {
  color: white;
  background-color: #9c242e;
  border-color: #9c242e;
}
.paragon-styles .btn-danger.disabled, .paragon-styles .btn-danger:disabled {
  color: white;
  background-color: #C32D3A;
  border-color: #C32D3A;
}
.paragon-styles .btn-danger:not(:disabled):not(.disabled):active, .paragon-styles .btn-danger:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-danger.dropdown-toggle {
  color: white;
  background-color: #892029;
  border-color: #892029;
}
.paragon-styles .btn-danger.focus, .paragon-styles .btn-danger:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-danger.focus:before, .paragon-styles .btn-danger:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #e1969d;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-danger.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-danger.focus.btn:before, .paragon-styles .btn-danger:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-danger.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-danger.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-danger.focus.btn:before, .paragon-styles .btn-danger:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-danger.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-danger.focus:active:before, .paragon-styles .btn-danger.focus.active:before, .paragon-styles .btn-danger:focus:active:before, .paragon-styles .btn-danger:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-danger.focus:disabled:before, .paragon-styles .btn-danger.focus.disabled:before, .paragon-styles .btn-danger:focus:disabled:before, .paragon-styles .btn-danger:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-danger {
  color: #C32D3A;
  border-color: #C32D3A;
}
.paragon-styles .btn-outline-danger:hover {
  color: #9c242e;
  background-color: #fbf2f3;
  border-color: #892029;
}
.paragon-styles .btn-outline-danger.disabled, .paragon-styles .btn-outline-danger:disabled {
  color: #C32D3A;
  background-color: transparent;
}
.paragon-styles .btn-outline-danger:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-danger:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-danger.dropdown-toggle {
  color: #495057;
  background-color: #fbf2f3;
  border-color: #892029;
}
.paragon-styles .btn-outline-danger.focus, .paragon-styles .btn-outline-danger:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-danger.focus:before, .paragon-styles .btn-outline-danger:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #e1969d;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-danger.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-danger.focus.btn:before, .paragon-styles .btn-outline-danger:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-danger.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-danger.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-danger.focus.btn:before, .paragon-styles .btn-outline-danger:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-danger.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-danger.focus:active:before, .paragon-styles .btn-outline-danger.focus.active:before, .paragon-styles .btn-outline-danger:focus:active:before, .paragon-styles .btn-outline-danger:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-danger.focus:disabled:before, .paragon-styles .btn-outline-danger.focus.disabled:before, .paragon-styles .btn-outline-danger:focus:disabled:before, .paragon-styles .btn-outline-danger:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-danger {
  color: #C32D3A;
  border-color: transparent;
  background-color: white;
}
.paragon-styles .btn-inverse-danger:hover {
  color: #a42631;
  background-color: #ececec;
  border-color: transparent;
}
.paragon-styles .btn-inverse-danger.disabled, .paragon-styles .btn-inverse-danger:disabled {
  color: #C32D3A;
  background-color: transparent;
}
.paragon-styles .btn-inverse-danger:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-danger:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-danger.dropdown-toggle {
  color: #9a232e;
  background: #eee;
}
.paragon-styles .btn-inverse-danger.focus, .paragon-styles .btn-inverse-danger:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-danger.focus:before, .paragon-styles .btn-inverse-danger:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-danger.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-danger.focus.btn:before, .paragon-styles .btn-inverse-danger:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-danger.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-danger.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-danger.focus.btn:before, .paragon-styles .btn-inverse-danger:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-danger.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-danger.focus:active:before, .paragon-styles .btn-inverse-danger.focus.active:before, .paragon-styles .btn-inverse-danger:focus:active:before, .paragon-styles .btn-inverse-danger:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-danger.focus:disabled:before, .paragon-styles .btn-inverse-danger.focus.disabled:before, .paragon-styles .btn-inverse-danger:focus:disabled:before, .paragon-styles .btn-inverse-danger:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-danger {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-danger:hover {
  color: #9c242e;
  background-color: #fbf2f3;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-danger.disabled, .paragon-styles .btn-inverse-outline-danger:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-danger:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-danger:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-danger.dropdown-toggle {
  color: #495057;
  background-color: #fbf2f3;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-danger.focus, .paragon-styles .btn-inverse-outline-danger:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-danger.focus:before, .paragon-styles .btn-inverse-outline-danger:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-danger.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-danger.focus.btn:before, .paragon-styles .btn-inverse-outline-danger:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-danger.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-danger.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-danger.focus.btn:before, .paragon-styles .btn-inverse-outline-danger:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-danger.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-danger.focus:active:before, .paragon-styles .btn-inverse-outline-danger.focus.active:before, .paragon-styles .btn-inverse-outline-danger:focus:active:before, .paragon-styles .btn-inverse-outline-danger:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-danger.focus:disabled:before, .paragon-styles .btn-inverse-outline-danger.focus.disabled:before, .paragon-styles .btn-inverse-outline-danger:focus:disabled:before, .paragon-styles .btn-inverse-outline-danger:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-light {
  color: #495057;
  background-color: #eee;
  border-color: #eee;
}
.paragon-styles .btn-light:hover {
  color: #495057;
  background-color: #D7D3D1;
  border-color: #D7D3D1;
}
.paragon-styles .btn-light.disabled, .paragon-styles .btn-light:disabled {
  color: #495057;
  background-color: #eee;
  border-color: #eee;
}
.paragon-styles .btn-light:not(:disabled):not(.disabled):active, .paragon-styles .btn-light:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-light.dropdown-toggle {
  color: #495057;
  background-color: #a7a7a7;
  border-color: #a7a7a7;
}
.paragon-styles .btn-light.focus, .paragon-styles .btn-light:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-light.focus:before, .paragon-styles .btn-light:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #f7f7f7;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-light.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-light.focus.btn:before, .paragon-styles .btn-light:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-light.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-light.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-light.focus.btn:before, .paragon-styles .btn-light:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-light.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-light.focus:active:before, .paragon-styles .btn-light.focus.active:before, .paragon-styles .btn-light:focus:active:before, .paragon-styles .btn-light:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-light.focus:disabled:before, .paragon-styles .btn-light.focus.disabled:before, .paragon-styles .btn-light:focus:disabled:before, .paragon-styles .btn-light:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-light {
  color: #eee;
  border-color: #eee;
}
.paragon-styles .btn-outline-light:hover {
  color: #D7D3D1;
  background-color: #fefefe;
  border-color: #a7a7a7;
}
.paragon-styles .btn-outline-light.disabled, .paragon-styles .btn-outline-light:disabled {
  color: #eee;
  background-color: transparent;
}
.paragon-styles .btn-outline-light:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-light:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-light.dropdown-toggle {
  color: #495057;
  background-color: #fefefe;
  border-color: #a7a7a7;
}
.paragon-styles .btn-outline-light.focus, .paragon-styles .btn-outline-light:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-light.focus:before, .paragon-styles .btn-outline-light:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #f7f7f7;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-light.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-light.focus.btn:before, .paragon-styles .btn-outline-light:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-light.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-light.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-light.focus.btn:before, .paragon-styles .btn-outline-light:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-light.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-light.focus:active:before, .paragon-styles .btn-outline-light.focus.active:before, .paragon-styles .btn-outline-light:focus:active:before, .paragon-styles .btn-outline-light:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-light.focus:disabled:before, .paragon-styles .btn-outline-light.focus.disabled:before, .paragon-styles .btn-outline-light:focus:disabled:before, .paragon-styles .btn-outline-light:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-light {
  color: #eee;
  border-color: transparent;
  background-color: #495057;
}
.paragon-styles .btn-inverse-light:hover {
  color: #dbdbdb;
  background-color: #383d42;
  border-color: transparent;
}
.paragon-styles .btn-inverse-light.disabled, .paragon-styles .btn-inverse-light:disabled {
  color: #eee;
  background-color: transparent;
}
.paragon-styles .btn-inverse-light:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-light:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-light.dropdown-toggle {
  color: #d5d5d5;
  background: #eee;
}
.paragon-styles .btn-inverse-light.focus, .paragon-styles .btn-inverse-light:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-light.focus:before, .paragon-styles .btn-inverse-light:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-light.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-light.focus.btn:before, .paragon-styles .btn-inverse-light:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-light.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-light.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-light.focus.btn:before, .paragon-styles .btn-inverse-light:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-light.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-light.focus:active:before, .paragon-styles .btn-inverse-light.focus.active:before, .paragon-styles .btn-inverse-light:focus:active:before, .paragon-styles .btn-inverse-light:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-light.focus:disabled:before, .paragon-styles .btn-inverse-light.focus.disabled:before, .paragon-styles .btn-inverse-light:focus:disabled:before, .paragon-styles .btn-inverse-light:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-light {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-light:hover {
  color: #D7D3D1;
  background-color: #fefefe;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-light.disabled, .paragon-styles .btn-inverse-outline-light:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-light:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-light:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-light.dropdown-toggle {
  color: #495057;
  background-color: #fefefe;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-light.focus, .paragon-styles .btn-inverse-outline-light:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-light.focus:before, .paragon-styles .btn-inverse-outline-light:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-light.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-light.focus.btn:before, .paragon-styles .btn-inverse-outline-light:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-light.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-light.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-light.focus.btn:before, .paragon-styles .btn-inverse-outline-light:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-light.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-light.focus:active:before, .paragon-styles .btn-inverse-outline-light.focus.active:before, .paragon-styles .btn-inverse-outline-light:focus:active:before, .paragon-styles .btn-inverse-outline-light:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-light.focus:disabled:before, .paragon-styles .btn-inverse-outline-light.focus.disabled:before, .paragon-styles .btn-inverse-outline-light:focus:disabled:before, .paragon-styles .btn-inverse-outline-light:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-dark {
  color: white;
  background-color: #495057;
  border-color: #495057;
}
.paragon-styles .btn-dark:hover {
  color: white;
  background-color: #3a4046;
  border-color: #3a4046;
}
.paragon-styles .btn-dark.disabled, .paragon-styles .btn-dark:disabled {
  color: white;
  background-color: #495057;
  border-color: #495057;
}
.paragon-styles .btn-dark:not(:disabled):not(.disabled):active, .paragon-styles .btn-dark:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-dark.dropdown-toggle {
  color: white;
  background-color: #33383d;
  border-color: #33383d;
}
.paragon-styles .btn-dark.focus, .paragon-styles .btn-dark:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-dark.focus:before, .paragon-styles .btn-dark:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #a4a8ab;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-dark.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-dark.focus.btn:before, .paragon-styles .btn-dark:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-dark.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-dark.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-dark.focus.btn:before, .paragon-styles .btn-dark:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-dark.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-dark.focus:active:before, .paragon-styles .btn-dark.focus.active:before, .paragon-styles .btn-dark:focus:active:before, .paragon-styles .btn-dark:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-dark.focus:disabled:before, .paragon-styles .btn-dark.focus.disabled:before, .paragon-styles .btn-dark:focus:disabled:before, .paragon-styles .btn-dark:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-dark {
  color: #495057;
  border-color: #495057;
}
.paragon-styles .btn-outline-dark:hover {
  color: #3a4046;
  background-color: #f4f5f5;
  border-color: #33383d;
}
.paragon-styles .btn-outline-dark.disabled, .paragon-styles .btn-outline-dark:disabled {
  color: #495057;
  background-color: transparent;
}
.paragon-styles .btn-outline-dark:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-dark:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-dark.dropdown-toggle {
  color: #495057;
  background-color: #f4f5f5;
  border-color: #33383d;
}
.paragon-styles .btn-outline-dark.focus, .paragon-styles .btn-outline-dark:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-dark.focus:before, .paragon-styles .btn-outline-dark:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #a4a8ab;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-dark.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-dark.focus.btn:before, .paragon-styles .btn-outline-dark:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-dark.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-dark.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-dark.focus.btn:before, .paragon-styles .btn-outline-dark:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-dark.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-dark.focus:active:before, .paragon-styles .btn-outline-dark.focus.active:before, .paragon-styles .btn-outline-dark:focus:active:before, .paragon-styles .btn-outline-dark:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-dark.focus:disabled:before, .paragon-styles .btn-outline-dark.focus.disabled:before, .paragon-styles .btn-outline-dark:focus:disabled:before, .paragon-styles .btn-outline-dark:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-dark {
  color: #495057;
  border-color: transparent;
  background-color: white;
}
.paragon-styles .btn-inverse-dark:hover {
  color: #383d42;
  background-color: #ececec;
  border-color: transparent;
}
.paragon-styles .btn-inverse-dark.disabled, .paragon-styles .btn-inverse-dark:disabled {
  color: #495057;
  background-color: transparent;
}
.paragon-styles .btn-inverse-dark:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-dark:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-dark.dropdown-toggle {
  color: #32373b;
  background: #eee;
}
.paragon-styles .btn-inverse-dark.focus, .paragon-styles .btn-inverse-dark:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-dark.focus:before, .paragon-styles .btn-inverse-dark:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-dark.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-dark.focus.btn:before, .paragon-styles .btn-inverse-dark:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-dark.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-dark.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-dark.focus.btn:before, .paragon-styles .btn-inverse-dark:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-dark.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-dark.focus:active:before, .paragon-styles .btn-inverse-dark.focus.active:before, .paragon-styles .btn-inverse-dark:focus:active:before, .paragon-styles .btn-inverse-dark:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-dark.focus:disabled:before, .paragon-styles .btn-inverse-dark.focus.disabled:before, .paragon-styles .btn-inverse-dark:focus:disabled:before, .paragon-styles .btn-inverse-dark:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-dark {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-dark:hover {
  color: #3a4046;
  background-color: #f4f5f5;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-dark.disabled, .paragon-styles .btn-inverse-outline-dark:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-dark:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-dark:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-dark.dropdown-toggle {
  color: #495057;
  background-color: #f4f5f5;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-dark.focus, .paragon-styles .btn-inverse-outline-dark:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-dark.focus:before, .paragon-styles .btn-inverse-outline-dark:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-dark.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-dark.focus.btn:before, .paragon-styles .btn-inverse-outline-dark:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-dark.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-dark.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-dark.focus.btn:before, .paragon-styles .btn-inverse-outline-dark:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-dark.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-dark.focus:active:before, .paragon-styles .btn-inverse-outline-dark.focus.active:before, .paragon-styles .btn-inverse-outline-dark:focus:active:before, .paragon-styles .btn-inverse-outline-dark:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-dark.focus:disabled:before, .paragon-styles .btn-inverse-outline-dark.focus.disabled:before, .paragon-styles .btn-inverse-outline-dark:focus:disabled:before, .paragon-styles .btn-inverse-outline-dark:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-brand {
  color: #495057;
  background-color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .btn-brand:hover {
  color: white;
  background-color: #6687a3;
  border-color: #6687a3;
}
.paragon-styles .btn-brand.disabled, .paragon-styles .btn-brand:disabled {
  color: #495057;
  background-color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .btn-brand:not(:disabled):not(.disabled):active, .paragon-styles .btn-brand:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-brand.dropdown-toggle {
  color: white;
  background-color: #5a768f;
  border-color: #5a768f;
}
.paragon-styles .btn-brand.focus, .paragon-styles .btn-brand:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-brand.focus:before, .paragon-styles .btn-brand:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #c0d4e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-brand.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-brand.focus.btn:before, .paragon-styles .btn-brand:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-brand.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-brand.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-brand.focus.btn:before, .paragon-styles .btn-brand:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-brand.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-brand.focus:active:before, .paragon-styles .btn-brand.focus.active:before, .paragon-styles .btn-brand:focus:active:before, .paragon-styles .btn-brand:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-brand.focus:disabled:before, .paragon-styles .btn-brand.focus.disabled:before, .paragon-styles .btn-brand:focus:disabled:before, .paragon-styles .btn-brand:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-brand {
  color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .btn-outline-brand:hover {
  color: #6687a3;
  background-color: #f7fafc;
  border-color: #5a768f;
}
.paragon-styles .btn-outline-brand.disabled, .paragon-styles .btn-outline-brand:disabled {
  color: #80a9cc;
  background-color: transparent;
}
.paragon-styles .btn-outline-brand:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-brand:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-brand.dropdown-toggle {
  color: #495057;
  background-color: #f7fafc;
  border-color: #5a768f;
}
.paragon-styles .btn-outline-brand.focus, .paragon-styles .btn-outline-brand:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-brand.focus:before, .paragon-styles .btn-outline-brand:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #c0d4e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-brand.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-brand.focus.btn:before, .paragon-styles .btn-outline-brand:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-brand.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-brand.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-brand.focus.btn:before, .paragon-styles .btn-outline-brand:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-brand.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-brand.focus:active:before, .paragon-styles .btn-outline-brand.focus.active:before, .paragon-styles .btn-outline-brand:focus:active:before, .paragon-styles .btn-outline-brand:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-brand.focus:disabled:before, .paragon-styles .btn-outline-brand.focus.disabled:before, .paragon-styles .btn-outline-brand:focus:disabled:before, .paragon-styles .btn-outline-brand:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-brand {
  color: #80a9cc;
  border-color: transparent;
  background-color: #495057;
}
.paragon-styles .btn-inverse-brand:hover {
  color: #6597c1;
  background-color: #383d42;
  border-color: transparent;
}
.paragon-styles .btn-inverse-brand.disabled, .paragon-styles .btn-inverse-brand:disabled {
  color: #80a9cc;
  background-color: transparent;
}
.paragon-styles .btn-inverse-brand:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-brand:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-brand.dropdown-toggle {
  color: #5c90bd;
  background: #eee;
}
.paragon-styles .btn-inverse-brand.focus, .paragon-styles .btn-inverse-brand:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-brand.focus:before, .paragon-styles .btn-inverse-brand:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-brand.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-brand.focus.btn:before, .paragon-styles .btn-inverse-brand:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-brand.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-brand.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-brand.focus.btn:before, .paragon-styles .btn-inverse-brand:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-brand.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-brand.focus:active:before, .paragon-styles .btn-inverse-brand.focus.active:before, .paragon-styles .btn-inverse-brand:focus:active:before, .paragon-styles .btn-inverse-brand:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-brand.focus:disabled:before, .paragon-styles .btn-inverse-brand.focus.disabled:before, .paragon-styles .btn-inverse-brand:focus:disabled:before, .paragon-styles .btn-inverse-brand:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-brand {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-brand:hover {
  color: #6687a3;
  background-color: #f7fafc;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-brand.disabled, .paragon-styles .btn-inverse-outline-brand:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-brand:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-brand:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-brand.dropdown-toggle {
  color: #495057;
  background-color: #f7fafc;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-brand.focus, .paragon-styles .btn-inverse-outline-brand:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-brand.focus:before, .paragon-styles .btn-inverse-outline-brand:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-brand.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-brand.focus.btn:before, .paragon-styles .btn-inverse-outline-brand:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-brand.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-brand.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-brand.focus.btn:before, .paragon-styles .btn-inverse-outline-brand:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-brand.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-brand.focus:active:before, .paragon-styles .btn-inverse-outline-brand.focus.active:before, .paragon-styles .btn-inverse-outline-brand:focus:active:before, .paragon-styles .btn-inverse-outline-brand:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-brand.focus:disabled:before, .paragon-styles .btn-inverse-outline-brand.focus.disabled:before, .paragon-styles .btn-inverse-outline-brand:focus:disabled:before, .paragon-styles .btn-inverse-outline-brand:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-gray {
  color: #495057;
  background-color: #adb5bd;
  border-color: #adb5bd;
}
.paragon-styles .btn-gray:hover {
  color: white;
  background-color: #495057;
  border-color: #495057;
}
.paragon-styles .btn-gray.disabled, .paragon-styles .btn-gray:disabled {
  color: #495057;
  background-color: #adb5bd;
  border-color: #adb5bd;
}
.paragon-styles .btn-gray:not(:disabled):not(.disabled):active, .paragon-styles .btn-gray:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-gray.dropdown-toggle {
  color: white;
  background-color: #212529;
  border-color: #212529;
}
.paragon-styles .btn-gray.focus, .paragon-styles .btn-gray:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-gray.focus:before, .paragon-styles .btn-gray:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #dee2e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-gray.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-gray.focus.btn:before, .paragon-styles .btn-gray:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-gray.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-gray.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-gray.focus.btn:before, .paragon-styles .btn-gray:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-gray.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-gray.focus:active:before, .paragon-styles .btn-gray.focus.active:before, .paragon-styles .btn-gray:focus:active:before, .paragon-styles .btn-gray:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-gray.focus:disabled:before, .paragon-styles .btn-gray.focus.disabled:before, .paragon-styles .btn-gray:focus:disabled:before, .paragon-styles .btn-gray:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-outline-gray {
  color: #adb5bd;
  border-color: #adb5bd;
}
.paragon-styles .btn-outline-gray:hover {
  color: #495057;
  background-color: #f8f9fa;
  border-color: #212529;
}
.paragon-styles .btn-outline-gray.disabled, .paragon-styles .btn-outline-gray:disabled {
  color: #adb5bd;
  background-color: transparent;
}
.paragon-styles .btn-outline-gray:not(:disabled):not(.disabled):active, .paragon-styles .btn-outline-gray:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-outline-gray.dropdown-toggle {
  color: #495057;
  background-color: #f8f9fa;
  border-color: #212529;
}
.paragon-styles .btn-outline-gray.focus, .paragon-styles .btn-outline-gray:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-outline-gray.focus:before, .paragon-styles .btn-outline-gray:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #dee2e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-outline-gray.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-gray.focus.btn:before, .paragon-styles .btn-outline-gray:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-outline-gray.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-outline-gray.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-gray.focus.btn:before, .paragon-styles .btn-outline-gray:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-outline-gray.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-outline-gray.focus:active:before, .paragon-styles .btn-outline-gray.focus.active:before, .paragon-styles .btn-outline-gray:focus:active:before, .paragon-styles .btn-outline-gray:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-outline-gray.focus:disabled:before, .paragon-styles .btn-outline-gray.focus.disabled:before, .paragon-styles .btn-outline-gray:focus:disabled:before, .paragon-styles .btn-outline-gray:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-gray {
  color: #adb5bd;
  border-color: transparent;
  background-color: #495057;
}
.paragon-styles .btn-inverse-gray:hover {
  color: #98a2ac;
  background-color: #383d42;
  border-color: transparent;
}
.paragon-styles .btn-inverse-gray.disabled, .paragon-styles .btn-inverse-gray:disabled {
  color: #adb5bd;
  background-color: transparent;
}
.paragon-styles .btn-inverse-gray:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-gray:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-gray.dropdown-toggle {
  color: #919ca6;
  background: #eee;
}
.paragon-styles .btn-inverse-gray.focus, .paragon-styles .btn-inverse-gray:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-gray.focus:before, .paragon-styles .btn-inverse-gray:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-gray.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-gray.focus.btn:before, .paragon-styles .btn-inverse-gray:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-gray.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-gray.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-gray.focus.btn:before, .paragon-styles .btn-inverse-gray:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-gray.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-gray.focus:active:before, .paragon-styles .btn-inverse-gray.focus.active:before, .paragon-styles .btn-inverse-gray:focus:active:before, .paragon-styles .btn-inverse-gray:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-gray.focus:disabled:before, .paragon-styles .btn-inverse-gray.focus.disabled:before, .paragon-styles .btn-inverse-gray:focus:disabled:before, .paragon-styles .btn-inverse-gray:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-outline-gray {
  color: white;
  border-color: white;
}
.paragon-styles .btn-inverse-outline-gray:hover {
  color: #495057;
  background-color: #f8f9fa;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-gray.disabled, .paragon-styles .btn-inverse-outline-gray:disabled {
  color: white;
  background-color: transparent;
}
.paragon-styles .btn-inverse-outline-gray:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-outline-gray:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-outline-gray.dropdown-toggle {
  color: #495057;
  background-color: #f8f9fa;
  border-color: transparent;
}
.paragon-styles .btn-inverse-outline-gray.focus, .paragon-styles .btn-inverse-outline-gray:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-outline-gray.focus:before, .paragon-styles .btn-inverse-outline-gray:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-outline-gray.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-gray.focus.btn:before, .paragon-styles .btn-inverse-outline-gray:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-outline-gray.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-outline-gray.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-gray.focus.btn:before, .paragon-styles .btn-inverse-outline-gray:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-outline-gray.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-outline-gray.focus:active:before, .paragon-styles .btn-inverse-outline-gray.focus.active:before, .paragon-styles .btn-inverse-outline-gray:focus:active:before, .paragon-styles .btn-inverse-outline-gray:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-outline-gray.focus:disabled:before, .paragon-styles .btn-inverse-outline-gray.focus.disabled:before, .paragon-styles .btn-inverse-outline-gray:focus:disabled:before, .paragon-styles .btn-inverse-outline-gray:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-tertiary {
  color: #495057;
  background-color: transparent;
  border-color: transparent;
}
.paragon-styles .btn-tertiary:hover {
  color: #495057;
  background-color: #eee;
  border-color: transparent;
}
.paragon-styles .btn-tertiary.disabled, .paragon-styles .btn-tertiary:disabled {
  color: white;
  background-color: transparent;
  border-color: transparent;
}
.paragon-styles .btn-tertiary:not(:disabled):not(.disabled):active, .paragon-styles .btn-tertiary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-tertiary.dropdown-toggle {
  color: #495057;
  background-color: #eee;
  border-color: transparent;
}
.paragon-styles .btn-tertiary.focus, .paragon-styles .btn-tertiary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-tertiary.focus:before, .paragon-styles .btn-tertiary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #c0d4e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-tertiary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-tertiary.focus.btn:before, .paragon-styles .btn-tertiary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-tertiary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-tertiary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-tertiary.focus.btn:before, .paragon-styles .btn-tertiary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-tertiary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-tertiary.focus:active:before, .paragon-styles .btn-tertiary.focus.active:before, .paragon-styles .btn-tertiary:focus:active:before, .paragon-styles .btn-tertiary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-tertiary.focus:disabled:before, .paragon-styles .btn-tertiary.focus.disabled:before, .paragon-styles .btn-tertiary:focus:disabled:before, .paragon-styles .btn-tertiary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-inverse-tertiary {
  color: white;
  background-color: transparent;
  border-color: transparent;
}
.paragon-styles .btn-inverse-tertiary:hover {
  color: white;
  background-color: rgba(255, 255, 255, 0.1);
  border-color: transparent;
}
.paragon-styles .btn-inverse-tertiary.disabled, .paragon-styles .btn-inverse-tertiary:disabled {
  color: white;
  background-color: transparent;
  border-color: transparent;
}
.paragon-styles .btn-inverse-tertiary:not(:disabled):not(.disabled):active, .paragon-styles .btn-inverse-tertiary:not(:disabled):not(.disabled).active, .show > .paragon-styles .btn-inverse-tertiary.dropdown-toggle {
  color: white;
  background-color: rgba(255, 255, 255, 0.1);
  border-color: transparent;
}
.paragon-styles .btn-inverse-tertiary.focus, .paragon-styles .btn-inverse-tertiary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-inverse-tertiary.focus:before, .paragon-styles .btn-inverse-tertiary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-inverse-tertiary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-tertiary.focus.btn:before, .paragon-styles .btn-inverse-tertiary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-inverse-tertiary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-inverse-tertiary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-tertiary.focus.btn:before, .paragon-styles .btn-inverse-tertiary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-inverse-tertiary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-inverse-tertiary.focus:active:before, .paragon-styles .btn-inverse-tertiary.focus.active:before, .paragon-styles .btn-inverse-tertiary:focus:active:before, .paragon-styles .btn-inverse-tertiary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-inverse-tertiary.focus:disabled:before, .paragon-styles .btn-inverse-tertiary.focus.disabled:before, .paragon-styles .btn-inverse-tertiary:focus:disabled:before, .paragon-styles .btn-inverse-tertiary:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-link {
  font-weight: 400;
  color: #80a9cc;
  text-decoration: none;
}
.paragon-styles .btn-link:hover {
  color: #4984b6;
  text-decoration: underline;
}
.paragon-styles .btn-link:focus, .paragon-styles .btn-link.focus {
  text-decoration: underline;
  box-shadow: none;
}
.paragon-styles .btn-link:disabled, .paragon-styles .btn-link.disabled {
  color: #adb5bd;
  pointer-events: none;
}
.paragon-styles .btn-light.focus, .paragon-styles .btn-light:focus,
.paragon-styles .btn-dark.focus,
.paragon-styles .btn-dark:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .btn-light.focus:before, .paragon-styles .btn-light:focus:before,
.paragon-styles .btn-dark.focus:before,
.paragon-styles .btn-dark:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #c0d4e6;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .btn-light.focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-light.focus.btn:before, .paragon-styles .btn-light:focus.btn-lg:before, .paragon-styles .btn-group-lg > .btn-light.btn:focus:before,
.paragon-styles .btn-dark.focus.btn-lg:before,
.paragon-styles .btn-group-lg > .btn-dark.focus.btn:before,
.paragon-styles .btn-dark:focus.btn-lg:before,
.paragon-styles .btn-group-lg > .btn-dark.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .btn-light.focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-light.focus.btn:before, .paragon-styles .btn-light:focus.btn-sm:before, .paragon-styles .btn-group-sm > .btn-light.btn:focus:before,
.paragon-styles .btn-dark.focus.btn-sm:before,
.paragon-styles .btn-group-sm > .btn-dark.focus.btn:before,
.paragon-styles .btn-dark:focus.btn-sm:before,
.paragon-styles .btn-group-sm > .btn-dark.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .btn-light.focus:active:before, .paragon-styles .btn-light.focus.active:before, .paragon-styles .btn-light:focus:active:before, .paragon-styles .btn-light:focus.active:before,
.paragon-styles .btn-dark.focus:active:before,
.paragon-styles .btn-dark.focus.active:before,
.paragon-styles .btn-dark:focus:active:before,
.paragon-styles .btn-dark:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .btn-light.focus:disabled:before, .paragon-styles .btn-light.focus.disabled:before, .paragon-styles .btn-light:focus:disabled:before, .paragon-styles .btn-light:focus.disabled:before,
.paragon-styles .btn-dark.focus:disabled:before,
.paragon-styles .btn-dark.focus.disabled:before,
.paragon-styles .btn-dark:focus:disabled:before,
.paragon-styles .btn-dark:focus.disabled:before {
  display: none;
}
.paragon-styles .btn-lg, .paragon-styles .btn-group-lg > .btn {
  padding: 0.5rem 1rem;
  font-size: 1.25rem;
  line-height: 1.5;
  border-radius: 0.35rem;
}
.paragon-styles .btn-sm, .paragon-styles .btn-group-sm > .btn {
  padding: 0.25rem 0.5rem;
  font-size: 0.875rem;
  line-height: 1.5;
  border-radius: 0.2rem;
}
.paragon-styles .btn-block {
  display: block;
  width: 100%;
}
.paragon-styles .btn-block + .btn-block {
  margin-top: 0.5rem;
}
.paragon-styles .btn-inline {
  line-height: calc(1.5em - 4px);
  font-size: inherit;
  vertical-align: baseline;
  padding: 0 0.25em;
}
.paragon-styles input[type=submit].btn-block,
.paragon-styles input[type=reset].btn-block,
.paragon-styles input[type=button].btn-block {
  width: 100%;
}
.paragon-styles .fade {
  transition: opacity 0.15s linear;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .fade {
    transition: none;
  }
}
.paragon-styles .fade:not(.show) {
  opacity: 0;
}
.paragon-styles .collapse:not(.show) {
  display: none;
}
.paragon-styles .collapsing {
  position: relative;
  height: 0;
  overflow: hidden;
  transition: height 0.35s ease;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .collapsing {
    transition: none;
  }
}
.paragon-styles .dropup,
.paragon-styles .dropright,
.paragon-styles .dropdown,
.paragon-styles .dropleft {
  position: relative;
}
.paragon-styles .dropdown-toggle {
  white-space: nowrap;
}
.paragon-styles .dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0.3em solid;
  border-right: 0.3em solid transparent;
  border-bottom: 0;
  border-left: 0.3em solid transparent;
}
.paragon-styles .dropdown-toggle:empty::after {
  margin-left: 0;
}
.paragon-styles .dropdown-menu {
  position: absolute;
  top: 100%;
  left: 0;
  z-index: 1000;
  display: none;
  float: left;
  min-width: 10rem;
  padding: 0.5rem 0;
  margin: 0.125rem 0 0;
  font-size: 1rem;
  color: #212529;
  text-align: left;
  list-style: none;
  background-color: white;
  background-clip: padding-box;
  border: 1px solid rgba(3, 3, 3, 0.15);
  border-radius: 0.3rem;
}
.paragon-styles .dropdown-menu-left {
  right: auto;
  left: 0;
}
.paragon-styles .dropdown-menu-right {
  right: 0;
  left: auto;
}
@media (min-width: 576px) {
  .paragon-styles .dropdown-menu-sm-left {
    right: auto;
    left: 0;
  }
  .paragon-styles .dropdown-menu-sm-right {
    right: 0;
    left: auto;
  }
}
@media (min-width: 768px) {
  .paragon-styles .dropdown-menu-md-left {
    right: auto;
    left: 0;
  }
  .paragon-styles .dropdown-menu-md-right {
    right: 0;
    left: auto;
  }
}
@media (min-width: 992px) {
  .paragon-styles .dropdown-menu-lg-left {
    right: auto;
    left: 0;
  }
  .paragon-styles .dropdown-menu-lg-right {
    right: 0;
    left: auto;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .dropdown-menu-xl-left {
    right: auto;
    left: 0;
  }
  .paragon-styles .dropdown-menu-xl-right {
    right: 0;
    left: auto;
  }
}
.paragon-styles .dropup .dropdown-menu {
  top: auto;
  bottom: 100%;
  margin-top: 0;
  margin-bottom: 0.125rem;
}
.paragon-styles .dropup .dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0;
  border-right: 0.3em solid transparent;
  border-bottom: 0.3em solid;
  border-left: 0.3em solid transparent;
}
.paragon-styles .dropup .dropdown-toggle:empty::after {
  margin-left: 0;
}
.paragon-styles .dropright .dropdown-menu {
  top: 0;
  right: auto;
  left: 100%;
  margin-top: 0;
  margin-left: 0.125rem;
}
.paragon-styles .dropright .dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0.3em solid transparent;
  border-right: 0;
  border-bottom: 0.3em solid transparent;
  border-left: 0.3em solid;
}
.paragon-styles .dropright .dropdown-toggle:empty::after {
  margin-left: 0;
}
.paragon-styles .dropright .dropdown-toggle::after {
  vertical-align: 0;
}
.paragon-styles .dropleft .dropdown-menu {
  top: 0;
  right: 100%;
  left: auto;
  margin-top: 0;
  margin-right: 0.125rem;
}
.paragon-styles .dropleft .dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
}
.paragon-styles .dropleft .dropdown-toggle::after {
  display: none;
}
.paragon-styles .dropleft .dropdown-toggle::before {
  display: inline-block;
  margin-right: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0.3em solid transparent;
  border-right: 0.3em solid;
  border-bottom: 0.3em solid transparent;
}
.paragon-styles .dropleft .dropdown-toggle:empty::after {
  margin-left: 0;
}
.paragon-styles .dropleft .dropdown-toggle::before {
  vertical-align: 0;
}
.paragon-styles .dropdown-menu[x-placement^=top], .paragon-styles .dropdown-menu[x-placement^=right], .paragon-styles .dropdown-menu[x-placement^=bottom], .paragon-styles .dropdown-menu[x-placement^=left] {
  right: auto;
  bottom: auto;
}
.paragon-styles .dropdown-divider {
  height: 0;
  margin: 0.5rem 0;
  overflow: hidden;
  border-top: 1px solid #f8f9fa;
}
.paragon-styles .dropdown-item {
  display: block;
  width: 100%;
  padding: 0.25rem 1.5rem;
  clear: both;
  font-weight: 400;
  color: #212529;
  text-align: inherit;
  white-space: nowrap;
  background-color: transparent;
  border: 0;
}
.paragon-styles .dropdown-item:hover, .paragon-styles .dropdown-item:focus {
  color: #16181b;
  text-decoration: none;
  background-color: #f8f9fa;
}
.paragon-styles .dropdown-item.active, .paragon-styles .dropdown-item:active {
  color: white;
  text-decoration: none;
  background-color: #80a9cc;
}
.paragon-styles .dropdown-item.disabled, .paragon-styles .dropdown-item:disabled {
  color: #adb5bd;
  pointer-events: none;
  background-color: transparent;
}
.paragon-styles .dropdown-menu.show {
  display: block;
}
.paragon-styles .dropdown-header {
  display: block;
  padding: 0.5rem 1.5rem;
  margin-bottom: 0;
  font-size: 0.875rem;
  color: #adb5bd;
  white-space: nowrap;
}
.paragon-styles .dropdown-item-text {
  display: block;
  padding: 0.25rem 1.5rem;
  color: #212529;
}
.paragon-styles .btn-group,
.paragon-styles .btn-group-vertical {
  position: relative;
  display: inline-flex;
  vertical-align: middle;
}
.paragon-styles .btn-group > .btn,
.paragon-styles .btn-group-vertical > .btn {
  position: relative;
  flex: 1 1 auto;
}
.paragon-styles .btn-group > .btn:hover,
.paragon-styles .btn-group-vertical > .btn:hover {
  z-index: 1;
}
.paragon-styles .btn-group > .btn:focus, .paragon-styles .btn-group > .btn:active, .paragon-styles .btn-group > .btn.active,
.paragon-styles .btn-group-vertical > .btn:focus,
.paragon-styles .btn-group-vertical > .btn:active,
.paragon-styles .btn-group-vertical > .btn.active {
  z-index: 1;
}
.paragon-styles .btn-toolbar {
  display: flex;
  flex-wrap: wrap;
  justify-content: flex-start;
}
.paragon-styles .btn-toolbar .input-group {
  width: auto;
}
.paragon-styles .btn-group > .btn:not(:first-child),
.paragon-styles .btn-group > .btn-group:not(:first-child) {
  margin-left: -2px;
}
.paragon-styles .btn-group > .btn:not(:last-child):not(.dropdown-toggle),
.paragon-styles .btn-group > .btn-group:not(:last-child) > .btn {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.paragon-styles .btn-group > .btn:not(:first-child),
.paragon-styles .btn-group > .btn-group:not(:first-child) > .btn {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.paragon-styles .dropdown-toggle-split {
  padding-right: 0.5625rem;
  padding-left: 0.5625rem;
}
.paragon-styles .dropdown-toggle-split::after, .dropup .paragon-styles .dropdown-toggle-split::after, .dropright .paragon-styles .dropdown-toggle-split::after {
  margin-left: 0;
}
.dropleft .paragon-styles .dropdown-toggle-split::before {
  margin-right: 0;
}
.paragon-styles .btn-sm + .dropdown-toggle-split, .paragon-styles .btn-group-sm > .btn + .dropdown-toggle-split {
  padding-right: 0.375rem;
  padding-left: 0.375rem;
}
.paragon-styles .btn-lg + .dropdown-toggle-split, .paragon-styles .btn-group-lg > .btn + .dropdown-toggle-split {
  padding-right: 0.75rem;
  padding-left: 0.75rem;
}
.paragon-styles .btn-group-vertical {
  flex-direction: column;
  align-items: flex-start;
  justify-content: center;
}
.paragon-styles .btn-group-vertical > .btn,
.paragon-styles .btn-group-vertical > .btn-group {
  width: 100%;
}
.paragon-styles .btn-group-vertical > .btn:not(:first-child),
.paragon-styles .btn-group-vertical > .btn-group:not(:first-child) {
  margin-top: -2px;
}
.paragon-styles .btn-group-vertical > .btn:not(:last-child):not(.dropdown-toggle),
.paragon-styles .btn-group-vertical > .btn-group:not(:last-child) > .btn {
  border-bottom-right-radius: 0;
  border-bottom-left-radius: 0;
}
.paragon-styles .btn-group-vertical > .btn:not(:first-child),
.paragon-styles .btn-group-vertical > .btn-group:not(:first-child) > .btn {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}
.paragon-styles .btn-group-toggle > .btn,
.paragon-styles .btn-group-toggle > .btn-group > .btn {
  margin-bottom: 0;
}
.paragon-styles .btn-group-toggle > .btn input[type=radio],
.paragon-styles .btn-group-toggle > .btn input[type=checkbox],
.paragon-styles .btn-group-toggle > .btn-group > .btn input[type=radio],
.paragon-styles .btn-group-toggle > .btn-group > .btn input[type=checkbox] {
  position: absolute;
  clip: rect(0, 0, 0, 0);
  pointer-events: none;
}
.paragon-styles .input-group {
  position: relative;
  display: flex;
  flex-wrap: wrap;
  align-items: stretch;
  width: 100%;
}
.paragon-styles .input-group > .form-control,
.paragon-styles .input-group > .form-control-plaintext,
.paragon-styles .input-group > .custom-select,
.paragon-styles .input-group > .custom-file {
  position: relative;
  flex: 1 1 auto;
  width: 1%;
  min-width: 0;
  margin-bottom: 0;
}
.paragon-styles .input-group > .form-control + .form-control,
.paragon-styles .input-group > .form-control + .custom-select,
.paragon-styles .input-group > .form-control + .custom-file,
.paragon-styles .input-group > .form-control-plaintext + .form-control,
.paragon-styles .input-group > .form-control-plaintext + .custom-select,
.paragon-styles .input-group > .form-control-plaintext + .custom-file,
.paragon-styles .input-group > .custom-select + .form-control,
.paragon-styles .input-group > .custom-select + .custom-select,
.paragon-styles .input-group > .custom-select + .custom-file,
.paragon-styles .input-group > .custom-file + .form-control,
.paragon-styles .input-group > .custom-file + .custom-select,
.paragon-styles .input-group > .custom-file + .custom-file {
  margin-left: -1px;
}
.paragon-styles .input-group > .form-control:focus,
.paragon-styles .input-group > .custom-select:focus,
.paragon-styles .input-group > .custom-file .custom-file-input:focus ~ .custom-file-label {
  z-index: 3;
}
.paragon-styles .input-group > .custom-file .custom-file-input:focus {
  z-index: 4;
}
.paragon-styles .input-group > .form-control:not(:first-child),
.paragon-styles .input-group > .custom-select:not(:first-child) {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.paragon-styles .input-group > .custom-file {
  display: flex;
  align-items: center;
}
.paragon-styles .input-group > .custom-file:not(:last-child) .custom-file-label, .paragon-styles .input-group > .custom-file:not(:first-child) .custom-file-label {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.paragon-styles .input-group:not(.has-validation) > .form-control:not(:last-child),
.paragon-styles .input-group:not(.has-validation) > .custom-select:not(:last-child),
.paragon-styles .input-group:not(.has-validation) > .custom-file:not(:last-child) .custom-file-label::after {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.paragon-styles .input-group.has-validation > .form-control:nth-last-child(n+3),
.paragon-styles .input-group.has-validation > .custom-select:nth-last-child(n+3),
.paragon-styles .input-group.has-validation > .custom-file:nth-last-child(n+3) .custom-file-label::after {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.paragon-styles .input-group-prepend,
.paragon-styles .input-group-append {
  display: flex;
}
.paragon-styles .input-group-prepend .btn,
.paragon-styles .input-group-append .btn {
  position: relative;
  z-index: 2;
}
.paragon-styles .input-group-prepend .btn:focus,
.paragon-styles .input-group-append .btn:focus {
  z-index: 3;
}
.paragon-styles .input-group-prepend .btn + .btn,
.paragon-styles .input-group-prepend .btn + .input-group-text,
.paragon-styles .input-group-prepend .input-group-text + .input-group-text,
.paragon-styles .input-group-prepend .input-group-text + .btn,
.paragon-styles .input-group-append .btn + .btn,
.paragon-styles .input-group-append .btn + .input-group-text,
.paragon-styles .input-group-append .input-group-text + .input-group-text,
.paragon-styles .input-group-append .input-group-text + .btn {
  margin-left: -1px;
}
.paragon-styles .input-group-prepend {
  margin-right: -1px;
}
.paragon-styles .input-group-append {
  margin-left: -1px;
}
.paragon-styles .input-group-text {
  display: flex;
  align-items: center;
  padding: 0.375rem 0.75rem;
  margin-bottom: 0;
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
  color: #495057;
  text-align: center;
  white-space: nowrap;
  background-color: #f8f9fa;
  border: 1px solid #e9ecef;
  border-radius: 0.3rem;
}
.paragon-styles .input-group-text input[type=radio],
.paragon-styles .input-group-text input[type=checkbox] {
  margin-top: 0;
}
.paragon-styles .input-group-lg > .form-control:not(textarea),
.paragon-styles .input-group-lg > .custom-select {
  height: calc(1.5em + 1rem + 2px);
}
.paragon-styles .input-group-lg > .form-control,
.paragon-styles .input-group-lg > .custom-select,
.paragon-styles .input-group-lg > .input-group-prepend > .input-group-text,
.paragon-styles .input-group-lg > .input-group-append > .input-group-text,
.paragon-styles .input-group-lg > .input-group-prepend > .btn,
.paragon-styles .input-group-lg > .input-group-append > .btn {
  padding: 0.5rem 1rem;
  font-size: 1.25rem;
  line-height: 1.5;
  border-radius: 0.35rem;
}
.paragon-styles .input-group-sm > .form-control:not(textarea),
.paragon-styles .input-group-sm > .custom-select {
  height: calc(1.5em + 0.5rem + 2px);
}
.paragon-styles .input-group-sm > .form-control,
.paragon-styles .input-group-sm > .custom-select,
.paragon-styles .input-group-sm > .input-group-prepend > .input-group-text,
.paragon-styles .input-group-sm > .input-group-append > .input-group-text,
.paragon-styles .input-group-sm > .input-group-prepend > .btn,
.paragon-styles .input-group-sm > .input-group-append > .btn {
  padding: 0.25rem 0.5rem;
  font-size: 0.875rem;
  line-height: 1.5;
  border-radius: 0.2rem;
}
.paragon-styles .input-group-lg > .custom-select,
.paragon-styles .input-group-sm > .custom-select {
  padding-right: 1.75rem;
}
.paragon-styles .input-group > .input-group-prepend > .btn,
.paragon-styles .input-group > .input-group-prepend > .input-group-text,
.paragon-styles .input-group:not(.has-validation) > .input-group-append:not(:last-child) > .btn,
.paragon-styles .input-group:not(.has-validation) > .input-group-append:not(:last-child) > .input-group-text,
.paragon-styles .input-group.has-validation > .input-group-append:nth-last-child(n+3) > .btn,
.paragon-styles .input-group.has-validation > .input-group-append:nth-last-child(n+3) > .input-group-text,
.paragon-styles .input-group > .input-group-append:last-child > .btn:not(:last-child):not(.dropdown-toggle),
.paragon-styles .input-group > .input-group-append:last-child > .input-group-text:not(:last-child) {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.paragon-styles .input-group > .input-group-append > .btn,
.paragon-styles .input-group > .input-group-append > .input-group-text,
.paragon-styles .input-group > .input-group-prepend:not(:first-child) > .btn,
.paragon-styles .input-group > .input-group-prepend:not(:first-child) > .input-group-text,
.paragon-styles .input-group > .input-group-prepend:first-child > .btn:not(:first-child),
.paragon-styles .input-group > .input-group-prepend:first-child > .input-group-text:not(:first-child) {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.paragon-styles .custom-control {
  position: relative;
  z-index: 1;
  display: block;
  min-height: 1.5rem;
  padding-left: 1.5rem;
  color-adjust: exact;
}
.paragon-styles .custom-control-inline {
  display: inline-flex;
  margin-right: 1rem;
}
.paragon-styles .custom-control-input {
  position: absolute;
  left: 0;
  z-index: -1;
  width: 1rem;
  height: 1.25rem;
  opacity: 0;
}
.paragon-styles .custom-control-input:checked ~ .custom-control-label::before {
  color: white;
  border-color: #80a9cc;
  background-color: #80a9cc;
}
.paragon-styles .custom-control-input:focus ~ .custom-control-label::before {
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .custom-control-input:focus:not(:checked) ~ .custom-control-label::before {
  border-color: #dbe7f1;
}
.paragon-styles .custom-control-input:not(:disabled):active ~ .custom-control-label::before {
  color: white;
  background-color: white;
  border-color: white;
}
.paragon-styles .custom-control-input[disabled] ~ .custom-control-label, .paragon-styles .custom-control-input:disabled ~ .custom-control-label {
  color: #adb5bd;
}
.paragon-styles .custom-control-input[disabled] ~ .custom-control-label::before, .paragon-styles .custom-control-input:disabled ~ .custom-control-label::before {
  background-color: #f8f9fa;
}
.paragon-styles .custom-control-label {
  position: relative;
  margin-bottom: 0;
  vertical-align: top;
}
.paragon-styles .custom-control-label::before {
  position: absolute;
  top: 0.25rem;
  left: -1.5rem;
  display: block;
  width: 1rem;
  height: 1rem;
  pointer-events: none;
  content: "";
  background-color: white;
  border: #e9ecef solid 1px;
}
.paragon-styles .custom-control-label::after {
  position: absolute;
  top: 0.25rem;
  left: -1.5rem;
  display: block;
  width: 1rem;
  height: 1rem;
  content: "";
  background: 50%/50% 50% no-repeat;
}
.paragon-styles .custom-checkbox .custom-control-label::before {
  border-radius: 0.3rem;
}
.paragon-styles .custom-checkbox .custom-control-input:checked ~ .custom-control-label::after {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='white' d='M6.564.75l-3.59 3.612-1.538-1.55L0 4.26 2.974 7.25 8 2.193z'/%3e%3c/svg%3e");
}
.paragon-styles .custom-checkbox .custom-control-input:indeterminate ~ .custom-control-label::before {
  border-color: #80a9cc;
  background-color: #80a9cc;
}
.paragon-styles .custom-checkbox .custom-control-input:indeterminate ~ .custom-control-label::after {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 4'%3e%3cpath stroke='white' d='M0 2h4'/%3e%3c/svg%3e");
}
.paragon-styles .custom-checkbox .custom-control-input:disabled:checked ~ .custom-control-label::before {
  background-color: rgba(128, 169, 204, 0.5);
}
.paragon-styles .custom-checkbox .custom-control-input:disabled:indeterminate ~ .custom-control-label::before {
  background-color: rgba(128, 169, 204, 0.5);
}
.paragon-styles .custom-radio .custom-control-label::before {
  border-radius: 50%;
}
.paragon-styles .custom-radio .custom-control-input:checked ~ .custom-control-label::after {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='white'/%3e%3c/svg%3e");
}
.paragon-styles .custom-radio .custom-control-input:disabled:checked ~ .custom-control-label::before {
  background-color: rgba(128, 169, 204, 0.5);
}
.paragon-styles .custom-switch {
  padding-left: 2.25rem;
}
.paragon-styles .custom-switch .custom-control-label::before {
  left: -2.25rem;
  width: 1.75rem;
  pointer-events: all;
  border-radius: 0.5rem;
}
.paragon-styles .custom-switch .custom-control-label::after {
  top: calc(0.25rem + 2px);
  left: calc(-2.25rem + 2px);
  width: calc(1rem - 4px);
  height: calc(1rem - 4px);
  background-color: #e9ecef;
  border-radius: 0.5rem;
  transition: transform 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .custom-switch .custom-control-label::after {
    transition: none;
  }
}
.paragon-styles .custom-switch .custom-control-input:checked ~ .custom-control-label::after {
  background-color: white;
  transform: translateX(0.75rem);
}
.paragon-styles .custom-switch .custom-control-input:disabled:checked ~ .custom-control-label::before {
  background-color: rgba(128, 169, 204, 0.5);
}
.paragon-styles .custom-select {
  display: inline-block;
  width: 100%;
  height: calc(1.5em + 0.75rem + 2px);
  padding: 0.375rem 1.75rem 0.375rem 0.75rem;
  font-size: 1rem;
  font-weight: 400;
  line-height: 1.5;
  color: #495057;
  vertical-align: middle;
  background: white url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 5'%3e%3cpath fill='%23495057' d='M2 0L0 2h4zm0 5L0 3h4z'/%3e%3c/svg%3e") no-repeat right 0.75rem center/8px 10px;
  border: 1px solid #e9ecef;
  border-radius: 0.3rem;
  appearance: none;
}
.paragon-styles .custom-select:focus {
  border-color: #dbe7f1;
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .custom-select:focus::-ms-value {
  color: #495057;
  background-color: white;
}
.paragon-styles .custom-select[multiple], .paragon-styles .custom-select[size]:not([size="1"]) {
  height: auto;
  padding-right: 0.75rem;
  background-image: none;
}
.paragon-styles .custom-select:disabled {
  color: #adb5bd;
  background-color: #f8f9fa;
}
.paragon-styles .custom-select::-ms-expand {
  display: none;
}
.paragon-styles .custom-select:-moz-focusring {
  color: transparent;
  text-shadow: 0 0 0 #495057;
}
.paragon-styles .custom-select-sm {
  height: calc(1.5em + 0.5rem + 2px);
  padding-top: 0.25rem;
  padding-bottom: 0.25rem;
  padding-left: 0.5rem;
  font-size: 0.875rem;
}
.paragon-styles .custom-select-lg {
  height: calc(1.5em + 1rem + 2px);
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
  padding-left: 1rem;
  font-size: 1.25rem;
}
.paragon-styles .custom-file {
  position: relative;
  display: inline-block;
  width: 100%;
  height: calc(1.5em + 0.75rem + 2px);
  margin-bottom: 0;
}
.paragon-styles .custom-file-input {
  position: relative;
  z-index: 2;
  width: 100%;
  height: calc(1.5em + 0.75rem + 2px);
  margin: 0;
  overflow: hidden;
  opacity: 0;
}
.paragon-styles .custom-file-input:focus ~ .custom-file-label {
  border-color: #dbe7f1;
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .custom-file-input[disabled] ~ .custom-file-label, .paragon-styles .custom-file-input:disabled ~ .custom-file-label {
  background-color: #f8f9fa;
}
.paragon-styles .custom-file-input:lang(en) ~ .custom-file-label::after {
  content: "Browse";
}
.paragon-styles .custom-file-input ~ .custom-file-label[data-browse]::after {
  content: attr(data-browse);
}
.paragon-styles .custom-file-label {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  z-index: 1;
  height: calc(1.5em + 0.75rem + 2px);
  padding: 0.375rem 0.75rem;
  overflow: hidden;
  font-weight: 400;
  line-height: 1.5;
  color: #495057;
  background-color: white;
  border: 1px solid #e9ecef;
  border-radius: 0.3rem;
}
.paragon-styles .custom-file-label::after {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  z-index: 3;
  display: block;
  height: calc(1.5em + 0.75rem);
  padding: 0.375rem 0.75rem;
  line-height: 1.5;
  color: #495057;
  content: "Browse";
  background-color: #f8f9fa;
  border-left: inherit;
  border-radius: 0 0.3rem 0.3rem 0;
}
.paragon-styles .custom-range {
  width: 100%;
  height: 1.4rem;
  padding: 0;
  background-color: transparent;
  appearance: none;
}
.paragon-styles .custom-range:focus {
  outline: 0;
}
.paragon-styles .custom-range:focus::-webkit-slider-thumb {
  box-shadow: 0 0 0 1px white, 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .custom-range:focus::-moz-range-thumb {
  box-shadow: 0 0 0 1px white, 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .custom-range:focus::-ms-thumb {
  box-shadow: 0 0 0 1px white, 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .custom-range::-moz-focus-outer {
  border: 0;
}
.paragon-styles .custom-range::-webkit-slider-thumb {
  width: 1rem;
  height: 1rem;
  margin-top: -0.25rem;
  background-color: #80a9cc;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
  appearance: none;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .custom-range::-webkit-slider-thumb {
    transition: none;
  }
}
.paragon-styles .custom-range::-webkit-slider-thumb:active {
  background-color: white;
}
.paragon-styles .custom-range::-webkit-slider-runnable-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: #dee2e6;
  border-color: transparent;
  border-radius: 1rem;
}
.paragon-styles .custom-range::-moz-range-thumb {
  width: 1rem;
  height: 1rem;
  background-color: #80a9cc;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
  appearance: none;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .custom-range::-moz-range-thumb {
    transition: none;
  }
}
.paragon-styles .custom-range::-moz-range-thumb:active {
  background-color: white;
}
.paragon-styles .custom-range::-moz-range-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: #dee2e6;
  border-color: transparent;
  border-radius: 1rem;
}
.paragon-styles .custom-range::-ms-thumb {
  width: 1rem;
  height: 1rem;
  margin-top: 0;
  margin-right: 0.2rem;
  margin-left: 0.2rem;
  background-color: #80a9cc;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
  appearance: none;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .custom-range::-ms-thumb {
    transition: none;
  }
}
.paragon-styles .custom-range::-ms-thumb:active {
  background-color: white;
}
.paragon-styles .custom-range::-ms-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: transparent;
  border-color: transparent;
  border-width: 0.5rem;
}
.paragon-styles .custom-range::-ms-fill-lower {
  background-color: #dee2e6;
  border-radius: 1rem;
}
.paragon-styles .custom-range::-ms-fill-upper {
  margin-right: 15px;
  background-color: #dee2e6;
  border-radius: 1rem;
}
.paragon-styles .custom-range:disabled::-webkit-slider-thumb {
  background-color: #adb5bd;
}
.paragon-styles .custom-range:disabled::-webkit-slider-runnable-track {
  cursor: default;
}
.paragon-styles .custom-range:disabled::-moz-range-thumb {
  background-color: #adb5bd;
}
.paragon-styles .custom-range:disabled::-moz-range-track {
  cursor: default;
}
.paragon-styles .custom-range:disabled::-ms-thumb {
  background-color: #adb5bd;
}
.paragon-styles .custom-control-label::before,
.paragon-styles .custom-file-label,
.paragon-styles .custom-select {
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .custom-control-label::before,
.paragon-styles .custom-file-label,
.paragon-styles .custom-select {
    transition: none;
  }
}
.paragon-styles .nav {
  display: flex;
  flex-wrap: wrap;
  padding-left: 0;
  margin-bottom: 0;
  list-style: none;
}
.paragon-styles .nav-link {
  display: block;
  padding: 0.5rem 1rem;
}
.paragon-styles .nav-link:hover, .paragon-styles .nav-link:focus {
  text-decoration: none;
}
.paragon-styles .nav-link.disabled {
  color: #adb5bd;
  pointer-events: none;
  cursor: default;
}
.paragon-styles .nav-tabs {
  border-bottom: 1px solid #e9ecef;
}
.paragon-styles .nav-tabs .nav-link {
  margin-bottom: -1px;
  border: 1px solid transparent;
  border-top-left-radius: 0.3rem;
  border-top-right-radius: 0.3rem;
}
.paragon-styles .nav-tabs .nav-link:hover, .paragon-styles .nav-tabs .nav-link:focus {
  border-color: #e9ecef #e9ecef #e9ecef;
}
.paragon-styles .nav-tabs .nav-link.disabled {
  color: #adb5bd;
  background-color: transparent;
  border-color: transparent;
}
.paragon-styles .nav-tabs .nav-link.active,
.paragon-styles .nav-tabs .nav-item.show .nav-link {
  color: #495057;
  background-color: white;
  border-color: #dee2e6 #dee2e6 white;
}
.paragon-styles .nav-tabs .dropdown-menu {
  margin-top: -1px;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}
.paragon-styles .nav-pills .nav-link {
  border-radius: 0.3rem;
}
.paragon-styles .nav-pills .nav-link.active,
.paragon-styles .nav-pills .show > .nav-link {
  color: white;
  background-color: #80a9cc;
}
.paragon-styles .nav-fill > .nav-link,
.paragon-styles .nav-fill .nav-item {
  flex: 1 1 auto;
  text-align: center;
}
.paragon-styles .nav-justified > .nav-link,
.paragon-styles .nav-justified .nav-item {
  flex-basis: 0;
  flex-grow: 1;
  text-align: center;
}
.paragon-styles .tab-content > .tab-pane {
  display: none;
}
.paragon-styles .tab-content > .active {
  display: block;
}
.paragon-styles .navbar {
  position: relative;
  display: flex;
  flex-wrap: wrap;
  align-items: center;
  justify-content: space-between;
  padding: 0.5rem 1rem;
}
.paragon-styles .navbar .container,
.paragon-styles .navbar .container-fluid,
.paragon-styles .navbar .container-sm,
.paragon-styles .navbar .container-md,
.paragon-styles .navbar .container-lg,
.paragon-styles .navbar .container-xl {
  display: flex;
  flex-wrap: wrap;
  align-items: center;
  justify-content: space-between;
}
.paragon-styles .navbar-brand {
  display: inline-block;
  padding-top: 0.3125rem;
  padding-bottom: 0.3125rem;
  margin-right: 1rem;
  font-size: 1.25rem;
  line-height: inherit;
  white-space: nowrap;
}
.paragon-styles .navbar-brand:hover, .paragon-styles .navbar-brand:focus {
  text-decoration: none;
}
.paragon-styles .navbar-nav {
  display: flex;
  flex-direction: column;
  padding-left: 0;
  margin-bottom: 0;
  list-style: none;
}
.paragon-styles .navbar-nav .nav-link {
  padding-right: 0;
  padding-left: 0;
}
.paragon-styles .navbar-nav .dropdown-menu {
  position: static;
  float: none;
}
.paragon-styles .navbar-text {
  display: inline-block;
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
}
.paragon-styles .navbar-collapse {
  flex-basis: 100%;
  flex-grow: 1;
  align-items: center;
}
.paragon-styles .navbar-toggler {
  padding: 0.25rem 0.75rem;
  font-size: 1.25rem;
  line-height: 1;
  background-color: transparent;
  border: 1px solid transparent;
  border-radius: 0.3rem;
}
.paragon-styles .navbar-toggler:hover, .paragon-styles .navbar-toggler:focus {
  text-decoration: none;
}
.paragon-styles .navbar-toggler-icon {
  display: inline-block;
  width: 1.5em;
  height: 1.5em;
  vertical-align: middle;
  content: "";
  background: 50%/100% 100% no-repeat;
}
.paragon-styles .navbar-nav-scroll {
  max-height: 75vh;
  overflow-y: auto;
}
@media (max-width: 575.98px) {
  .paragon-styles .navbar-expand-sm > .container,
.paragon-styles .navbar-expand-sm > .container-fluid,
.paragon-styles .navbar-expand-sm > .container-sm,
.paragon-styles .navbar-expand-sm > .container-md,
.paragon-styles .navbar-expand-sm > .container-lg,
.paragon-styles .navbar-expand-sm > .container-xl {
    padding-right: 0;
    padding-left: 0;
  }
}
@media (min-width: 576px) {
  .paragon-styles .navbar-expand-sm {
    flex-flow: row nowrap;
    justify-content: flex-start;
  }
  .paragon-styles .navbar-expand-sm .navbar-nav {
    flex-direction: row;
  }
  .paragon-styles .navbar-expand-sm .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .paragon-styles .navbar-expand-sm .navbar-nav .nav-link {
    padding-right: 0.5rem;
    padding-left: 0.5rem;
  }
  .paragon-styles .navbar-expand-sm > .container,
.paragon-styles .navbar-expand-sm > .container-fluid,
.paragon-styles .navbar-expand-sm > .container-sm,
.paragon-styles .navbar-expand-sm > .container-md,
.paragon-styles .navbar-expand-sm > .container-lg,
.paragon-styles .navbar-expand-sm > .container-xl {
    flex-wrap: nowrap;
  }
  .paragon-styles .navbar-expand-sm .navbar-nav-scroll {
    overflow: visible;
  }
  .paragon-styles .navbar-expand-sm .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .paragon-styles .navbar-expand-sm .navbar-toggler {
    display: none;
  }
}
@media (max-width: 767.98px) {
  .paragon-styles .navbar-expand-md > .container,
.paragon-styles .navbar-expand-md > .container-fluid,
.paragon-styles .navbar-expand-md > .container-sm,
.paragon-styles .navbar-expand-md > .container-md,
.paragon-styles .navbar-expand-md > .container-lg,
.paragon-styles .navbar-expand-md > .container-xl {
    padding-right: 0;
    padding-left: 0;
  }
}
@media (min-width: 768px) {
  .paragon-styles .navbar-expand-md {
    flex-flow: row nowrap;
    justify-content: flex-start;
  }
  .paragon-styles .navbar-expand-md .navbar-nav {
    flex-direction: row;
  }
  .paragon-styles .navbar-expand-md .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .paragon-styles .navbar-expand-md .navbar-nav .nav-link {
    padding-right: 0.5rem;
    padding-left: 0.5rem;
  }
  .paragon-styles .navbar-expand-md > .container,
.paragon-styles .navbar-expand-md > .container-fluid,
.paragon-styles .navbar-expand-md > .container-sm,
.paragon-styles .navbar-expand-md > .container-md,
.paragon-styles .navbar-expand-md > .container-lg,
.paragon-styles .navbar-expand-md > .container-xl {
    flex-wrap: nowrap;
  }
  .paragon-styles .navbar-expand-md .navbar-nav-scroll {
    overflow: visible;
  }
  .paragon-styles .navbar-expand-md .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .paragon-styles .navbar-expand-md .navbar-toggler {
    display: none;
  }
}
@media (max-width: 991.98px) {
  .paragon-styles .navbar-expand-lg > .container,
.paragon-styles .navbar-expand-lg > .container-fluid,
.paragon-styles .navbar-expand-lg > .container-sm,
.paragon-styles .navbar-expand-lg > .container-md,
.paragon-styles .navbar-expand-lg > .container-lg,
.paragon-styles .navbar-expand-lg > .container-xl {
    padding-right: 0;
    padding-left: 0;
  }
}
@media (min-width: 992px) {
  .paragon-styles .navbar-expand-lg {
    flex-flow: row nowrap;
    justify-content: flex-start;
  }
  .paragon-styles .navbar-expand-lg .navbar-nav {
    flex-direction: row;
  }
  .paragon-styles .navbar-expand-lg .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .paragon-styles .navbar-expand-lg .navbar-nav .nav-link {
    padding-right: 0.5rem;
    padding-left: 0.5rem;
  }
  .paragon-styles .navbar-expand-lg > .container,
.paragon-styles .navbar-expand-lg > .container-fluid,
.paragon-styles .navbar-expand-lg > .container-sm,
.paragon-styles .navbar-expand-lg > .container-md,
.paragon-styles .navbar-expand-lg > .container-lg,
.paragon-styles .navbar-expand-lg > .container-xl {
    flex-wrap: nowrap;
  }
  .paragon-styles .navbar-expand-lg .navbar-nav-scroll {
    overflow: visible;
  }
  .paragon-styles .navbar-expand-lg .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .paragon-styles .navbar-expand-lg .navbar-toggler {
    display: none;
  }
}
@media (max-width: 1199.98px) {
  .paragon-styles .navbar-expand-xl > .container,
.paragon-styles .navbar-expand-xl > .container-fluid,
.paragon-styles .navbar-expand-xl > .container-sm,
.paragon-styles .navbar-expand-xl > .container-md,
.paragon-styles .navbar-expand-xl > .container-lg,
.paragon-styles .navbar-expand-xl > .container-xl {
    padding-right: 0;
    padding-left: 0;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .navbar-expand-xl {
    flex-flow: row nowrap;
    justify-content: flex-start;
  }
  .paragon-styles .navbar-expand-xl .navbar-nav {
    flex-direction: row;
  }
  .paragon-styles .navbar-expand-xl .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .paragon-styles .navbar-expand-xl .navbar-nav .nav-link {
    padding-right: 0.5rem;
    padding-left: 0.5rem;
  }
  .paragon-styles .navbar-expand-xl > .container,
.paragon-styles .navbar-expand-xl > .container-fluid,
.paragon-styles .navbar-expand-xl > .container-sm,
.paragon-styles .navbar-expand-xl > .container-md,
.paragon-styles .navbar-expand-xl > .container-lg,
.paragon-styles .navbar-expand-xl > .container-xl {
    flex-wrap: nowrap;
  }
  .paragon-styles .navbar-expand-xl .navbar-nav-scroll {
    overflow: visible;
  }
  .paragon-styles .navbar-expand-xl .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .paragon-styles .navbar-expand-xl .navbar-toggler {
    display: none;
  }
}
.paragon-styles .navbar-expand {
  flex-flow: row nowrap;
  justify-content: flex-start;
}
.paragon-styles .navbar-expand > .container,
.paragon-styles .navbar-expand > .container-fluid,
.paragon-styles .navbar-expand > .container-sm,
.paragon-styles .navbar-expand > .container-md,
.paragon-styles .navbar-expand > .container-lg,
.paragon-styles .navbar-expand > .container-xl {
  padding-right: 0;
  padding-left: 0;
}
.paragon-styles .navbar-expand .navbar-nav {
  flex-direction: row;
}
.paragon-styles .navbar-expand .navbar-nav .dropdown-menu {
  position: absolute;
}
.paragon-styles .navbar-expand .navbar-nav .nav-link {
  padding-right: 0.5rem;
  padding-left: 0.5rem;
}
.paragon-styles .navbar-expand > .container,
.paragon-styles .navbar-expand > .container-fluid,
.paragon-styles .navbar-expand > .container-sm,
.paragon-styles .navbar-expand > .container-md,
.paragon-styles .navbar-expand > .container-lg,
.paragon-styles .navbar-expand > .container-xl {
  flex-wrap: nowrap;
}
.paragon-styles .navbar-expand .navbar-nav-scroll {
  overflow: visible;
}
.paragon-styles .navbar-expand .navbar-collapse {
  display: flex !important;
  flex-basis: auto;
}
.paragon-styles .navbar-expand .navbar-toggler {
  display: none;
}
.paragon-styles .navbar-light .navbar-brand {
  color: rgba(3, 3, 3, 0.9);
}
.paragon-styles .navbar-light .navbar-brand:hover, .paragon-styles .navbar-light .navbar-brand:focus {
  color: rgba(3, 3, 3, 0.9);
}
.paragon-styles .navbar-light .navbar-nav .nav-link {
  color: rgba(3, 3, 3, 0.5);
}
.paragon-styles .navbar-light .navbar-nav .nav-link:hover, .paragon-styles .navbar-light .navbar-nav .nav-link:focus {
  color: rgba(3, 3, 3, 0.7);
}
.paragon-styles .navbar-light .navbar-nav .nav-link.disabled {
  color: rgba(3, 3, 3, 0.3);
}
.paragon-styles .navbar-light .navbar-nav .show > .nav-link,
.paragon-styles .navbar-light .navbar-nav .active > .nav-link,
.paragon-styles .navbar-light .navbar-nav .nav-link.show,
.paragon-styles .navbar-light .navbar-nav .nav-link.active {
  color: rgba(3, 3, 3, 0.9);
}
.paragon-styles .navbar-light .navbar-toggler {
  color: rgba(3, 3, 3, 0.5);
  border-color: rgba(3, 3, 3, 0.1);
}
.paragon-styles .navbar-light .navbar-toggler-icon {
  background-image: url("data:image/svg+xml,%3csvg viewBox='0 0 30 30' xmlns='http://www.w3.org/2000/svg'%3e%3cpath stroke='rgba%283, 3, 3, 0.5%29' stroke-width='2' stroke-linecap='round' stroke-miterlimit='10' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
}
.paragon-styles .navbar-light .navbar-text {
  color: rgba(3, 3, 3, 0.5);
}
.paragon-styles .navbar-light .navbar-text a {
  color: rgba(3, 3, 3, 0.9);
}
.paragon-styles .navbar-light .navbar-text a:hover, .paragon-styles .navbar-light .navbar-text a:focus {
  color: rgba(3, 3, 3, 0.9);
}
.paragon-styles .navbar-dark .navbar-brand {
  color: white;
}
.paragon-styles .navbar-dark .navbar-brand:hover, .paragon-styles .navbar-dark .navbar-brand:focus {
  color: white;
}
.paragon-styles .navbar-dark .navbar-nav .nav-link {
  color: rgba(255, 255, 255, 0.5);
}
.paragon-styles .navbar-dark .navbar-nav .nav-link:hover, .paragon-styles .navbar-dark .navbar-nav .nav-link:focus {
  color: rgba(255, 255, 255, 0.75);
}
.paragon-styles .navbar-dark .navbar-nav .nav-link.disabled {
  color: rgba(255, 255, 255, 0.25);
}
.paragon-styles .navbar-dark .navbar-nav .show > .nav-link,
.paragon-styles .navbar-dark .navbar-nav .active > .nav-link,
.paragon-styles .navbar-dark .navbar-nav .nav-link.show,
.paragon-styles .navbar-dark .navbar-nav .nav-link.active {
  color: white;
}
.paragon-styles .navbar-dark .navbar-toggler {
  color: rgba(255, 255, 255, 0.5);
  border-color: rgba(255, 255, 255, 0.1);
}
.paragon-styles .navbar-dark .navbar-toggler-icon {
  background-image: url("data:image/svg+xml,%3csvg viewBox='0 0 30 30' xmlns='http://www.w3.org/2000/svg'%3e%3cpath stroke='rgba%28255, 255, 255, 0.5%29' stroke-width='2' stroke-linecap='round' stroke-miterlimit='10' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
}
.paragon-styles .navbar-dark .navbar-text {
  color: rgba(255, 255, 255, 0.5);
}
.paragon-styles .navbar-dark .navbar-text a {
  color: white;
}
.paragon-styles .navbar-dark .navbar-text a:hover, .paragon-styles .navbar-dark .navbar-text a:focus {
  color: white;
}
.paragon-styles .card, .paragon-styles .collapsible-card, .paragon-styles .collapsible-card-lg {
  position: relative;
  display: flex;
  flex-direction: column;
  min-width: 0;
  word-wrap: break-word;
  background-color: white;
  background-clip: border-box;
  border: 1px solid rgba(3, 3, 3, 0.125);
  border-radius: 0.3rem;
}
.paragon-styles .card > hr, .paragon-styles .collapsible-card > hr, .paragon-styles .collapsible-card-lg > hr {
  margin-right: 0;
  margin-left: 0;
}
.paragon-styles .card > .list-group, .paragon-styles .collapsible-card > .list-group, .paragon-styles .collapsible-card-lg > .list-group {
  border-top: inherit;
  border-bottom: inherit;
}
.paragon-styles .card > .list-group:first-child, .paragon-styles .collapsible-card > .list-group:first-child, .paragon-styles .collapsible-card-lg > .list-group:first-child {
  border-top-width: 0;
  border-top-left-radius: calc(0.3rem - 1px);
  border-top-right-radius: calc(0.3rem - 1px);
}
.paragon-styles .card > .list-group:last-child, .paragon-styles .collapsible-card > .list-group:last-child, .paragon-styles .collapsible-card-lg > .list-group:last-child {
  border-bottom-width: 0;
  border-bottom-right-radius: calc(0.3rem - 1px);
  border-bottom-left-radius: calc(0.3rem - 1px);
}
.paragon-styles .card > .card-header + .list-group, .paragon-styles .collapsible-card > .card-header + .list-group, .paragon-styles .collapsible-card-lg > .card-header + .list-group,
.paragon-styles .card > .list-group + .card-footer,
.paragon-styles .collapsible-card > .list-group + .card-footer,
.paragon-styles .collapsible-card-lg > .list-group + .card-footer {
  border-top: 0;
}
.paragon-styles .card-body, .paragon-styles .collapsible-card .collapsible-body, .paragon-styles .collapsible-card-lg .collapsible-body {
  flex: 1 1 auto;
  min-height: 1px;
  padding: 1.25rem;
}
.paragon-styles .card-title {
  margin-bottom: 0.75rem;
}
.paragon-styles .card-subtitle {
  margin-top: -0.375rem;
  margin-bottom: 0;
}
.paragon-styles .card-text:last-child {
  margin-bottom: 0;
}
.paragon-styles .card-link:hover {
  text-decoration: none;
}
.paragon-styles .card-link + .card-link {
  margin-left: 1.25rem;
}
.paragon-styles .card-header {
  padding: 0.75rem 1.25rem;
  margin-bottom: 0;
  background-color: rgba(3, 3, 3, 0.03);
  border-bottom: 1px solid rgba(3, 3, 3, 0.125);
}
.paragon-styles .card-header:first-child {
  border-radius: calc(0.3rem - 1px) calc(0.3rem - 1px) 0 0;
}
.paragon-styles .card-footer {
  padding: 0.75rem 1.25rem;
  background-color: rgba(3, 3, 3, 0.03);
  border-top: 1px solid rgba(3, 3, 3, 0.125);
}
.paragon-styles .card-footer:last-child {
  border-radius: 0 0 calc(0.3rem - 1px) calc(0.3rem - 1px);
}
.paragon-styles .card-header-tabs {
  margin-right: -0.625rem;
  margin-bottom: -0.75rem;
  margin-left: -0.625rem;
  border-bottom: 0;
}
.paragon-styles .card-header-pills {
  margin-right: -0.625rem;
  margin-left: -0.625rem;
}
.paragon-styles .card-img-overlay {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  padding: 1.25rem;
  border-radius: calc(0.3rem - 1px);
}
.paragon-styles .card-img,
.paragon-styles .card-img-top,
.paragon-styles .card-img-bottom {
  flex-shrink: 0;
  width: 100%;
}
.paragon-styles .card-img,
.paragon-styles .card-img-top {
  border-top-left-radius: calc(0.3rem - 1px);
  border-top-right-radius: calc(0.3rem - 1px);
}
.paragon-styles .card-img,
.paragon-styles .card-img-bottom {
  border-bottom-right-radius: calc(0.3rem - 1px);
  border-bottom-left-radius: calc(0.3rem - 1px);
}
.paragon-styles .card-deck .card, .paragon-styles .card-deck .collapsible-card, .paragon-styles .card-deck .collapsible-card-lg {
  margin-bottom: 15px;
}
@media (min-width: 576px) {
  .paragon-styles .card-deck {
    display: flex;
    flex-flow: row wrap;
    margin-right: -15px;
    margin-left: -15px;
  }
  .paragon-styles .card-deck .card, .paragon-styles .card-deck .collapsible-card, .paragon-styles .card-deck .collapsible-card-lg {
    flex: 1 0 0%;
    margin-right: 15px;
    margin-bottom: 0;
    margin-left: 15px;
  }
}
.paragon-styles .card-group > .card, .paragon-styles .card-group > .collapsible-card, .paragon-styles .card-group > .collapsible-card-lg {
  margin-bottom: 15px;
}
@media (min-width: 576px) {
  .paragon-styles .card-group {
    display: flex;
    flex-flow: row wrap;
  }
  .paragon-styles .card-group > .card, .paragon-styles .card-group > .collapsible-card, .paragon-styles .card-group > .collapsible-card-lg {
    flex: 1 0 0%;
    margin-bottom: 0;
  }
  .paragon-styles .card-group > .card + .card, .paragon-styles .card-group > .collapsible-card + .card, .paragon-styles .card-group > .collapsible-card-lg + .card, .paragon-styles .card-group > .card + .collapsible-card, .paragon-styles .card-group > .card + .collapsible-card-lg, .paragon-styles .card-group > .collapsible-card + .collapsible-card, .paragon-styles .card-group > .collapsible-card-lg + .collapsible-card, .paragon-styles .card-group > .collapsible-card + .collapsible-card-lg, .paragon-styles .card-group > .collapsible-card-lg + .collapsible-card-lg {
    margin-left: 0;
    border-left: 0;
  }
  .paragon-styles .card-group > .card:not(:last-child), .paragon-styles .card-group > .collapsible-card:not(:last-child), .paragon-styles .card-group > .collapsible-card-lg:not(:last-child) {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0;
  }
  .paragon-styles .card-group > .card:not(:last-child) .card-img-top, .paragon-styles .card-group > .collapsible-card:not(:last-child) .card-img-top, .paragon-styles .card-group > .collapsible-card-lg:not(:last-child) .card-img-top,
.paragon-styles .card-group > .card:not(:last-child) .card-header,
.paragon-styles .card-group > .collapsible-card:not(:last-child) .card-header,
.paragon-styles .card-group > .collapsible-card-lg:not(:last-child) .card-header {
    border-top-right-radius: 0;
  }
  .paragon-styles .card-group > .card:not(:last-child) .card-img-bottom, .paragon-styles .card-group > .collapsible-card:not(:last-child) .card-img-bottom, .paragon-styles .card-group > .collapsible-card-lg:not(:last-child) .card-img-bottom,
.paragon-styles .card-group > .card:not(:last-child) .card-footer,
.paragon-styles .card-group > .collapsible-card:not(:last-child) .card-footer,
.paragon-styles .card-group > .collapsible-card-lg:not(:last-child) .card-footer {
    border-bottom-right-radius: 0;
  }
  .paragon-styles .card-group > .card:not(:first-child), .paragon-styles .card-group > .collapsible-card:not(:first-child), .paragon-styles .card-group > .collapsible-card-lg:not(:first-child) {
    border-top-left-radius: 0;
    border-bottom-left-radius: 0;
  }
  .paragon-styles .card-group > .card:not(:first-child) .card-img-top, .paragon-styles .card-group > .collapsible-card:not(:first-child) .card-img-top, .paragon-styles .card-group > .collapsible-card-lg:not(:first-child) .card-img-top,
.paragon-styles .card-group > .card:not(:first-child) .card-header,
.paragon-styles .card-group > .collapsible-card:not(:first-child) .card-header,
.paragon-styles .card-group > .collapsible-card-lg:not(:first-child) .card-header {
    border-top-left-radius: 0;
  }
  .paragon-styles .card-group > .card:not(:first-child) .card-img-bottom, .paragon-styles .card-group > .collapsible-card:not(:first-child) .card-img-bottom, .paragon-styles .card-group > .collapsible-card-lg:not(:first-child) .card-img-bottom,
.paragon-styles .card-group > .card:not(:first-child) .card-footer,
.paragon-styles .card-group > .collapsible-card:not(:first-child) .card-footer,
.paragon-styles .card-group > .collapsible-card-lg:not(:first-child) .card-footer {
    border-bottom-left-radius: 0;
  }
}
.paragon-styles .card-columns .card, .paragon-styles .card-columns .collapsible-card, .paragon-styles .card-columns .collapsible-card-lg {
  margin-bottom: 0.75rem;
}
@media (min-width: 576px) {
  .paragon-styles .card-columns {
    column-count: 3;
    column-gap: 1.25rem;
    orphans: 1;
    widows: 1;
  }
  .paragon-styles .card-columns .card, .paragon-styles .card-columns .collapsible-card, .paragon-styles .card-columns .collapsible-card-lg {
    display: inline-block;
    width: 100%;
  }
}
.paragon-styles .accordion {
  overflow-anchor: none;
}
.paragon-styles .accordion > .card, .paragon-styles .accordion > .collapsible-card, .paragon-styles .accordion > .collapsible-card-lg {
  overflow: hidden;
}
.paragon-styles .accordion > .card:not(:last-of-type), .paragon-styles .accordion > .collapsible-card:not(:last-of-type), .paragon-styles .accordion > .collapsible-card-lg:not(:last-of-type) {
  border-bottom: 0;
  border-bottom-right-radius: 0;
  border-bottom-left-radius: 0;
}
.paragon-styles .accordion > .card:not(:first-of-type), .paragon-styles .accordion > .collapsible-card:not(:first-of-type), .paragon-styles .accordion > .collapsible-card-lg:not(:first-of-type) {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}
.paragon-styles .accordion > .card > .card-header, .paragon-styles .accordion > .collapsible-card > .card-header, .paragon-styles .accordion > .collapsible-card-lg > .card-header {
  border-radius: 0;
  margin-bottom: -1px;
}
.paragon-styles .breadcrumb {
  display: flex;
  flex-wrap: wrap;
  padding: 0.75rem 1rem;
  margin-bottom: 1rem;
  list-style: none;
  background-color: #f8f9fa;
  border-radius: 0.3rem;
}
.paragon-styles .breadcrumb-item + .breadcrumb-item {
  padding-left: 0.5rem;
}
.paragon-styles .breadcrumb-item + .breadcrumb-item::before {
  float: left;
  padding-right: 0.5rem;
  color: #dee2e6;
  content: "/";
}
.paragon-styles .breadcrumb-item + .breadcrumb-item:hover::before {
  text-decoration: underline;
}
.paragon-styles .breadcrumb-item + .breadcrumb-item:hover::before {
  text-decoration: none;
}
.paragon-styles .breadcrumb-item.active {
  color: #adb5bd;
}
.paragon-styles .pagination {
  display: flex;
  padding-left: 0;
  list-style: none;
  border-radius: 0.3rem;
}
.paragon-styles .page-link {
  position: relative;
  display: block;
  padding: 0.5rem 0.75rem;
  margin-left: -1px;
  line-height: 1.25;
  color: #80a9cc;
  background-color: white;
  border: 1px solid #e9ecef;
}
.paragon-styles .page-link:hover {
  z-index: 2;
  color: #4984b6;
  text-decoration: none;
  background-color: #f8f9fa;
  border-color: #e9ecef;
}
.paragon-styles .page-link:focus {
  z-index: 3;
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .page-item:first-child .page-link {
  margin-left: 0;
  border-top-left-radius: 0.3rem;
  border-bottom-left-radius: 0.3rem;
}
.paragon-styles .page-item:last-child .page-link {
  border-top-right-radius: 0.3rem;
  border-bottom-right-radius: 0.3rem;
}
.paragon-styles .page-item.active .page-link {
  z-index: 3;
  color: white;
  background-color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .page-item.disabled .page-link {
  color: #adb5bd;
  pointer-events: none;
  cursor: auto;
  background-color: white;
  border-color: #f8f9fa;
}
.paragon-styles .pagination-lg .page-link {
  padding: 0.75rem 1.5rem;
  font-size: 1.25rem;
  line-height: 1.5;
}
.paragon-styles .pagination-lg .page-item:first-child .page-link {
  border-top-left-radius: 0.35rem;
  border-bottom-left-radius: 0.35rem;
}
.paragon-styles .pagination-lg .page-item:last-child .page-link {
  border-top-right-radius: 0.35rem;
  border-bottom-right-radius: 0.35rem;
}
.paragon-styles .pagination-sm .page-link {
  padding: 0.25rem 0.5rem;
  font-size: 0.875rem;
  line-height: 1.5;
}
.paragon-styles .pagination-sm .page-item:first-child .page-link {
  border-top-left-radius: 0.2rem;
  border-bottom-left-radius: 0.2rem;
}
.paragon-styles .pagination-sm .page-item:last-child .page-link {
  border-top-right-radius: 0.2rem;
  border-bottom-right-radius: 0.2rem;
}
.paragon-styles .badge {
  display: inline-block;
  padding: 0.25em 0.4em;
  font-size: 75%;
  font-weight: 700;
  line-height: 1;
  text-align: center;
  white-space: nowrap;
  vertical-align: baseline;
  border-radius: 0.3rem;
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .badge {
    transition: none;
  }
}
a.paragon-styles .badge:hover, a.paragon-styles .badge:focus {
  text-decoration: none;
}

.paragon-styles .badge:empty {
  display: none;
}
.paragon-styles .btn .badge {
  position: relative;
  top: -1px;
}
.paragon-styles .badge-pill {
  padding-right: 0.6em;
  padding-left: 0.6em;
  border-radius: 10rem;
}
.paragon-styles .badge-primary {
  color: #495057;
  background-color: #80a9cc;
}
a.paragon-styles .badge-primary:hover, a.paragon-styles .badge-primary:focus {
  color: #495057;
  background-color: #5c90bd;
}
a.paragon-styles .badge-primary:focus, a.paragon-styles .badge-primary.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.5);
}

.paragon-styles .badge-secondary {
  color: white;
  background-color: #6c757d;
}
a.paragon-styles .badge-secondary:hover, a.paragon-styles .badge-secondary:focus {
  color: white;
  background-color: #545b62;
}
a.paragon-styles .badge-secondary:focus, a.paragon-styles .badge-secondary.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(108, 117, 125, 0.5);
}

.paragon-styles .badge-success {
  color: white;
  background-color: #178253;
}
a.paragon-styles .badge-success:hover, a.paragon-styles .badge-success:focus {
  color: white;
  background-color: #0f5737;
}
a.paragon-styles .badge-success:focus, a.paragon-styles .badge-success.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(23, 130, 83, 0.5);
}

.paragon-styles .badge-info {
  color: white;
  background-color: #006DAA;
}
a.paragon-styles .badge-info:hover, a.paragon-styles .badge-info:focus {
  color: white;
  background-color: #004c77;
}
a.paragon-styles .badge-info:focus, a.paragon-styles .badge-info.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(0, 109, 170, 0.5);
}

.paragon-styles .badge-warning {
  color: #495057;
  background-color: #D69600;
}
a.paragon-styles .badge-warning:hover, a.paragon-styles .badge-warning:focus {
  color: #495057;
  background-color: #a37200;
}
a.paragon-styles .badge-warning:focus, a.paragon-styles .badge-warning.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(214, 150, 0, 0.5);
}

.paragon-styles .badge-danger {
  color: white;
  background-color: #C32D3A;
}
a.paragon-styles .badge-danger:hover, a.paragon-styles .badge-danger:focus {
  color: white;
  background-color: #9a232e;
}
a.paragon-styles .badge-danger:focus, a.paragon-styles .badge-danger.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(195, 45, 58, 0.5);
}

.paragon-styles .badge-light {
  color: #495057;
  background-color: #eee;
}
a.paragon-styles .badge-light:hover, a.paragon-styles .badge-light:focus {
  color: #495057;
  background-color: #d5d5d5;
}
a.paragon-styles .badge-light:focus, a.paragon-styles .badge-light.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(238, 238, 238, 0.5);
}

.paragon-styles .badge-dark {
  color: white;
  background-color: #495057;
}
a.paragon-styles .badge-dark:hover, a.paragon-styles .badge-dark:focus {
  color: white;
  background-color: #32373b;
}
a.paragon-styles .badge-dark:focus, a.paragon-styles .badge-dark.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(73, 80, 87, 0.5);
}

.paragon-styles .badge-brand {
  color: #495057;
  background-color: #80a9cc;
}
a.paragon-styles .badge-brand:hover, a.paragon-styles .badge-brand:focus {
  color: #495057;
  background-color: #5c90bd;
}
a.paragon-styles .badge-brand:focus, a.paragon-styles .badge-brand.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.5);
}

.paragon-styles .badge-gray {
  color: #495057;
  background-color: #adb5bd;
}
a.paragon-styles .badge-gray:hover, a.paragon-styles .badge-gray:focus {
  color: #495057;
  background-color: #919ca6;
}
a.paragon-styles .badge-gray:focus, a.paragon-styles .badge-gray.focus {
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(173, 181, 189, 0.5);
}

.paragon-styles .jumbotron {
  padding: 2rem 1rem;
  margin-bottom: 2rem;
  background-color: #f8f9fa;
  border-radius: 0.35rem;
}
@media (min-width: 576px) {
  .paragon-styles .jumbotron {
    padding: 4rem 2rem;
  }
}
.paragon-styles .jumbotron-fluid {
  padding-right: 0;
  padding-left: 0;
  border-radius: 0;
}
.paragon-styles .alert {
  position: relative;
  padding: 0.75rem 1.25rem;
  margin-bottom: 1rem;
  border: 1px solid transparent;
  border-radius: 0.3rem;
}
.paragon-styles .alert-heading {
  color: inherit;
}
.paragon-styles .alert-link {
  font-weight: 700;
}
.paragon-styles .alert-dismissible {
  padding-right: 4rem;
}
.paragon-styles .alert-dismissible .close {
  position: absolute;
  top: 0;
  right: 0;
  padding: 0.75rem 1.25rem;
  color: inherit;
}
.paragon-styles .alert-primary {
  color: #212529;
  background-color: #f7fafc;
  border-color: #dfeaf2;
}
.paragon-styles .alert-primary hr {
  border-top-color: #cddeeb;
}
.paragon-styles .alert-primary .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-secondary {
  color: #212529;
  background-color: #6c757d;
  border-color: #6c757d;
}
.paragon-styles .alert-secondary hr {
  border-top-color: #60686f;
}
.paragon-styles .alert-secondary .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-success {
  color: #212529;
  background-color: #f1f8f5;
  border-color: #c5e0d4;
}
.paragon-styles .alert-success hr {
  border-top-color: #b4d7c8;
}
.paragon-styles .alert-success .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-info {
  color: #212529;
  background-color: #f0f6fa;
  border-color: #bfdbea;
}
.paragon-styles .alert-info hr {
  border-top-color: #acd0e4;
}
.paragon-styles .alert-info .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-warning {
  color: #212529;
  background-color: #fdf9f0;
  border-color: #f5e5bf;
}
.paragon-styles .alert-warning hr {
  border-top-color: #f2dca9;
}
.paragon-styles .alert-warning .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-danger {
  color: #212529;
  background-color: #fbf2f3;
  border-color: #f0cbce;
}
.paragon-styles .alert-danger hr {
  border-top-color: #eab7bb;
}
.paragon-styles .alert-danger .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-light {
  color: #212529;
  background-color: #fefefe;
  border-color: #fbfbfb;
}
.paragon-styles .alert-light hr {
  border-top-color: #eeeeee;
}
.paragon-styles .alert-light .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-dark {
  color: #212529;
  background-color: #f4f5f5;
  border-color: #d2d3d5;
}
.paragon-styles .alert-dark hr {
  border-top-color: #c5c6c9;
}
.paragon-styles .alert-dark .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-brand {
  color: #212529;
  background-color: #f7fafc;
  border-color: #dfeaf2;
}
.paragon-styles .alert-brand hr {
  border-top-color: #cddeeb;
}
.paragon-styles .alert-brand .alert-link {
  color: #0a0c0d;
}
.paragon-styles .alert-gray {
  color: #212529;
  background-color: #f8f9fa;
  border-color: #e9ecef;
}
.paragon-styles .alert-gray hr {
  border-top-color: #dadfe4;
}
.paragon-styles .alert-gray .alert-link {
  color: #0a0c0d;
}
@keyframes progress-bar-stripes {
  from {
    background-position: 1rem 0;
  }
  to {
    background-position: 0 0;
  }
}
.paragon-styles .progress {
  display: flex;
  height: 1rem;
  overflow: hidden;
  line-height: 0;
  font-size: 0.75rem;
  background-color: #f8f9fa;
  border-radius: 0.3rem;
}
.paragon-styles .progress-bar {
  display: flex;
  flex-direction: column;
  justify-content: center;
  overflow: hidden;
  color: white;
  text-align: center;
  white-space: nowrap;
  background-color: #80a9cc;
  transition: width 0.6s ease;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .progress-bar {
    transition: none;
  }
}
.paragon-styles .progress-bar-striped {
  background-image: linear, 45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent;
  background-size: 1rem 1rem;
}
.paragon-styles .progress-bar-animated {
  animation: 1s linear infinite progress-bar-stripes;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .progress-bar-animated {
    animation: none;
  }
}
.paragon-styles .media {
  display: flex;
  align-items: flex-start;
}
.paragon-styles .media-body {
  flex: 1;
}
.paragon-styles .list-group {
  display: flex;
  flex-direction: column;
  padding-left: 0;
  margin-bottom: 0;
  border-radius: 0.3rem;
}
.paragon-styles .list-group-item-action {
  width: 100%;
  color: #495057;
  text-align: inherit;
}
.paragon-styles .list-group-item-action:hover, .paragon-styles .list-group-item-action:focus {
  z-index: 1;
  color: #495057;
  text-decoration: none;
  background-color: #f8f9fa;
}
.paragon-styles .list-group-item-action:active {
  color: #212529;
  background-color: #f8f9fa;
}
.paragon-styles .list-group-item {
  position: relative;
  display: block;
  padding: 0.75rem 1.25rem;
  background-color: white;
  border: 1px solid rgba(3, 3, 3, 0.125);
}
.paragon-styles .list-group-item:first-child {
  border-top-left-radius: inherit;
  border-top-right-radius: inherit;
}
.paragon-styles .list-group-item:last-child {
  border-bottom-right-radius: inherit;
  border-bottom-left-radius: inherit;
}
.paragon-styles .list-group-item.disabled, .paragon-styles .list-group-item:disabled {
  color: #adb5bd;
  pointer-events: none;
  background-color: white;
}
.paragon-styles .list-group-item.active {
  z-index: 2;
  color: white;
  background-color: #80a9cc;
  border-color: #80a9cc;
}
.paragon-styles .list-group-item + .paragon-styles .list-group-item {
  border-top-width: 0;
}
.paragon-styles .list-group-item + .paragon-styles .list-group-item.active {
  margin-top: -1px;
  border-top-width: 1px;
}
.paragon-styles .list-group-horizontal {
  flex-direction: row;
}
.paragon-styles .list-group-horizontal > .list-group-item:first-child {
  border-bottom-left-radius: 0.3rem;
  border-top-right-radius: 0;
}
.paragon-styles .list-group-horizontal > .list-group-item:last-child {
  border-top-right-radius: 0.3rem;
  border-bottom-left-radius: 0;
}
.paragon-styles .list-group-horizontal > .list-group-item.active {
  margin-top: 0;
}
.paragon-styles .list-group-horizontal > .list-group-item + .list-group-item {
  border-top-width: 1px;
  border-left-width: 0;
}
.paragon-styles .list-group-horizontal > .list-group-item + .list-group-item.active {
  margin-left: -1px;
  border-left-width: 1px;
}
@media (min-width: 576px) {
  .paragon-styles .list-group-horizontal-sm {
    flex-direction: row;
  }
  .paragon-styles .list-group-horizontal-sm > .list-group-item:first-child {
    border-bottom-left-radius: 0.3rem;
    border-top-right-radius: 0;
  }
  .paragon-styles .list-group-horizontal-sm > .list-group-item:last-child {
    border-top-right-radius: 0.3rem;
    border-bottom-left-radius: 0;
  }
  .paragon-styles .list-group-horizontal-sm > .list-group-item.active {
    margin-top: 0;
  }
  .paragon-styles .list-group-horizontal-sm > .list-group-item + .list-group-item {
    border-top-width: 1px;
    border-left-width: 0;
  }
  .paragon-styles .list-group-horizontal-sm > .list-group-item + .list-group-item.active {
    margin-left: -1px;
    border-left-width: 1px;
  }
}
@media (min-width: 768px) {
  .paragon-styles .list-group-horizontal-md {
    flex-direction: row;
  }
  .paragon-styles .list-group-horizontal-md > .list-group-item:first-child {
    border-bottom-left-radius: 0.3rem;
    border-top-right-radius: 0;
  }
  .paragon-styles .list-group-horizontal-md > .list-group-item:last-child {
    border-top-right-radius: 0.3rem;
    border-bottom-left-radius: 0;
  }
  .paragon-styles .list-group-horizontal-md > .list-group-item.active {
    margin-top: 0;
  }
  .paragon-styles .list-group-horizontal-md > .list-group-item + .list-group-item {
    border-top-width: 1px;
    border-left-width: 0;
  }
  .paragon-styles .list-group-horizontal-md > .list-group-item + .list-group-item.active {
    margin-left: -1px;
    border-left-width: 1px;
  }
}
@media (min-width: 992px) {
  .paragon-styles .list-group-horizontal-lg {
    flex-direction: row;
  }
  .paragon-styles .list-group-horizontal-lg > .list-group-item:first-child {
    border-bottom-left-radius: 0.3rem;
    border-top-right-radius: 0;
  }
  .paragon-styles .list-group-horizontal-lg > .list-group-item:last-child {
    border-top-right-radius: 0.3rem;
    border-bottom-left-radius: 0;
  }
  .paragon-styles .list-group-horizontal-lg > .list-group-item.active {
    margin-top: 0;
  }
  .paragon-styles .list-group-horizontal-lg > .list-group-item + .list-group-item {
    border-top-width: 1px;
    border-left-width: 0;
  }
  .paragon-styles .list-group-horizontal-lg > .list-group-item + .list-group-item.active {
    margin-left: -1px;
    border-left-width: 1px;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .list-group-horizontal-xl {
    flex-direction: row;
  }
  .paragon-styles .list-group-horizontal-xl > .list-group-item:first-child {
    border-bottom-left-radius: 0.3rem;
    border-top-right-radius: 0;
  }
  .paragon-styles .list-group-horizontal-xl > .list-group-item:last-child {
    border-top-right-radius: 0.3rem;
    border-bottom-left-radius: 0;
  }
  .paragon-styles .list-group-horizontal-xl > .list-group-item.active {
    margin-top: 0;
  }
  .paragon-styles .list-group-horizontal-xl > .list-group-item + .list-group-item {
    border-top-width: 1px;
    border-left-width: 0;
  }
  .paragon-styles .list-group-horizontal-xl > .list-group-item + .list-group-item.active {
    margin-left: -1px;
    border-left-width: 1px;
  }
}
.paragon-styles .list-group-flush {
  border-radius: 0;
}
.paragon-styles .list-group-flush > .list-group-item {
  border-width: 0 0 1px;
}
.paragon-styles .list-group-flush > .list-group-item:last-child {
  border-bottom-width: 0;
}
.paragon-styles .list-group-item-primary {
  color: #44596c;
  background-color: #dbe7f1;
}
.paragon-styles .list-group-item-primary.list-group-item-action:hover, .paragon-styles .list-group-item-primary.list-group-item-action:focus {
  color: #44596c;
  background-color: #c9dbea;
}
.paragon-styles .list-group-item-primary.list-group-item-action.active {
  color: white;
  background-color: #44596c;
  border-color: #44596c;
}
.paragon-styles .list-group-item-secondary {
  color: #3a3e42;
  background-color: #d6d8db;
}
.paragon-styles .list-group-item-secondary.list-group-item-action:hover, .paragon-styles .list-group-item-secondary.list-group-item-action:focus {
  color: #3a3e42;
  background-color: #c8cbcf;
}
.paragon-styles .list-group-item-secondary.list-group-item-action.active {
  color: white;
  background-color: #3a3e42;
  border-color: #3a3e42;
}
.paragon-styles .list-group-item-success {
  color: #0d452d;
  background-color: #bedccf;
}
.paragon-styles .list-group-item-success.list-group-item-action:hover, .paragon-styles .list-group-item-success.list-group-item-action:focus {
  color: #0d452d;
  background-color: #add3c3;
}
.paragon-styles .list-group-item-success.list-group-item-action.active {
  color: white;
  background-color: #0d452d;
  border-color: #0d452d;
}
.paragon-styles .list-group-item-info {
  color: #013a5a;
  background-color: #b8d6e7;
}
.paragon-styles .list-group-item-info.list-group-item-action:hover, .paragon-styles .list-group-item-info.list-group-item-action:focus {
  color: #013a5a;
  background-color: #a5cbe1;
}
.paragon-styles .list-group-item-info.list-group-item-action.active {
  color: white;
  background-color: #013a5a;
  border-color: #013a5a;
}
.paragon-styles .list-group-item-warning {
  color: #714f01;
  background-color: #f4e2b8;
}
.paragon-styles .list-group-item-warning.list-group-item-action:hover, .paragon-styles .list-group-item-warning.list-group-item-action:focus {
  color: #714f01;
  background-color: #f1d9a2;
}
.paragon-styles .list-group-item-warning.list-group-item-action.active {
  color: white;
  background-color: #714f01;
  border-color: #714f01;
}
.paragon-styles .list-group-item-danger {
  color: #671920;
  background-color: #eec4c8;
}
.paragon-styles .list-group-item-danger.list-group-item-action:hover, .paragon-styles .list-group-item-danger.list-group-item-action:focus {
  color: #671920;
  background-color: #e8b0b6;
}
.paragon-styles .list-group-item-danger.list-group-item-action.active {
  color: white;
  background-color: #671920;
  border-color: #671920;
}
.paragon-styles .list-group-item-light {
  color: #7d7d7d;
  background-color: #fafafa;
}
.paragon-styles .list-group-item-light.list-group-item-action:hover, .paragon-styles .list-group-item-light.list-group-item-action:focus {
  color: #7d7d7d;
  background-color: #ededed;
}
.paragon-styles .list-group-item-light.list-group-item-action.active {
  color: white;
  background-color: #7d7d7d;
  border-color: #7d7d7d;
}
.paragon-styles .list-group-item-dark {
  color: #272b2f;
  background-color: #ccced0;
}
.paragon-styles .list-group-item-dark.list-group-item-action:hover, .paragon-styles .list-group-item-dark.list-group-item-action:focus {
  color: #272b2f;
  background-color: #bfc1c4;
}
.paragon-styles .list-group-item-dark.list-group-item-action.active {
  color: white;
  background-color: #272b2f;
  border-color: #272b2f;
}
.paragon-styles .list-group-item-brand {
  color: #44596c;
  background-color: #dbe7f1;
}
.paragon-styles .list-group-item-brand.list-group-item-action:hover, .paragon-styles .list-group-item-brand.list-group-item-action:focus {
  color: #44596c;
  background-color: #c9dbea;
}
.paragon-styles .list-group-item-brand.list-group-item-action.active {
  color: white;
  background-color: #44596c;
  border-color: #44596c;
}
.paragon-styles .list-group-item-gray {
  color: #5b6064;
  background-color: #e8eaed;
}
.paragon-styles .list-group-item-gray.list-group-item-action:hover, .paragon-styles .list-group-item-gray.list-group-item-action:focus {
  color: #5b6064;
  background-color: #dadde2;
}
.paragon-styles .list-group-item-gray.list-group-item-action.active {
  color: white;
  background-color: #5b6064;
  border-color: #5b6064;
}
.paragon-styles .close {
  float: right;
  font-size: 1.5rem;
  font-weight: 700;
  line-height: 1;
  color: #030303;
  text-shadow: 0 1px 0 white;
  opacity: 0.5;
}
.paragon-styles .close:hover {
  color: #030303;
  text-decoration: none;
}
.paragon-styles .close:not(:disabled):not(.disabled):hover, .paragon-styles .close:not(:disabled):not(.disabled):focus {
  opacity: 0.75;
}
.paragon-styles button.close {
  padding: 0;
  background-color: transparent;
  border: 0;
}
.paragon-styles a.close.disabled {
  pointer-events: none;
}
.paragon-styles .toast {
  flex-basis: 400px;
  max-width: 400px;
  font-size: 0.875rem;
  background-color: #495057;
  background-clip: padding-box;
  border: 1px solid rgba(0, 0, 0, 0.1);
  box-shadow: 0.125rem 0.125rem 0.25rem 0 rgba(3, 3, 3, 0.25);
  opacity: 0;
  border-radius: 0.1875rem;
}
.paragon-styles .toast:not(:last-child) {
  margin-bottom: 0.75rem;
}
.paragon-styles .toast.showing {
  opacity: 1;
}
.paragon-styles .toast.show {
  display: block;
  opacity: 1;
}
.paragon-styles .toast.hide {
  display: none;
}
.paragon-styles .toast-header {
  display: flex;
  align-items: center;
  padding: 0.25rem 0.75rem;
  color: white;
  background-color: #495057;
  background-clip: padding-box;
  border-bottom: 1px solid rgba(0, 0, 0, 0.05);
  border-top-left-radius: calc(0.1875rem - 1px);
  border-top-right-radius: calc(0.1875rem - 1px);
}
.paragon-styles .toast-body {
  padding: 0.75rem;
}
.paragon-styles .modal-open {
  overflow: hidden;
}
.paragon-styles .modal-open .modal {
  overflow-x: hidden;
  overflow-y: auto;
}
.paragon-styles .modal {
  position: fixed;
  top: 0;
  left: 0;
  z-index: 1050;
  display: none;
  width: 100%;
  height: 100%;
  overflow: hidden;
  outline: 0;
}
.paragon-styles .modal-dialog {
  position: relative;
  width: auto;
  margin: 0.5rem;
  pointer-events: none;
}
.modal.fade .paragon-styles .modal-dialog {
  transition: transform 0.3s ease-out;
  transform: translate(0, -50px);
}
@media (prefers-reduced-motion: reduce) {
  .modal.fade .paragon-styles .modal-dialog {
    transition: none;
  }
}
.modal.show .paragon-styles .modal-dialog {
  transform: none;
}
.modal.modal-static .paragon-styles .modal-dialog {
  transform: scale(1.02);
}
.paragon-styles .modal-dialog-scrollable {
  display: flex;
  max-height: calc(100% - 1rem);
}
.paragon-styles .modal-dialog-scrollable .modal-content {
  max-height: calc(100vh - 1rem);
  overflow: hidden;
}
.paragon-styles .modal-dialog-scrollable .modal-header,
.paragon-styles .modal-dialog-scrollable .modal-footer {
  flex-shrink: 0;
}
.paragon-styles .modal-dialog-scrollable .modal-body {
  overflow-y: auto;
}
.paragon-styles .modal-dialog-centered {
  display: flex;
  align-items: center;
  min-height: calc(100% - 1rem);
}
.paragon-styles .modal-dialog-centered::before {
  display: block;
  height: calc(100vh - 1rem);
  height: min-content;
  content: "";
}
.paragon-styles .modal-dialog-centered.modal-dialog-scrollable {
  flex-direction: column;
  justify-content: center;
  height: 100%;
}
.paragon-styles .modal-dialog-centered.modal-dialog-scrollable .modal-content {
  max-height: none;
}
.paragon-styles .modal-dialog-centered.modal-dialog-scrollable::before {
  content: none;
}
.paragon-styles .modal-content {
  position: relative;
  display: flex;
  flex-direction: column;
  width: 100%;
  pointer-events: auto;
  background-color: white;
  background-clip: padding-box;
  border: 1px solid rgba(3, 3, 3, 0.2);
  border-radius: 0.35rem;
  outline: 0;
}
.paragon-styles .modal-backdrop {
  position: fixed;
  top: 0;
  left: 0;
  z-index: 1040;
  width: 100vw;
  height: 100vh;
  background-color: #030303;
}
.paragon-styles .modal-backdrop.fade {
  opacity: 0;
}
.paragon-styles .modal-backdrop.show {
  opacity: 0.5;
}
.paragon-styles .modal-header {
  display: flex;
  align-items: flex-start;
  justify-content: space-between;
  padding: 1rem 1rem;
  border-bottom: 1px solid #e9ecef;
  border-top-left-radius: calc(0.35rem - 1px);
  border-top-right-radius: calc(0.35rem - 1px);
}
.paragon-styles .modal-header .close {
  padding: 1rem 1rem;
  margin: -1rem -1rem -1rem auto;
}
.paragon-styles .modal-title {
  margin-bottom: 0;
  line-height: 1.5;
}
.paragon-styles .modal-body {
  position: relative;
  flex: 1 1 auto;
  padding: 1rem;
}
.paragon-styles .modal-footer {
  display: flex;
  flex-wrap: wrap;
  align-items: center;
  justify-content: flex-end;
  padding: 0.75rem;
  border-top: 1px solid #e9ecef;
  border-bottom-right-radius: calc(0.35rem - 1px);
  border-bottom-left-radius: calc(0.35rem - 1px);
}
.paragon-styles .modal-footer > * {
  margin: 0.25rem;
}
.paragon-styles .modal-scrollbar-measure {
  position: absolute;
  top: -9999px;
  width: 50px;
  height: 50px;
  overflow: scroll;
}
@media (min-width: 576px) {
  .paragon-styles .modal-dialog {
    max-width: 500px;
    margin: 1.75rem auto;
  }
  .paragon-styles .modal-dialog-scrollable {
    max-height: calc(100% - 3.5rem);
  }
  .paragon-styles .modal-dialog-scrollable .modal-content {
    max-height: calc(100vh - 3.5rem);
  }
  .paragon-styles .modal-dialog-centered {
    min-height: calc(100% - 3.5rem);
  }
  .paragon-styles .modal-dialog-centered::before {
    height: calc(100vh - 3.5rem);
    height: min-content;
  }
  .paragon-styles .modal-sm {
    max-width: 300px;
  }
}
@media (min-width: 992px) {
  .paragon-styles .modal-lg,
.paragon-styles .modal-xl {
    max-width: 800px;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .modal-xl {
    max-width: 1140px;
  }
}
.paragon-styles .tooltip {
  position: absolute;
  z-index: 1070;
  display: block;
  margin: 0;
  font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-style: normal;
  font-weight: 400;
  line-height: 1.5;
  text-align: left;
  text-align: start;
  text-decoration: none;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  word-spacing: normal;
  white-space: normal;
  line-break: auto;
  font-size: 0.875rem;
  word-wrap: break-word;
  opacity: 0;
}
.paragon-styles .tooltip.show {
  opacity: 0.9;
}
.paragon-styles .tooltip .arrow {
  position: absolute;
  display: block;
  width: 0.8rem;
  height: 0.4rem;
}
.paragon-styles .tooltip .arrow::before {
  position: absolute;
  content: "";
  border-color: transparent;
  border-style: solid;
}
.paragon-styles .bs-tooltip-top, .paragon-styles .bs-tooltip-auto[x-placement^=top] {
  padding: 0.4rem 0;
}
.paragon-styles .bs-tooltip-top .arrow, .paragon-styles .bs-tooltip-auto[x-placement^=top] .arrow {
  bottom: 0;
}
.paragon-styles .bs-tooltip-top .arrow::before, .paragon-styles .bs-tooltip-auto[x-placement^=top] .arrow::before {
  top: 0;
  border-width: 0.4rem 0.4rem 0;
  border-top-color: #030303;
}
.paragon-styles .bs-tooltip-right, .paragon-styles .bs-tooltip-auto[x-placement^=right] {
  padding: 0 0.4rem;
}
.paragon-styles .bs-tooltip-right .arrow, .paragon-styles .bs-tooltip-auto[x-placement^=right] .arrow {
  left: 0;
  width: 0.4rem;
  height: 0.8rem;
}
.paragon-styles .bs-tooltip-right .arrow::before, .paragon-styles .bs-tooltip-auto[x-placement^=right] .arrow::before {
  right: 0;
  border-width: 0.4rem 0.4rem 0.4rem 0;
  border-right-color: #030303;
}
.paragon-styles .bs-tooltip-bottom, .paragon-styles .bs-tooltip-auto[x-placement^=bottom] {
  padding: 0.4rem 0;
}
.paragon-styles .bs-tooltip-bottom .arrow, .paragon-styles .bs-tooltip-auto[x-placement^=bottom] .arrow {
  top: 0;
}
.paragon-styles .bs-tooltip-bottom .arrow::before, .paragon-styles .bs-tooltip-auto[x-placement^=bottom] .arrow::before {
  bottom: 0;
  border-width: 0 0.4rem 0.4rem;
  border-bottom-color: #030303;
}
.paragon-styles .bs-tooltip-left, .paragon-styles .bs-tooltip-auto[x-placement^=left] {
  padding: 0 0.4rem;
}
.paragon-styles .bs-tooltip-left .arrow, .paragon-styles .bs-tooltip-auto[x-placement^=left] .arrow {
  right: 0;
  width: 0.4rem;
  height: 0.8rem;
}
.paragon-styles .bs-tooltip-left .arrow::before, .paragon-styles .bs-tooltip-auto[x-placement^=left] .arrow::before {
  left: 0;
  border-width: 0.4rem 0 0.4rem 0.4rem;
  border-left-color: #030303;
}
.paragon-styles .tooltip-inner {
  max-width: 200px;
  padding: 0.25rem 0.5rem;
  color: white;
  text-align: center;
  background-color: #030303;
  border-radius: 0.3rem;
}
.paragon-styles .popover {
  position: absolute;
  top: 0;
  left: 0;
  z-index: 1060;
  display: block;
  max-width: 276px;
  font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-style: normal;
  font-weight: 400;
  line-height: 1.5;
  text-align: left;
  text-align: start;
  text-decoration: none;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  word-spacing: normal;
  white-space: normal;
  line-break: auto;
  font-size: 0.875rem;
  word-wrap: break-word;
  background-color: white;
  background-clip: padding-box;
  border: 1px solid rgba(3, 3, 3, 0.2);
  border-radius: 0.35rem;
}
.paragon-styles .popover .arrow {
  position: absolute;
  display: block;
  width: 1rem;
  height: 0.5rem;
  margin: 0 0.35rem;
}
.paragon-styles .popover .arrow::before, .paragon-styles .popover .arrow::after {
  position: absolute;
  display: block;
  content: "";
  border-color: transparent;
  border-style: solid;
}
.paragon-styles .bs-popover-top, .paragon-styles .bs-popover-auto[x-placement^=top] {
  margin-bottom: 0.5rem;
}
.paragon-styles .bs-popover-top > .arrow, .paragon-styles .bs-popover-auto[x-placement^=top] > .arrow {
  bottom: calc(-0.5rem - 1px);
}
.paragon-styles .bs-popover-top > .arrow::before, .paragon-styles .bs-popover-auto[x-placement^=top] > .arrow::before {
  bottom: 0;
  border-width: 0.5rem 0.5rem 0;
  border-top-color: rgba(3, 3, 3, 0.25);
}
.paragon-styles .bs-popover-top > .arrow::after, .paragon-styles .bs-popover-auto[x-placement^=top] > .arrow::after {
  bottom: 1px;
  border-width: 0.5rem 0.5rem 0;
  border-top-color: white;
}
.paragon-styles .bs-popover-right, .paragon-styles .bs-popover-auto[x-placement^=right] {
  margin-left: 0.5rem;
}
.paragon-styles .bs-popover-right > .arrow, .paragon-styles .bs-popover-auto[x-placement^=right] > .arrow {
  left: calc(-0.5rem - 1px);
  width: 0.5rem;
  height: 1rem;
  margin: 0.35rem 0;
}
.paragon-styles .bs-popover-right > .arrow::before, .paragon-styles .bs-popover-auto[x-placement^=right] > .arrow::before {
  left: 0;
  border-width: 0.5rem 0.5rem 0.5rem 0;
  border-right-color: rgba(3, 3, 3, 0.25);
}
.paragon-styles .bs-popover-right > .arrow::after, .paragon-styles .bs-popover-auto[x-placement^=right] > .arrow::after {
  left: 1px;
  border-width: 0.5rem 0.5rem 0.5rem 0;
  border-right-color: white;
}
.paragon-styles .bs-popover-bottom, .paragon-styles .bs-popover-auto[x-placement^=bottom] {
  margin-top: 0.5rem;
}
.paragon-styles .bs-popover-bottom > .arrow, .paragon-styles .bs-popover-auto[x-placement^=bottom] > .arrow {
  top: calc(-0.5rem - 1px);
}
.paragon-styles .bs-popover-bottom > .arrow::before, .paragon-styles .bs-popover-auto[x-placement^=bottom] > .arrow::before {
  top: 0;
  border-width: 0 0.5rem 0.5rem 0.5rem;
  border-bottom-color: rgba(3, 3, 3, 0.25);
}
.paragon-styles .bs-popover-bottom > .arrow::after, .paragon-styles .bs-popover-auto[x-placement^=bottom] > .arrow::after {
  top: 1px;
  border-width: 0 0.5rem 0.5rem 0.5rem;
  border-bottom-color: white;
}
.paragon-styles .bs-popover-bottom .popover-header::before, .paragon-styles .bs-popover-auto[x-placement^=bottom] .popover-header::before {
  position: absolute;
  top: 0;
  left: 50%;
  display: block;
  width: 1rem;
  margin-left: -0.5rem;
  content: "";
  border-bottom: 1px solid #f7f7f7;
}
.paragon-styles .bs-popover-left, .paragon-styles .bs-popover-auto[x-placement^=left] {
  margin-right: 0.5rem;
}
.paragon-styles .bs-popover-left > .arrow, .paragon-styles .bs-popover-auto[x-placement^=left] > .arrow {
  right: calc(-0.5rem - 1px);
  width: 0.5rem;
  height: 1rem;
  margin: 0.35rem 0;
}
.paragon-styles .bs-popover-left > .arrow::before, .paragon-styles .bs-popover-auto[x-placement^=left] > .arrow::before {
  right: 0;
  border-width: 0.5rem 0 0.5rem 0.5rem;
  border-left-color: rgba(3, 3, 3, 0.25);
}
.paragon-styles .bs-popover-left > .arrow::after, .paragon-styles .bs-popover-auto[x-placement^=left] > .arrow::after {
  right: 1px;
  border-width: 0.5rem 0 0.5rem 0.5rem;
  border-left-color: white;
}
.paragon-styles .popover-header {
  padding: 0.5rem 0.75rem;
  margin-bottom: 0;
  font-size: 1rem;
  background-color: #f7f7f7;
  border-bottom: 1px solid #ebebeb;
  border-top-left-radius: calc(0.35rem - 1px);
  border-top-right-radius: calc(0.35rem - 1px);
}
.paragon-styles .popover-header:empty {
  display: none;
}
.paragon-styles .popover-body {
  padding: 0.5rem 0.75rem;
  color: #212529;
}
.paragon-styles .carousel {
  position: relative;
}
.paragon-styles .carousel.pointer-event {
  touch-action: pan-y;
}
.paragon-styles .carousel-inner {
  position: relative;
  width: 100%;
  overflow: hidden;
}
.paragon-styles .carousel-inner::after {
  display: block;
  clear: both;
  content: "";
}
.paragon-styles .carousel-item {
  position: relative;
  display: none;
  float: left;
  width: 100%;
  margin-right: -100%;
  backface-visibility: hidden;
  transition: transform 0.6s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .carousel-item {
    transition: none;
  }
}
.paragon-styles .carousel-item.active,
.paragon-styles .carousel-item-next,
.paragon-styles .carousel-item-prev {
  display: block;
}
.paragon-styles .carousel-item-next:not(.carousel-item-left),
.paragon-styles .active.carousel-item-right {
  transform: translateX(100%);
}
.paragon-styles .carousel-item-prev:not(.carousel-item-right),
.paragon-styles .active.carousel-item-left {
  transform: translateX(-100%);
}
.paragon-styles .carousel-fade .carousel-item {
  opacity: 0;
  transition-property: opacity;
  transform: none;
}
.paragon-styles .carousel-fade .carousel-item.active,
.paragon-styles .carousel-fade .carousel-item-next.carousel-item-left,
.paragon-styles .carousel-fade .carousel-item-prev.carousel-item-right {
  z-index: 1;
  opacity: 1;
}
.paragon-styles .carousel-fade .active.carousel-item-left,
.paragon-styles .carousel-fade .active.carousel-item-right {
  z-index: 0;
  opacity: 0;
  transition: opacity 0s 0.6s;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .carousel-fade .active.carousel-item-left,
.paragon-styles .carousel-fade .active.carousel-item-right {
    transition: none;
  }
}
.paragon-styles .carousel-control-prev,
.paragon-styles .carousel-control-next {
  position: absolute;
  top: 0;
  bottom: 0;
  z-index: 1;
  display: flex;
  align-items: center;
  justify-content: center;
  width: 15%;
  color: white;
  text-align: center;
  opacity: 0.5;
  transition: opacity 0.15s ease;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .carousel-control-prev,
.paragon-styles .carousel-control-next {
    transition: none;
  }
}
.paragon-styles .carousel-control-prev:hover, .paragon-styles .carousel-control-prev:focus,
.paragon-styles .carousel-control-next:hover,
.paragon-styles .carousel-control-next:focus {
  color: white;
  text-decoration: none;
  outline: 0;
  opacity: 0.9;
}
.paragon-styles .carousel-control-prev {
  left: 0;
}
.paragon-styles .carousel-control-next {
  right: 0;
}
.paragon-styles .carousel-control-prev-icon,
.paragon-styles .carousel-control-next-icon {
  display: inline-block;
  width: 20px;
  height: 20px;
  background: 50%/100% 100% no-repeat;
}
.paragon-styles .carousel-control-prev-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='white' viewBox='0 0 8 8'%3e%3cpath d='M5.25 0l-4 4 4 4 1.5-1.5-2.5-2.5 2.5-2.5-1.5-1.5z'/%3e%3c/svg%3e");
}
.paragon-styles .carousel-control-next-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='white' viewBox='0 0 8 8'%3e%3cpath d='M2.75 0l-1.5 1.5 2.5 2.5-2.5 2.5 1.5 1.5 4-4-4-4z'/%3e%3c/svg%3e");
}
.paragon-styles .carousel-indicators {
  position: absolute;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 15;
  display: flex;
  justify-content: center;
  padding-left: 0;
  margin-right: 15%;
  margin-left: 15%;
  list-style: none;
}
.paragon-styles .carousel-indicators li {
  box-sizing: content-box;
  flex: 0 1 auto;
  width: 30px;
  height: 3px;
  margin-right: 3px;
  margin-left: 3px;
  text-indent: -999px;
  cursor: pointer;
  background-color: white;
  background-clip: padding-box;
  border-top: 10px solid transparent;
  border-bottom: 10px solid transparent;
  opacity: 0.5;
  transition: opacity 0.6s ease;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .carousel-indicators li {
    transition: none;
  }
}
.paragon-styles .carousel-indicators .active {
  opacity: 1;
}
.paragon-styles .carousel-caption {
  position: absolute;
  right: 15%;
  bottom: 20px;
  left: 15%;
  z-index: 10;
  padding-top: 20px;
  padding-bottom: 20px;
  color: white;
  text-align: center;
}
@keyframes spinner-border {
  to {
    transform: rotate(360deg);
  }
}
.paragon-styles .spinner-border {
  display: inline-block;
  width: 2rem;
  height: 2rem;
  vertical-align: text-bottom;
  border: 0.25em solid currentColor;
  border-right-color: transparent;
  border-radius: 50%;
  animation: 0.75s linear infinite spinner-border;
}
.paragon-styles .spinner-border-sm {
  width: 1rem;
  height: 1rem;
  border-width: 0.2em;
}
@keyframes spinner-grow {
  0% {
    transform: scale(0);
  }
  50% {
    opacity: 1;
    transform: none;
  }
}
.paragon-styles .spinner-grow {
  display: inline-block;
  width: 2rem;
  height: 2rem;
  vertical-align: text-bottom;
  background-color: currentColor;
  border-radius: 50%;
  opacity: 0;
  animation: 0.75s linear infinite spinner-grow;
}
.paragon-styles .spinner-grow-sm {
  width: 1rem;
  height: 1rem;
}
@media (prefers-reduced-motion: reduce) {
  .paragon-styles .spinner-border,
.paragon-styles .spinner-grow {
    animation-duration: 1.5s;
  }
}
.paragon-styles .align-baseline {
  vertical-align: baseline !important;
}
.paragon-styles .align-top {
  vertical-align: top !important;
}
.paragon-styles .align-middle {
  vertical-align: middle !important;
}
.paragon-styles .align-bottom {
  vertical-align: bottom !important;
}
.paragon-styles .align-text-bottom {
  vertical-align: text-bottom !important;
}
.paragon-styles .align-text-top {
  vertical-align: text-top !important;
}
.paragon-styles .bg-primary {
  background-color: #80a9cc !important;
}
.paragon-styles a.bg-primary:hover, .paragon-styles a.bg-primary:focus,
.paragon-styles button.bg-primary:hover,
.paragon-styles button.bg-primary:focus {
  background-color: #5c90bd !important;
}
.paragon-styles .bg-secondary {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary:hover, .paragon-styles a.bg-secondary:focus,
.paragon-styles button.bg-secondary:hover,
.paragon-styles button.bg-secondary:focus {
  background-color: #545b62 !important;
}
.paragon-styles .bg-success {
  background-color: #178253 !important;
}
.paragon-styles a.bg-success:hover, .paragon-styles a.bg-success:focus,
.paragon-styles button.bg-success:hover,
.paragon-styles button.bg-success:focus {
  background-color: #0f5737 !important;
}
.paragon-styles .bg-info {
  background-color: #006DAA !important;
}
.paragon-styles a.bg-info:hover, .paragon-styles a.bg-info:focus,
.paragon-styles button.bg-info:hover,
.paragon-styles button.bg-info:focus {
  background-color: #004c77 !important;
}
.paragon-styles .bg-warning {
  background-color: #D69600 !important;
}
.paragon-styles a.bg-warning:hover, .paragon-styles a.bg-warning:focus,
.paragon-styles button.bg-warning:hover,
.paragon-styles button.bg-warning:focus {
  background-color: #a37200 !important;
}
.paragon-styles .bg-danger {
  background-color: #C32D3A !important;
}
.paragon-styles a.bg-danger:hover, .paragon-styles a.bg-danger:focus,
.paragon-styles button.bg-danger:hover,
.paragon-styles button.bg-danger:focus {
  background-color: #9a232e !important;
}
.paragon-styles .bg-light {
  background-color: #eee !important;
}
.paragon-styles a.bg-light:hover, .paragon-styles a.bg-light:focus,
.paragon-styles button.bg-light:hover,
.paragon-styles button.bg-light:focus {
  background-color: #d5d5d5 !important;
}
.paragon-styles .bg-dark {
  background-color: #495057 !important;
}
.paragon-styles a.bg-dark:hover, .paragon-styles a.bg-dark:focus,
.paragon-styles button.bg-dark:hover,
.paragon-styles button.bg-dark:focus {
  background-color: #32373b !important;
}
.paragon-styles .bg-brand {
  background-color: #80a9cc !important;
}
.paragon-styles a.bg-brand:hover, .paragon-styles a.bg-brand:focus,
.paragon-styles button.bg-brand:hover,
.paragon-styles button.bg-brand:focus {
  background-color: #5c90bd !important;
}
.paragon-styles .bg-gray {
  background-color: #adb5bd !important;
}
.paragon-styles a.bg-gray:hover, .paragon-styles a.bg-gray:focus,
.paragon-styles button.bg-gray:hover,
.paragon-styles button.bg-gray:focus {
  background-color: #919ca6 !important;
}
.paragon-styles .bg-white {
  background-color: white !important;
}
.paragon-styles .bg-transparent {
  background-color: transparent !important;
}
.paragon-styles .border, .paragon-styles .pgn__searchfield {
  border: 1px solid #e9ecef !important;
}
.paragon-styles .border-top {
  border-top: 1px solid #e9ecef !important;
}
.paragon-styles .border-right {
  border-right: 1px solid #e9ecef !important;
}
.paragon-styles .border-bottom {
  border-bottom: 1px solid #e9ecef !important;
}
.paragon-styles .border-left {
  border-left: 1px solid #e9ecef !important;
}
.paragon-styles .border-0, .paragon-styles .pgn__searchfield .form-control {
  border: 0 !important;
}
.paragon-styles .border-top-0 {
  border-top: 0 !important;
}
.paragon-styles .border-right-0 {
  border-right: 0 !important;
}
.paragon-styles .border-bottom-0 {
  border-bottom: 0 !important;
}
.paragon-styles .border-left-0 {
  border-left: 0 !important;
}
.paragon-styles .border-primary {
  border-color: #80a9cc !important;
}
.paragon-styles .border-secondary {
  border-color: #6c757d !important;
}
.paragon-styles .border-success {
  border-color: #178253 !important;
}
.paragon-styles .border-info {
  border-color: #006DAA !important;
}
.paragon-styles .border-warning {
  border-color: #D69600 !important;
}
.paragon-styles .border-danger {
  border-color: #C32D3A !important;
}
.paragon-styles .border-light {
  border-color: #eee !important;
}
.paragon-styles .border-dark {
  border-color: #495057 !important;
}
.paragon-styles .border-brand {
  border-color: #80a9cc !important;
}
.paragon-styles .border-gray {
  border-color: #adb5bd !important;
}
.paragon-styles .border-white {
  border-color: white !important;
}
.paragon-styles .rounded-sm {
  border-radius: 0.2rem !important;
}
.paragon-styles .rounded, .paragon-styles .pgn__searchfield {
  border-radius: 0.3rem !important;
}
.paragon-styles .rounded-top {
  border-top-left-radius: 0.3rem !important;
  border-top-right-radius: 0.3rem !important;
}
.paragon-styles .rounded-right {
  border-top-right-radius: 0.3rem !important;
  border-bottom-right-radius: 0.3rem !important;
}
.paragon-styles .rounded-bottom {
  border-bottom-right-radius: 0.3rem !important;
  border-bottom-left-radius: 0.3rem !important;
}
.paragon-styles .rounded-left {
  border-top-left-radius: 0.3rem !important;
  border-bottom-left-radius: 0.3rem !important;
}
.paragon-styles .rounded-lg {
  border-radius: 0.35rem !important;
}
.paragon-styles .rounded-circle {
  border-radius: 50% !important;
}
.paragon-styles .rounded-pill {
  border-radius: 50rem !important;
}
.paragon-styles .rounded-0 {
  border-radius: 0 !important;
}
.paragon-styles .clearfix::after {
  display: block;
  clear: both;
  content: "";
}
.paragon-styles .d-none {
  display: none !important;
}
.paragon-styles .d-inline {
  display: inline !important;
}
.paragon-styles .d-inline-block {
  display: inline-block !important;
}
.paragon-styles .d-block {
  display: block !important;
}
.paragon-styles .d-table {
  display: table !important;
}
.paragon-styles .d-table-row {
  display: table-row !important;
}
.paragon-styles .d-table-cell {
  display: table-cell !important;
}
.paragon-styles .d-flex {
  display: flex !important;
}
.paragon-styles .d-inline-flex {
  display: inline-flex !important;
}
@media (min-width: 576px) {
  .paragon-styles .d-sm-none {
    display: none !important;
  }
  .paragon-styles .d-sm-inline {
    display: inline !important;
  }
  .paragon-styles .d-sm-inline-block {
    display: inline-block !important;
  }
  .paragon-styles .d-sm-block {
    display: block !important;
  }
  .paragon-styles .d-sm-table {
    display: table !important;
  }
  .paragon-styles .d-sm-table-row {
    display: table-row !important;
  }
  .paragon-styles .d-sm-table-cell {
    display: table-cell !important;
  }
  .paragon-styles .d-sm-flex {
    display: flex !important;
  }
  .paragon-styles .d-sm-inline-flex {
    display: inline-flex !important;
  }
}
@media (min-width: 768px) {
  .paragon-styles .d-md-none {
    display: none !important;
  }
  .paragon-styles .d-md-inline {
    display: inline !important;
  }
  .paragon-styles .d-md-inline-block {
    display: inline-block !important;
  }
  .paragon-styles .d-md-block {
    display: block !important;
  }
  .paragon-styles .d-md-table {
    display: table !important;
  }
  .paragon-styles .d-md-table-row {
    display: table-row !important;
  }
  .paragon-styles .d-md-table-cell {
    display: table-cell !important;
  }
  .paragon-styles .d-md-flex {
    display: flex !important;
  }
  .paragon-styles .d-md-inline-flex {
    display: inline-flex !important;
  }
}
@media (min-width: 992px) {
  .paragon-styles .d-lg-none {
    display: none !important;
  }
  .paragon-styles .d-lg-inline {
    display: inline !important;
  }
  .paragon-styles .d-lg-inline-block {
    display: inline-block !important;
  }
  .paragon-styles .d-lg-block {
    display: block !important;
  }
  .paragon-styles .d-lg-table {
    display: table !important;
  }
  .paragon-styles .d-lg-table-row {
    display: table-row !important;
  }
  .paragon-styles .d-lg-table-cell {
    display: table-cell !important;
  }
  .paragon-styles .d-lg-flex {
    display: flex !important;
  }
  .paragon-styles .d-lg-inline-flex {
    display: inline-flex !important;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .d-xl-none {
    display: none !important;
  }
  .paragon-styles .d-xl-inline {
    display: inline !important;
  }
  .paragon-styles .d-xl-inline-block {
    display: inline-block !important;
  }
  .paragon-styles .d-xl-block {
    display: block !important;
  }
  .paragon-styles .d-xl-table {
    display: table !important;
  }
  .paragon-styles .d-xl-table-row {
    display: table-row !important;
  }
  .paragon-styles .d-xl-table-cell {
    display: table-cell !important;
  }
  .paragon-styles .d-xl-flex {
    display: flex !important;
  }
  .paragon-styles .d-xl-inline-flex {
    display: inline-flex !important;
  }
}
@media print {
  .paragon-styles .d-print-none {
    display: none !important;
  }
  .paragon-styles .d-print-inline {
    display: inline !important;
  }
  .paragon-styles .d-print-inline-block {
    display: inline-block !important;
  }
  .paragon-styles .d-print-block {
    display: block !important;
  }
  .paragon-styles .d-print-table {
    display: table !important;
  }
  .paragon-styles .d-print-table-row {
    display: table-row !important;
  }
  .paragon-styles .d-print-table-cell {
    display: table-cell !important;
  }
  .paragon-styles .d-print-flex {
    display: flex !important;
  }
  .paragon-styles .d-print-inline-flex {
    display: inline-flex !important;
  }
}
.paragon-styles .embed-responsive {
  position: relative;
  display: block;
  width: 100%;
  padding: 0;
  overflow: hidden;
}
.paragon-styles .embed-responsive::before {
  display: block;
  content: "";
}
.paragon-styles .embed-responsive .embed-responsive-item,
.paragon-styles .embed-responsive iframe,
.paragon-styles .embed-responsive embed,
.paragon-styles .embed-responsive object,
.paragon-styles .embed-responsive video {
  position: absolute;
  top: 0;
  bottom: 0;
  left: 0;
  width: 100%;
  height: 100%;
  border: 0;
}
.paragon-styles .embed-responsive-21by9::before {
  padding-top: 42.8571428571%;
}
.paragon-styles .embed-responsive-16by9::before {
  padding-top: 56.25%;
}
.paragon-styles .embed-responsive-4by3::before {
  padding-top: 75%;
}
.paragon-styles .embed-responsive-1by1::before {
  padding-top: 100%;
}
.paragon-styles .embed-responsive-21by9::before {
  padding-top: 42.8571428571%;
}
.paragon-styles .embed-responsive-16by9::before {
  padding-top: 56.25%;
}
.paragon-styles .embed-responsive-4by3::before {
  padding-top: 75%;
}
.paragon-styles .embed-responsive-1by1::before {
  padding-top: 100%;
}
.paragon-styles .flex-row {
  flex-direction: row !important;
}
.paragon-styles .flex-column {
  flex-direction: column !important;
}
.paragon-styles .flex-row-reverse {
  flex-direction: row-reverse !important;
}
.paragon-styles .flex-column-reverse {
  flex-direction: column-reverse !important;
}
.paragon-styles .flex-wrap {
  flex-wrap: wrap !important;
}
.paragon-styles .flex-nowrap {
  flex-wrap: nowrap !important;
}
.paragon-styles .flex-wrap-reverse {
  flex-wrap: wrap-reverse !important;
}
.paragon-styles .flex-fill {
  flex: 1 1 auto !important;
}
.paragon-styles .flex-grow-0 {
  flex-grow: 0 !important;
}
.paragon-styles .flex-grow-1 {
  flex-grow: 1 !important;
}
.paragon-styles .flex-shrink-0 {
  flex-shrink: 0 !important;
}
.paragon-styles .flex-shrink-1 {
  flex-shrink: 1 !important;
}
.paragon-styles .justify-content-start {
  justify-content: flex-start !important;
}
.paragon-styles .justify-content-end {
  justify-content: flex-end !important;
}
.paragon-styles .justify-content-center {
  justify-content: center !important;
}
.paragon-styles .justify-content-between {
  justify-content: space-between !important;
}
.paragon-styles .justify-content-around {
  justify-content: space-around !important;
}
.paragon-styles .align-items-start {
  align-items: flex-start !important;
}
.paragon-styles .align-items-end {
  align-items: flex-end !important;
}
.paragon-styles .align-items-center {
  align-items: center !important;
}
.paragon-styles .align-items-baseline {
  align-items: baseline !important;
}
.paragon-styles .align-items-stretch {
  align-items: stretch !important;
}
.paragon-styles .align-content-start {
  align-content: flex-start !important;
}
.paragon-styles .align-content-end {
  align-content: flex-end !important;
}
.paragon-styles .align-content-center {
  align-content: center !important;
}
.paragon-styles .align-content-between {
  align-content: space-between !important;
}
.paragon-styles .align-content-around {
  align-content: space-around !important;
}
.paragon-styles .align-content-stretch {
  align-content: stretch !important;
}
.paragon-styles .align-self-auto {
  align-self: auto !important;
}
.paragon-styles .align-self-start {
  align-self: flex-start !important;
}
.paragon-styles .align-self-end {
  align-self: flex-end !important;
}
.paragon-styles .align-self-center {
  align-self: center !important;
}
.paragon-styles .align-self-baseline {
  align-self: baseline !important;
}
.paragon-styles .align-self-stretch {
  align-self: stretch !important;
}
@media (min-width: 576px) {
  .paragon-styles .flex-sm-row {
    flex-direction: row !important;
  }
  .paragon-styles .flex-sm-column {
    flex-direction: column !important;
  }
  .paragon-styles .flex-sm-row-reverse {
    flex-direction: row-reverse !important;
  }
  .paragon-styles .flex-sm-column-reverse {
    flex-direction: column-reverse !important;
  }
  .paragon-styles .flex-sm-wrap {
    flex-wrap: wrap !important;
  }
  .paragon-styles .flex-sm-nowrap {
    flex-wrap: nowrap !important;
  }
  .paragon-styles .flex-sm-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .paragon-styles .flex-sm-fill {
    flex: 1 1 auto !important;
  }
  .paragon-styles .flex-sm-grow-0 {
    flex-grow: 0 !important;
  }
  .paragon-styles .flex-sm-grow-1 {
    flex-grow: 1 !important;
  }
  .paragon-styles .flex-sm-shrink-0 {
    flex-shrink: 0 !important;
  }
  .paragon-styles .flex-sm-shrink-1 {
    flex-shrink: 1 !important;
  }
  .paragon-styles .justify-content-sm-start {
    justify-content: flex-start !important;
  }
  .paragon-styles .justify-content-sm-end {
    justify-content: flex-end !important;
  }
  .paragon-styles .justify-content-sm-center {
    justify-content: center !important;
  }
  .paragon-styles .justify-content-sm-between {
    justify-content: space-between !important;
  }
  .paragon-styles .justify-content-sm-around {
    justify-content: space-around !important;
  }
  .paragon-styles .align-items-sm-start {
    align-items: flex-start !important;
  }
  .paragon-styles .align-items-sm-end {
    align-items: flex-end !important;
  }
  .paragon-styles .align-items-sm-center {
    align-items: center !important;
  }
  .paragon-styles .align-items-sm-baseline {
    align-items: baseline !important;
  }
  .paragon-styles .align-items-sm-stretch {
    align-items: stretch !important;
  }
  .paragon-styles .align-content-sm-start {
    align-content: flex-start !important;
  }
  .paragon-styles .align-content-sm-end {
    align-content: flex-end !important;
  }
  .paragon-styles .align-content-sm-center {
    align-content: center !important;
  }
  .paragon-styles .align-content-sm-between {
    align-content: space-between !important;
  }
  .paragon-styles .align-content-sm-around {
    align-content: space-around !important;
  }
  .paragon-styles .align-content-sm-stretch {
    align-content: stretch !important;
  }
  .paragon-styles .align-self-sm-auto {
    align-self: auto !important;
  }
  .paragon-styles .align-self-sm-start {
    align-self: flex-start !important;
  }
  .paragon-styles .align-self-sm-end {
    align-self: flex-end !important;
  }
  .paragon-styles .align-self-sm-center {
    align-self: center !important;
  }
  .paragon-styles .align-self-sm-baseline {
    align-self: baseline !important;
  }
  .paragon-styles .align-self-sm-stretch {
    align-self: stretch !important;
  }
}
@media (min-width: 768px) {
  .paragon-styles .flex-md-row {
    flex-direction: row !important;
  }
  .paragon-styles .flex-md-column {
    flex-direction: column !important;
  }
  .paragon-styles .flex-md-row-reverse {
    flex-direction: row-reverse !important;
  }
  .paragon-styles .flex-md-column-reverse {
    flex-direction: column-reverse !important;
  }
  .paragon-styles .flex-md-wrap {
    flex-wrap: wrap !important;
  }
  .paragon-styles .flex-md-nowrap {
    flex-wrap: nowrap !important;
  }
  .paragon-styles .flex-md-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .paragon-styles .flex-md-fill {
    flex: 1 1 auto !important;
  }
  .paragon-styles .flex-md-grow-0 {
    flex-grow: 0 !important;
  }
  .paragon-styles .flex-md-grow-1 {
    flex-grow: 1 !important;
  }
  .paragon-styles .flex-md-shrink-0 {
    flex-shrink: 0 !important;
  }
  .paragon-styles .flex-md-shrink-1 {
    flex-shrink: 1 !important;
  }
  .paragon-styles .justify-content-md-start {
    justify-content: flex-start !important;
  }
  .paragon-styles .justify-content-md-end {
    justify-content: flex-end !important;
  }
  .paragon-styles .justify-content-md-center {
    justify-content: center !important;
  }
  .paragon-styles .justify-content-md-between {
    justify-content: space-between !important;
  }
  .paragon-styles .justify-content-md-around {
    justify-content: space-around !important;
  }
  .paragon-styles .align-items-md-start {
    align-items: flex-start !important;
  }
  .paragon-styles .align-items-md-end {
    align-items: flex-end !important;
  }
  .paragon-styles .align-items-md-center {
    align-items: center !important;
  }
  .paragon-styles .align-items-md-baseline {
    align-items: baseline !important;
  }
  .paragon-styles .align-items-md-stretch {
    align-items: stretch !important;
  }
  .paragon-styles .align-content-md-start {
    align-content: flex-start !important;
  }
  .paragon-styles .align-content-md-end {
    align-content: flex-end !important;
  }
  .paragon-styles .align-content-md-center {
    align-content: center !important;
  }
  .paragon-styles .align-content-md-between {
    align-content: space-between !important;
  }
  .paragon-styles .align-content-md-around {
    align-content: space-around !important;
  }
  .paragon-styles .align-content-md-stretch {
    align-content: stretch !important;
  }
  .paragon-styles .align-self-md-auto {
    align-self: auto !important;
  }
  .paragon-styles .align-self-md-start {
    align-self: flex-start !important;
  }
  .paragon-styles .align-self-md-end {
    align-self: flex-end !important;
  }
  .paragon-styles .align-self-md-center {
    align-self: center !important;
  }
  .paragon-styles .align-self-md-baseline {
    align-self: baseline !important;
  }
  .paragon-styles .align-self-md-stretch {
    align-self: stretch !important;
  }
}
@media (min-width: 992px) {
  .paragon-styles .flex-lg-row {
    flex-direction: row !important;
  }
  .paragon-styles .flex-lg-column {
    flex-direction: column !important;
  }
  .paragon-styles .flex-lg-row-reverse {
    flex-direction: row-reverse !important;
  }
  .paragon-styles .flex-lg-column-reverse {
    flex-direction: column-reverse !important;
  }
  .paragon-styles .flex-lg-wrap {
    flex-wrap: wrap !important;
  }
  .paragon-styles .flex-lg-nowrap {
    flex-wrap: nowrap !important;
  }
  .paragon-styles .flex-lg-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .paragon-styles .flex-lg-fill {
    flex: 1 1 auto !important;
  }
  .paragon-styles .flex-lg-grow-0 {
    flex-grow: 0 !important;
  }
  .paragon-styles .flex-lg-grow-1 {
    flex-grow: 1 !important;
  }
  .paragon-styles .flex-lg-shrink-0 {
    flex-shrink: 0 !important;
  }
  .paragon-styles .flex-lg-shrink-1 {
    flex-shrink: 1 !important;
  }
  .paragon-styles .justify-content-lg-start {
    justify-content: flex-start !important;
  }
  .paragon-styles .justify-content-lg-end {
    justify-content: flex-end !important;
  }
  .paragon-styles .justify-content-lg-center {
    justify-content: center !important;
  }
  .paragon-styles .justify-content-lg-between {
    justify-content: space-between !important;
  }
  .paragon-styles .justify-content-lg-around {
    justify-content: space-around !important;
  }
  .paragon-styles .align-items-lg-start {
    align-items: flex-start !important;
  }
  .paragon-styles .align-items-lg-end {
    align-items: flex-end !important;
  }
  .paragon-styles .align-items-lg-center {
    align-items: center !important;
  }
  .paragon-styles .align-items-lg-baseline {
    align-items: baseline !important;
  }
  .paragon-styles .align-items-lg-stretch {
    align-items: stretch !important;
  }
  .paragon-styles .align-content-lg-start {
    align-content: flex-start !important;
  }
  .paragon-styles .align-content-lg-end {
    align-content: flex-end !important;
  }
  .paragon-styles .align-content-lg-center {
    align-content: center !important;
  }
  .paragon-styles .align-content-lg-between {
    align-content: space-between !important;
  }
  .paragon-styles .align-content-lg-around {
    align-content: space-around !important;
  }
  .paragon-styles .align-content-lg-stretch {
    align-content: stretch !important;
  }
  .paragon-styles .align-self-lg-auto {
    align-self: auto !important;
  }
  .paragon-styles .align-self-lg-start {
    align-self: flex-start !important;
  }
  .paragon-styles .align-self-lg-end {
    align-self: flex-end !important;
  }
  .paragon-styles .align-self-lg-center {
    align-self: center !important;
  }
  .paragon-styles .align-self-lg-baseline {
    align-self: baseline !important;
  }
  .paragon-styles .align-self-lg-stretch {
    align-self: stretch !important;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .flex-xl-row {
    flex-direction: row !important;
  }
  .paragon-styles .flex-xl-column {
    flex-direction: column !important;
  }
  .paragon-styles .flex-xl-row-reverse {
    flex-direction: row-reverse !important;
  }
  .paragon-styles .flex-xl-column-reverse {
    flex-direction: column-reverse !important;
  }
  .paragon-styles .flex-xl-wrap {
    flex-wrap: wrap !important;
  }
  .paragon-styles .flex-xl-nowrap {
    flex-wrap: nowrap !important;
  }
  .paragon-styles .flex-xl-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .paragon-styles .flex-xl-fill {
    flex: 1 1 auto !important;
  }
  .paragon-styles .flex-xl-grow-0 {
    flex-grow: 0 !important;
  }
  .paragon-styles .flex-xl-grow-1 {
    flex-grow: 1 !important;
  }
  .paragon-styles .flex-xl-shrink-0 {
    flex-shrink: 0 !important;
  }
  .paragon-styles .flex-xl-shrink-1 {
    flex-shrink: 1 !important;
  }
  .paragon-styles .justify-content-xl-start {
    justify-content: flex-start !important;
  }
  .paragon-styles .justify-content-xl-end {
    justify-content: flex-end !important;
  }
  .paragon-styles .justify-content-xl-center {
    justify-content: center !important;
  }
  .paragon-styles .justify-content-xl-between {
    justify-content: space-between !important;
  }
  .paragon-styles .justify-content-xl-around {
    justify-content: space-around !important;
  }
  .paragon-styles .align-items-xl-start {
    align-items: flex-start !important;
  }
  .paragon-styles .align-items-xl-end {
    align-items: flex-end !important;
  }
  .paragon-styles .align-items-xl-center {
    align-items: center !important;
  }
  .paragon-styles .align-items-xl-baseline {
    align-items: baseline !important;
  }
  .paragon-styles .align-items-xl-stretch {
    align-items: stretch !important;
  }
  .paragon-styles .align-content-xl-start {
    align-content: flex-start !important;
  }
  .paragon-styles .align-content-xl-end {
    align-content: flex-end !important;
  }
  .paragon-styles .align-content-xl-center {
    align-content: center !important;
  }
  .paragon-styles .align-content-xl-between {
    align-content: space-between !important;
  }
  .paragon-styles .align-content-xl-around {
    align-content: space-around !important;
  }
  .paragon-styles .align-content-xl-stretch {
    align-content: stretch !important;
  }
  .paragon-styles .align-self-xl-auto {
    align-self: auto !important;
  }
  .paragon-styles .align-self-xl-start {
    align-self: flex-start !important;
  }
  .paragon-styles .align-self-xl-end {
    align-self: flex-end !important;
  }
  .paragon-styles .align-self-xl-center {
    align-self: center !important;
  }
  .paragon-styles .align-self-xl-baseline {
    align-self: baseline !important;
  }
  .paragon-styles .align-self-xl-stretch {
    align-self: stretch !important;
  }
}
.paragon-styles .float-left {
  float: left !important;
}
.paragon-styles .float-right {
  float: right !important;
}
.paragon-styles .float-none {
  float: none !important;
}
@media (min-width: 576px) {
  .paragon-styles .float-sm-left {
    float: left !important;
  }
  .paragon-styles .float-sm-right {
    float: right !important;
  }
  .paragon-styles .float-sm-none {
    float: none !important;
  }
}
@media (min-width: 768px) {
  .paragon-styles .float-md-left {
    float: left !important;
  }
  .paragon-styles .float-md-right {
    float: right !important;
  }
  .paragon-styles .float-md-none {
    float: none !important;
  }
}
@media (min-width: 992px) {
  .paragon-styles .float-lg-left {
    float: left !important;
  }
  .paragon-styles .float-lg-right {
    float: right !important;
  }
  .paragon-styles .float-lg-none {
    float: none !important;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .float-xl-left {
    float: left !important;
  }
  .paragon-styles .float-xl-right {
    float: right !important;
  }
  .paragon-styles .float-xl-none {
    float: none !important;
  }
}
.paragon-styles .user-select-all {
  user-select: all !important;
}
.paragon-styles .user-select-auto {
  user-select: auto !important;
}
.paragon-styles .user-select-none {
  user-select: none !important;
}
.paragon-styles .overflow-auto {
  overflow: auto !important;
}
.paragon-styles .overflow-hidden {
  overflow: hidden !important;
}
.paragon-styles .position-static {
  position: static !important;
}
.paragon-styles .position-relative {
  position: relative !important;
}
.paragon-styles .position-absolute {
  position: absolute !important;
}
.paragon-styles .position-fixed {
  position: fixed !important;
}
.paragon-styles .position-sticky {
  position: sticky !important;
}
.paragon-styles .fixed-top {
  position: fixed;
  top: 0;
  right: 0;
  left: 0;
  z-index: 1030;
}
.paragon-styles .fixed-bottom {
  position: fixed;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 1030;
}
@supports (position: sticky) {
  .paragon-styles .sticky-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
}
.paragon-styles .sr-only {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border: 0;
}
.paragon-styles .sr-only-focusable:active, .paragon-styles .sr-only-focusable:focus {
  position: static;
  width: auto;
  height: auto;
  overflow: visible;
  clip: auto;
  white-space: normal;
}
.paragon-styles .shadow-sm {
  box-shadow: 0 0.0625rem 0.125rem rgba(3, 3, 3, 0.2) !important;
}
.paragon-styles .shadow {
  box-shadow: 0 0.125rem 0.25rem rgba(3, 3, 3, 0.3) !important;
}
.paragon-styles .shadow-lg {
  box-shadow: 0 0.25rem 0.5rem rgba(3, 3, 3, 0.3) !important;
}
.paragon-styles .shadow-none {
  box-shadow: none !important;
}
.paragon-styles .w-25 {
  width: 25% !important;
}
.paragon-styles .w-50 {
  width: 50% !important;
}
.paragon-styles .w-75 {
  width: 75% !important;
}
.paragon-styles .w-100 {
  width: 100% !important;
}
.paragon-styles .w-auto {
  width: auto !important;
}
.paragon-styles .h-25 {
  height: 25% !important;
}
.paragon-styles .h-50 {
  height: 50% !important;
}
.paragon-styles .h-75 {
  height: 75% !important;
}
.paragon-styles .h-100 {
  height: 100% !important;
}
.paragon-styles .h-auto {
  height: auto !important;
}
.paragon-styles .mw-100 {
  max-width: 100% !important;
}
.paragon-styles .mh-100 {
  max-height: 100% !important;
}
.paragon-styles .min-vw-100 {
  min-width: 100vw !important;
}
.paragon-styles .min-vh-100 {
  min-height: 100vh !important;
}
.paragon-styles .vw-100 {
  width: 100vw !important;
}
.paragon-styles .vh-100 {
  height: 100vh !important;
}
.paragon-styles .m-0 {
  margin: 0 !important;
}
.paragon-styles .mt-0,
.paragon-styles .my-0 {
  margin-top: 0 !important;
}
.paragon-styles .mr-0,
.paragon-styles .mx-0 {
  margin-right: 0 !important;
}
.paragon-styles .mb-0,
.paragon-styles .my-0 {
  margin-bottom: 0 !important;
}
.paragon-styles .ml-0,
.paragon-styles .mx-0 {
  margin-left: 0 !important;
}
.paragon-styles .m-1 {
  margin: 0.25rem !important;
}
.paragon-styles .mt-1,
.paragon-styles .my-1 {
  margin-top: 0.25rem !important;
}
.paragon-styles .mr-1,
.paragon-styles .mx-1 {
  margin-right: 0.25rem !important;
}
.paragon-styles .mb-1,
.paragon-styles .my-1 {
  margin-bottom: 0.25rem !important;
}
.paragon-styles .ml-1,
.paragon-styles .mx-1 {
  margin-left: 0.25rem !important;
}
.paragon-styles .m-2 {
  margin: 0.5rem !important;
}
.paragon-styles .mt-2,
.paragon-styles .my-2 {
  margin-top: 0.5rem !important;
}
.paragon-styles .mr-2,
.paragon-styles .mx-2 {
  margin-right: 0.5rem !important;
}
.paragon-styles .mb-2,
.paragon-styles .my-2 {
  margin-bottom: 0.5rem !important;
}
.paragon-styles .ml-2,
.paragon-styles .mx-2 {
  margin-left: 0.5rem !important;
}
.paragon-styles .m-3 {
  margin: 1rem !important;
}
.paragon-styles .mt-3,
.paragon-styles .my-3 {
  margin-top: 1rem !important;
}
.paragon-styles .mr-3,
.paragon-styles .mx-3 {
  margin-right: 1rem !important;
}
.paragon-styles .mb-3,
.paragon-styles .my-3 {
  margin-bottom: 1rem !important;
}
.paragon-styles .ml-3,
.paragon-styles .mx-3 {
  margin-left: 1rem !important;
}
.paragon-styles .m-4 {
  margin: 1.5rem !important;
}
.paragon-styles .mt-4,
.paragon-styles .my-4 {
  margin-top: 1.5rem !important;
}
.paragon-styles .mr-4,
.paragon-styles .mx-4 {
  margin-right: 1.5rem !important;
}
.paragon-styles .mb-4,
.paragon-styles .my-4 {
  margin-bottom: 1.5rem !important;
}
.paragon-styles .ml-4,
.paragon-styles .mx-4 {
  margin-left: 1.5rem !important;
}
.paragon-styles .m-5 {
  margin: 3rem !important;
}
.paragon-styles .mt-5,
.paragon-styles .my-5 {
  margin-top: 3rem !important;
}
.paragon-styles .mr-5,
.paragon-styles .mx-5 {
  margin-right: 3rem !important;
}
.paragon-styles .mb-5,
.paragon-styles .my-5 {
  margin-bottom: 3rem !important;
}
.paragon-styles .ml-5,
.paragon-styles .mx-5 {
  margin-left: 3rem !important;
}
.paragon-styles .p-0 {
  padding: 0 !important;
}
.paragon-styles .pt-0,
.paragon-styles .py-0 {
  padding-top: 0 !important;
}
.paragon-styles .pr-0,
.paragon-styles .px-0 {
  padding-right: 0 !important;
}
.paragon-styles .pb-0,
.paragon-styles .py-0 {
  padding-bottom: 0 !important;
}
.paragon-styles .pl-0,
.paragon-styles .px-0 {
  padding-left: 0 !important;
}
.paragon-styles .p-1 {
  padding: 0.25rem !important;
}
.paragon-styles .pt-1,
.paragon-styles .py-1 {
  padding-top: 0.25rem !important;
}
.paragon-styles .pr-1,
.paragon-styles .px-1 {
  padding-right: 0.25rem !important;
}
.paragon-styles .pb-1,
.paragon-styles .py-1 {
  padding-bottom: 0.25rem !important;
}
.paragon-styles .pl-1,
.paragon-styles .px-1 {
  padding-left: 0.25rem !important;
}
.paragon-styles .p-2 {
  padding: 0.5rem !important;
}
.paragon-styles .pt-2,
.paragon-styles .py-2 {
  padding-top: 0.5rem !important;
}
.paragon-styles .pr-2,
.paragon-styles .px-2,
.paragon-styles .pgn__searchfield .btn[type=submit],
.paragon-styles .pgn__searchfield .btn[type=reset] {
  padding-right: 0.5rem !important;
}
.paragon-styles .pb-2,
.paragon-styles .py-2 {
  padding-bottom: 0.5rem !important;
}
.paragon-styles .pl-2,
.paragon-styles .px-2,
.paragon-styles .pgn__searchfield .btn[type=submit],
.paragon-styles .pgn__searchfield .btn[type=reset] {
  padding-left: 0.5rem !important;
}
.paragon-styles .p-3 {
  padding: 1rem !important;
}
.paragon-styles .pt-3,
.paragon-styles .py-3 {
  padding-top: 1rem !important;
}
.paragon-styles .pr-3,
.paragon-styles .px-3 {
  padding-right: 1rem !important;
}
.paragon-styles .pb-3,
.paragon-styles .py-3 {
  padding-bottom: 1rem !important;
}
.paragon-styles .pl-3,
.paragon-styles .px-3 {
  padding-left: 1rem !important;
}
.paragon-styles .p-4 {
  padding: 1.5rem !important;
}
.paragon-styles .pt-4,
.paragon-styles .py-4 {
  padding-top: 1.5rem !important;
}
.paragon-styles .pr-4,
.paragon-styles .px-4 {
  padding-right: 1.5rem !important;
}
.paragon-styles .pb-4,
.paragon-styles .py-4 {
  padding-bottom: 1.5rem !important;
}
.paragon-styles .pl-4,
.paragon-styles .px-4 {
  padding-left: 1.5rem !important;
}
.paragon-styles .p-5 {
  padding: 3rem !important;
}
.paragon-styles .pt-5,
.paragon-styles .py-5 {
  padding-top: 3rem !important;
}
.paragon-styles .pr-5,
.paragon-styles .px-5 {
  padding-right: 3rem !important;
}
.paragon-styles .pb-5,
.paragon-styles .py-5 {
  padding-bottom: 3rem !important;
}
.paragon-styles .pl-5,
.paragon-styles .px-5 {
  padding-left: 3rem !important;
}
.paragon-styles .m-n1 {
  margin: -0.25rem !important;
}
.paragon-styles .mt-n1,
.paragon-styles .my-n1 {
  margin-top: -0.25rem !important;
}
.paragon-styles .mr-n1,
.paragon-styles .mx-n1 {
  margin-right: -0.25rem !important;
}
.paragon-styles .mb-n1,
.paragon-styles .my-n1 {
  margin-bottom: -0.25rem !important;
}
.paragon-styles .ml-n1,
.paragon-styles .mx-n1 {
  margin-left: -0.25rem !important;
}
.paragon-styles .m-n2 {
  margin: -0.5rem !important;
}
.paragon-styles .mt-n2,
.paragon-styles .my-n2 {
  margin-top: -0.5rem !important;
}
.paragon-styles .mr-n2,
.paragon-styles .mx-n2 {
  margin-right: -0.5rem !important;
}
.paragon-styles .mb-n2,
.paragon-styles .my-n2 {
  margin-bottom: -0.5rem !important;
}
.paragon-styles .ml-n2,
.paragon-styles .mx-n2 {
  margin-left: -0.5rem !important;
}
.paragon-styles .m-n3 {
  margin: -1rem !important;
}
.paragon-styles .mt-n3,
.paragon-styles .my-n3 {
  margin-top: -1rem !important;
}
.paragon-styles .mr-n3,
.paragon-styles .mx-n3 {
  margin-right: -1rem !important;
}
.paragon-styles .mb-n3,
.paragon-styles .my-n3 {
  margin-bottom: -1rem !important;
}
.paragon-styles .ml-n3,
.paragon-styles .mx-n3 {
  margin-left: -1rem !important;
}
.paragon-styles .m-n4 {
  margin: -1.5rem !important;
}
.paragon-styles .mt-n4,
.paragon-styles .my-n4 {
  margin-top: -1.5rem !important;
}
.paragon-styles .mr-n4,
.paragon-styles .mx-n4 {
  margin-right: -1.5rem !important;
}
.paragon-styles .mb-n4,
.paragon-styles .my-n4 {
  margin-bottom: -1.5rem !important;
}
.paragon-styles .ml-n4,
.paragon-styles .mx-n4 {
  margin-left: -1.5rem !important;
}
.paragon-styles .m-n5 {
  margin: -3rem !important;
}
.paragon-styles .mt-n5,
.paragon-styles .my-n5 {
  margin-top: -3rem !important;
}
.paragon-styles .mr-n5,
.paragon-styles .mx-n5 {
  margin-right: -3rem !important;
}
.paragon-styles .mb-n5,
.paragon-styles .my-n5 {
  margin-bottom: -3rem !important;
}
.paragon-styles .ml-n5,
.paragon-styles .mx-n5 {
  margin-left: -3rem !important;
}
.paragon-styles .m-auto {
  margin: auto !important;
}
.paragon-styles .mt-auto,
.paragon-styles .my-auto {
  margin-top: auto !important;
}
.paragon-styles .mr-auto,
.paragon-styles .mx-auto {
  margin-right: auto !important;
}
.paragon-styles .mb-auto,
.paragon-styles .my-auto {
  margin-bottom: auto !important;
}
.paragon-styles .ml-auto,
.paragon-styles .mx-auto {
  margin-left: auto !important;
}
@media (min-width: 576px) {
  .paragon-styles .m-sm-0 {
    margin: 0 !important;
  }
  .paragon-styles .mt-sm-0,
.paragon-styles .my-sm-0 {
    margin-top: 0 !important;
  }
  .paragon-styles .mr-sm-0,
.paragon-styles .mx-sm-0 {
    margin-right: 0 !important;
  }
  .paragon-styles .mb-sm-0,
.paragon-styles .my-sm-0 {
    margin-bottom: 0 !important;
  }
  .paragon-styles .ml-sm-0,
.paragon-styles .mx-sm-0 {
    margin-left: 0 !important;
  }
  .paragon-styles .m-sm-1 {
    margin: 0.25rem !important;
  }
  .paragon-styles .mt-sm-1,
.paragon-styles .my-sm-1 {
    margin-top: 0.25rem !important;
  }
  .paragon-styles .mr-sm-1,
.paragon-styles .mx-sm-1 {
    margin-right: 0.25rem !important;
  }
  .paragon-styles .mb-sm-1,
.paragon-styles .my-sm-1 {
    margin-bottom: 0.25rem !important;
  }
  .paragon-styles .ml-sm-1,
.paragon-styles .mx-sm-1 {
    margin-left: 0.25rem !important;
  }
  .paragon-styles .m-sm-2 {
    margin: 0.5rem !important;
  }
  .paragon-styles .mt-sm-2,
.paragon-styles .my-sm-2 {
    margin-top: 0.5rem !important;
  }
  .paragon-styles .mr-sm-2,
.paragon-styles .mx-sm-2 {
    margin-right: 0.5rem !important;
  }
  .paragon-styles .mb-sm-2,
.paragon-styles .my-sm-2 {
    margin-bottom: 0.5rem !important;
  }
  .paragon-styles .ml-sm-2,
.paragon-styles .mx-sm-2 {
    margin-left: 0.5rem !important;
  }
  .paragon-styles .m-sm-3 {
    margin: 1rem !important;
  }
  .paragon-styles .mt-sm-3,
.paragon-styles .my-sm-3 {
    margin-top: 1rem !important;
  }
  .paragon-styles .mr-sm-3,
.paragon-styles .mx-sm-3 {
    margin-right: 1rem !important;
  }
  .paragon-styles .mb-sm-3,
.paragon-styles .my-sm-3 {
    margin-bottom: 1rem !important;
  }
  .paragon-styles .ml-sm-3,
.paragon-styles .mx-sm-3 {
    margin-left: 1rem !important;
  }
  .paragon-styles .m-sm-4 {
    margin: 1.5rem !important;
  }
  .paragon-styles .mt-sm-4,
.paragon-styles .my-sm-4 {
    margin-top: 1.5rem !important;
  }
  .paragon-styles .mr-sm-4,
.paragon-styles .mx-sm-4 {
    margin-right: 1.5rem !important;
  }
  .paragon-styles .mb-sm-4,
.paragon-styles .my-sm-4 {
    margin-bottom: 1.5rem !important;
  }
  .paragon-styles .ml-sm-4,
.paragon-styles .mx-sm-4 {
    margin-left: 1.5rem !important;
  }
  .paragon-styles .m-sm-5 {
    margin: 3rem !important;
  }
  .paragon-styles .mt-sm-5,
.paragon-styles .my-sm-5 {
    margin-top: 3rem !important;
  }
  .paragon-styles .mr-sm-5,
.paragon-styles .mx-sm-5 {
    margin-right: 3rem !important;
  }
  .paragon-styles .mb-sm-5,
.paragon-styles .my-sm-5 {
    margin-bottom: 3rem !important;
  }
  .paragon-styles .ml-sm-5,
.paragon-styles .mx-sm-5 {
    margin-left: 3rem !important;
  }
  .paragon-styles .p-sm-0 {
    padding: 0 !important;
  }
  .paragon-styles .pt-sm-0,
.paragon-styles .py-sm-0 {
    padding-top: 0 !important;
  }
  .paragon-styles .pr-sm-0,
.paragon-styles .px-sm-0 {
    padding-right: 0 !important;
  }
  .paragon-styles .pb-sm-0,
.paragon-styles .py-sm-0 {
    padding-bottom: 0 !important;
  }
  .paragon-styles .pl-sm-0,
.paragon-styles .px-sm-0 {
    padding-left: 0 !important;
  }
  .paragon-styles .p-sm-1 {
    padding: 0.25rem !important;
  }
  .paragon-styles .pt-sm-1,
.paragon-styles .py-sm-1 {
    padding-top: 0.25rem !important;
  }
  .paragon-styles .pr-sm-1,
.paragon-styles .px-sm-1 {
    padding-right: 0.25rem !important;
  }
  .paragon-styles .pb-sm-1,
.paragon-styles .py-sm-1 {
    padding-bottom: 0.25rem !important;
  }
  .paragon-styles .pl-sm-1,
.paragon-styles .px-sm-1 {
    padding-left: 0.25rem !important;
  }
  .paragon-styles .p-sm-2 {
    padding: 0.5rem !important;
  }
  .paragon-styles .pt-sm-2,
.paragon-styles .py-sm-2 {
    padding-top: 0.5rem !important;
  }
  .paragon-styles .pr-sm-2,
.paragon-styles .px-sm-2 {
    padding-right: 0.5rem !important;
  }
  .paragon-styles .pb-sm-2,
.paragon-styles .py-sm-2 {
    padding-bottom: 0.5rem !important;
  }
  .paragon-styles .pl-sm-2,
.paragon-styles .px-sm-2 {
    padding-left: 0.5rem !important;
  }
  .paragon-styles .p-sm-3 {
    padding: 1rem !important;
  }
  .paragon-styles .pt-sm-3,
.paragon-styles .py-sm-3 {
    padding-top: 1rem !important;
  }
  .paragon-styles .pr-sm-3,
.paragon-styles .px-sm-3 {
    padding-right: 1rem !important;
  }
  .paragon-styles .pb-sm-3,
.paragon-styles .py-sm-3 {
    padding-bottom: 1rem !important;
  }
  .paragon-styles .pl-sm-3,
.paragon-styles .px-sm-3 {
    padding-left: 1rem !important;
  }
  .paragon-styles .p-sm-4 {
    padding: 1.5rem !important;
  }
  .paragon-styles .pt-sm-4,
.paragon-styles .py-sm-4 {
    padding-top: 1.5rem !important;
  }
  .paragon-styles .pr-sm-4,
.paragon-styles .px-sm-4 {
    padding-right: 1.5rem !important;
  }
  .paragon-styles .pb-sm-4,
.paragon-styles .py-sm-4 {
    padding-bottom: 1.5rem !important;
  }
  .paragon-styles .pl-sm-4,
.paragon-styles .px-sm-4 {
    padding-left: 1.5rem !important;
  }
  .paragon-styles .p-sm-5 {
    padding: 3rem !important;
  }
  .paragon-styles .pt-sm-5,
.paragon-styles .py-sm-5 {
    padding-top: 3rem !important;
  }
  .paragon-styles .pr-sm-5,
.paragon-styles .px-sm-5 {
    padding-right: 3rem !important;
  }
  .paragon-styles .pb-sm-5,
.paragon-styles .py-sm-5 {
    padding-bottom: 3rem !important;
  }
  .paragon-styles .pl-sm-5,
.paragon-styles .px-sm-5 {
    padding-left: 3rem !important;
  }
  .paragon-styles .m-sm-n1 {
    margin: -0.25rem !important;
  }
  .paragon-styles .mt-sm-n1,
.paragon-styles .my-sm-n1 {
    margin-top: -0.25rem !important;
  }
  .paragon-styles .mr-sm-n1,
.paragon-styles .mx-sm-n1 {
    margin-right: -0.25rem !important;
  }
  .paragon-styles .mb-sm-n1,
.paragon-styles .my-sm-n1 {
    margin-bottom: -0.25rem !important;
  }
  .paragon-styles .ml-sm-n1,
.paragon-styles .mx-sm-n1 {
    margin-left: -0.25rem !important;
  }
  .paragon-styles .m-sm-n2 {
    margin: -0.5rem !important;
  }
  .paragon-styles .mt-sm-n2,
.paragon-styles .my-sm-n2 {
    margin-top: -0.5rem !important;
  }
  .paragon-styles .mr-sm-n2,
.paragon-styles .mx-sm-n2 {
    margin-right: -0.5rem !important;
  }
  .paragon-styles .mb-sm-n2,
.paragon-styles .my-sm-n2 {
    margin-bottom: -0.5rem !important;
  }
  .paragon-styles .ml-sm-n2,
.paragon-styles .mx-sm-n2 {
    margin-left: -0.5rem !important;
  }
  .paragon-styles .m-sm-n3 {
    margin: -1rem !important;
  }
  .paragon-styles .mt-sm-n3,
.paragon-styles .my-sm-n3 {
    margin-top: -1rem !important;
  }
  .paragon-styles .mr-sm-n3,
.paragon-styles .mx-sm-n3 {
    margin-right: -1rem !important;
  }
  .paragon-styles .mb-sm-n3,
.paragon-styles .my-sm-n3 {
    margin-bottom: -1rem !important;
  }
  .paragon-styles .ml-sm-n3,
.paragon-styles .mx-sm-n3 {
    margin-left: -1rem !important;
  }
  .paragon-styles .m-sm-n4 {
    margin: -1.5rem !important;
  }
  .paragon-styles .mt-sm-n4,
.paragon-styles .my-sm-n4 {
    margin-top: -1.5rem !important;
  }
  .paragon-styles .mr-sm-n4,
.paragon-styles .mx-sm-n4 {
    margin-right: -1.5rem !important;
  }
  .paragon-styles .mb-sm-n4,
.paragon-styles .my-sm-n4 {
    margin-bottom: -1.5rem !important;
  }
  .paragon-styles .ml-sm-n4,
.paragon-styles .mx-sm-n4 {
    margin-left: -1.5rem !important;
  }
  .paragon-styles .m-sm-n5 {
    margin: -3rem !important;
  }
  .paragon-styles .mt-sm-n5,
.paragon-styles .my-sm-n5 {
    margin-top: -3rem !important;
  }
  .paragon-styles .mr-sm-n5,
.paragon-styles .mx-sm-n5 {
    margin-right: -3rem !important;
  }
  .paragon-styles .mb-sm-n5,
.paragon-styles .my-sm-n5 {
    margin-bottom: -3rem !important;
  }
  .paragon-styles .ml-sm-n5,
.paragon-styles .mx-sm-n5 {
    margin-left: -3rem !important;
  }
  .paragon-styles .m-sm-auto {
    margin: auto !important;
  }
  .paragon-styles .mt-sm-auto,
.paragon-styles .my-sm-auto {
    margin-top: auto !important;
  }
  .paragon-styles .mr-sm-auto,
.paragon-styles .mx-sm-auto {
    margin-right: auto !important;
  }
  .paragon-styles .mb-sm-auto,
.paragon-styles .my-sm-auto {
    margin-bottom: auto !important;
  }
  .paragon-styles .ml-sm-auto,
.paragon-styles .mx-sm-auto {
    margin-left: auto !important;
  }
}
@media (min-width: 768px) {
  .paragon-styles .m-md-0 {
    margin: 0 !important;
  }
  .paragon-styles .mt-md-0,
.paragon-styles .my-md-0 {
    margin-top: 0 !important;
  }
  .paragon-styles .mr-md-0,
.paragon-styles .mx-md-0 {
    margin-right: 0 !important;
  }
  .paragon-styles .mb-md-0,
.paragon-styles .my-md-0 {
    margin-bottom: 0 !important;
  }
  .paragon-styles .ml-md-0,
.paragon-styles .mx-md-0 {
    margin-left: 0 !important;
  }
  .paragon-styles .m-md-1 {
    margin: 0.25rem !important;
  }
  .paragon-styles .mt-md-1,
.paragon-styles .my-md-1 {
    margin-top: 0.25rem !important;
  }
  .paragon-styles .mr-md-1,
.paragon-styles .mx-md-1 {
    margin-right: 0.25rem !important;
  }
  .paragon-styles .mb-md-1,
.paragon-styles .my-md-1 {
    margin-bottom: 0.25rem !important;
  }
  .paragon-styles .ml-md-1,
.paragon-styles .mx-md-1 {
    margin-left: 0.25rem !important;
  }
  .paragon-styles .m-md-2 {
    margin: 0.5rem !important;
  }
  .paragon-styles .mt-md-2,
.paragon-styles .my-md-2 {
    margin-top: 0.5rem !important;
  }
  .paragon-styles .mr-md-2,
.paragon-styles .mx-md-2 {
    margin-right: 0.5rem !important;
  }
  .paragon-styles .mb-md-2,
.paragon-styles .my-md-2 {
    margin-bottom: 0.5rem !important;
  }
  .paragon-styles .ml-md-2,
.paragon-styles .mx-md-2 {
    margin-left: 0.5rem !important;
  }
  .paragon-styles .m-md-3 {
    margin: 1rem !important;
  }
  .paragon-styles .mt-md-3,
.paragon-styles .my-md-3 {
    margin-top: 1rem !important;
  }
  .paragon-styles .mr-md-3,
.paragon-styles .mx-md-3 {
    margin-right: 1rem !important;
  }
  .paragon-styles .mb-md-3,
.paragon-styles .my-md-3 {
    margin-bottom: 1rem !important;
  }
  .paragon-styles .ml-md-3,
.paragon-styles .mx-md-3 {
    margin-left: 1rem !important;
  }
  .paragon-styles .m-md-4 {
    margin: 1.5rem !important;
  }
  .paragon-styles .mt-md-4,
.paragon-styles .my-md-4 {
    margin-top: 1.5rem !important;
  }
  .paragon-styles .mr-md-4,
.paragon-styles .mx-md-4 {
    margin-right: 1.5rem !important;
  }
  .paragon-styles .mb-md-4,
.paragon-styles .my-md-4 {
    margin-bottom: 1.5rem !important;
  }
  .paragon-styles .ml-md-4,
.paragon-styles .mx-md-4 {
    margin-left: 1.5rem !important;
  }
  .paragon-styles .m-md-5 {
    margin: 3rem !important;
  }
  .paragon-styles .mt-md-5,
.paragon-styles .my-md-5 {
    margin-top: 3rem !important;
  }
  .paragon-styles .mr-md-5,
.paragon-styles .mx-md-5 {
    margin-right: 3rem !important;
  }
  .paragon-styles .mb-md-5,
.paragon-styles .my-md-5 {
    margin-bottom: 3rem !important;
  }
  .paragon-styles .ml-md-5,
.paragon-styles .mx-md-5 {
    margin-left: 3rem !important;
  }
  .paragon-styles .p-md-0 {
    padding: 0 !important;
  }
  .paragon-styles .pt-md-0,
.paragon-styles .py-md-0 {
    padding-top: 0 !important;
  }
  .paragon-styles .pr-md-0,
.paragon-styles .px-md-0 {
    padding-right: 0 !important;
  }
  .paragon-styles .pb-md-0,
.paragon-styles .py-md-0 {
    padding-bottom: 0 !important;
  }
  .paragon-styles .pl-md-0,
.paragon-styles .px-md-0 {
    padding-left: 0 !important;
  }
  .paragon-styles .p-md-1 {
    padding: 0.25rem !important;
  }
  .paragon-styles .pt-md-1,
.paragon-styles .py-md-1 {
    padding-top: 0.25rem !important;
  }
  .paragon-styles .pr-md-1,
.paragon-styles .px-md-1 {
    padding-right: 0.25rem !important;
  }
  .paragon-styles .pb-md-1,
.paragon-styles .py-md-1 {
    padding-bottom: 0.25rem !important;
  }
  .paragon-styles .pl-md-1,
.paragon-styles .px-md-1 {
    padding-left: 0.25rem !important;
  }
  .paragon-styles .p-md-2 {
    padding: 0.5rem !important;
  }
  .paragon-styles .pt-md-2,
.paragon-styles .py-md-2 {
    padding-top: 0.5rem !important;
  }
  .paragon-styles .pr-md-2,
.paragon-styles .px-md-2 {
    padding-right: 0.5rem !important;
  }
  .paragon-styles .pb-md-2,
.paragon-styles .py-md-2 {
    padding-bottom: 0.5rem !important;
  }
  .paragon-styles .pl-md-2,
.paragon-styles .px-md-2 {
    padding-left: 0.5rem !important;
  }
  .paragon-styles .p-md-3 {
    padding: 1rem !important;
  }
  .paragon-styles .pt-md-3,
.paragon-styles .py-md-3 {
    padding-top: 1rem !important;
  }
  .paragon-styles .pr-md-3,
.paragon-styles .px-md-3 {
    padding-right: 1rem !important;
  }
  .paragon-styles .pb-md-3,
.paragon-styles .py-md-3 {
    padding-bottom: 1rem !important;
  }
  .paragon-styles .pl-md-3,
.paragon-styles .px-md-3 {
    padding-left: 1rem !important;
  }
  .paragon-styles .p-md-4 {
    padding: 1.5rem !important;
  }
  .paragon-styles .pt-md-4,
.paragon-styles .py-md-4 {
    padding-top: 1.5rem !important;
  }
  .paragon-styles .pr-md-4,
.paragon-styles .px-md-4 {
    padding-right: 1.5rem !important;
  }
  .paragon-styles .pb-md-4,
.paragon-styles .py-md-4 {
    padding-bottom: 1.5rem !important;
  }
  .paragon-styles .pl-md-4,
.paragon-styles .px-md-4 {
    padding-left: 1.5rem !important;
  }
  .paragon-styles .p-md-5 {
    padding: 3rem !important;
  }
  .paragon-styles .pt-md-5,
.paragon-styles .py-md-5 {
    padding-top: 3rem !important;
  }
  .paragon-styles .pr-md-5,
.paragon-styles .px-md-5 {
    padding-right: 3rem !important;
  }
  .paragon-styles .pb-md-5,
.paragon-styles .py-md-5 {
    padding-bottom: 3rem !important;
  }
  .paragon-styles .pl-md-5,
.paragon-styles .px-md-5 {
    padding-left: 3rem !important;
  }
  .paragon-styles .m-md-n1 {
    margin: -0.25rem !important;
  }
  .paragon-styles .mt-md-n1,
.paragon-styles .my-md-n1 {
    margin-top: -0.25rem !important;
  }
  .paragon-styles .mr-md-n1,
.paragon-styles .mx-md-n1 {
    margin-right: -0.25rem !important;
  }
  .paragon-styles .mb-md-n1,
.paragon-styles .my-md-n1 {
    margin-bottom: -0.25rem !important;
  }
  .paragon-styles .ml-md-n1,
.paragon-styles .mx-md-n1 {
    margin-left: -0.25rem !important;
  }
  .paragon-styles .m-md-n2 {
    margin: -0.5rem !important;
  }
  .paragon-styles .mt-md-n2,
.paragon-styles .my-md-n2 {
    margin-top: -0.5rem !important;
  }
  .paragon-styles .mr-md-n2,
.paragon-styles .mx-md-n2 {
    margin-right: -0.5rem !important;
  }
  .paragon-styles .mb-md-n2,
.paragon-styles .my-md-n2 {
    margin-bottom: -0.5rem !important;
  }
  .paragon-styles .ml-md-n2,
.paragon-styles .mx-md-n2 {
    margin-left: -0.5rem !important;
  }
  .paragon-styles .m-md-n3 {
    margin: -1rem !important;
  }
  .paragon-styles .mt-md-n3,
.paragon-styles .my-md-n3 {
    margin-top: -1rem !important;
  }
  .paragon-styles .mr-md-n3,
.paragon-styles .mx-md-n3 {
    margin-right: -1rem !important;
  }
  .paragon-styles .mb-md-n3,
.paragon-styles .my-md-n3 {
    margin-bottom: -1rem !important;
  }
  .paragon-styles .ml-md-n3,
.paragon-styles .mx-md-n3 {
    margin-left: -1rem !important;
  }
  .paragon-styles .m-md-n4 {
    margin: -1.5rem !important;
  }
  .paragon-styles .mt-md-n4,
.paragon-styles .my-md-n4 {
    margin-top: -1.5rem !important;
  }
  .paragon-styles .mr-md-n4,
.paragon-styles .mx-md-n4 {
    margin-right: -1.5rem !important;
  }
  .paragon-styles .mb-md-n4,
.paragon-styles .my-md-n4 {
    margin-bottom: -1.5rem !important;
  }
  .paragon-styles .ml-md-n4,
.paragon-styles .mx-md-n4 {
    margin-left: -1.5rem !important;
  }
  .paragon-styles .m-md-n5 {
    margin: -3rem !important;
  }
  .paragon-styles .mt-md-n5,
.paragon-styles .my-md-n5 {
    margin-top: -3rem !important;
  }
  .paragon-styles .mr-md-n5,
.paragon-styles .mx-md-n5 {
    margin-right: -3rem !important;
  }
  .paragon-styles .mb-md-n5,
.paragon-styles .my-md-n5 {
    margin-bottom: -3rem !important;
  }
  .paragon-styles .ml-md-n5,
.paragon-styles .mx-md-n5 {
    margin-left: -3rem !important;
  }
  .paragon-styles .m-md-auto {
    margin: auto !important;
  }
  .paragon-styles .mt-md-auto,
.paragon-styles .my-md-auto {
    margin-top: auto !important;
  }
  .paragon-styles .mr-md-auto,
.paragon-styles .mx-md-auto {
    margin-right: auto !important;
  }
  .paragon-styles .mb-md-auto,
.paragon-styles .my-md-auto {
    margin-bottom: auto !important;
  }
  .paragon-styles .ml-md-auto,
.paragon-styles .mx-md-auto {
    margin-left: auto !important;
  }
}
@media (min-width: 992px) {
  .paragon-styles .m-lg-0 {
    margin: 0 !important;
  }
  .paragon-styles .mt-lg-0,
.paragon-styles .my-lg-0 {
    margin-top: 0 !important;
  }
  .paragon-styles .mr-lg-0,
.paragon-styles .mx-lg-0 {
    margin-right: 0 !important;
  }
  .paragon-styles .mb-lg-0,
.paragon-styles .my-lg-0 {
    margin-bottom: 0 !important;
  }
  .paragon-styles .ml-lg-0,
.paragon-styles .mx-lg-0 {
    margin-left: 0 !important;
  }
  .paragon-styles .m-lg-1 {
    margin: 0.25rem !important;
  }
  .paragon-styles .mt-lg-1,
.paragon-styles .my-lg-1 {
    margin-top: 0.25rem !important;
  }
  .paragon-styles .mr-lg-1,
.paragon-styles .mx-lg-1 {
    margin-right: 0.25rem !important;
  }
  .paragon-styles .mb-lg-1,
.paragon-styles .my-lg-1 {
    margin-bottom: 0.25rem !important;
  }
  .paragon-styles .ml-lg-1,
.paragon-styles .mx-lg-1 {
    margin-left: 0.25rem !important;
  }
  .paragon-styles .m-lg-2 {
    margin: 0.5rem !important;
  }
  .paragon-styles .mt-lg-2,
.paragon-styles .my-lg-2 {
    margin-top: 0.5rem !important;
  }
  .paragon-styles .mr-lg-2,
.paragon-styles .mx-lg-2 {
    margin-right: 0.5rem !important;
  }
  .paragon-styles .mb-lg-2,
.paragon-styles .my-lg-2 {
    margin-bottom: 0.5rem !important;
  }
  .paragon-styles .ml-lg-2,
.paragon-styles .mx-lg-2 {
    margin-left: 0.5rem !important;
  }
  .paragon-styles .m-lg-3 {
    margin: 1rem !important;
  }
  .paragon-styles .mt-lg-3,
.paragon-styles .my-lg-3 {
    margin-top: 1rem !important;
  }
  .paragon-styles .mr-lg-3,
.paragon-styles .mx-lg-3 {
    margin-right: 1rem !important;
  }
  .paragon-styles .mb-lg-3,
.paragon-styles .my-lg-3 {
    margin-bottom: 1rem !important;
  }
  .paragon-styles .ml-lg-3,
.paragon-styles .mx-lg-3 {
    margin-left: 1rem !important;
  }
  .paragon-styles .m-lg-4 {
    margin: 1.5rem !important;
  }
  .paragon-styles .mt-lg-4,
.paragon-styles .my-lg-4 {
    margin-top: 1.5rem !important;
  }
  .paragon-styles .mr-lg-4,
.paragon-styles .mx-lg-4 {
    margin-right: 1.5rem !important;
  }
  .paragon-styles .mb-lg-4,
.paragon-styles .my-lg-4 {
    margin-bottom: 1.5rem !important;
  }
  .paragon-styles .ml-lg-4,
.paragon-styles .mx-lg-4 {
    margin-left: 1.5rem !important;
  }
  .paragon-styles .m-lg-5 {
    margin: 3rem !important;
  }
  .paragon-styles .mt-lg-5,
.paragon-styles .my-lg-5 {
    margin-top: 3rem !important;
  }
  .paragon-styles .mr-lg-5,
.paragon-styles .mx-lg-5 {
    margin-right: 3rem !important;
  }
  .paragon-styles .mb-lg-5,
.paragon-styles .my-lg-5 {
    margin-bottom: 3rem !important;
  }
  .paragon-styles .ml-lg-5,
.paragon-styles .mx-lg-5 {
    margin-left: 3rem !important;
  }
  .paragon-styles .p-lg-0 {
    padding: 0 !important;
  }
  .paragon-styles .pt-lg-0,
.paragon-styles .py-lg-0 {
    padding-top: 0 !important;
  }
  .paragon-styles .pr-lg-0,
.paragon-styles .px-lg-0 {
    padding-right: 0 !important;
  }
  .paragon-styles .pb-lg-0,
.paragon-styles .py-lg-0 {
    padding-bottom: 0 !important;
  }
  .paragon-styles .pl-lg-0,
.paragon-styles .px-lg-0 {
    padding-left: 0 !important;
  }
  .paragon-styles .p-lg-1 {
    padding: 0.25rem !important;
  }
  .paragon-styles .pt-lg-1,
.paragon-styles .py-lg-1 {
    padding-top: 0.25rem !important;
  }
  .paragon-styles .pr-lg-1,
.paragon-styles .px-lg-1 {
    padding-right: 0.25rem !important;
  }
  .paragon-styles .pb-lg-1,
.paragon-styles .py-lg-1 {
    padding-bottom: 0.25rem !important;
  }
  .paragon-styles .pl-lg-1,
.paragon-styles .px-lg-1 {
    padding-left: 0.25rem !important;
  }
  .paragon-styles .p-lg-2 {
    padding: 0.5rem !important;
  }
  .paragon-styles .pt-lg-2,
.paragon-styles .py-lg-2 {
    padding-top: 0.5rem !important;
  }
  .paragon-styles .pr-lg-2,
.paragon-styles .px-lg-2 {
    padding-right: 0.5rem !important;
  }
  .paragon-styles .pb-lg-2,
.paragon-styles .py-lg-2 {
    padding-bottom: 0.5rem !important;
  }
  .paragon-styles .pl-lg-2,
.paragon-styles .px-lg-2 {
    padding-left: 0.5rem !important;
  }
  .paragon-styles .p-lg-3 {
    padding: 1rem !important;
  }
  .paragon-styles .pt-lg-3,
.paragon-styles .py-lg-3 {
    padding-top: 1rem !important;
  }
  .paragon-styles .pr-lg-3,
.paragon-styles .px-lg-3 {
    padding-right: 1rem !important;
  }
  .paragon-styles .pb-lg-3,
.paragon-styles .py-lg-3 {
    padding-bottom: 1rem !important;
  }
  .paragon-styles .pl-lg-3,
.paragon-styles .px-lg-3 {
    padding-left: 1rem !important;
  }
  .paragon-styles .p-lg-4 {
    padding: 1.5rem !important;
  }
  .paragon-styles .pt-lg-4,
.paragon-styles .py-lg-4 {
    padding-top: 1.5rem !important;
  }
  .paragon-styles .pr-lg-4,
.paragon-styles .px-lg-4 {
    padding-right: 1.5rem !important;
  }
  .paragon-styles .pb-lg-4,
.paragon-styles .py-lg-4 {
    padding-bottom: 1.5rem !important;
  }
  .paragon-styles .pl-lg-4,
.paragon-styles .px-lg-4 {
    padding-left: 1.5rem !important;
  }
  .paragon-styles .p-lg-5 {
    padding: 3rem !important;
  }
  .paragon-styles .pt-lg-5,
.paragon-styles .py-lg-5 {
    padding-top: 3rem !important;
  }
  .paragon-styles .pr-lg-5,
.paragon-styles .px-lg-5 {
    padding-right: 3rem !important;
  }
  .paragon-styles .pb-lg-5,
.paragon-styles .py-lg-5 {
    padding-bottom: 3rem !important;
  }
  .paragon-styles .pl-lg-5,
.paragon-styles .px-lg-5 {
    padding-left: 3rem !important;
  }
  .paragon-styles .m-lg-n1 {
    margin: -0.25rem !important;
  }
  .paragon-styles .mt-lg-n1,
.paragon-styles .my-lg-n1 {
    margin-top: -0.25rem !important;
  }
  .paragon-styles .mr-lg-n1,
.paragon-styles .mx-lg-n1 {
    margin-right: -0.25rem !important;
  }
  .paragon-styles .mb-lg-n1,
.paragon-styles .my-lg-n1 {
    margin-bottom: -0.25rem !important;
  }
  .paragon-styles .ml-lg-n1,
.paragon-styles .mx-lg-n1 {
    margin-left: -0.25rem !important;
  }
  .paragon-styles .m-lg-n2 {
    margin: -0.5rem !important;
  }
  .paragon-styles .mt-lg-n2,
.paragon-styles .my-lg-n2 {
    margin-top: -0.5rem !important;
  }
  .paragon-styles .mr-lg-n2,
.paragon-styles .mx-lg-n2 {
    margin-right: -0.5rem !important;
  }
  .paragon-styles .mb-lg-n2,
.paragon-styles .my-lg-n2 {
    margin-bottom: -0.5rem !important;
  }
  .paragon-styles .ml-lg-n2,
.paragon-styles .mx-lg-n2 {
    margin-left: -0.5rem !important;
  }
  .paragon-styles .m-lg-n3 {
    margin: -1rem !important;
  }
  .paragon-styles .mt-lg-n3,
.paragon-styles .my-lg-n3 {
    margin-top: -1rem !important;
  }
  .paragon-styles .mr-lg-n3,
.paragon-styles .mx-lg-n3 {
    margin-right: -1rem !important;
  }
  .paragon-styles .mb-lg-n3,
.paragon-styles .my-lg-n3 {
    margin-bottom: -1rem !important;
  }
  .paragon-styles .ml-lg-n3,
.paragon-styles .mx-lg-n3 {
    margin-left: -1rem !important;
  }
  .paragon-styles .m-lg-n4 {
    margin: -1.5rem !important;
  }
  .paragon-styles .mt-lg-n4,
.paragon-styles .my-lg-n4 {
    margin-top: -1.5rem !important;
  }
  .paragon-styles .mr-lg-n4,
.paragon-styles .mx-lg-n4 {
    margin-right: -1.5rem !important;
  }
  .paragon-styles .mb-lg-n4,
.paragon-styles .my-lg-n4 {
    margin-bottom: -1.5rem !important;
  }
  .paragon-styles .ml-lg-n4,
.paragon-styles .mx-lg-n4 {
    margin-left: -1.5rem !important;
  }
  .paragon-styles .m-lg-n5 {
    margin: -3rem !important;
  }
  .paragon-styles .mt-lg-n5,
.paragon-styles .my-lg-n5 {
    margin-top: -3rem !important;
  }
  .paragon-styles .mr-lg-n5,
.paragon-styles .mx-lg-n5 {
    margin-right: -3rem !important;
  }
  .paragon-styles .mb-lg-n5,
.paragon-styles .my-lg-n5 {
    margin-bottom: -3rem !important;
  }
  .paragon-styles .ml-lg-n5,
.paragon-styles .mx-lg-n5 {
    margin-left: -3rem !important;
  }
  .paragon-styles .m-lg-auto {
    margin: auto !important;
  }
  .paragon-styles .mt-lg-auto,
.paragon-styles .my-lg-auto {
    margin-top: auto !important;
  }
  .paragon-styles .mr-lg-auto,
.paragon-styles .mx-lg-auto {
    margin-right: auto !important;
  }
  .paragon-styles .mb-lg-auto,
.paragon-styles .my-lg-auto {
    margin-bottom: auto !important;
  }
  .paragon-styles .ml-lg-auto,
.paragon-styles .mx-lg-auto {
    margin-left: auto !important;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .m-xl-0 {
    margin: 0 !important;
  }
  .paragon-styles .mt-xl-0,
.paragon-styles .my-xl-0 {
    margin-top: 0 !important;
  }
  .paragon-styles .mr-xl-0,
.paragon-styles .mx-xl-0 {
    margin-right: 0 !important;
  }
  .paragon-styles .mb-xl-0,
.paragon-styles .my-xl-0 {
    margin-bottom: 0 !important;
  }
  .paragon-styles .ml-xl-0,
.paragon-styles .mx-xl-0 {
    margin-left: 0 !important;
  }
  .paragon-styles .m-xl-1 {
    margin: 0.25rem !important;
  }
  .paragon-styles .mt-xl-1,
.paragon-styles .my-xl-1 {
    margin-top: 0.25rem !important;
  }
  .paragon-styles .mr-xl-1,
.paragon-styles .mx-xl-1 {
    margin-right: 0.25rem !important;
  }
  .paragon-styles .mb-xl-1,
.paragon-styles .my-xl-1 {
    margin-bottom: 0.25rem !important;
  }
  .paragon-styles .ml-xl-1,
.paragon-styles .mx-xl-1 {
    margin-left: 0.25rem !important;
  }
  .paragon-styles .m-xl-2 {
    margin: 0.5rem !important;
  }
  .paragon-styles .mt-xl-2,
.paragon-styles .my-xl-2 {
    margin-top: 0.5rem !important;
  }
  .paragon-styles .mr-xl-2,
.paragon-styles .mx-xl-2 {
    margin-right: 0.5rem !important;
  }
  .paragon-styles .mb-xl-2,
.paragon-styles .my-xl-2 {
    margin-bottom: 0.5rem !important;
  }
  .paragon-styles .ml-xl-2,
.paragon-styles .mx-xl-2 {
    margin-left: 0.5rem !important;
  }
  .paragon-styles .m-xl-3 {
    margin: 1rem !important;
  }
  .paragon-styles .mt-xl-3,
.paragon-styles .my-xl-3 {
    margin-top: 1rem !important;
  }
  .paragon-styles .mr-xl-3,
.paragon-styles .mx-xl-3 {
    margin-right: 1rem !important;
  }
  .paragon-styles .mb-xl-3,
.paragon-styles .my-xl-3 {
    margin-bottom: 1rem !important;
  }
  .paragon-styles .ml-xl-3,
.paragon-styles .mx-xl-3 {
    margin-left: 1rem !important;
  }
  .paragon-styles .m-xl-4 {
    margin: 1.5rem !important;
  }
  .paragon-styles .mt-xl-4,
.paragon-styles .my-xl-4 {
    margin-top: 1.5rem !important;
  }
  .paragon-styles .mr-xl-4,
.paragon-styles .mx-xl-4 {
    margin-right: 1.5rem !important;
  }
  .paragon-styles .mb-xl-4,
.paragon-styles .my-xl-4 {
    margin-bottom: 1.5rem !important;
  }
  .paragon-styles .ml-xl-4,
.paragon-styles .mx-xl-4 {
    margin-left: 1.5rem !important;
  }
  .paragon-styles .m-xl-5 {
    margin: 3rem !important;
  }
  .paragon-styles .mt-xl-5,
.paragon-styles .my-xl-5 {
    margin-top: 3rem !important;
  }
  .paragon-styles .mr-xl-5,
.paragon-styles .mx-xl-5 {
    margin-right: 3rem !important;
  }
  .paragon-styles .mb-xl-5,
.paragon-styles .my-xl-5 {
    margin-bottom: 3rem !important;
  }
  .paragon-styles .ml-xl-5,
.paragon-styles .mx-xl-5 {
    margin-left: 3rem !important;
  }
  .paragon-styles .p-xl-0 {
    padding: 0 !important;
  }
  .paragon-styles .pt-xl-0,
.paragon-styles .py-xl-0 {
    padding-top: 0 !important;
  }
  .paragon-styles .pr-xl-0,
.paragon-styles .px-xl-0 {
    padding-right: 0 !important;
  }
  .paragon-styles .pb-xl-0,
.paragon-styles .py-xl-0 {
    padding-bottom: 0 !important;
  }
  .paragon-styles .pl-xl-0,
.paragon-styles .px-xl-0 {
    padding-left: 0 !important;
  }
  .paragon-styles .p-xl-1 {
    padding: 0.25rem !important;
  }
  .paragon-styles .pt-xl-1,
.paragon-styles .py-xl-1 {
    padding-top: 0.25rem !important;
  }
  .paragon-styles .pr-xl-1,
.paragon-styles .px-xl-1 {
    padding-right: 0.25rem !important;
  }
  .paragon-styles .pb-xl-1,
.paragon-styles .py-xl-1 {
    padding-bottom: 0.25rem !important;
  }
  .paragon-styles .pl-xl-1,
.paragon-styles .px-xl-1 {
    padding-left: 0.25rem !important;
  }
  .paragon-styles .p-xl-2 {
    padding: 0.5rem !important;
  }
  .paragon-styles .pt-xl-2,
.paragon-styles .py-xl-2 {
    padding-top: 0.5rem !important;
  }
  .paragon-styles .pr-xl-2,
.paragon-styles .px-xl-2 {
    padding-right: 0.5rem !important;
  }
  .paragon-styles .pb-xl-2,
.paragon-styles .py-xl-2 {
    padding-bottom: 0.5rem !important;
  }
  .paragon-styles .pl-xl-2,
.paragon-styles .px-xl-2 {
    padding-left: 0.5rem !important;
  }
  .paragon-styles .p-xl-3 {
    padding: 1rem !important;
  }
  .paragon-styles .pt-xl-3,
.paragon-styles .py-xl-3 {
    padding-top: 1rem !important;
  }
  .paragon-styles .pr-xl-3,
.paragon-styles .px-xl-3 {
    padding-right: 1rem !important;
  }
  .paragon-styles .pb-xl-3,
.paragon-styles .py-xl-3 {
    padding-bottom: 1rem !important;
  }
  .paragon-styles .pl-xl-3,
.paragon-styles .px-xl-3 {
    padding-left: 1rem !important;
  }
  .paragon-styles .p-xl-4 {
    padding: 1.5rem !important;
  }
  .paragon-styles .pt-xl-4,
.paragon-styles .py-xl-4 {
    padding-top: 1.5rem !important;
  }
  .paragon-styles .pr-xl-4,
.paragon-styles .px-xl-4 {
    padding-right: 1.5rem !important;
  }
  .paragon-styles .pb-xl-4,
.paragon-styles .py-xl-4 {
    padding-bottom: 1.5rem !important;
  }
  .paragon-styles .pl-xl-4,
.paragon-styles .px-xl-4 {
    padding-left: 1.5rem !important;
  }
  .paragon-styles .p-xl-5 {
    padding: 3rem !important;
  }
  .paragon-styles .pt-xl-5,
.paragon-styles .py-xl-5 {
    padding-top: 3rem !important;
  }
  .paragon-styles .pr-xl-5,
.paragon-styles .px-xl-5 {
    padding-right: 3rem !important;
  }
  .paragon-styles .pb-xl-5,
.paragon-styles .py-xl-5 {
    padding-bottom: 3rem !important;
  }
  .paragon-styles .pl-xl-5,
.paragon-styles .px-xl-5 {
    padding-left: 3rem !important;
  }
  .paragon-styles .m-xl-n1 {
    margin: -0.25rem !important;
  }
  .paragon-styles .mt-xl-n1,
.paragon-styles .my-xl-n1 {
    margin-top: -0.25rem !important;
  }
  .paragon-styles .mr-xl-n1,
.paragon-styles .mx-xl-n1 {
    margin-right: -0.25rem !important;
  }
  .paragon-styles .mb-xl-n1,
.paragon-styles .my-xl-n1 {
    margin-bottom: -0.25rem !important;
  }
  .paragon-styles .ml-xl-n1,
.paragon-styles .mx-xl-n1 {
    margin-left: -0.25rem !important;
  }
  .paragon-styles .m-xl-n2 {
    margin: -0.5rem !important;
  }
  .paragon-styles .mt-xl-n2,
.paragon-styles .my-xl-n2 {
    margin-top: -0.5rem !important;
  }
  .paragon-styles .mr-xl-n2,
.paragon-styles .mx-xl-n2 {
    margin-right: -0.5rem !important;
  }
  .paragon-styles .mb-xl-n2,
.paragon-styles .my-xl-n2 {
    margin-bottom: -0.5rem !important;
  }
  .paragon-styles .ml-xl-n2,
.paragon-styles .mx-xl-n2 {
    margin-left: -0.5rem !important;
  }
  .paragon-styles .m-xl-n3 {
    margin: -1rem !important;
  }
  .paragon-styles .mt-xl-n3,
.paragon-styles .my-xl-n3 {
    margin-top: -1rem !important;
  }
  .paragon-styles .mr-xl-n3,
.paragon-styles .mx-xl-n3 {
    margin-right: -1rem !important;
  }
  .paragon-styles .mb-xl-n3,
.paragon-styles .my-xl-n3 {
    margin-bottom: -1rem !important;
  }
  .paragon-styles .ml-xl-n3,
.paragon-styles .mx-xl-n3 {
    margin-left: -1rem !important;
  }
  .paragon-styles .m-xl-n4 {
    margin: -1.5rem !important;
  }
  .paragon-styles .mt-xl-n4,
.paragon-styles .my-xl-n4 {
    margin-top: -1.5rem !important;
  }
  .paragon-styles .mr-xl-n4,
.paragon-styles .mx-xl-n4 {
    margin-right: -1.5rem !important;
  }
  .paragon-styles .mb-xl-n4,
.paragon-styles .my-xl-n4 {
    margin-bottom: -1.5rem !important;
  }
  .paragon-styles .ml-xl-n4,
.paragon-styles .mx-xl-n4 {
    margin-left: -1.5rem !important;
  }
  .paragon-styles .m-xl-n5 {
    margin: -3rem !important;
  }
  .paragon-styles .mt-xl-n5,
.paragon-styles .my-xl-n5 {
    margin-top: -3rem !important;
  }
  .paragon-styles .mr-xl-n5,
.paragon-styles .mx-xl-n5 {
    margin-right: -3rem !important;
  }
  .paragon-styles .mb-xl-n5,
.paragon-styles .my-xl-n5 {
    margin-bottom: -3rem !important;
  }
  .paragon-styles .ml-xl-n5,
.paragon-styles .mx-xl-n5 {
    margin-left: -3rem !important;
  }
  .paragon-styles .m-xl-auto {
    margin: auto !important;
  }
  .paragon-styles .mt-xl-auto,
.paragon-styles .my-xl-auto {
    margin-top: auto !important;
  }
  .paragon-styles .mr-xl-auto,
.paragon-styles .mx-xl-auto {
    margin-right: auto !important;
  }
  .paragon-styles .mb-xl-auto,
.paragon-styles .my-xl-auto {
    margin-bottom: auto !important;
  }
  .paragon-styles .ml-xl-auto,
.paragon-styles .mx-xl-auto {
    margin-left: auto !important;
  }
}
.paragon-styles .stretched-link::after {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 1;
  pointer-events: auto;
  content: "";
  background-color: rgba(0, 0, 0, 0);
}
.paragon-styles .text-monospace {
  font-family: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace !important;
}
.paragon-styles .text-justify {
  text-align: justify !important;
}
.paragon-styles .text-wrap {
  white-space: normal !important;
}
.paragon-styles .text-nowrap {
  white-space: nowrap !important;
}
.paragon-styles .text-truncate {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}
.paragon-styles .text-left {
  text-align: left !important;
}
.paragon-styles .text-right {
  text-align: right !important;
}
.paragon-styles .text-center {
  text-align: center !important;
}
@media (min-width: 576px) {
  .paragon-styles .text-sm-left {
    text-align: left !important;
  }
  .paragon-styles .text-sm-right {
    text-align: right !important;
  }
  .paragon-styles .text-sm-center {
    text-align: center !important;
  }
}
@media (min-width: 768px) {
  .paragon-styles .text-md-left {
    text-align: left !important;
  }
  .paragon-styles .text-md-right {
    text-align: right !important;
  }
  .paragon-styles .text-md-center {
    text-align: center !important;
  }
}
@media (min-width: 992px) {
  .paragon-styles .text-lg-left {
    text-align: left !important;
  }
  .paragon-styles .text-lg-right {
    text-align: right !important;
  }
  .paragon-styles .text-lg-center {
    text-align: center !important;
  }
}
@media (min-width: 1200px) {
  .paragon-styles .text-xl-left {
    text-align: left !important;
  }
  .paragon-styles .text-xl-right {
    text-align: right !important;
  }
  .paragon-styles .text-xl-center {
    text-align: center !important;
  }
}
.paragon-styles .text-lowercase {
  text-transform: lowercase !important;
}
.paragon-styles .text-uppercase {
  text-transform: uppercase !important;
}
.paragon-styles .text-capitalize {
  text-transform: capitalize !important;
}
.paragon-styles .font-weight-light {
  font-weight: 300 !important;
}
.paragon-styles .font-weight-lighter {
  font-weight: lighter !important;
}
.paragon-styles .font-weight-normal {
  font-weight: 400 !important;
}
.paragon-styles .font-weight-bold {
  font-weight: 700 !important;
}
.paragon-styles .font-weight-bolder {
  font-weight: bolder !important;
}
.paragon-styles .font-italic {
  font-style: italic !important;
}
.paragon-styles .text-white {
  color: white !important;
}
.paragon-styles .text-primary {
  color: #80a9cc !important;
}
.paragon-styles a.text-primary:hover, .paragon-styles a.text-primary:focus {
  color: #4984b6 !important;
}
.paragon-styles .text-secondary {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary:hover, .paragon-styles a.text-secondary:focus {
  color: #494f54 !important;
}
.paragon-styles .text-success {
  color: #178253 !important;
}
.paragon-styles a.text-success:hover, .paragon-styles a.text-success:focus {
  color: #0c412a !important;
}
.paragon-styles .text-info {
  color: #006DAA !important;
}
.paragon-styles a.text-info:hover, .paragon-styles a.text-info:focus {
  color: #003c5e !important;
}
.paragon-styles .text-warning {
  color: #D69600 !important;
}
.paragon-styles a.text-warning:hover, .paragon-styles a.text-warning:focus {
  color: #8a6000 !important;
}
.paragon-styles .text-danger {
  color: #C32D3A !important;
}
.paragon-styles a.text-danger:hover, .paragon-styles a.text-danger:focus {
  color: #851f28 !important;
}
.paragon-styles .text-light {
  color: #eee !important;
}
.paragon-styles a.text-light:hover, .paragon-styles a.text-light:focus {
  color: #c8c8c8 !important;
}
.paragon-styles .text-dark {
  color: #495057 !important;
}
.paragon-styles a.text-dark:hover, .paragon-styles a.text-dark:focus {
  color: #262a2d !important;
}
.paragon-styles .text-brand {
  color: #80a9cc !important;
}
.paragon-styles a.text-brand:hover, .paragon-styles a.text-brand:focus {
  color: #4984b6 !important;
}
.paragon-styles .text-gray {
  color: #adb5bd !important;
}
.paragon-styles a.text-gray:hover, .paragon-styles a.text-gray:focus {
  color: #838f9b !important;
}
.paragon-styles .text-body {
  color: #212529 !important;
}
.paragon-styles .text-muted {
  color: #adb5bd !important;
}
.paragon-styles .text-black-50 {
  color: rgba(3, 3, 3, 0.5) !important;
}
.paragon-styles .text-white-50 {
  color: rgba(255, 255, 255, 0.5) !important;
}
.paragon-styles .text-hide {
  font: 0/0 a;
  color: transparent;
  text-shadow: none;
  background-color: transparent;
  border: 0;
}
.paragon-styles .text-decoration-none {
  text-decoration: none !important;
}
.paragon-styles .text-break {
  word-break: break-word !important;
  word-wrap: break-word !important;
}
.paragon-styles .text-reset {
  color: inherit !important;
}
.paragon-styles .visible {
  visibility: visible !important;
}
.paragon-styles .invisible {
  visibility: hidden !important;
}
@media print {
  .paragon-styles *,
.paragon-styles *::before,
.paragon-styles *::after {
    text-shadow: none !important;
    box-shadow: none !important;
  }
  .paragon-styles a:not(.btn) {
    text-decoration: underline;
  }
  .paragon-styles abbr[title]::after {
    content: " (" attr(title) ")";
  }
  .paragon-styles pre {
    white-space: pre-wrap !important;
  }
  .paragon-styles pre,
.paragon-styles blockquote {
    border: 1px solid #adb5bd;
    page-break-inside: avoid;
  }
  .paragon-styles thead {
    display: table-header-group;
  }
  .paragon-styles tr,
.paragon-styles img {
    page-break-inside: avoid;
  }
  .paragon-styles p,
.paragon-styles h2,
.paragon-styles h3 {
    orphans: 3;
    widows: 3;
  }
  .paragon-styles h2,
.paragon-styles h3 {
    page-break-after: avoid;
  }
  @page {
    .paragon-styles {
      size: a3;
    }
  }
  .paragon-styles body {
    min-width: 992px !important;
  }
  .paragon-styles .container {
    min-width: 992px !important;
  }
  .paragon-styles .navbar {
    display: none;
  }
  .paragon-styles .badge {
    border: 1px solid #030303;
  }
  .paragon-styles .table {
    border-collapse: collapse !important;
  }
  .paragon-styles .table td,
.paragon-styles .table th {
    background-color: white !important;
  }
  .paragon-styles .table-bordered th,
.paragon-styles .table-bordered td {
    border: 1px solid #dee2e6 !important;
  }
  .paragon-styles .table-dark {
    color: inherit;
  }
  .paragon-styles .table-dark th,
.paragon-styles .table-dark td,
.paragon-styles .table-dark thead th,
.paragon-styles .table-dark tbody + tbody {
    border-color: #e9ecef;
  }
  .paragon-styles .table .thead-dark th {
    color: inherit;
    border-color: #e9ecef;
  }
}
.paragon-styles .form-control.is-invalid.is-invalid-nodanger {
  border-color: #e9ecef;
}
.paragon-styles .form-inline label {
  margin-right: 5px;
}
.paragon-styles .pgn__avatar {
  border-radius: 100%;
  border: solid 1px #f7f7f7;
  display: inline-block;
  height: 3rem;
  overflow: hidden;
  width: 3rem;
  object-fit: cover;
}
.paragon-styles .pgn__avatar.pgn__avatar-xs {
  height: 1.5rem;
  width: 1.5rem;
}
.paragon-styles .pgn__avatar.pgn__avatar-sm {
  height: 2.25rem;
  width: 2.25rem;
}
.paragon-styles .pgn__avatar.pgn__avatar-md {
  height: 3rem;
  width: 3rem;
}
.paragon-styles .pgn__avatar.pgn__avatar-lg {
  height: 4rem;
  width: 4rem;
}
.paragon-styles .pgn__avatar.pgn__avatar-xl {
  height: 18.75rem;
  width: 18.75rem;
}
.paragon-styles .pgn__avatar-button-avatar {
  padding-left: 0.5em;
}
.paragon-styles .pgn__avatar-button-avatar.pgn__avatar-button-avatar-sm {
  padding-left: 0.5;
}
.paragon-styles .pgn__avatar-button-avatar.pgn__avatar-button-avatar-lg {
  padding-left: 0.5em;
}
.paragon-styles .pgn__avatar-button-avatar .pgn__avatar-button-avatar-wrap {
  height: 0px;
  vertical-align: middle;
  display: inline-flex;
  margin-right: 0.5em;
}
.paragon-styles .pgn__avatar-button-avatar.pgn__avatar-button-hide-label {
  padding-left: 4px;
  padding-right: 4px;
}
.paragon-styles .pgn__avatar-button-avatar.pgn__avatar-button-hide-label .pgn__avatar-button-avatar-wrap {
  margin-right: 0em;
}
.paragon-styles .pgn__avatar-button-avatar .pgn__avatar {
  transform: translateY(-50%);
  margin-top: -1px;
}
.paragon-styles .collapsible-card .collapsible-trigger, .paragon-styles .collapsible-card-lg .collapsible-trigger {
  padding: 0.45rem 0.75rem;
  border-radius: calc(0.3rem - 1px);
  border-bottom: 1px solid transparent;
  transition: border-color 100ms ease 150ms;
  cursor: pointer;
  display: flex;
  align-items: center;
  justify-content: space-between;
}
.paragon-styles .collapsible-card .collapsible-trigger > *, .paragon-styles .collapsible-card-lg .collapsible-trigger > * {
  margin-bottom: 0;
  margin-top: 0;
}
.paragon-styles .collapsible-card .collapsible-trigger[aria-expanded=true], .paragon-styles .collapsible-card-lg .collapsible-trigger[aria-expanded=true] {
  border-radius: calc(0.3rem - 1px) calc(0.3rem - 1px) 0 0;
  border-color: rgba(3, 3, 3, 0.125);
  transition: none;
}
.paragon-styles .collapsible-card .collapsible-body, .paragon-styles .collapsible-card-lg .collapsible-body {
  padding: 0.45rem 0.75rem;
}
.paragon-styles .collapsible-card .collapsible-body > *:last-child, .paragon-styles .collapsible-card-lg .collapsible-body > *:last-child {
  margin-bottom: 0;
}
.paragon-styles .collapsible-card-lg .collapsible-trigger {
  padding: 0.75rem 1.25rem;
}
.paragon-styles .collapsible-card-lg .collapsible-body {
  padding: 0.75rem 1.25rem;
}
.paragon-styles .collapsible-basic .collapsible-trigger {
  display: flex;
  cursor: pointer;
  align-items: center;
  text-decoration: underline;
  color: #80a9cc;
}
.paragon-styles .paragon-fieldset {
  margin-bottom: 1.5rem;
}
.paragon-styles .paragon-fieldset .form-control {
  height: auto;
}
.paragon-styles .paragon-fieldset fieldset legend {
  width: auto;
  margin-bottom: 0;
}
.paragon-styles .form-control.is-invalid.is-invalid-nodanger {
  border-color: #e9ecef;
}
.paragon-styles .form-control.is-invalid.is-invalid-nodanger:focus {
  color: #495057;
  background-color: white;
  border-color: #dbe7f1;
  outline: 0;
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .pgn__icon {
  display: block;
  height: 1.5rem;
  max-height: 100%;
  max-width: 100%;
  position: relative;
  width: 1.5rem;
}
.paragon-styles .pgn__icon svg {
  display: block;
  height: 100%;
  position: relative;
  max-height: 100%;
  max-width: 100%;
  width: 100%;
}
.paragon-styles .modal.show {
  position: fixed;
  background-color: transparent;
  max-height: 100%;
  width: 100%;
}
.paragon-styles .modal.show:focus .modal-dialog {
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .modal.is-ie11 {
  overflow-y: scroll;
  height: auto;
}
.paragon-styles .modal.is-ie11 .modal-content {
  height: auto;
  max-height: none;
}
.paragon-styles .modal-backdrop {
  background-color: rgba(0, 0, 0, 0.3);
}
.paragon-styles .modal-backdrop.fade {
  opacity: 0;
}
.paragon-styles .modal-backdrop.show {
  opacity: 1;
}
.paragon-styles .modal-dialog {
  height: 100%;
  margin: auto;
  padding: 0.5rem;
}
@media (min-width: 576px) {
  .paragon-styles .modal-dialog {
    padding: 1rem;
  }
}
.paragon-styles .modal-content {
  max-height: calc(100vh - (1rem * 2));
}
.paragon-styles .modal-content:focus {
  outline: 1px dotted;
  outline: 5px auto -webkit-focus-ring-color;
}
.paragon-styles .modal-header {
  flex: 0 0 auto;
}
.paragon-styles .modal-body {
  overflow: auto;
}
.paragon-styles .modal-footer {
  flex: 0 0 auto;
}
.paragon-styles .page-link {
  border-color: #e9ecef !important;
  border-radius: 0 !important;
}
.paragon-styles .page-link div {
  display: inline-block;
}
.paragon-styles .page-item > .btn {
  transition: none;
}
.paragon-styles .pgn__searchfield {
  transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
.paragon-styles .pgn__searchfield.has-focus {
  box-shadow: 0 0 0 0.2rem rgba(128, 169, 204, 0.25);
}
.paragon-styles .pgn__searchfield .form-control:focus {
  box-shadow: none;
}
.paragon-styles .pgn__searchfield .form-control::-ms-clear, .paragon-styles .pgn__searchfield .form-control::-ms-reveal {
  display: none;
  width: 0;
  height: 0;
}
.paragon-styles .pgn__searchfield .form-control::-webkit-search-decoration, .paragon-styles .pgn__searchfield .form-control::-webkit-search-cancel-button, .paragon-styles .pgn__searchfield .form-control::-webkit-search-results-button, .paragon-styles .pgn__searchfield .form-control::-webkit-search-results-decoration {
  display: none;
}
.paragon-styles .pgn__searchfield label.has-label-text {
  padding-left: 0.75rem;
}
.paragon-styles .pgn__stateful-btn-icon {
  margin-right: 0.5em;
}
.paragon-styles .btn-header {
  padding: 0;
  font-weight: 700;
}
.paragon-styles .table th, .paragon-styles .table td {
  word-wrap: break-word;
}
.paragon-styles .pgn__data-table-wrapper {
  border: 1px solid #e9ecef;
}
.paragon-styles .pgn__bulk-actions {
  display: flex;
}
.paragon-styles .pgn__data-table-container {
  width: 100%;
  display: block;
  overflow-x: auto;
  -webkit-overflow-scrolling: touch;
  border-bottom: 1px solid #e9ecef;
}
.paragon-styles .pgn__data-table {
  width: 100%;
  border-bottom: 1px solid #e9ecef;
}
.paragon-styles .pgn__data-table.is-striped tr:nth-child(even) {
  background-color: #fbfbfb;
}
.paragon-styles .pgn__data-table th {
  background-color: #f7f7f7;
  padding: 8px 14px;
}
.paragon-styles .pgn__data-table td {
  padding: 0.75rem;
  line-height: 24px;
}
.paragon-styles .pgn__data-table tr.is-selected {
  background-color: #f0f6fa !important;
}
.paragon-styles .pgn__data-table-cell-wrap {
  max-width: 20vw;
  display: block;
  white-space: normal;
  overflow: hidden;
  text-overflow: ellipsis;
}
.paragon-styles .pgn__data-table-empty {
  padding: 14px 14px;
}
.paragon-styles .pgn__data-table-actions {
  display: flex;
  justify-content: space-between;
  padding-bottom: 8px;
}
.paragon-styles .pgn__data-table-status-bar {
  padding: 14px 14px 0 14px;
}
.paragon-styles .pgn__data-table-filters {
  display: flex;
}
.paragon-styles .pgn__data-table-filters-breakout-filter {
  align-self: flex-end;
}
.paragon-styles .pgn__data-table-filters-dropdown-item {
  padding: 8px 14px;
  min-width: 15em;
}
.paragon-styles .pgn__data-table-status {
  display: flex;
  padding: 0px 0px 8px 0px;
  justify-content: space-between;
}
.paragon-styles .pgn__data-table-status .pgn__data-table-status-left {
  align-self: flex-end;
}
.paragon-styles .pgn__data-table-footer {
  display: flex;
  justify-content: space-between;
  padding: 14px 14px;
}
.paragon-styles .pgn__data-table-footer .pgn__data-table-footer-row-status {
  align-self: flex-end;
}
.paragon-styles .pgn__data-table-pagination {
  display: flex;
}
.paragon-styles .pgn__data-table-pagination .pgn__page-count {
  align-self: flex-end;
}
.paragon-styles .pgn-transition-replace-group {
  transition: height 300ms ease;
}
.paragon-styles .pgn__transition-replace-enter {
  opacity: 0;
}
.paragon-styles .pgn__transition-replace-enter-active {
  opacity: 1;
  z-index: 1;
  transition: opacity 300ms ease;
}
.paragon-styles .pgn__transition-replace-exit {
  opacity: 1;
}
.paragon-styles .pgn__transition-replace-exit-active {
  opacity: 0;
  transition: opacity 300ms ease;
}
.paragon-styles .form-control.is-invalid ~ .invalid-feedback-nodanger {
  color: #212529;
}
.paragon-styles .fa-icon-spacing {
  padding: 0px 5px 0px 0px;
}
.paragon-styles .nav-tabs .nav-link {
  background: transparent;
}
.paragon-styles .iconbutton-hover {
  width: 2rem;
  height: 2rem;
  border-radius: 2rem;
  display: flex;
  flex-shrink: 0;
  justify-content: center;
  align-items: center;
  text-align: center;
  position: relative;
  border: none;
  cursor: pointer;
}
.paragon-styles .iconbutton-hover:active {
  transform: scale(0.9);
}
.paragon-styles .iconbutton-hover__primary {
  background-color: transparent;
  color: #80a9cc;
}
.paragon-styles .iconbutton-hover__primary.focus, .paragon-styles .iconbutton-hover__primary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__primary.focus:before, .paragon-styles .iconbutton-hover__primary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #80a9cc;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__primary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__primary.focus.btn:before, .paragon-styles .iconbutton-hover__primary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__primary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__primary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__primary.focus.btn:before, .paragon-styles .iconbutton-hover__primary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__primary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__primary.focus:active:before, .paragon-styles .iconbutton-hover__primary.focus.active:before, .paragon-styles .iconbutton-hover__primary:focus:active:before, .paragon-styles .iconbutton-hover__primary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__primary.focus:disabled:before, .paragon-styles .iconbutton-hover__primary.focus.disabled:before, .paragon-styles .iconbutton-hover__primary:focus:disabled:before, .paragon-styles .iconbutton-hover__primary:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__primary:hover {
  background-color: #80a9cc;
  color: white;
}
.paragon-styles .iconbutton-hover__primary--invert {
  background-color: transparent;
  color: white;
}
.paragon-styles .iconbutton-hover__primary--invert.focus, .paragon-styles .iconbutton-hover__primary--invert:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__primary--invert.focus:before, .paragon-styles .iconbutton-hover__primary--invert:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__primary--invert.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__primary--invert.focus.btn:before, .paragon-styles .iconbutton-hover__primary--invert:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__primary--invert.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__primary--invert.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__primary--invert.focus.btn:before, .paragon-styles .iconbutton-hover__primary--invert:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__primary--invert.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__primary--invert.focus:active:before, .paragon-styles .iconbutton-hover__primary--invert.focus.active:before, .paragon-styles .iconbutton-hover__primary--invert:focus:active:before, .paragon-styles .iconbutton-hover__primary--invert:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__primary--invert.focus:disabled:before, .paragon-styles .iconbutton-hover__primary--invert.focus.disabled:before, .paragon-styles .iconbutton-hover__primary--invert:focus:disabled:before, .paragon-styles .iconbutton-hover__primary--invert:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__primary--invert:hover {
  background-color: white;
  color: #80a9cc;
}
.paragon-styles .iconbutton-hover__secondary {
  background-color: transparent;
  color: #6c757d;
}
.paragon-styles .iconbutton-hover__secondary.focus, .paragon-styles .iconbutton-hover__secondary:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__secondary.focus:before, .paragon-styles .iconbutton-hover__secondary:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #6c757d;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__secondary.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__secondary.focus.btn:before, .paragon-styles .iconbutton-hover__secondary:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__secondary.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__secondary.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__secondary.focus.btn:before, .paragon-styles .iconbutton-hover__secondary:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__secondary.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__secondary.focus:active:before, .paragon-styles .iconbutton-hover__secondary.focus.active:before, .paragon-styles .iconbutton-hover__secondary:focus:active:before, .paragon-styles .iconbutton-hover__secondary:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__secondary.focus:disabled:before, .paragon-styles .iconbutton-hover__secondary.focus.disabled:before, .paragon-styles .iconbutton-hover__secondary:focus:disabled:before, .paragon-styles .iconbutton-hover__secondary:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__secondary:hover {
  background-color: #6c757d;
  color: white;
}
.paragon-styles .iconbutton-hover__secondary--invert {
  background-color: transparent;
  color: white;
}
.paragon-styles .iconbutton-hover__secondary--invert.focus, .paragon-styles .iconbutton-hover__secondary--invert:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__secondary--invert.focus:before, .paragon-styles .iconbutton-hover__secondary--invert:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__secondary--invert.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__secondary--invert.focus.btn:before, .paragon-styles .iconbutton-hover__secondary--invert:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__secondary--invert.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__secondary--invert.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__secondary--invert.focus.btn:before, .paragon-styles .iconbutton-hover__secondary--invert:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__secondary--invert.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__secondary--invert.focus:active:before, .paragon-styles .iconbutton-hover__secondary--invert.focus.active:before, .paragon-styles .iconbutton-hover__secondary--invert:focus:active:before, .paragon-styles .iconbutton-hover__secondary--invert:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__secondary--invert.focus:disabled:before, .paragon-styles .iconbutton-hover__secondary--invert.focus.disabled:before, .paragon-styles .iconbutton-hover__secondary--invert:focus:disabled:before, .paragon-styles .iconbutton-hover__secondary--invert:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__secondary--invert:hover {
  background-color: white;
  color: #6c757d;
}
.paragon-styles .iconbutton-hover__success {
  background-color: transparent;
  color: #178253;
}
.paragon-styles .iconbutton-hover__success.focus, .paragon-styles .iconbutton-hover__success:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__success.focus:before, .paragon-styles .iconbutton-hover__success:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #178253;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__success.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__success.focus.btn:before, .paragon-styles .iconbutton-hover__success:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__success.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__success.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__success.focus.btn:before, .paragon-styles .iconbutton-hover__success:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__success.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__success.focus:active:before, .paragon-styles .iconbutton-hover__success.focus.active:before, .paragon-styles .iconbutton-hover__success:focus:active:before, .paragon-styles .iconbutton-hover__success:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__success.focus:disabled:before, .paragon-styles .iconbutton-hover__success.focus.disabled:before, .paragon-styles .iconbutton-hover__success:focus:disabled:before, .paragon-styles .iconbutton-hover__success:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__success:hover {
  background-color: #178253;
  color: white;
}
.paragon-styles .iconbutton-hover__success--invert {
  background-color: transparent;
  color: white;
}
.paragon-styles .iconbutton-hover__success--invert.focus, .paragon-styles .iconbutton-hover__success--invert:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__success--invert.focus:before, .paragon-styles .iconbutton-hover__success--invert:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__success--invert.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__success--invert.focus.btn:before, .paragon-styles .iconbutton-hover__success--invert:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__success--invert.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__success--invert.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__success--invert.focus.btn:before, .paragon-styles .iconbutton-hover__success--invert:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__success--invert.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__success--invert.focus:active:before, .paragon-styles .iconbutton-hover__success--invert.focus.active:before, .paragon-styles .iconbutton-hover__success--invert:focus:active:before, .paragon-styles .iconbutton-hover__success--invert:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__success--invert.focus:disabled:before, .paragon-styles .iconbutton-hover__success--invert.focus.disabled:before, .paragon-styles .iconbutton-hover__success--invert:focus:disabled:before, .paragon-styles .iconbutton-hover__success--invert:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__success--invert:hover {
  background-color: white;
  color: #178253;
}
.paragon-styles .iconbutton-hover__warning {
  background-color: transparent;
  color: #D69600;
}
.paragon-styles .iconbutton-hover__warning.focus, .paragon-styles .iconbutton-hover__warning:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__warning.focus:before, .paragon-styles .iconbutton-hover__warning:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #D69600;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__warning.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__warning.focus.btn:before, .paragon-styles .iconbutton-hover__warning:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__warning.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__warning.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__warning.focus.btn:before, .paragon-styles .iconbutton-hover__warning:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__warning.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__warning.focus:active:before, .paragon-styles .iconbutton-hover__warning.focus.active:before, .paragon-styles .iconbutton-hover__warning:focus:active:before, .paragon-styles .iconbutton-hover__warning:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__warning.focus:disabled:before, .paragon-styles .iconbutton-hover__warning.focus.disabled:before, .paragon-styles .iconbutton-hover__warning:focus:disabled:before, .paragon-styles .iconbutton-hover__warning:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__warning:hover {
  background-color: #D69600;
  color: white;
}
.paragon-styles .iconbutton-hover__warning--invert {
  background-color: transparent;
  color: white;
}
.paragon-styles .iconbutton-hover__warning--invert.focus, .paragon-styles .iconbutton-hover__warning--invert:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__warning--invert.focus:before, .paragon-styles .iconbutton-hover__warning--invert:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__warning--invert.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__warning--invert.focus.btn:before, .paragon-styles .iconbutton-hover__warning--invert:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__warning--invert.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__warning--invert.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__warning--invert.focus.btn:before, .paragon-styles .iconbutton-hover__warning--invert:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__warning--invert.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__warning--invert.focus:active:before, .paragon-styles .iconbutton-hover__warning--invert.focus.active:before, .paragon-styles .iconbutton-hover__warning--invert:focus:active:before, .paragon-styles .iconbutton-hover__warning--invert:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__warning--invert.focus:disabled:before, .paragon-styles .iconbutton-hover__warning--invert.focus.disabled:before, .paragon-styles .iconbutton-hover__warning--invert:focus:disabled:before, .paragon-styles .iconbutton-hover__warning--invert:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__warning--invert:hover {
  background-color: white;
  color: #D69600;
}
.paragon-styles .iconbutton-hover__danger {
  background-color: transparent;
  color: #C32D3A;
}
.paragon-styles .iconbutton-hover__danger.focus, .paragon-styles .iconbutton-hover__danger:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__danger.focus:before, .paragon-styles .iconbutton-hover__danger:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #C32D3A;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__danger.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__danger.focus.btn:before, .paragon-styles .iconbutton-hover__danger:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__danger.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__danger.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__danger.focus.btn:before, .paragon-styles .iconbutton-hover__danger:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__danger.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__danger.focus:active:before, .paragon-styles .iconbutton-hover__danger.focus.active:before, .paragon-styles .iconbutton-hover__danger:focus:active:before, .paragon-styles .iconbutton-hover__danger:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__danger.focus:disabled:before, .paragon-styles .iconbutton-hover__danger.focus.disabled:before, .paragon-styles .iconbutton-hover__danger:focus:disabled:before, .paragon-styles .iconbutton-hover__danger:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__danger:hover {
  background-color: #C32D3A;
  color: white;
}
.paragon-styles .iconbutton-hover__danger--invert {
  background-color: transparent;
  color: white;
}
.paragon-styles .iconbutton-hover__danger--invert.focus, .paragon-styles .iconbutton-hover__danger--invert:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__danger--invert.focus:before, .paragon-styles .iconbutton-hover__danger--invert:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__danger--invert.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__danger--invert.focus.btn:before, .paragon-styles .iconbutton-hover__danger--invert:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__danger--invert.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__danger--invert.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__danger--invert.focus.btn:before, .paragon-styles .iconbutton-hover__danger--invert:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__danger--invert.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__danger--invert.focus:active:before, .paragon-styles .iconbutton-hover__danger--invert.focus.active:before, .paragon-styles .iconbutton-hover__danger--invert:focus:active:before, .paragon-styles .iconbutton-hover__danger--invert:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__danger--invert.focus:disabled:before, .paragon-styles .iconbutton-hover__danger--invert.focus.disabled:before, .paragon-styles .iconbutton-hover__danger--invert:focus:disabled:before, .paragon-styles .iconbutton-hover__danger--invert:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__danger--invert:hover {
  background-color: white;
  color: #C32D3A;
}
.paragon-styles .iconbutton-hover__light {
  background-color: transparent;
  color: #eee;
}
.paragon-styles .iconbutton-hover__light.focus, .paragon-styles .iconbutton-hover__light:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__light.focus:before, .paragon-styles .iconbutton-hover__light:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #eee;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__light.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__light.focus.btn:before, .paragon-styles .iconbutton-hover__light:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__light.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__light.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__light.focus.btn:before, .paragon-styles .iconbutton-hover__light:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__light.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__light.focus:active:before, .paragon-styles .iconbutton-hover__light.focus.active:before, .paragon-styles .iconbutton-hover__light:focus:active:before, .paragon-styles .iconbutton-hover__light:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__light.focus:disabled:before, .paragon-styles .iconbutton-hover__light.focus.disabled:before, .paragon-styles .iconbutton-hover__light:focus:disabled:before, .paragon-styles .iconbutton-hover__light:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__light:hover {
  background-color: #eee;
  color: white;
}
.paragon-styles .iconbutton-hover__light--invert {
  background-color: transparent;
  color: white;
}
.paragon-styles .iconbutton-hover__light--invert.focus, .paragon-styles .iconbutton-hover__light--invert:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__light--invert.focus:before, .paragon-styles .iconbutton-hover__light--invert:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__light--invert.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__light--invert.focus.btn:before, .paragon-styles .iconbutton-hover__light--invert:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__light--invert.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__light--invert.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__light--invert.focus.btn:before, .paragon-styles .iconbutton-hover__light--invert:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__light--invert.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__light--invert.focus:active:before, .paragon-styles .iconbutton-hover__light--invert.focus.active:before, .paragon-styles .iconbutton-hover__light--invert:focus:active:before, .paragon-styles .iconbutton-hover__light--invert:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__light--invert.focus:disabled:before, .paragon-styles .iconbutton-hover__light--invert.focus.disabled:before, .paragon-styles .iconbutton-hover__light--invert:focus:disabled:before, .paragon-styles .iconbutton-hover__light--invert:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__light--invert:hover {
  background-color: white;
  color: #eee;
}
.paragon-styles .iconbutton-hover__dark {
  background-color: transparent;
  color: #495057;
}
.paragon-styles .iconbutton-hover__dark.focus, .paragon-styles .iconbutton-hover__dark:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__dark.focus:before, .paragon-styles .iconbutton-hover__dark:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px #495057;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__dark.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__dark.focus.btn:before, .paragon-styles .iconbutton-hover__dark:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__dark.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__dark.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__dark.focus.btn:before, .paragon-styles .iconbutton-hover__dark:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__dark.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__dark.focus:active:before, .paragon-styles .iconbutton-hover__dark.focus.active:before, .paragon-styles .iconbutton-hover__dark:focus:active:before, .paragon-styles .iconbutton-hover__dark:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__dark.focus:disabled:before, .paragon-styles .iconbutton-hover__dark.focus.disabled:before, .paragon-styles .iconbutton-hover__dark:focus:disabled:before, .paragon-styles .iconbutton-hover__dark:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__dark:hover {
  background-color: #495057;
  color: white;
}
.paragon-styles .iconbutton-hover__dark--invert {
  background-color: transparent;
  color: white;
}
.paragon-styles .iconbutton-hover__dark--invert.focus, .paragon-styles .iconbutton-hover__dark--invert:focus {
  position: relative;
  outline: 0;
  box-shadow: none;
}
.paragon-styles .iconbutton-hover__dark--invert.focus:before, .paragon-styles .iconbutton-hover__dark--invert:focus:before {
  content: "";
  position: absolute;
  top: -6px;
  right: -6px;
  bottom: -6px;
  left: -6px;
  border: solid 3px white;
  border-radius: calc(0.3rem + 4px);
}
.paragon-styles .iconbutton-hover__dark--invert.focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__dark--invert.focus.btn:before, .paragon-styles .iconbutton-hover__dark--invert:focus.btn-lg:before, .paragon-styles .btn-group-lg > .iconbutton-hover__dark--invert.btn:focus:before {
  border-radius: calc(0.35rem + 4px);
}
.paragon-styles .iconbutton-hover__dark--invert.focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__dark--invert.focus.btn:before, .paragon-styles .iconbutton-hover__dark--invert:focus.btn-sm:before, .paragon-styles .btn-group-sm > .iconbutton-hover__dark--invert.btn:focus:before {
  border-radius: calc(0.2rem + 4px);
}
.paragon-styles .iconbutton-hover__dark--invert.focus:active:before, .paragon-styles .iconbutton-hover__dark--invert.focus.active:before, .paragon-styles .iconbutton-hover__dark--invert:focus:active:before, .paragon-styles .iconbutton-hover__dark--invert:focus.active:before {
  opacity: 0.75;
}
.paragon-styles .iconbutton-hover__dark--invert.focus:disabled:before, .paragon-styles .iconbutton-hover__dark--invert.focus.disabled:before, .paragon-styles .iconbutton-hover__dark--invert:focus:disabled:before, .paragon-styles .iconbutton-hover__dark--invert:focus.disabled:before {
  display: none;
}
.paragon-styles .iconbutton-hover__dark--invert:hover {
  background-color: white;
  color: #495057;
}
.paragon-styles .toast {
  background-color: #495057;
  box-shadow: 0.125rem 0.125rem 0.25rem 0 rgba(3, 3, 3, 0.25);
  margin: 0;
  min-width: 400px;
  padding: 1.25rem;
  position: relative;
  border-radius: 0.1875rem;
  z-index: 2;
}
.paragon-styles .toast .toast-header-btn-container {
  margin: -0.25rem -0.5rem;
  align-self: flex-start;
}
.paragon-styles .toast .btn {
  margin-top: 0.75rem;
  align-self: flex-start;
}
.paragon-styles .toast .toast-header {
  align-items: center;
  border-bottom: 0;
  justify-content: space-between;
  padding: 0;
}
.paragon-styles .toast .toast-header p {
  font-size: 87.5%;
  margin: 0;
  max-width: 320px;
}
.paragon-styles .toast .toast-header + .btn {
  margin-top: 1rem;
}
@media only screen and (max-width: 768px) {
  .paragon-styles .toast {
    max-width: 100%;
  }
}
@media only screen and (min-width: 768px) {
  .paragon-styles .toast {
    bottom: 0.625rem;
    left: 0.625rem;
    max-width: 400px;
  }
}
.paragon-styles .toast-container {
  bottom: 0.625rem;
  left: 0.625rem;
  position: fixed;
  z-index: 2;
}
@media only screen and (max-width: 768px) {
  .paragon-styles .toast-container {
    right: 0.625rem;
  }
}
@media (max-width: 576px) {
  .paragon-styles .display-1,
.paragon-styles .display-2,
.paragon-styles .display-3,
.paragon-styles .display-4,
.paragon-styles h1, .paragon-styles .h1 {
    font-size: 2rem;
  }
  .paragon-styles h2, .paragon-styles .h2 {
    font-size: 1.75rem;
  }
  .paragon-styles h3, .paragon-styles .h3 {
    font-size: 1.5rem;
  }
  .paragon-styles h4, .paragon-styles .h4 {
    font-size: 1.375rem;
  }
  .paragon-styles h5, .paragon-styles .h5 {
    font-size: 1.25rem;
  }
  .paragon-styles h6, .paragon-styles .h6 {
    font-size: 1rem;
  }
}
.paragon-styles .bg-primary-100 {
  background-color: #f7fafc !important;
}
.paragon-styles a.bg-primary-100:hover, .paragon-styles a.bg-primary-100:focus,
.paragon-styles button.bg-primary-100:hover,
.paragon-styles button.bg-primary-100:focus {
  background-color: #d2e3ee !important;
}
.paragon-styles .text-primary-100 {
  color: #f7fafc !important;
}
.paragon-styles a.text-primary-100:hover, .paragon-styles a.text-primary-100:focus {
  color: #bfd7e7 !important;
}
.paragon-styles .border-primary-100 {
  border-color: #f7fafc !important;
}
.paragon-styles .bg-primary-200 {
  background-color: #dfeaf2 !important;
}
.paragon-styles a.bg-primary-200:hover, .paragon-styles a.bg-primary-200:focus,
.paragon-styles button.bg-primary-200:hover,
.paragon-styles button.bg-primary-200:focus {
  background-color: #bbd2e3 !important;
}
.paragon-styles .text-primary-200 {
  color: #dfeaf2 !important;
}
.paragon-styles a.text-primary-200:hover, .paragon-styles a.text-primary-200:focus {
  color: #a9c6dc !important;
}
.paragon-styles .border-primary-200 {
  border-color: #dfeaf2 !important;
}
.paragon-styles .bg-primary-300 {
  background-color: #c0d4e6 !important;
}
.paragon-styles a.bg-primary-300:hover, .paragon-styles a.bg-primary-300:focus,
.paragon-styles button.bg-primary-300:hover,
.paragon-styles button.bg-primary-300:focus {
  background-color: #9bbbd8 !important;
}
.paragon-styles .text-primary-300 {
  color: #c0d4e6 !important;
}
.paragon-styles a.text-primary-300:hover, .paragon-styles a.text-primary-300:focus {
  color: #89afd0 !important;
}
.paragon-styles .border-primary-300 {
  border-color: #c0d4e6 !important;
}
.paragon-styles .bg-primary-400 {
  background-color: #a0bfd9 !important;
}
.paragon-styles a.bg-primary-400:hover, .paragon-styles a.bg-primary-400:focus,
.paragon-styles button.bg-primary-400:hover,
.paragon-styles button.bg-primary-400:focus {
  background-color: #7ca6ca !important;
}
.paragon-styles .text-primary-400 {
  color: #a0bfd9 !important;
}
.paragon-styles a.text-primary-400:hover, .paragon-styles a.text-primary-400:focus {
  color: #699ac3 !important;
}
.paragon-styles .border-primary-400 {
  border-color: #a0bfd9 !important;
}
.paragon-styles .bg-primary-500 {
  background-color: #80a9cc !important;
}
.paragon-styles a.bg-primary-500:hover, .paragon-styles a.bg-primary-500:focus,
.paragon-styles button.bg-primary-500:hover,
.paragon-styles button.bg-primary-500:focus {
  background-color: #5c90bd !important;
}
.paragon-styles .text-primary-500 {
  color: #80a9cc !important;
}
.paragon-styles a.text-primary-500:hover, .paragon-styles a.text-primary-500:focus {
  color: #4984b6 !important;
}
.paragon-styles .border-primary-500 {
  border-color: #80a9cc !important;
}
.paragon-styles .bg-primary-600 {
  background-color: #7398b8 !important;
}
.paragon-styles a.bg-primary-600:hover, .paragon-styles a.bg-primary-600:focus,
.paragon-styles button.bg-primary-600:hover,
.paragon-styles button.bg-primary-600:focus {
  background-color: #537fa5 !important;
}
.paragon-styles .text-primary-600 {
  color: #7398b8 !important;
}
.paragon-styles a.text-primary-600:hover, .paragon-styles a.text-primary-600:focus {
  color: #4b7294 !important;
}
.paragon-styles .border-primary-600 {
  border-color: #7398b8 !important;
}
.paragon-styles .bg-primary-700 {
  background-color: #6687a3 !important;
}
.paragon-styles a.bg-primary-700:hover, .paragon-styles a.bg-primary-700:focus,
.paragon-styles button.bg-primary-700:hover,
.paragon-styles button.bg-primary-700:focus {
  background-color: #506d86 !important;
}
.paragon-styles .text-primary-700 {
  color: #6687a3 !important;
}
.paragon-styles a.text-primary-700:hover, .paragon-styles a.text-primary-700:focus {
  color: #476076 !important;
}
.paragon-styles .border-primary-700 {
  border-color: #6687a3 !important;
}
.paragon-styles .bg-primary-800 {
  background-color: #607f99 !important;
}
.paragon-styles a.bg-primary-800:hover, .paragon-styles a.bg-primary-800:focus,
.paragon-styles button.bg-primary-800:hover,
.paragon-styles button.bg-primary-800:focus {
  background-color: #4c657a !important;
}
.paragon-styles .text-primary-800 {
  color: #607f99 !important;
}
.paragon-styles a.text-primary-800:hover, .paragon-styles a.text-primary-800:focus {
  color: #43586a !important;
}
.paragon-styles .border-primary-800 {
  border-color: #607f99 !important;
}
.paragon-styles .bg-primary-900 {
  background-color: #5a768f !important;
}
.paragon-styles a.bg-primary-900:hover, .paragon-styles a.bg-primary-900:focus,
.paragon-styles button.bg-primary-900:hover,
.paragon-styles button.bg-primary-900:focus {
  background-color: #465c70 !important;
}
.paragon-styles .text-primary-900 {
  color: #5a768f !important;
}
.paragon-styles a.text-primary-900:hover, .paragon-styles a.text-primary-900:focus {
  color: #3c4f60 !important;
}
.paragon-styles .border-primary-900 {
  border-color: #5a768f !important;
}
.paragon-styles .bg-secondary-100 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-100:hover, .paragon-styles a.bg-secondary-100:focus,
.paragon-styles button.bg-secondary-100:hover,
.paragon-styles button.bg-secondary-100:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-100 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-100:hover, .paragon-styles a.text-secondary-100:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-100 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-200 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-200:hover, .paragon-styles a.bg-secondary-200:focus,
.paragon-styles button.bg-secondary-200:hover,
.paragon-styles button.bg-secondary-200:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-200 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-200:hover, .paragon-styles a.text-secondary-200:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-200 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-300 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-300:hover, .paragon-styles a.bg-secondary-300:focus,
.paragon-styles button.bg-secondary-300:hover,
.paragon-styles button.bg-secondary-300:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-300 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-300:hover, .paragon-styles a.text-secondary-300:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-300 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-400 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-400:hover, .paragon-styles a.bg-secondary-400:focus,
.paragon-styles button.bg-secondary-400:hover,
.paragon-styles button.bg-secondary-400:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-400 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-400:hover, .paragon-styles a.text-secondary-400:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-400 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-500 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-500:hover, .paragon-styles a.bg-secondary-500:focus,
.paragon-styles button.bg-secondary-500:hover,
.paragon-styles button.bg-secondary-500:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-500 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-500:hover, .paragon-styles a.text-secondary-500:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-500 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-600 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-600:hover, .paragon-styles a.bg-secondary-600:focus,
.paragon-styles button.bg-secondary-600:hover,
.paragon-styles button.bg-secondary-600:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-600 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-600:hover, .paragon-styles a.text-secondary-600:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-600 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-700 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-700:hover, .paragon-styles a.bg-secondary-700:focus,
.paragon-styles button.bg-secondary-700:hover,
.paragon-styles button.bg-secondary-700:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-700 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-700:hover, .paragon-styles a.text-secondary-700:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-700 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-800 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-800:hover, .paragon-styles a.bg-secondary-800:focus,
.paragon-styles button.bg-secondary-800:hover,
.paragon-styles button.bg-secondary-800:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-800 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-800:hover, .paragon-styles a.text-secondary-800:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-800 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-secondary-900 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-secondary-900:hover, .paragon-styles a.bg-secondary-900:focus,
.paragon-styles button.bg-secondary-900:hover,
.paragon-styles button.bg-secondary-900:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-secondary-900 {
  color: #6c757d !important;
}
.paragon-styles a.text-secondary-900:hover, .paragon-styles a.text-secondary-900:focus {
  color: #494f54 !important;
}
.paragon-styles .border-secondary-900 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-success-100 {
  background-color: #f1f8f5 !important;
}
.paragon-styles a.bg-success-100:hover, .paragon-styles a.bg-success-100:focus,
.paragon-styles button.bg-success-100:hover,
.paragon-styles button.bg-success-100:focus {
  background-color: #cfe7dd !important;
}
.paragon-styles .text-success-100 {
  color: #f1f8f5 !important;
}
.paragon-styles a.text-success-100:hover, .paragon-styles a.text-success-100:focus {
  color: #bedfd1 !important;
}
.paragon-styles .border-success-100 {
  border-color: #f1f8f5 !important;
}
.paragon-styles .bg-success-200 {
  background-color: #c5e0d4 !important;
}
.paragon-styles a.bg-success-200:hover, .paragon-styles a.bg-success-200:focus,
.paragon-styles button.bg-success-200:hover,
.paragon-styles button.bg-success-200:focus {
  background-color: #a4cebb !important;
}
.paragon-styles .text-success-200 {
  color: #c5e0d4 !important;
}
.paragon-styles a.text-success-200:hover, .paragon-styles a.text-success-200:focus {
  color: #93c5af !important;
}
.paragon-styles .border-success-200 {
  border-color: #c5e0d4 !important;
}
.paragon-styles .bg-success-300 {
  background-color: #8bc1a9 !important;
}
.paragon-styles a.bg-success-300:hover, .paragon-styles a.bg-success-300:focus,
.paragon-styles button.bg-success-300:hover,
.paragon-styles button.bg-success-300:focus {
  background-color: #6aaf90 !important;
}
.paragon-styles .text-success-300 {
  color: #8bc1a9 !important;
}
.paragon-styles a.text-success-300:hover, .paragon-styles a.text-success-300:focus {
  color: #59a684 !important;
}
.paragon-styles .border-success-300 {
  border-color: #8bc1a9 !important;
}
.paragon-styles .bg-success-400 {
  background-color: #51a17e !important;
}
.paragon-styles a.bg-success-400:hover, .paragon-styles a.bg-success-400:focus,
.paragon-styles button.bg-success-400:hover,
.paragon-styles button.bg-success-400:focus {
  background-color: #407f63 !important;
}
.paragon-styles .text-success-400 {
  color: #51a17e !important;
}
.paragon-styles a.text-success-400:hover, .paragon-styles a.text-success-400:focus {
  color: #376e56 !important;
}
.paragon-styles .border-success-400 {
  border-color: #51a17e !important;
}
.paragon-styles .bg-success-500 {
  background-color: #178253 !important;
}
.paragon-styles a.bg-success-500:hover, .paragon-styles a.bg-success-500:focus,
.paragon-styles button.bg-success-500:hover,
.paragon-styles button.bg-success-500:focus {
  background-color: #0f5737 !important;
}
.paragon-styles .text-success-500 {
  color: #178253 !important;
}
.paragon-styles a.text-success-500:hover, .paragon-styles a.text-success-500:focus {
  color: #0c412a !important;
}
.paragon-styles .border-success-500 {
  border-color: #178253 !important;
}
.paragon-styles .bg-success-600 {
  background-color: #15754b !important;
}
.paragon-styles a.bg-success-600:hover, .paragon-styles a.bg-success-600:focus,
.paragon-styles button.bg-success-600:hover,
.paragon-styles button.bg-success-600:focus {
  background-color: #0d4a2f !important;
}
.paragon-styles .text-success-600 {
  color: #15754b !important;
}
.paragon-styles a.text-success-600:hover, .paragon-styles a.text-success-600:focus {
  color: #093421 !important;
}
.paragon-styles .border-success-600 {
  border-color: #15754b !important;
}
.paragon-styles .bg-success-700 {
  background-color: #126842 !important;
}
.paragon-styles a.bg-success-700:hover, .paragon-styles a.bg-success-700:focus,
.paragon-styles button.bg-success-700:hover,
.paragon-styles button.bg-success-700:focus {
  background-color: #0a3d26 !important;
}
.paragon-styles .text-success-700 {
  color: #126842 !important;
}
.paragon-styles a.text-success-700:hover, .paragon-styles a.text-success-700:focus {
  color: #072719 !important;
}
.paragon-styles .border-success-700 {
  border-color: #126842 !important;
}
.paragon-styles .bg-success-800 {
  background-color: #11623e !important;
}
.paragon-styles a.bg-success-800:hover, .paragon-styles a.bg-success-800:focus,
.paragon-styles button.bg-success-800:hover,
.paragon-styles button.bg-success-800:focus {
  background-color: #093723 !important;
}
.paragon-styles .text-success-800 {
  color: #11623e !important;
}
.paragon-styles a.text-success-800:hover, .paragon-styles a.text-success-800:focus {
  color: #062115 !important;
}
.paragon-styles .border-success-800 {
  border-color: #11623e !important;
}
.paragon-styles .bg-success-900 {
  background-color: #105b3a !important;
}
.paragon-styles a.bg-success-900:hover, .paragon-styles a.bg-success-900:focus,
.paragon-styles button.bg-success-900:hover,
.paragon-styles button.bg-success-900:focus {
  background-color: #08301e !important;
}
.paragon-styles .text-success-900 {
  color: #105b3a !important;
}
.paragon-styles a.text-success-900:hover, .paragon-styles a.text-success-900:focus {
  color: #051a11 !important;
}
.paragon-styles .border-success-900 {
  border-color: #105b3a !important;
}
.paragon-styles .bg-info-100 {
  background-color: #f0f6fa !important;
}
.paragon-styles a.bg-info-100:hover, .paragon-styles a.bg-info-100:focus,
.paragon-styles button.bg-info-100:hover,
.paragon-styles button.bg-info-100:focus {
  background-color: #cadfed !important;
}
.paragon-styles .text-info-100 {
  color: #f0f6fa !important;
}
.paragon-styles a.text-info-100:hover, .paragon-styles a.text-info-100:focus {
  color: #b7d4e7 !important;
}
.paragon-styles .border-info-100 {
  border-color: #f0f6fa !important;
}
.paragon-styles .bg-info-200 {
  background-color: #bfdbea !important;
}
.paragon-styles a.bg-info-200:hover, .paragon-styles a.bg-info-200:focus,
.paragon-styles button.bg-info-200:hover,
.paragon-styles button.bg-info-200:focus {
  background-color: #99c5dd !important;
}
.paragon-styles .text-info-200 {
  color: #bfdbea !important;
}
.paragon-styles a.text-info-200:hover, .paragon-styles a.text-info-200:focus {
  color: #85bbd7 !important;
}
.paragon-styles .border-info-200 {
  border-color: #bfdbea !important;
}
.paragon-styles .bg-info-300 {
  background-color: #80b6d5 !important;
}
.paragon-styles a.bg-info-300:hover, .paragon-styles a.bg-info-300:focus,
.paragon-styles button.bg-info-300:hover,
.paragon-styles button.bg-info-300:focus {
  background-color: #5aa0c8 !important;
}
.paragon-styles .text-info-300 {
  color: #80b6d5 !important;
}
.paragon-styles a.text-info-300:hover, .paragon-styles a.text-info-300:focus {
  color: #4795c2 !important;
}
.paragon-styles .border-info-300 {
  border-color: #80b6d5 !important;
}
.paragon-styles .bg-info-400 {
  background-color: #4092bf !important;
}
.paragon-styles a.bg-info-400:hover, .paragon-styles a.bg-info-400:focus,
.paragon-styles button.bg-info-400:hover,
.paragon-styles button.bg-info-400:focus {
  background-color: #337599 !important;
}
.paragon-styles .text-info-400 {
  color: #4092bf !important;
}
.paragon-styles a.text-info-400:hover, .paragon-styles a.text-info-400:focus {
  color: #2d6686 !important;
}
.paragon-styles .border-info-400 {
  border-color: #4092bf !important;
}
.paragon-styles .bg-info-500 {
  background-color: #006DAA !important;
}
.paragon-styles a.bg-info-500:hover, .paragon-styles a.bg-info-500:focus,
.paragon-styles button.bg-info-500:hover,
.paragon-styles button.bg-info-500:focus {
  background-color: #004c77 !important;
}
.paragon-styles .text-info-500 {
  color: #006DAA !important;
}
.paragon-styles a.text-info-500:hover, .paragon-styles a.text-info-500:focus {
  color: #003c5e !important;
}
.paragon-styles .border-info-500 {
  border-color: #006DAA !important;
}
.paragon-styles .bg-info-600 {
  background-color: #006299 !important;
}
.paragon-styles a.bg-info-600:hover, .paragon-styles a.bg-info-600:focus,
.paragon-styles button.bg-info-600:hover,
.paragon-styles button.bg-info-600:focus {
  background-color: #004166 !important;
}
.paragon-styles .text-info-600 {
  color: #006299 !important;
}
.paragon-styles a.text-info-600:hover, .paragon-styles a.text-info-600:focus {
  color: #00314d !important;
}
.paragon-styles .border-info-600 {
  border-color: #006299 !important;
}
.paragon-styles .bg-info-700 {
  background-color: #005788 !important;
}
.paragon-styles a.bg-info-700:hover, .paragon-styles a.bg-info-700:focus,
.paragon-styles button.bg-info-700:hover,
.paragon-styles button.bg-info-700:focus {
  background-color: #003655 !important;
}
.paragon-styles .text-info-700 {
  color: #005788 !important;
}
.paragon-styles a.text-info-700:hover, .paragon-styles a.text-info-700:focus {
  color: #00263c !important;
}
.paragon-styles .border-info-700 {
  border-color: #005788 !important;
}
.paragon-styles .bg-info-800 {
  background-color: #005280 !important;
}
.paragon-styles a.bg-info-800:hover, .paragon-styles a.bg-info-800:focus,
.paragon-styles button.bg-info-800:hover,
.paragon-styles button.bg-info-800:focus {
  background-color: #00314d !important;
}
.paragon-styles .text-info-800 {
  color: #005280 !important;
}
.paragon-styles a.text-info-800:hover, .paragon-styles a.text-info-800:focus {
  color: #002134 !important;
}
.paragon-styles .border-info-800 {
  border-color: #005280 !important;
}
.paragon-styles .bg-info-900 {
  background-color: #004c77 !important;
}
.paragon-styles a.bg-info-900:hover, .paragon-styles a.bg-info-900:focus,
.paragon-styles button.bg-info-900:hover,
.paragon-styles button.bg-info-900:focus {
  background-color: #002b44 !important;
}
.paragon-styles .text-info-900 {
  color: #004c77 !important;
}
.paragon-styles a.text-info-900:hover, .paragon-styles a.text-info-900:focus {
  color: #001b2b !important;
}
.paragon-styles .border-info-900 {
  border-color: #004c77 !important;
}
.paragon-styles .bg-warning-100 {
  background-color: #fdf9f0 !important;
}
.paragon-styles a.bg-warning-100:hover, .paragon-styles a.bg-warning-100:focus,
.paragon-styles button.bg-warning-100:hover,
.paragon-styles button.bg-warning-100:focus {
  background-color: #f7e7c3 !important;
}
.paragon-styles .text-warning-100 {
  color: #fdf9f0 !important;
}
.paragon-styles a.text-warning-100:hover, .paragon-styles a.text-warning-100:focus {
  color: #f4dead !important;
}
.paragon-styles .border-warning-100 {
  border-color: #fdf9f0 !important;
}
.paragon-styles .bg-warning-200 {
  background-color: #f5e5bf !important;
}
.paragon-styles a.bg-warning-200:hover, .paragon-styles a.bg-warning-200:focus,
.paragon-styles button.bg-warning-200:hover,
.paragon-styles button.bg-warning-200:focus {
  background-color: #eed393 !important;
}
.paragon-styles .text-warning-200 {
  color: #f5e5bf !important;
}
.paragon-styles a.text-warning-200:hover, .paragon-styles a.text-warning-200:focus {
  color: #ebca7d !important;
}
.paragon-styles .border-warning-200 {
  border-color: #f5e5bf !important;
}
.paragon-styles .bg-warning-300 {
  background-color: #ebcb80 !important;
}
.paragon-styles a.bg-warning-300:hover, .paragon-styles a.bg-warning-300:focus,
.paragon-styles button.bg-warning-300:hover,
.paragon-styles button.bg-warning-300:focus {
  background-color: #e4b954 !important;
}
.paragon-styles .text-warning-300 {
  color: #ebcb80 !important;
}
.paragon-styles a.text-warning-300:hover, .paragon-styles a.text-warning-300:focus {
  color: #e1b03e !important;
}
.paragon-styles .border-warning-300 {
  border-color: #ebcb80 !important;
}
.paragon-styles .bg-warning-400 {
  background-color: #e0b040 !important;
}
.paragon-styles a.bg-warning-400:hover, .paragon-styles a.bg-warning-400:focus,
.paragon-styles button.bg-warning-400:hover,
.paragon-styles button.bg-warning-400:focus {
  background-color: #cc9921 !important;
}
.paragon-styles .text-warning-400 {
  color: #e0b040 !important;
}
.paragon-styles a.text-warning-400:hover, .paragon-styles a.text-warning-400:focus {
  color: #b6881e !important;
}
.paragon-styles .border-warning-400 {
  border-color: #e0b040 !important;
}
.paragon-styles .bg-warning-500 {
  background-color: #D69600 !important;
}
.paragon-styles a.bg-warning-500:hover, .paragon-styles a.bg-warning-500:focus,
.paragon-styles button.bg-warning-500:hover,
.paragon-styles button.bg-warning-500:focus {
  background-color: #a37200 !important;
}
.paragon-styles .text-warning-500 {
  color: #D69600 !important;
}
.paragon-styles a.text-warning-500:hover, .paragon-styles a.text-warning-500:focus {
  color: #8a6000 !important;
}
.paragon-styles .border-warning-500 {
  border-color: #D69600 !important;
}
.paragon-styles .bg-warning-600 {
  background-color: #c18700 !important;
}
.paragon-styles a.bg-warning-600:hover, .paragon-styles a.bg-warning-600:focus,
.paragon-styles button.bg-warning-600:hover,
.paragon-styles button.bg-warning-600:focus {
  background-color: #8e6300 !important;
}
.paragon-styles .text-warning-600 {
  color: #c18700 !important;
}
.paragon-styles a.text-warning-600:hover, .paragon-styles a.text-warning-600:focus {
  color: #755100 !important;
}
.paragon-styles .border-warning-600 {
  border-color: #c18700 !important;
}
.paragon-styles .bg-warning-700 {
  background-color: #ab7800 !important;
}
.paragon-styles a.bg-warning-700:hover, .paragon-styles a.bg-warning-700:focus,
.paragon-styles button.bg-warning-700:hover,
.paragon-styles button.bg-warning-700:focus {
  background-color: #785400 !important;
}
.paragon-styles .text-warning-700 {
  color: #ab7800 !important;
}
.paragon-styles a.text-warning-700:hover, .paragon-styles a.text-warning-700:focus {
  color: #5f4200 !important;
}
.paragon-styles .border-warning-700 {
  border-color: #ab7800 !important;
}
.paragon-styles .bg-warning-800 {
  background-color: #a17100 !important;
}
.paragon-styles a.bg-warning-800:hover, .paragon-styles a.bg-warning-800:focus,
.paragon-styles button.bg-warning-800:hover,
.paragon-styles button.bg-warning-800:focus {
  background-color: #6e4d00 !important;
}
.paragon-styles .text-warning-800 {
  color: #a17100 !important;
}
.paragon-styles a.text-warning-800:hover, .paragon-styles a.text-warning-800:focus {
  color: #553b00 !important;
}
.paragon-styles .border-warning-800 {
  border-color: #a17100 !important;
}
.paragon-styles .bg-warning-900 {
  background-color: #966900 !important;
}
.paragon-styles a.bg-warning-900:hover, .paragon-styles a.bg-warning-900:focus,
.paragon-styles button.bg-warning-900:hover,
.paragon-styles button.bg-warning-900:focus {
  background-color: #634500 !important;
}
.paragon-styles .text-warning-900 {
  color: #966900 !important;
}
.paragon-styles a.text-warning-900:hover, .paragon-styles a.text-warning-900:focus {
  color: #4a3300 !important;
}
.paragon-styles .border-warning-900 {
  border-color: #966900 !important;
}
.paragon-styles .bg-danger-100 {
  background-color: #fbf2f3 !important;
}
.paragon-styles a.bg-danger-100:hover, .paragon-styles a.bg-danger-100:focus,
.paragon-styles button.bg-danger-100:hover,
.paragon-styles button.bg-danger-100:focus {
  background-color: #efcbcf !important;
}
.paragon-styles .text-danger-100 {
  color: #fbf2f3 !important;
}
.paragon-styles a.text-danger-100:hover, .paragon-styles a.text-danger-100:focus {
  color: #e9b8bd !important;
}
.paragon-styles .border-danger-100 {
  border-color: #fbf2f3 !important;
}
.paragon-styles .bg-danger-200 {
  background-color: #f0cbce !important;
}
.paragon-styles a.bg-danger-200:hover, .paragon-styles a.bg-danger-200:focus,
.paragon-styles button.bg-danger-200:hover,
.paragon-styles button.bg-danger-200:focus {
  background-color: #e5a3a9 !important;
}
.paragon-styles .text-danger-200 {
  color: #f0cbce !important;
}
.paragon-styles a.text-danger-200:hover, .paragon-styles a.text-danger-200:focus {
  color: #df9096 !important;
}
.paragon-styles .border-danger-200 {
  border-color: #f0cbce !important;
}
.paragon-styles .bg-danger-300 {
  background-color: #e1969d !important;
}
.paragon-styles a.bg-danger-300:hover, .paragon-styles a.bg-danger-300:focus,
.paragon-styles button.bg-danger-300:hover,
.paragon-styles button.bg-danger-300:focus {
  background-color: #d66e78 !important;
}
.paragon-styles .text-danger-300 {
  color: #e1969d !important;
}
.paragon-styles a.text-danger-300:hover, .paragon-styles a.text-danger-300:focus {
  color: #d05b65 !important;
}
.paragon-styles .border-danger-300 {
  border-color: #e1969d !important;
}
.paragon-styles .bg-danger-400 {
  background-color: #d2626b !important;
}
.paragon-styles a.bg-danger-400:hover, .paragon-styles a.bg-danger-400:focus,
.paragon-styles button.bg-danger-400:hover,
.paragon-styles button.bg-danger-400:focus {
  background-color: #c73a46 !important;
}
.paragon-styles .text-danger-400 {
  color: #d2626b !important;
}
.paragon-styles a.text-danger-400:hover, .paragon-styles a.text-danger-400:focus {
  color: #b4343e !important;
}
.paragon-styles .border-danger-400 {
  border-color: #d2626b !important;
}
.paragon-styles .bg-danger-500 {
  background-color: #C32D3A !important;
}
.paragon-styles a.bg-danger-500:hover, .paragon-styles a.bg-danger-500:focus,
.paragon-styles button.bg-danger-500:hover,
.paragon-styles button.bg-danger-500:focus {
  background-color: #9a232e !important;
}
.paragon-styles .text-danger-500 {
  color: #C32D3A !important;
}
.paragon-styles a.text-danger-500:hover, .paragon-styles a.text-danger-500:focus {
  color: #851f28 !important;
}
.paragon-styles .border-danger-500 {
  border-color: #C32D3A !important;
}
.paragon-styles .bg-danger-600 {
  background-color: #b02934 !important;
}
.paragon-styles a.bg-danger-600:hover, .paragon-styles a.bg-danger-600:focus,
.paragon-styles button.bg-danger-600:hover,
.paragon-styles button.bg-danger-600:focus {
  background-color: #871f28 !important;
}
.paragon-styles .text-danger-600 {
  color: #b02934 !important;
}
.paragon-styles a.text-danger-600:hover, .paragon-styles a.text-danger-600:focus {
  color: #721b22 !important;
}
.paragon-styles .border-danger-600 {
  border-color: #b02934 !important;
}
.paragon-styles .bg-danger-700 {
  background-color: #9c242e !important;
}
.paragon-styles a.bg-danger-700:hover, .paragon-styles a.bg-danger-700:focus,
.paragon-styles button.bg-danger-700:hover,
.paragon-styles button.bg-danger-700:focus {
  background-color: #731a22 !important;
}
.paragon-styles .text-danger-700 {
  color: #9c242e !important;
}
.paragon-styles a.text-danger-700:hover, .paragon-styles a.text-danger-700:focus {
  color: #5e161c !important;
}
.paragon-styles .border-danger-700 {
  border-color: #9c242e !important;
}
.paragon-styles .bg-danger-800 {
  background-color: #92222c !important;
}
.paragon-styles a.bg-danger-800:hover, .paragon-styles a.bg-danger-800:focus,
.paragon-styles button.bg-danger-800:hover,
.paragon-styles button.bg-danger-800:focus {
  background-color: #691820 !important;
}
.paragon-styles .text-danger-800 {
  color: #92222c !important;
}
.paragon-styles a.text-danger-800:hover, .paragon-styles a.text-danger-800:focus {
  color: #541419 !important;
}
.paragon-styles .border-danger-800 {
  border-color: #92222c !important;
}
.paragon-styles .bg-danger-900 {
  background-color: #892029 !important;
}
.paragon-styles a.bg-danger-900:hover, .paragon-styles a.bg-danger-900:focus,
.paragon-styles button.bg-danger-900:hover,
.paragon-styles button.bg-danger-900:focus {
  background-color: #60161d !important;
}
.paragon-styles .text-danger-900 {
  color: #892029 !important;
}
.paragon-styles a.text-danger-900:hover, .paragon-styles a.text-danger-900:focus {
  color: #4b1216 !important;
}
.paragon-styles .border-danger-900 {
  border-color: #892029 !important;
}
.paragon-styles .bg-light-100 {
  background-color: #fefefe !important;
}
.paragon-styles a.bg-light-100:hover, .paragon-styles a.bg-light-100:focus,
.paragon-styles button.bg-light-100:hover,
.paragon-styles button.bg-light-100:focus {
  background-color: #e5e5e5 !important;
}
.paragon-styles .text-light-100 {
  color: #fefefe !important;
}
.paragon-styles a.text-light-100:hover, .paragon-styles a.text-light-100:focus {
  color: #d8d8d8 !important;
}
.paragon-styles .border-light-100 {
  border-color: #fefefe !important;
}
.paragon-styles .bg-light-200 {
  background-color: #fbfbfb !important;
}
.paragon-styles a.bg-light-200:hover, .paragon-styles a.bg-light-200:focus,
.paragon-styles button.bg-light-200:hover,
.paragon-styles button.bg-light-200:focus {
  background-color: #e2e2e2 !important;
}
.paragon-styles .text-light-200 {
  color: #fbfbfb !important;
}
.paragon-styles a.text-light-200:hover, .paragon-styles a.text-light-200:focus {
  color: #d5d5d5 !important;
}
.paragon-styles .border-light-200 {
  border-color: #fbfbfb !important;
}
.paragon-styles .bg-light-300 {
  background-color: #f7f7f7 !important;
}
.paragon-styles a.bg-light-300:hover, .paragon-styles a.bg-light-300:focus,
.paragon-styles button.bg-light-300:hover,
.paragon-styles button.bg-light-300:focus {
  background-color: #dedede !important;
}
.paragon-styles .text-light-300 {
  color: #f7f7f7 !important;
}
.paragon-styles a.text-light-300:hover, .paragon-styles a.text-light-300:focus {
  color: #d1d1d1 !important;
}
.paragon-styles .border-light-300 {
  border-color: #f7f7f7 !important;
}
.paragon-styles .bg-light-400 {
  background-color: #f2f2f2 !important;
}
.paragon-styles a.bg-light-400:hover, .paragon-styles a.bg-light-400:focus,
.paragon-styles button.bg-light-400:hover,
.paragon-styles button.bg-light-400:focus {
  background-color: #d9d9d9 !important;
}
.paragon-styles .text-light-400 {
  color: #f2f2f2 !important;
}
.paragon-styles a.text-light-400:hover, .paragon-styles a.text-light-400:focus {
  color: #cccccc !important;
}
.paragon-styles .border-light-400 {
  border-color: #f2f2f2 !important;
}
.paragon-styles .bg-light-500 {
  background-color: #eee !important;
}
.paragon-styles a.bg-light-500:hover, .paragon-styles a.bg-light-500:focus,
.paragon-styles button.bg-light-500:hover,
.paragon-styles button.bg-light-500:focus {
  background-color: #d5d5d5 !important;
}
.paragon-styles .text-light-500 {
  color: #eee !important;
}
.paragon-styles a.text-light-500:hover, .paragon-styles a.text-light-500:focus {
  color: #c8c8c8 !important;
}
.paragon-styles .border-light-500 {
  border-color: #eee !important;
}
.paragon-styles .bg-light-600 {
  background-color: #d6d6d6 !important;
}
.paragon-styles a.bg-light-600:hover, .paragon-styles a.bg-light-600:focus,
.paragon-styles button.bg-light-600:hover,
.paragon-styles button.bg-light-600:focus {
  background-color: #bdbdbd !important;
}
.paragon-styles .text-light-600 {
  color: #d6d6d6 !important;
}
.paragon-styles a.text-light-600:hover, .paragon-styles a.text-light-600:focus {
  color: #b0b0b0 !important;
}
.paragon-styles .border-light-600 {
  border-color: #d6d6d6 !important;
}
.paragon-styles .bg-light-700 {
  background-color: #D7D3D1 !important;
}
.paragon-styles a.bg-light-700:hover, .paragon-styles a.bg-light-700:focus,
.paragon-styles button.bg-light-700:hover,
.paragon-styles button.bg-light-700:focus {
  background-color: #bfb9b6 !important;
}
.paragon-styles .text-light-700 {
  color: #D7D3D1 !important;
}
.paragon-styles a.text-light-700:hover, .paragon-styles a.text-light-700:focus {
  color: #b3aca8 !important;
}
.paragon-styles .border-light-700 {
  border-color: #D7D3D1 !important;
}
.paragon-styles .bg-light-800 {
  background-color: #b3b3b3 !important;
}
.paragon-styles a.bg-light-800:hover, .paragon-styles a.bg-light-800:focus,
.paragon-styles button.bg-light-800:hover,
.paragon-styles button.bg-light-800:focus {
  background-color: #9a9a9a !important;
}
.paragon-styles .text-light-800 {
  color: #b3b3b3 !important;
}
.paragon-styles a.text-light-800:hover, .paragon-styles a.text-light-800:focus {
  color: #8d8d8d !important;
}
.paragon-styles .border-light-800 {
  border-color: #b3b3b3 !important;
}
.paragon-styles .bg-light-900 {
  background-color: #a7a7a7 !important;
}
.paragon-styles a.bg-light-900:hover, .paragon-styles a.bg-light-900:focus,
.paragon-styles button.bg-light-900:hover,
.paragon-styles button.bg-light-900:focus {
  background-color: #8e8e8e !important;
}
.paragon-styles .text-light-900 {
  color: #a7a7a7 !important;
}
.paragon-styles a.text-light-900:hover, .paragon-styles a.text-light-900:focus {
  color: #818181 !important;
}
.paragon-styles .border-light-900 {
  border-color: #a7a7a7 !important;
}
.paragon-styles .bg-dark-100 {
  background-color: #f4f5f5 !important;
}
.paragon-styles a.bg-dark-100:hover, .paragon-styles a.bg-dark-100:focus,
.paragon-styles button.bg-dark-100:hover,
.paragon-styles button.bg-dark-100:focus {
  background-color: #d9dddd !important;
}
.paragon-styles .text-dark-100 {
  color: #f4f5f5 !important;
}
.paragon-styles a.text-dark-100:hover, .paragon-styles a.text-dark-100:focus {
  color: #ccd1d1 !important;
}
.paragon-styles .border-dark-100 {
  border-color: #f4f5f5 !important;
}
.paragon-styles .bg-dark-200 {
  background-color: #d2d3d5 !important;
}
.paragon-styles a.bg-dark-200:hover, .paragon-styles a.bg-dark-200:focus,
.paragon-styles button.bg-dark-200:hover,
.paragon-styles button.bg-dark-200:focus {
  background-color: #b8b9bc !important;
}
.paragon-styles .text-dark-200 {
  color: #d2d3d5 !important;
}
.paragon-styles a.text-dark-200:hover, .paragon-styles a.text-dark-200:focus {
  color: #aaacb0 !important;
}
.paragon-styles .border-dark-200 {
  border-color: #d2d3d5 !important;
}
.paragon-styles .bg-dark-300 {
  background-color: #a4a8ab !important;
}
.paragon-styles a.bg-dark-300:hover, .paragon-styles a.bg-dark-300:focus,
.paragon-styles button.bg-dark-300:hover,
.paragon-styles button.bg-dark-300:focus {
  background-color: #898f93 !important;
}
.paragon-styles .text-dark-300 {
  color: #a4a8ab !important;
}
.paragon-styles a.text-dark-300:hover, .paragon-styles a.text-dark-300:focus {
  color: #7c8286 !important;
}
.paragon-styles .border-dark-300 {
  border-color: #a4a8ab !important;
}
.paragon-styles .bg-dark-400 {
  background-color: #777c81 !important;
}
.paragon-styles a.bg-dark-400:hover, .paragon-styles a.bg-dark-400:focus,
.paragon-styles button.bg-dark-400:hover,
.paragon-styles button.bg-dark-400:focus {
  background-color: #5f6366 !important;
}
.paragon-styles .text-dark-400 {
  color: #777c81 !important;
}
.paragon-styles a.text-dark-400:hover, .paragon-styles a.text-dark-400:focus {
  color: #525659 !important;
}
.paragon-styles .border-dark-400 {
  border-color: #777c81 !important;
}
.paragon-styles .bg-dark-500 {
  background-color: #495057 !important;
}
.paragon-styles a.bg-dark-500:hover, .paragon-styles a.bg-dark-500:focus,
.paragon-styles button.bg-dark-500:hover,
.paragon-styles button.bg-dark-500:focus {
  background-color: #32373b !important;
}
.paragon-styles .text-dark-500 {
  color: #495057 !important;
}
.paragon-styles a.text-dark-500:hover, .paragon-styles a.text-dark-500:focus {
  color: #262a2d !important;
}
.paragon-styles .border-dark-500 {
  border-color: #495057 !important;
}
.paragon-styles .bg-dark-600 {
  background-color: #42484e !important;
}
.paragon-styles a.bg-dark-600:hover, .paragon-styles a.bg-dark-600:focus,
.paragon-styles button.bg-dark-600:hover,
.paragon-styles button.bg-dark-600:focus {
  background-color: #2b2f32 !important;
}
.paragon-styles .text-dark-600 {
  color: #42484e !important;
}
.paragon-styles a.text-dark-600:hover, .paragon-styles a.text-dark-600:focus {
  color: #1f2225 !important;
}
.paragon-styles .border-dark-600 {
  border-color: #42484e !important;
}
.paragon-styles .bg-dark-700 {
  background-color: #3a4046 !important;
}
.paragon-styles a.bg-dark-700:hover, .paragon-styles a.bg-dark-700:focus,
.paragon-styles button.bg-dark-700:hover,
.paragon-styles button.bg-dark-700:focus {
  background-color: #23272a !important;
}
.paragon-styles .text-dark-700 {
  color: #3a4046 !important;
}
.paragon-styles a.text-dark-700:hover, .paragon-styles a.text-dark-700:focus {
  color: #171a1c !important;
}
.paragon-styles .border-dark-700 {
  border-color: #3a4046 !important;
}
.paragon-styles .bg-dark-800 {
  background-color: #373c41 !important;
}
.paragon-styles a.bg-dark-800:hover, .paragon-styles a.bg-dark-800:focus,
.paragon-styles button.bg-dark-800:hover,
.paragon-styles button.bg-dark-800:focus {
  background-color: #202325 !important;
}
.paragon-styles .text-dark-800 {
  color: #373c41 !important;
}
.paragon-styles a.text-dark-800:hover, .paragon-styles a.text-dark-800:focus {
  color: #141618 !important;
}
.paragon-styles .border-dark-800 {
  border-color: #373c41 !important;
}
.paragon-styles .bg-dark-900 {
  background-color: #33383d !important;
}
.paragon-styles a.bg-dark-900:hover, .paragon-styles a.bg-dark-900:focus,
.paragon-styles button.bg-dark-900:hover,
.paragon-styles button.bg-dark-900:focus {
  background-color: #1c1f21 !important;
}
.paragon-styles .text-dark-900 {
  color: #33383d !important;
}
.paragon-styles a.text-dark-900:hover, .paragon-styles a.text-dark-900:focus {
  color: #101213 !important;
}
.paragon-styles .border-dark-900 {
  border-color: #33383d !important;
}
.paragon-styles .bg-brand-100 {
  background-color: #f7fafc !important;
}
.paragon-styles a.bg-brand-100:hover, .paragon-styles a.bg-brand-100:focus,
.paragon-styles button.bg-brand-100:hover,
.paragon-styles button.bg-brand-100:focus {
  background-color: #d2e3ee !important;
}
.paragon-styles .text-brand-100 {
  color: #f7fafc !important;
}
.paragon-styles a.text-brand-100:hover, .paragon-styles a.text-brand-100:focus {
  color: #bfd7e7 !important;
}
.paragon-styles .border-brand-100 {
  border-color: #f7fafc !important;
}
.paragon-styles .bg-brand-200 {
  background-color: #dfeaf2 !important;
}
.paragon-styles a.bg-brand-200:hover, .paragon-styles a.bg-brand-200:focus,
.paragon-styles button.bg-brand-200:hover,
.paragon-styles button.bg-brand-200:focus {
  background-color: #bbd2e3 !important;
}
.paragon-styles .text-brand-200 {
  color: #dfeaf2 !important;
}
.paragon-styles a.text-brand-200:hover, .paragon-styles a.text-brand-200:focus {
  color: #a9c6dc !important;
}
.paragon-styles .border-brand-200 {
  border-color: #dfeaf2 !important;
}
.paragon-styles .bg-brand-300 {
  background-color: #c0d4e6 !important;
}
.paragon-styles a.bg-brand-300:hover, .paragon-styles a.bg-brand-300:focus,
.paragon-styles button.bg-brand-300:hover,
.paragon-styles button.bg-brand-300:focus {
  background-color: #9bbbd8 !important;
}
.paragon-styles .text-brand-300 {
  color: #c0d4e6 !important;
}
.paragon-styles a.text-brand-300:hover, .paragon-styles a.text-brand-300:focus {
  color: #89afd0 !important;
}
.paragon-styles .border-brand-300 {
  border-color: #c0d4e6 !important;
}
.paragon-styles .bg-brand-400 {
  background-color: #a0bfd9 !important;
}
.paragon-styles a.bg-brand-400:hover, .paragon-styles a.bg-brand-400:focus,
.paragon-styles button.bg-brand-400:hover,
.paragon-styles button.bg-brand-400:focus {
  background-color: #7ca6ca !important;
}
.paragon-styles .text-brand-400 {
  color: #a0bfd9 !important;
}
.paragon-styles a.text-brand-400:hover, .paragon-styles a.text-brand-400:focus {
  color: #699ac3 !important;
}
.paragon-styles .border-brand-400 {
  border-color: #a0bfd9 !important;
}
.paragon-styles .bg-brand-500 {
  background-color: #80a9cc !important;
}
.paragon-styles a.bg-brand-500:hover, .paragon-styles a.bg-brand-500:focus,
.paragon-styles button.bg-brand-500:hover,
.paragon-styles button.bg-brand-500:focus {
  background-color: #5c90bd !important;
}
.paragon-styles .text-brand-500 {
  color: #80a9cc !important;
}
.paragon-styles a.text-brand-500:hover, .paragon-styles a.text-brand-500:focus {
  color: #4984b6 !important;
}
.paragon-styles .border-brand-500 {
  border-color: #80a9cc !important;
}
.paragon-styles .bg-brand-600 {
  background-color: #7398b8 !important;
}
.paragon-styles a.bg-brand-600:hover, .paragon-styles a.bg-brand-600:focus,
.paragon-styles button.bg-brand-600:hover,
.paragon-styles button.bg-brand-600:focus {
  background-color: #537fa5 !important;
}
.paragon-styles .text-brand-600 {
  color: #7398b8 !important;
}
.paragon-styles a.text-brand-600:hover, .paragon-styles a.text-brand-600:focus {
  color: #4b7294 !important;
}
.paragon-styles .border-brand-600 {
  border-color: #7398b8 !important;
}
.paragon-styles .bg-brand-700 {
  background-color: #6687a3 !important;
}
.paragon-styles a.bg-brand-700:hover, .paragon-styles a.bg-brand-700:focus,
.paragon-styles button.bg-brand-700:hover,
.paragon-styles button.bg-brand-700:focus {
  background-color: #506d86 !important;
}
.paragon-styles .text-brand-700 {
  color: #6687a3 !important;
}
.paragon-styles a.text-brand-700:hover, .paragon-styles a.text-brand-700:focus {
  color: #476076 !important;
}
.paragon-styles .border-brand-700 {
  border-color: #6687a3 !important;
}
.paragon-styles .bg-brand-800 {
  background-color: #607f99 !important;
}
.paragon-styles a.bg-brand-800:hover, .paragon-styles a.bg-brand-800:focus,
.paragon-styles button.bg-brand-800:hover,
.paragon-styles button.bg-brand-800:focus {
  background-color: #4c657a !important;
}
.paragon-styles .text-brand-800 {
  color: #607f99 !important;
}
.paragon-styles a.text-brand-800:hover, .paragon-styles a.text-brand-800:focus {
  color: #43586a !important;
}
.paragon-styles .border-brand-800 {
  border-color: #607f99 !important;
}
.paragon-styles .bg-brand-900 {
  background-color: #5a768f !important;
}
.paragon-styles a.bg-brand-900:hover, .paragon-styles a.bg-brand-900:focus,
.paragon-styles button.bg-brand-900:hover,
.paragon-styles button.bg-brand-900:focus {
  background-color: #465c70 !important;
}
.paragon-styles .text-brand-900 {
  color: #5a768f !important;
}
.paragon-styles a.text-brand-900:hover, .paragon-styles a.text-brand-900:focus {
  color: #3c4f60 !important;
}
.paragon-styles .border-brand-900 {
  border-color: #5a768f !important;
}
.paragon-styles .bg-gray-100 {
  background-color: #f8f9fa !important;
}
.paragon-styles a.bg-gray-100:hover, .paragon-styles a.bg-gray-100:focus,
.paragon-styles button.bg-gray-100:hover,
.paragon-styles button.bg-gray-100:focus {
  background-color: #dae0e5 !important;
}
.paragon-styles .text-gray-100 {
  color: #f8f9fa !important;
}
.paragon-styles a.text-gray-100:hover, .paragon-styles a.text-gray-100:focus {
  color: #cbd3da !important;
}
.paragon-styles .border-gray-100 {
  border-color: #f8f9fa !important;
}
.paragon-styles .bg-gray-200 {
  background-color: #e9ecef !important;
}
.paragon-styles a.bg-gray-200:hover, .paragon-styles a.bg-gray-200:focus,
.paragon-styles button.bg-gray-200:hover,
.paragon-styles button.bg-gray-200:focus {
  background-color: #cbd3da !important;
}
.paragon-styles .text-gray-200 {
  color: #e9ecef !important;
}
.paragon-styles a.text-gray-200:hover, .paragon-styles a.text-gray-200:focus {
  color: #bdc6cf !important;
}
.paragon-styles .border-gray-200 {
  border-color: #e9ecef !important;
}
.paragon-styles .bg-gray-300 {
  background-color: #dee2e6 !important;
}
.paragon-styles a.bg-gray-300:hover, .paragon-styles a.bg-gray-300:focus,
.paragon-styles button.bg-gray-300:hover,
.paragon-styles button.bg-gray-300:focus {
  background-color: #c1c9d0 !important;
}
.paragon-styles .text-gray-300 {
  color: #dee2e6 !important;
}
.paragon-styles a.text-gray-300:hover, .paragon-styles a.text-gray-300:focus {
  color: #b2bcc5 !important;
}
.paragon-styles .border-gray-300 {
  border-color: #dee2e6 !important;
}
.paragon-styles .bg-gray-400 {
  background-color: #ced4da !important;
}
.paragon-styles a.bg-gray-400:hover, .paragon-styles a.bg-gray-400:focus,
.paragon-styles button.bg-gray-400:hover,
.paragon-styles button.bg-gray-400:focus {
  background-color: #b1bbc4 !important;
}
.paragon-styles .text-gray-400 {
  color: #ced4da !important;
}
.paragon-styles a.text-gray-400:hover, .paragon-styles a.text-gray-400:focus {
  color: #a2aeb9 !important;
}
.paragon-styles .border-gray-400 {
  border-color: #ced4da !important;
}
.paragon-styles .bg-gray-500 {
  background-color: #adb5bd !important;
}
.paragon-styles a.bg-gray-500:hover, .paragon-styles a.bg-gray-500:focus,
.paragon-styles button.bg-gray-500:hover,
.paragon-styles button.bg-gray-500:focus {
  background-color: #919ca6 !important;
}
.paragon-styles .text-gray-500 {
  color: #adb5bd !important;
}
.paragon-styles a.text-gray-500:hover, .paragon-styles a.text-gray-500:focus {
  color: #838f9b !important;
}
.paragon-styles .border-gray-500 {
  border-color: #adb5bd !important;
}
.paragon-styles .bg-gray-600 {
  background-color: #6c757d !important;
}
.paragon-styles a.bg-gray-600:hover, .paragon-styles a.bg-gray-600:focus,
.paragon-styles button.bg-gray-600:hover,
.paragon-styles button.bg-gray-600:focus {
  background-color: #545b62 !important;
}
.paragon-styles .text-gray-600 {
  color: #6c757d !important;
}
.paragon-styles a.text-gray-600:hover, .paragon-styles a.text-gray-600:focus {
  color: #494f54 !important;
}
.paragon-styles .border-gray-600 {
  border-color: #6c757d !important;
}
.paragon-styles .bg-gray-700 {
  background-color: #495057 !important;
}
.paragon-styles a.bg-gray-700:hover, .paragon-styles a.bg-gray-700:focus,
.paragon-styles button.bg-gray-700:hover,
.paragon-styles button.bg-gray-700:focus {
  background-color: #32373b !important;
}
.paragon-styles .text-gray-700 {
  color: #495057 !important;
}
.paragon-styles a.text-gray-700:hover, .paragon-styles a.text-gray-700:focus {
  color: #262a2d !important;
}
.paragon-styles .border-gray-700 {
  border-color: #495057 !important;
}
.paragon-styles .bg-gray-800 {
  background-color: #333333 !important;
}
.paragon-styles a.bg-gray-800:hover, .paragon-styles a.bg-gray-800:focus,
.paragon-styles button.bg-gray-800:hover,
.paragon-styles button.bg-gray-800:focus {
  background-color: #1a1a1a !important;
}
.paragon-styles .text-gray-800 {
  color: #333333 !important;
}
.paragon-styles a.text-gray-800:hover, .paragon-styles a.text-gray-800:focus {
  color: #0d0d0d !important;
}
.paragon-styles .border-gray-800 {
  border-color: #333333 !important;
}
.paragon-styles .bg-gray-900 {
  background-color: #212529 !important;
}
.paragon-styles a.bg-gray-900:hover, .paragon-styles a.bg-gray-900:focus,
.paragon-styles button.bg-gray-900:hover,
.paragon-styles button.bg-gray-900:focus {
  background-color: #0a0c0d !important;
}
.paragon-styles .text-gray-900 {
  color: #212529 !important;
}
.paragon-styles a.text-gray-900:hover, .paragon-styles a.text-gray-900:focus {
  color: black !important;
}
.paragon-styles .border-gray-900 {
  border-color: #212529 !important;
}
.paragon-styles .x-small {
  font-size: 75%;
}
